Package: spatialEco
Type: Package
Title: Spatial Analysis and Modelling Utilities
Version: 2.0-5
Date: 2026-05-19
Authors@R: c(
    person(family="Evans", given="Jeffrey S.", email = "sage_insights@outlook.com", 
         role = c("aut", "cre"), comment = c(ORCID = "0000-0002-5533-7044")),
  person(family="Murphy", given="Melanie A.", role = "ctb"),			 
  person(family="Ram", given="Karthik", role = "ctb") )
Description: Utilities to support spatial data manipulation, query, sampling
    and modelling in ecological applications. Functions include models for species 
	population density, spatial smoothing, multivariate separability, point process 
	model for creating pseudo- absences and sub-sampling, Quadrant-based sampling and 
	analysis, auto-logistic modeling, sampling models, cluster optimization, statistical 
	exploratory tools and raster-based metrics.
Depends: R (>= 4.2)
Imports: sf, terra
Suggests: spatstat.geom (>= 3.0-3), spatstat.explore, spdep, ks,
        cluster, readr, RCurl, RANN, rms, yaImpute, mgcv, zyp,
        SpatialPack (>= 0.3), MASS, caret, dplyr, earth, Matrix, gstat,
        spatstat.data, methods, units, sp, stringr, lwgeom, geodata
Maintainer: Jeffrey S. Evans <sage_insights@outlook.com>
License: GPL-3
URL: https://github.com/jeffreyevans/spatialEco,
        https://jeffreyevans.github.io/spatialEco/
BugReports: https://github.com/jeffreyevans/spatialEco/issues
NeedsCompilation: no
Repository: CRAN
LazyData: true
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
Packaged: 2026-05-19 18:19:06 UTC; sage_
Author: Jeffrey S. Evans [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5533-7044>),
  Melanie A. Murphy [ctb],
  Karthik Ram [ctb]
Date/Publication: 2026-05-20 06:20:03 UTC
Built: R 4.6.0; ; 2026-05-20 08:24:55 UTC; unix

Package: spatialEco
Type: Package
Title: Spatial Analysis and Modelling Utilities
Version: 2.0-5
Date: 2026-05-19
Authors@R: c(
    person(family="Evans", given="Jeffrey S.", email = "sage_insights@outlook.com", 
         role = c("aut", "cre"), comment = c(ORCID = "0000-0002-5533-7044")),
  person(family="Murphy", given="Melanie A.", role = "ctb"),			 
  person(family="Ram", given="Karthik", role = "ctb") )
Description: Utilities to support spatial data manipulation, query, sampling
    and modelling in ecological applications. Functions include models for species 
	population density, spatial smoothing, multivariate separability, point process 
	model for creating pseudo- absences and sub-sampling, Quadrant-based sampling and 
	analysis, auto-logistic modeling, sampling models, cluster optimization, statistical 
	exploratory tools and raster-based metrics.
Depends: R (>= 4.2)
Imports: sf, terra
Suggests: spatstat.geom (>= 3.0-3), spatstat.explore, spdep, ks,
        cluster, readr, RCurl, RANN, rms, yaImpute, mgcv, zyp,
        SpatialPack (>= 0.3), MASS, caret, dplyr, earth, Matrix, gstat,
        spatstat.data, methods, units, sp, stringr, lwgeom, geodata
Maintainer: Jeffrey S. Evans <sage_insights@outlook.com>
License: GPL-3
URL: https://github.com/jeffreyevans/spatialEco,
        https://jeffreyevans.github.io/spatialEco/
BugReports: https://github.com/jeffreyevans/spatialEco/issues
NeedsCompilation: no
Repository: CRAN
LazyData: true
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
Packaged: 2026-05-19 18:19:06 UTC; sage_
Author: Jeffrey S. Evans [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5533-7044>),
  Melanie A. Murphy [ctb],
  Karthik Ram [ctb]
Date/Publication: 2026-05-20 06:20:03 UTC
Built: R 4.6.0; ; 2026-05-22 00:21:47 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(plot,effect.size)
S3method(plot,loess.boot)
S3method(print,cross.cor)
S3method(print,effect.size)
S3method(print,loess.boot)
S3method(print,poly.trend)
S3method(summary,cross.cor)
S3method(summary,effect.size)
S3method(summary,loess.boot)
export(all_pairwise)
export(annulus.matrix)
export(aspline.downscale)
export(background)
export(bbox_poly)
export(bearing.distance)
export(breeding.density)
export(built.index)
export(cgls_urls)
export(chae)
export(classBreaks)
export(collinear)
export(combine)
export(concordance)
export(conf.interval)
export(correlogram)
export(cross.tab)
export(crossCorrelation)
export(csi)
export(curvature)
export(dahi)
export(date_seq)
export(daymet.point)
export(daymet.tiles)
export(dispersion)
export(dissection)
export(divergence)
export(effect.size)
export(erase.point)
export(extract.vertices)
export(fuzzySum)
export(gaussian.kernel)
export(geo.buffer)
export(group.pdf)
export(hexagons)
export(hli)
export(hli.pt)
export(hsp)
export(hybrid.kmeans)
export(idw.smoothing)
export(impute.loess)
export(insert)
export(insert.values)
export(is.empty)
export(kendall)
export(kl.divergence)
export(knn)
export(lai)
export(local.min.max)
export(loess.boot)
export(loess.ci)
export(logistic.regression)
export(max_extent)
export(mean_angle)
export(moments)
export(morans.plot)
export(nni)
export(nth.values)
export(o.ring)
export(oli.aws)
export(optimal.k)
export(optimized.sample.variance)
export(outliers)
export(overlap)
export(parea.sample)
export(parse.bits)
export(partial.cor)
export(poly.regression)
export(polyPerimeter)
export(poly_trend)
export(pp.subsample)
export(proximity.index)
export(pseudo.absence)
export(quadrats)
export(random.raster)
export(raster.Zscore)
export(raster.change)
export(raster.deviation)
export(raster.downscale)
export(raster.entropy)
export(raster.gaussian.smooth)
export(raster.invert)
export(raster.kendall)
export(raster.mds)
export(raster.modified.ttest)
export(raster.moments)
export(raster.transformation)
export(raster.vol)
export(rasterCorrelation)
export(rasterDistance)
export(remove.holes)
export(remove_duplicates)
export(rm.ext)
export(rotate.polygon)
export(sa.trans)
export(sample.annulus)
export(sampleTransect)
export(sar)
export(separability)
export(sf.kde)
export(sf_dissolve)
export(sg.smooth)
export(shannons)
export(shift)
export(sieve)
export(similarity)
export(smooth.time.series)
export(sobal)
export(sp.kde)
export(spatial.select)
export(spatialEcoNews)
export(spectral.separability)
export(spherical.sd)
export(squareBuffer)
export(srr)
export(stratified.random)
export(subsample.distance)
export(swvi)
export(time_to_event)
export(topo.distance)
export(tpi)
export(trasp)
export(trend.line)
export(tri)
export(vrm)
export(winsorize)
export(wt.centroid)
export(z_normalization)
import(terra)

#' @title Raster Downscale using adaptive regression splines
#' @description Downscales a raster to a higher resolution raster multivariate adaptive 
#'              regression splines (MARS).
#' 
#' @param x            A terra SpatRaster object representing independent variable(s) 
#' @param y            A terra SpatRaster object representing dependent variable
#' @param add.coords   (FALSE/TRUE) Add spatial coordinates to model  
#' @param keep.model   (FALSE/TRUE) Keep MARS model (earth class object)
#' @param grid.search  (FALSE/TRUE) perform a hyper-parameter grid se
#' @param plot         (FALSE/TRUE) Plot results
#' @param ...          Additional arguments passed to earth 
#'
#' @details
#' This function uses Multivariate Adaptive Regression Splines, to downscale a raster based 
#' on higher-resolution or more detailed raster data specified as covariate(s). This is similar
#' to the raster.downsample function which uses a robust regression and is a frequentest model for
#' fitting linear asymptotic relationships whereas, this approach is for fitting nonparametric
#' functions and should be used when the distributional relationship are complex/nonlinear.
#' Using add.coords adds spatial coordinates to the model, including creating the associated 
#' rasters for prediction.      
#'
#' @return A list object containing:  
#' * downscale         Downscaled terra SpatRaster object
#' * GCV Generalized   Cross Validation (GCV)
#' * GRSq              Estimate of the predictive power 
#' * RSS               Residual sum-of-squares (RSS) 
#' * RSq               R-square
#' * model             earth MARS model object (if keep.model = TRUE) 
#' @md
#'
#' @references
#' Friedman (1991) Multivariate Adaptive Regression Splines (with discussion) 
#'   Annals of Statistics 19(1):1–141
#' 
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @examples 
#' \dontrun{
#' if (require(geodata, quietly = TRUE)) {
#' library(terra)
#' library(geodata)
#' 
#' # Download example data (requires geodata package)
#'   elev <- elevation_30s(country="SWZ",  path=tempdir())
#'     slp <- terrain(elev, v="slope")
#' 	  x <- c(elev,slp)
#'         names(x) <- c("elev","slope")
#'   tmax <- worldclim_country(country="SWZ", var="tmax", 
#'                                      path=tempdir())
#'     tmax <- crop(tmax[[1]], ext(elev))
#' 	  names(tmax) <- "tmax"
#' 	  
#' tmax.ds <- aspline.downscale(x, tmax, add.coords=TRUE, keep.model=TRUE)
#'   plot(tmax.ds$model) 
#' 
#'   # plot prediction and parameters	
#'   opar <- par(no.readonly=TRUE)
#'     par(mfrow=c(2,2))
#'       plot(tmax, main="Original Temp max")
#'       plot(x[[1]], main="elevation")
#'       plot(x[[2]], main="slope")
#'       plot(tmax.ds$downscale, main="Downscaled Temp max")
#'   par(opar)
#'
#' } else { 
#'   cat("Please install geodata package to run example", "\n")
#' }
#' }
#'
#' @export
aspline.downscale <- function(x, y, add.coords = TRUE, keep.model = FALSE, 
                              grid.search = FALSE, plot = FALSE, ...) {
    if (!inherits(x, "SpatRaster")) 
	  stop(deparse(substitute(x)), " must be a terra SpatRaster object")	
    if (!inherits(y, "SpatRaster")) 
	  stop(deparse(substitute(y)), " must be a terra SpatRaster object")
    if (terra::nlyr(y) > 1)  {
      warning("Dependent variable must be a single response, 
	    only the first raster will be used")	
          y <- y[[1]]
    }
    if(grid.search) {
      if(length(find.package("caret", quiet = TRUE)) == 0)
        stop("please install caret package to implement grid search")
    }	
  sub.samp.sp <- terra::as.points(x, na.rm=TRUE)
  if(add.coords == TRUE) {
    sub.samp.sp <- cbind(sub.samp.sp, terra::crds(sub.samp.sp, df=TRUE))
      sub.samp <- data.frame(terra::extract(y, sub.samp.sp), sub.samp.sp)
      sub.samp <- sub.samp[,-which(names(sub.samp) %in% "ID")]
    names(sub.samp) <- c("y", names(x), "xcoord", "ycoord")
  } else {
    sub.samp <- data.frame(terra::extract(y, sub.samp.sp), sub.samp.sp)
      sub.samp <- sub.samp[,-which(names(sub.samp) %in% "ID")]
    names(sub.samp) <- c("y", names(x))
  }  
  na.idx <- unique(which(is.na(sub.samp), arr.ind = TRUE)[,1])
    if(length(na.idx) > 0)
  	  sub.samp <- sub.samp[-na.idx,]
  if(add.coords == TRUE) {
    xcoord <- x[[1]] 
      ycoord <- x[[1]]
        xcoord[] <- xFromCell(xcoord, 1:terra::ncell(xcoord))
          ycoord[] <- yFromCell(ycoord, 1:terra::ncell(ycoord))
    x <- c(x, xcoord, ycoord)
      names(x)[(nlyr(x)-1):nlyr(x)] <- c("xcoord", "ycoord")
  } 
    if(grid.search) {
      if(length(find.package("caret", quiet = TRUE)) == 0)
        stop("please install caret package to implement grid search")
      hyper_grid <- expand.grid(
        degree = 1:3, 
        nprune = floor(seq(2, 100, length.out = 10))
      )
	  message("Running hyper parameter grid search with ", nrow(hyper_grid), 
	          " parameter sets, this may take awhile")
      tuned_mars <- caret::train(
        x = subset(sub.samp, select = -y),
        y = sub.samp$y,
        method = "earth",
        metric = "RMSE",
        trControl = caret::trainControl(method = "cv", number = 10),
        tuneGrid = hyper_grid
      )
      ds.fit <- tuned_mars$finalModel
    } else {
      ds.fit <- earth::earth( y ~ ., data = sub.samp, ...)  
    }
  parms <- earth::evimp(ds.fit)
    pidx <- which(names(x) %in% rownames(parms))  
      p <- terra::predict(x[[pidx]], ds.fit)
    results <- list(downscale = p, 
                    GCV = ds.fit$gcv, 
                    RSS = ds.fit$rss, 
                    GRSq = ds.fit$grsq,
                    RSq = ds.fit$rsq,
					imp = parms)
    if(keep.model) {
      results$model <- ds.fit 
    }
    if(plot) plot(ds.fit)
  return(results)
}

#' @title Dissolve polygons
#' @description Dissolve polygon feature calss
#' 
#' @param x         An sf POLYGON or MULTIPOLYGON object
#' @param y         An attribute in x to dissolve by, default is NULL
#' @param overlaps  (FALSE/TRUE) Dissolve overlapping polygons, negates using attribute
#'
#' @details 
#' If a dissolve attribute is defined, the result will be a 
#' MULTIPOLYGON with the grouping attribute column. If y=NULL all polygons
#' will be dissolved into a single attribute, unless there is spatial
#' discontinuity (eg., gaps) in the data. The intent of overlaps=TRUE is to 
#' provide functionality for dissolving overlapping polygons and should only
#' be used in this specialized case.
#'
#' @return A dissolved POLYGON or MULTIPOLYGON object 
#' 
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @examples
#' library(sf)
#' nc <- st_read(system.file("shape/nc.shp", package="sf"))
#'   nc$group <- ifelse(nc$CNTY_ <= 1902, 1, 
#'                 ifelse(nc$CNTY_ > 1902 & nc$CNTY_ <= 1982, 2, 
#' 				     ifelse(nc$CNTY_ > 1982, 3, NA))) 
#' 
#' # Dissolve by group attribute
#' d <- sf_dissolve(nc, "group")
#'   plot(st_geometry(nc), border="grey")
#'     plot(st_geometry(d), border="red", col=NA, 
#'          lwd=2, add=TRUE) 
#' 
#' # Dissolve all polygons
#' d <- sf_dissolve(nc)
#'   plot(st_geometry(nc), border="grey")
#'     plot(st_geometry(d), border="red", col=NA, 
#'          lwd=2, add=TRUE)
#' 
#' # Dissolve overlapping polygons
#' sq <- function(pt, sz = 1) st_polygon(list(rbind(c(pt - sz), 
#'      c(pt[1] + sz, pt[2] - sz), c(pt + sz), c(pt[1] - sz, pt[2] + sz), 
#' 	 c(pt - sz))))
#' pol <- st_sf(box = 1:6, st_sfc(sq(c(4.2,4.2)), sq(c(0,0)), sq(c(1, -0.8)), 
#'           sq(c(0.5, 1.7)), sq(c(3,3)), sq(c(-3, -3))))
#'  st_geometry(pol) <- "geometry" 		  
#' 
#' plot(pol)
#' 
#' d <- sf_dissolve(pol, overlaps=TRUE)
#'   plot(d["diss"])
#' 
#' @export
sf_dissolve <- function(x, y=NULL, overlaps=FALSE) {
  if(!inherits(x, "sf"))		
    stop(deparse(substitute(x)), " must be an sf POLYGON object")	
  if(!any(unique(as.character(sf::st_geometry_type(x))) != c("POLYGON", "MULTIPOLYGON")))
    stop(deparse(substitute(x)), " must be an sf POLYGON object")	
  if(!is.null(y)) {	
    if(!y %in% names(x))
      stop(deparse(substitute(y)), " is not in sf objects attributes")
    d <- split(x, f=sf::st_drop_geometry(x[,y])[,1]) |> 
      lapply(sf::st_union)
        atts <- names(d)
          d <- do.call(c, d) |> 
            sf::st_as_sf()		  
		      d[,y] <- atts
  } else if(overlaps == FALSE) {
    d <- sf::st_union(x, by_feature = FALSE) |> 
        sf::st_cast(to="POLYGON") |> 
          sf::st_as_sf()
  } else if(overlaps == TRUE) {
    if(!any(sf::st_overlaps(x,sparse = FALSE) == TRUE))
      stop("There are no overlapping geometries")
    diss <- unlist(sf::st_intersects(x, 
      sf::st_as_sf(sf::st_cast(sf::st_union(x),"POLYGON"))))
        d <- cbind(x, diss) |>
          dplyr::group_by(diss)
            d <- dplyr::summarize(d, paste(d$box, collapse = ", ")) 
			  names(d)[2] <- "box"
        sf::st_geometry(d) <- "geometry"
  }
    sf::st_crs(d) <- sf::st_crs(x) 
  return(d)
}  

#' @name TM5
#' @docType data
#'
#' @title Landsat 5 TM Scene
#'
#' @description 
#' Subset of Landsat 5 TM Scene: LT52240631988227CUB02 Contains six bands of surface reflectance
#' path 224/row 63 acquisition date: 1988-08-14 13:00:47 GMT, EPSG:32622
#'
#' @format A tif (inst/extdata/Landsat_TM5.tif) with 30m 6 bands:
#' \describe{
#'   \item{Blue}{0.45 - 0.52 µm}
#'   \item{Green}{0.52 - 0.60 µm}
#'   \item{Red}{0.63 - 0.69 µm}
#'   \item{NIR Near-Infrared}{0.76 - 0.90 µm}
#'   \item{SWIR1 Near-Infrared}{1.55 - 1.75 µm}
#'   \item{SWIR2 Mid-Infrared}{2.08 - 2.35 µm}
#' }
#'
NULL

#' @title Point process random subsample
#' @description Generates random subsample based on density estimate 
#'              of observations
#'
#' @param x         An sf POINT class 
#' @param n         Number of random samples to generate
#' @param window    Type of window (hull or extent)
#' @param sigma     Bandwidth selection method for KDE, default is 'Scott'. 
#'                  Options are 'Scott', 'Stoyan', 'Diggle', 'likelihood', 
#'                  and 'geometry'
#' @param wts       Optional vector of weights corresponding to point pattern
#' @param gradient  A scaling factor applied to the sigma parameter used to 
#'                  adjust the gradient decent of the density estimate. The 
#'                  default is 1, for no adjustment (downweight < 1 | upweight > 1)   
#' @param edge      Apply Diggle edge correction (TRUE/FALSE)
#'
#' @details
#' The window type creates a convex hull by default or, optionally, uses the maximum 
#' extent (envelope). The resulting bandwidth can vary widely by method. the 'diggle' 
#' method is intended for  bandwidth representing 2nd order spatial variation whereas 
#' the 'scott' method will represent 1st order trend. the 'geometry' approach will also 
#' represent 1st order trend. for large datasets, caution should be used with the 2nd 
#' order 'likelihood' approach, as it is slow and computationally expensive. finally, 
#' the 'stoyan' method will produce very strong 2nd order results. '
#' 
#' Available bandwidth selection methods are:
#' * Scott - (Scott 1992), Scott's Rule for Bandwidth Selection (1st order)
#' * Diggle - (Berman & Diggle 1989), Minimise the mean-square error via cross 
#'           validation (2nd order)  
#' * likelihood - (Loader 1999), Maximum likelihood cross validation (2nd order)
#' * geometry - Bandwidth is based on simple window geometry (1st order)
#' * Stoyan - (Stoyan & Stoyan 1995), Based on pair-correlation function (strong 2nd order)
#' * User defined - using a numeric value for sigma
#' @md
#'
#' @return sf class POINT geometry containing random subsamples 
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @references
#' Berman, M. and Diggle, P. (1989) Estimating weighted integrals of the second-order 
#'   intensity of a spatial point process. Journal of the Royal Statistical Society, 
#'   series B 51, 81-92. 
#' 
#' Fithian, W & T. Hastie (2013) Finite-sample equivalence in statistical models for 
#'   presence-only data. Annals of Applied Statistics 7(4): 1917-1939
#' 
#' Hengl, T., H. Sierdsema, A. Radovic, and A. Dilo (2009) Spatial prediction of species 
#'   distributions from occurrence-only records: combining point pattern analysis, 
#'   ENFA and regression-kriging. Ecological Modelling, 220(24):3499-3511  
#' 
#' Loader, C. (1999) Local Regression and Likelihood. Springer, New York. 
#' 
#' Scott, D.W. (1992) Multivariate Density Estimation. Theory, Practice and Visualization. 
#'   New York, Wiley. 
#' 
#' Stoyan, D. and Stoyan, H. (1995) Fractals, random shapes and point fields: methods of 
#'   geometrical statistics. John Wiley and Sons. 
#' 
#' Warton, D.i., and L.C. Shepherd (2010) Poisson Point Process Models Solve the Pseudo-Absence 
#'   Problem for Presence-only Data in Ecology. The Annals of Applied Statistics, 4(3):1383-1402
#'
#' @examples  
#' library(sf) 
#' if(require(spatstat.explore, quietly = TRUE)) { 
#' data(bei, package = "spatstat.data")
#' 
#' trees <- st_as_sf(bei)
#'   trees <- trees[-1,]
#' 
#' n=round(nrow(trees) * 0.10, digits=0)       
#' trees.wrs <- pp.subsample(trees, n=n, window='hull')
#'   plot(st_geometry(trees), pch=19, col='black')
#'     plot(st_geometry(trees.wrs), pch=19, col='red', add=TRUE) 
#'       box()
#'        title('10% subsample')
#'     legend('bottomright', legend=c('Original sample', 'Subsample'), 
#'                  col=c('black','red'),pch=c(19,19))   
#' 
#' } else { 
#'   cat("Please install spatstat.explore package to run example", "\n")
#' }
#'
#' @export pp.subsample
pp.subsample <- function(x, n, window = "hull", sigma = "Scott", wts = NULL, 
                         gradient = 1, edge = FALSE) {
  # if(class(x) == "sf") { x <- as(x, "Spatial") }
    if (is.null(window)) 
        stop("Please specify a valid window type hull | extent")
    bw.Scott <- function(X) {
        stopifnot(spatstat.geom::is.ppp(X))
        n <- spatstat.geom::npoints(X)
        sdx <- sqrt(stats::var(X$x))
        sdy <- sqrt(stats::var(X$y))
        return(c(sdx, sdy) * n^(-1/6))
    }
    bw.Stoyan <- function(X, co = 0.15) {
        stopifnot(spatstat.geom::is.ppp(X))
        n <- spatstat.geom::npoints(X)
        W <- spatstat.geom::as.owin(X)
        a <- spatstat.geom::area.owin(W)
        stoyan <- co/sqrt(5 * n/a)
        return(stoyan)
    }
    bw.geometry <- function(X, f = 1/4) {
        X <- spatstat.geom::as.owin(X)
        g <- spatstat.explore::distcdf(X)
        r <- with(g, .x)
        Fr <- with(g, .y)
        iopt <- min(which(Fr >= f))
        return(r[iopt])
    }
    bw.likelihood <- function(X, srange = NULL, ns = 16) {
	  check.range <- function (x, fatal = TRUE) {
        xname <- deparse(substitute(x))
        if (is.numeric(x) && identical(x, range(x, na.rm = TRUE))) 
          return(TRUE)
        if (fatal) 
          stop(paste(xname, "should be a vector of length 2 giving (min, max)"))
        return(FALSE)
      }
        stopifnot(spatstat.geom::is.ppp(X))
        if (!is.null(srange)) 
            check.range(srange) else {
            nnd <- spatstat.geom::nndist(X)
            srange <- c(min(nnd[nnd > 0]), spatstat.geom::diameter(spatstat.geom::as.owin(X))/2)
        }
        sigma <- exp(seq(log(srange[1]), log(srange[2]), length = ns))
        cv <- numeric(ns)
        for (i in 1:ns) {
            si <- sigma[i]
            lamx <- spatstat.explore::density.ppp(X, sigma = si, at = "points", leaveoneout = TRUE)
            lam <- spatstat.explore::density.ppp(X, sigma = si)
            cv[i] <- sum(log(lamx)) - spatstat.geom::integral.im(lam)
        }
        result <- spatstat.explore::bw.optim(cv, sigma, iopt = which.max(cv), criterion = "Likelihood Cross-Validation")
        return(result)
    }
    if (window == "hull") {
        win <- spatstat.geom::convexhull.xy(sf::st_coordinates(x)[,1:2])
    } else {
      if (window == "extent") {
        e <- as.vector(sf::st_bbox(x))
        win <- spatstat.geom::as.owin(c(e[1], e[3], e[2], e[4]))
      }
    }
    x.ppp <- spatstat.geom::as.ppp(sf::st_coordinates(x)[,1:2], win)
    if (sigma == "Diggle") {
        bw <- spatstat.explore::bw.diggle(x.ppp)
    } else {
        if (sigma == "Scott") {
            bw <- bw.Scott(x.ppp)
        } else {
            if (sigma == "Stoyan") {
                bw <- bw.Stoyan(x.ppp)
            } else {
                if (sigma == "geometry") {
                  bw <- bw.geometry(x.ppp)
                } else {
                  if (sigma == "likelihood") {
                    bw <- bw.likelihood(x.ppp)
                  } else {
                    if (is.numeric(sigma)) {
                      bw <- sigma
                  }
               }
             }
          }
       }
    }
  den <- spatstat.explore::density.ppp(x.ppp, weights = wts, sigma = bw, adjust = gradient, diggle = edge, at = "points")
    point.den <- data.frame(X = x.ppp$x, Y = x.ppp$y, KDE = as.vector(den * 10000))
      point.den$KDE <- point.den$KDE/max(point.den$KDE)
        point.den <- point.den[order(point.den[["KDE"]]), ]
        point.den$KDE <- rev(point.den$KDE)
      point.den <- sf::st_as_sf(point.den, coords = c("X", "Y"), agr = "constant")	  
    point.rs <- point.den[sample(seq(1:nrow(point.den)), n, prob = point.den$KDE), ]
  return( point.rs )
} 

#' @title Proximity Index 
#' @description Calculates proximity index for a set of polygons  
#'
#' @param   x           A polygon class sp or sf object
#' @param   y           Optional column in data containing classes
#' @param   min.dist    Minimum threshold distance
#' @param   max.dist    Maximum neighbor distance
#' @param   background  Optional value in y column indicating background value 
#'
#' @return 
#' A vector equal to nrow(x) of proximity index values, if a background value is 
#' specified NA values will be returned in the position(s) of the specified class
#'   
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @references 
#' Gustafson, E.J., & G.R. Parker (1994) Using an Index of Habitat Patch Proximity 
#'   for Landscape Design. Landscape and Urban Planning 29:117-130
#'
#' @examples
#' \donttest{
#'  library(sf)
#'  if(require(sp, quietly = TRUE)) {
#'    data(meuse, package = "sp")
#'    meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, 
#'                      agr = "constant")
#'    meuse <- st_buffer(meuse, dist = meuse$elev * 5)
#'      meuse$LU <- sample(c("forest","nonforest"), nrow(meuse), 
#'                        replace=TRUE) 
#' 
#'  # All polygon proximity index 1000 radius	
#'  ( pidx <- proximity.index(meuse, min.dist = 1) )
#'    pidx[pidx > 1000] <- 1000
#'  
#'  # Class-level proximity index 1000 radius
#'  ( pidx.class <- proximity.index(meuse, y = "LU", min.dist = 1) )
#'    
#'  # plot index for all polygons
#'  meuse$pidx <- pidx
#'    plot(meuse["pidx"])
#'  
#'  # plot index for class-level polygons 
#'  meuse$cpidx <- pidx.class
#'    plot(meuse["cpidx"])
#'  
#'  # plot index for just forest class
#'  forest <- meuse[meuse$LU == "forest",]
#'   plot(forest["cpidx"])
#'
#' } else { 
#'   cat("Please install sp package to run example", "\n")
#' }
#' }   
#' @export proximity.index
proximity.index <- function(x, y = NULL, min.dist = 0, max.dist = 1000, 
                            background = NULL) {
  if(!inherits(x, "sf"))
    stop(deparse(substitute(x)), " must be an sf object")	  
  if(!unique(as.character(sf::st_geometry_type(x))) == "POLYGON")
      stop(deparse(substitute(x)), " must be POLYGON geometry (not multi-part) ")
  if(!is.null(y)) {  
    if(!any(y %in% names(x))) 
	  stop("Column (y) is not in polygon data")
	classes <- unique(sf::st_drop_geometry(x[,y])[,1])  
    if(!is.null(background)) {
	  if(!any(background %in% classes))
        stop("Background class not in data")
      bg.idx <- which( sf::st_drop_geometry(x[,y])[,1] == background ) 
        x <- x[sf::st_drop_geometry(x[,y])[,1] != background,]
          classes <- classes[-grep(background, classes)]		
	  }	  
    class.pidx <- vector()
      rn <- vector()	
    for(j in classes) { 
	  xx <- x[which(sf::st_drop_geometry(x[,y])[,1] == j),]
	  rn <- append(rn, rownames(xx))
      dmat <- units::drop_units(sf::st_distance(xx, sparse=FALSE))
	    colnames(dmat) <- rownames(xx)
		rownames(dmat) <- rownames(xx)
	    #diag(dmat) <- NA
        dmat[dmat <= min.dist | dmat > max.dist] <- NA
      a <- sf::st_area(xx)
        names(a) <- rownames(xx)	  
	  pidx <- vector()
        for(i in 1:nrow(dmat)) {
	      idx <- colnames(dmat)[which(!is.na(as.numeric(dmat[,i])))]
		  pidx[i] <- sum(a[which(names(a) %in% idx)] * dmat[,i][idx]^-2)
        }
      class.pidx <- append(class.pidx, pidx)
 	}
	pidx <- class.pidx[order(match(rownames(x),rn))]
  } else {
	dmat <- units::drop_units(sf::st_distance(x, sparse=FALSE))
	  colnames(dmat) <- rownames(x)
	  rownames(dmat) <- rownames(x)	
    dmat[dmat <= min.dist | dmat > max.dist] <- NA
	a <- sf::st_area(x)
	  names(a) <- rownames(x)
	pidx <- vector()
      for(i in 1:nrow(dmat)) {
	    idx <- colnames(dmat)[which(!is.na(as.numeric(dmat[,i])))]
		pidx[i] <- sum(a[which(names(a) %in% idx)] * dmat[,i][idx]^-2)
      }
	}
    if(!is.null(background)) {
	  z <- numeric(length(pidx) + length(bg.idx))
        z[bg.idx] <- NA
        z[-bg.idx] <- pidx
	  pidx <- z
	}
  return ( pidx )
}

#' @title Spatial kernel density estimate
#' @description A weighted or unweighted Gaussian Kernel Density estimate 
#'              for point spatial data
#'
#' @param x             sf POINT object
#' @param y             Optional values, associated with x coordinates, 
#'                      to be used as weights
#' @param bw            Distance bandwidth of Gaussian Kernel, must be units 
#'                      of projection
#' @param ref           A terra SpatRaster, vect, ext or sf vector, bbox 
#'                      vector to estimate the kde extent
#' @param res           Resolution of raster when ref not SpatRaster           
#' @param standardize   Standardize results to 0-1 (FALSE/TRUE)
#' @param scale.factor  Numeric scaling factor for the KDE (defaults to 10000),  
#'                      to account for very small estimate values
#' @param mask          (TRUE/FALSE) mask resulting raster if ref is provided  
#'                      as a SpatRaster
#'
#' @details
#' Please note that ks methods for estimation has been reverted to the Gussian method proposed
#' in Venables & Ripley (2002). There was not enought evendence that the Chacon & Duong (2018)
#' multivariate method(s) for bandwidth selection and kernal estimation were suitable for 
#' spatial random fields. 
#'
#' @return  a terra SpatRaster class object containing kernel density estimate 
#'
#' @author Jeffrey S. Evans  <jeffrey_evans@@tnc.org>
#'
#' @references
#' Duong, T. & Hazelton, M.L. (2005) Cross-validation bandwidth matrices for multivariate 
#'   kernel density estimation. Scandinavian Journal of Statistics, 32, 485-506. 
#'
#' Wand, M.P. & Jones, M.C. (1994) Multivariate plug-in bandwidth selection. Computational 
#'   Statistics, 9, 97-116. 
#'
#' Venables, W. N. and Ripley, B. D. (2002) Modern Applied Statistics with S. 
#'   Fourth edition. Springer. 
#'
#' @examples
#' \donttest{ 
#' library(sf) 
#' library(terra) 
#'   
#' data(meuse, package = "sp")
#' meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, 
#'                   agr = "constant") 
#'   			
#' # Unweighted KDE (spatial locations only) with 40m resoultion  				
#' pt.kde <- sf.kde(x = meuse, bw = 1000, standardize = TRUE, res=40)
#'   plot(pt.kde, main="Unweighted kde")
#'     plot(st_geometry(meuse), pch=20, col="red", add=TRUE) 
#' 
#' # cadmium weighted KDE usign extent with 40m resoultion and 500m and 1000m bw 
#' cadmium.kde.500 <- sf.kde(x = meuse, y = meuse$cadmium, res=40, 
#'                           bw = 500, standardize = TRUE)
#' cadmium.kde.1000 <- sf.kde(x = meuse, y = meuse$cadmium, res=40, 
#'                           bw = 1000, standardize = TRUE)						  
#'   plot(c(cadmium.kde.500, cadmium.kde.1000))
#' 
#' # Using defined raster
#' r <- terra::rast(terra::ext(meuse), resolution =  40, 
#' 		               crs=terra::crs(meuse))   
#'   pt.kde <- sf.kde(x = meuse, bw = 1000, standardize = TRUE,  ref = r) 
#'  
#' }
#' 
#' @export sp.kde sf.kde
#' @aliases sp.kde
sf.kde <- function(x, y = NULL, bw = NULL, ref = NULL, res = NULL,  
                   standardize = FALSE, scale.factor = 10000, 
				   mask = FALSE) {
  if(missing(x))
    stop("x argument must be provided")
  if(!inherits(x, c("sf", "sfc") ))	
    stop(deparse(substitute(x)), " must be a sf, or sfc object")
  if(unique(as.character(sf::st_geometry_type(x))) != "POINT")
    stop(deparse(substitute(x)), " must be single-part POINT geometry") 
  if(is.null(scale.factor)) scale.factor = 1     
  if(is.null(res) && inherits(ref, "SpatRaster")) {
    res <- terra::res(ref)
  }
  if(is.null(res) && !inherits(ref, "SpatRaster")) {
    res <- 100
	message("resoultion not defined, defaulting to 100")
  }
  if(!any(c(inherits(ref, "sf"), inherits(ref, "SpatVector"), inherits(ref, "SpatRaster"),  
     inherits(ref, "bbox"), inherits(ref, "SpatExtent"), inherits(ref, "NULL"))) ) 
       stop("Reference must be sf (vector, bbox) or terra (SpatRaster, SpatVector, SpatExtent). 
	         If not defined, will be extent of points (x)") 
  if(any(c(inherits(ref, "sf"), inherits(ref, "SpatVector"), inherits(ref, "SpatRaster"),     
       inherits(ref, "bbox"), inherits(ref, "SpatExtent"))) ) {
    if(sf::st_is_longlat(ref))
      stop("Coordinate Reference System must be projected coordinates (not lat/long)")
  }	 
  if(is.null(ref)) {
    message("Creating reference raster from extent of points (x)")
    ref <- terra::rast(terra::ext(x), resolution =  res, 
		               crs=terra::crs(x))  
  } else if(inherits(ref, "SpatRaster")) {
    message("Using ", deparse(substitute(ref)), " as reference raster")  
    if(!terra::crs(x) == terra::crs(ref) )
      stop("CRS do not match")
	  if(mask == TRUE) { 
	    m <- ref 
		if(!terra::hasValues(m)) { m[] <- 1 } 
	  }
        ref[] <- NA	  
  } else if(inherits(ref, "sf")) {
    message("Using extent from ", deparse(substitute(ref)), " as reference raster")  
    if(!terra::crs(x) == terra::crs(ref))
      stop("CRS do not match")	
    ref <- terra::rast(terra::ext(ref), resolution =  res, 
		               crs=terra::crs(x))      
  } else if(inherits(ref, "SpatVector")) { 
    message("Using extent from ", deparse(substitute(ref)), " as reference raster")    
    sf::st_is_longlat(ref)
	  stop("Coordinate Reference System be projected coordinates (not lat/long)")
    ref <- terra::rast(terra::ext(ref), resolution =  res, 
		               crs=terra::crs(x))  
  } else if(inherits(ref, "bbox")) {
    message("Using defined extent ", deparse(substitute(ref)), " as reference raster")    
    ref <- terra::rast(terra::ext(as.numeric(ref)[c(1,3,2,4)]), resolution =  res, 
		               crs=terra::crs(x))  
  } else if(inherits(ref, "SpatExtent")) {
    message("Using defined extent ", deparse(substitute(ref)), " as reference raster")  
    ref <- terra::rast(ref, resolution =  res, 
		               crs=terra::crs(x))  
  }
   if(mask == TRUE & !exists("m")) {
     m <- ref 
      if(!terra::hasValues(m)) { m[] <- 1 } 
    }
  # check if points are contained within refrence raster
  rext <- sf::st_as_sf(terra::as.polygons(terra::ext(ref)))
  xext <- sf::st_as_sf(terra::as.polygons(terra::ext(x)))
  inside <- sf::st_within(sf::st_buffer(xext, -res), rext, sparse=FALSE)
  if(!inside[,1][1]) 
    stop("Points are not inside the reference raster")

  # check if the crs match
  if(terra::crs(x) != terra::crs(x))
    stop("CRS do not match")

  # weighted kde function, modification of MASS::kde2d 
  fhat <- function (x, y, h, w, n = 25, lims = c(range(x), range(y))) {
    nx <- length(x)
      if (length(y) != nx) 
          stop("data vectors must be the same length")
      if (length(w) != nx & length(w) != 1) 
          stop("weight vectors must be 1 or length of data")
      if (missing(h)) { 
        h <- c(MASS::bandwidth.nrd(x), MASS::bandwidth.nrd(y))
      } else { 
        h <- rep(h, length.out = 2L)
      }	
    if (any(h <= 0)) stop("bandwidths must be strictly positive")
      if (missing(w)) { w <- numeric(nx) + 1 }
      gx <- seq(lims[1], lims[2], length = n[1])
      gy <- seq(lims[3], lims[4], length = n[2])
          h <- h/4
        ax <- outer(gx, x, "-") / h[1]
      ay <- outer(gy, y, "-") / h[2]
    z <- ( matrix(rep(w, n[1]), nrow = n[1], ncol = nx, byrow = TRUE) * 
           matrix(stats::dnorm(ax), n[1], nx) ) %*% t(matrix(stats::dnorm(ay), n[2], nx)) /
            ( sum(w) * h[1] * h[2] )
    return(list(x = gx, y = gy, z = z))
  }
  if(is.null(bw)){ 
    bwf <- function(x){
      r <- stats::quantile(x, c(0.25, 0.75))
      h <- (r[2] - r[1])/1.34
      4 * 1.06 * min(sqrt(stats::var(x)), h) * length(x)^(-1/5)
    }
	bw <- c(bwf(sf::st_coordinates(x)[,1]), 
	        bwf(sf::st_coordinates(x)[,2]))
	  message("Using ", round(bw[1],3), ", ", round(bw[2],3), " for bandwidth", "\n")
  } else {
    bw <- c(bw,bw)
  }
  n <- c(terra::nrow(ref), terra::ncol(ref)) 
    if(!is.null(y)) {
      message("\n","calculating weighted kde","\n")
      k  <- fhat(sf::st_coordinates(x)[,1], sf::st_coordinates(x)[,2], w = y, 
 	             h = bw, n = n, lims = as.vector(terra::ext(ref)) )
    } else {
 	  message("\n","calculating unweighted kde","\n")
 	  k <- MASS::kde2d(sf::st_coordinates(x)[,1], sf::st_coordinates(x)[,2], h = bw, 
 	                   n = n, lims = as.vector(terra::ext(ref)) )
    }
  k$z <- k$z * scale.factor	
    if( standardize == TRUE ) { k$z <- (k$z - min(k$z)) / (max(k$z) - min(k$z)) }
        kde.est <- flip(terra::rast(t(k[[3]]), crs=terra::crs(x), extent=terra::ext(ref)), direction="vertical")
      if(mask == TRUE) { kde.est <- terra::mask(kde.est, m) }
    terra::crs(kde.est) <- terra::crs(x)  
  return( kde.est )  
}  
sp.kde = sf.kde

#' @title z normalization 
#' @description Z-normalizes a time series by subtracting its mean and dividing by the standard deviation or meadian and MAD
#'

#' @param x         Timeseries vector to normalize
#' @param method    ("original", "modified") Use the standard z normalization or modified version
#' @param na        Behavior of NA values, keep or remove
#'
#' @note 
#' The original method is to subtract x rom the mean then divide by sd whereas, modified uses the median and mad. 
#' For NA's if the argument is keep, an NA index is built and the NA's are re-inserted in to the resulting vector.
#' The behavior of remove will entirely remove them and result in a vector length different than the input.  
#' 
#' @return vector of z normalized data
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @references
#' Iglewicz, B. & D.C. Hoaglin (1993) How to Detect and Handle Outliers, 
#'   American Society for Quality Control, Milwaukee, WI.
#'
#' @examples 
#'  # add data
#'  data(EuStockMarkets)
#'  d <- as.vector(EuStockMarkets[,1])
#' 
#' # Calculate Z score
#' summary(d)
#' summary(Z_org <- z_normalization(d) ) 
#' summary(Z_mod <- z_normalization(d, method="modified") )
#'   par(mfrow=c(3,1))
#'     plot(density(d), main="original timeseries")
#'     plot(density(Z_org), main="original z norm")
#'     plot(density(Z_mod), main="modified z norm")
#' 
#' # Check NA behavior, insert some NA's
#' d[c(100, 500, 1000, 1400)] <- NA
#' length(d)
#' length( z_normalization(d, na="keep") ) 
#' length( z_normalization(d, na="remove") ) 
#'  
#' @export
z_normalization <- function(x, method=c("original", "modified"), 
                            na = c("keep", "remove")) {
  if(!is.numeric(x))
    stop("x must be nuumeric vector")
  na.idx <- which(is.na(x))
    if(length(na.idx) > 0) x <- x[-na.idx]
  if(na[1] == "keep" & length(na.idx) > 0) {
    message(length(na.idx), " NA's found and will be retained")
  } else if(na[1] == "remove" & length(na.idx) > 0) {
	message(length(na.idx), " NA's found and will be removed")
  }
  if(method[1] == "original") {
    z <- (x - mean(x)) / stats::sd(x)
  } else if(method[1] == "modified") {
    z <- (0.6745 * (x - stats::median(x))) / stats::mad(x)
  }
  if(na[1] == "keep" & length(na.idx) > 0) {
    zv <- z
      z <- numeric(length(zv) + length(na.idx))
        z[na.idx] <- NA
          z[-na.idx] <- zv
  }  
  return(z)
}

#' @title moments
#'
#' @description Calculate statistical moments of a distribution
#'
#' @param x numeric vector  
#' @param plot plot of distribution (TRUE/FALSE)
#'
#' @return A vector with the following values:
#' * min Minimum           
#' * 25th  25th percentile     
#' * mean  Arithmetic mean 
#' * gmean  Geometric mean
#' * hmean  Harmonic mean
#' * median  50th percentile      
#' * 7th5  75th percentile      
#' * max  Maximum          
#' * stdv  Standard deviation         
#' * var  Variance     
#' * cv  Coefficient of variation (percent)         
#' * mad  Median absolute deviation         
#' * skew  Skewness    
#' * kurt  Kurtosis     
#' * nmodes  Number of modes   
#' * mode  Mode (dominate)
#' @md
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @examples 
#' x <- runif(1000,0,100)
#' ( d <- moments(x, plot=TRUE) )
#' ( mode.x <- moments(x, plot=FALSE)[16] )
#'  
#' @export 
moments <- function(x, plot = FALSE) {
    if (!length(x) >= 3) 
        stop("Not enough values to represent a distribution")
    .skew <- function(x, na.rm = FALSE) {
        if (na.rm) 
            x <- x[!is.na(x)]
        sum((x - mean(x))^3)/(length(x) * stats::sd(x)^3)
    }
    .kurt <- function(x, na.rm = FALSE) {
        if (na.rm) 
            x <- x[!is.na(x)]
        sum((x - mean(x))^4)/(length(x) * stats::var(x)^2) - 3
    }
    .cv <- function(x) {
        (stats::sd(x)/mean(x)) * 100
    }
    .means <- function(x) {
        if (any(x < 0)) 
            stop("need positive data")
        geometric <- function(x) {
            (exp(mean(log(x))))
        }
        harmonic <- function(x) {
            length(x)/sum(1/x)
        }
        arithmetic <- function(x) {
            sum(x)/length(x)
        }
        (x <- c(arithmetic(x), harmonic(x), geometric(x)))
    }
    .dmode <- function(x) {
        den <- stats::density(x, kernel = c("gaussian"))
        (den$x[den$y == max(den$y)])
    }
    .n.modes <- function(x) {
        den <- stats::density(x, kernel = c("gaussian"))
        den.s <- stats::smooth.spline(den$x, den$y, all.knots = TRUE, spar = 0.8)
        s.0 <- stats::predict(den.s, den.s$x, deriv = 0)
        s.1 <- stats::predict(den.s, den.s$x, deriv = 1)
        s.derv <- data.frame(s0 = s.0$y, s1 = s.1$y)
        nmodes <- length(rle(den.sign <- sign(s.derv$s1))$values)/2
        if ((nmodes > 10) == TRUE) {
            nmodes <- 10
        }
        if (is.na(nmodes) == TRUE) {
            nmodes <- 0
        }
        (nmodes)
    }
    r <- c(min(x), stats::quantile(x, 0.25, na.rm = TRUE), .means(x)[1], .means(x)[2], .means(x)[3], 
	       stats::quantile(x, 0.5, na.rm = TRUE), stats::quantile(x, 0.75, na.rm = TRUE), 
           max(x, na.rm = TRUE), stats::sd(x, na.rm = TRUE), stats::var(x, na.rm = TRUE), .cv(x), 
		   stats::mad(x, na.rm = TRUE), .skew(x), .kurt(x), .n.modes(x), .dmode(x))
    names(r) <- c("min", "25th", "mean", "hmean", "gmean", "median", "75th", "max", "stdv", "var", "cv", "mad", "skew", 
        "kurt", "nmodes", "mode")
    if (plot == TRUE) {
        graphics::plot(stats::density(x), type = "n", main = "", ylab = "DENSITY", xlab = "RANGE", )
        graphics::polygon(stats::density(x), col = "blue")
        graphics::abline(v = min(x), lty = 1, lwd = 1, col = "black")
        graphics::abline(v = max(x), lty = 1, lwd = 1, col = "black")
        graphics::abline(v = stats::quantile(x, 0.25), lty = 2, lwd = 1, col = "black")
        graphics::abline(v = stats::quantile(x, 0.75), lty = 2, lwd = 1, col = "black")
        graphics::abline(v = .dmode(x), lty = 3, lwd = 1, col = "red")
        graphics::legend("topright", lty = c(1, 1, 2, 2, 3), lwd = c(1, 1, 1, 1, 1), bty = "n", legend = c("MIN", "MAX", "25th", 
            "75th", "MODE"), col = c("black", "black", "black", "black", "red"))
    }
    return(r)
} 

#' @title Summarizing Loess bootstrap models
#' @description Summary method for class "loess.boot".
#'
#' @param object  Object of class loess.boot
#' @param ... Ignored
#'
#'
#' @return same as print lowess.boot data.frame including; 
#' * nreps        Number of bootstrap replicates 
#' * confidence   Confidence interval (region)
#' * span         alpha (span) parameter used loess fit
#' * degree       polynomial degree used in loess fit
#' * normalize    Normalized data (TRUE/FALSE)
#' * family       Family of statistic used in fit
#' * parametric   Parametric approximation (TRUE/FALSE) 
#' * surface      Surface fit, see loess.control
#' * data         data.frame of x,y used in model
#' * fit          data.frame including:
#'   1) x - Equally-spaced x index (see NOTES)         
#'   2) y.fit - loess fit
#'   3) up.lim - Upper confidence interval 
#'   4) low.lim - Lower confidence interval 
#'   5) stddev - Standard deviation of loess fit at each x value
#' @md 
#'
#' @method summary loess.boot
#' @export
summary.loess.boot <- function(object, ...) {
  cat("Number of permutations: ", object$nreps, "\n", sep="")
  cat("Loess alpha (span) parameter: ", object$span, "\n", sep="")
  cat("Polynomial degree: ", object$degree, "\n", sep="")
  cat("Data normalized: ", object$normalize, "\n", sep="")
  cat("Model distribution: ", object$family, "\n", sep="")
  cat("parametric model: ", object$parametric, "\n", sep="")
  cat("","\n", sep="")  
  cat("summary of fit: ","\n", sep="")
    summary(object$fit[,"y.fit"])
  cat("","\n", sep="")
  cat("summary of lower confidence limits: ","\n", sep="")
    summary(object$fit[,"low.lim"])
  cat("","\n", sep="")
  cat("summary of fit: ","\n", sep="")
    summary(object$fit[,"up.lim"])
}

#' @title Surface curvature
#' @description Calculates Zevenbergen & Thorne, McNab's or Bolstad's curvature 
#' 
#' @param x      A terra SpatRaster object
#' @param type   Method used c("planform", "profile", "total", "mcnab", "bolstad")
#' @param ...    Additional arguments passed to focal
#'
#' @details 
#' The planform and profile curvatures are the second derivative(s) of the 
#' elevation surface, or the slope of the slope. Profile curvature is in 
#' the direction of the maximum slope, and the planform curvature is 
#' perpendicular to the direction of the maximum slope.
#' Negative values in the profile curvature indicate the surface is upwardly
#' convex whereas, positive values indicate that the surface is upwardly concave.  
#' Positive values in the planform curvature indicate an that the surface is 
#' laterally convex whereas, negative values indicate that the surface is 
#' laterally concave.
#'
#' Total curvature is the sigma of the profile and planform curvatures. A value of 
#' 0 in profile, planform or total curvature, indicates the surface is flat. 
#' The planform, profile and total curvatures are derived using 
#' Zevenbergen & Thorne (1987) via a quadratic equation fit to eight neighbors 
#' as such, the s (focal window size) argument is ignored. 
#'
#' McNab's and Bolstad's variants of the surface curvature (concavity/convexity) 
#' index (McNab 1993; Bolstad & Lillesand 1992; McNab 1989). The index is based 
#' on features that confine the view from the center of a 
#' 3x3 window. In the Bolstad equation, edge correction is addressed 
#' by dividing by the radius distance to the outermost cell (36.2m). 
#' 
#' @return raster class object of surface curvature
#' 
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#' 
#' @references
#' Bolstad, P.V., and T.M. Lillesand (1992). Improved classification of forest 
#'   vegetation in northern Wisconsin through a rule-based combination of soils, 
#'   terrain, and Landsat TM data. Forest Science. 38(1):5-20.
#' 
#' Florinsky, I.V. (1998). Accuracy of Local Topographic Variables Derived from 
#'   Digital Elevation Models. International Journal of Geographical Information 
#'   Science, 12(1):47-62.
#' 
#' McNab, H.W. (1989). Terrain shape index: quantifying effect of minor landforms 
#'   on tree height. Forest Science. 35(1):91-104.
#' 
#' McNab, H.W. (1993). A topographic index to quantify the effect of mesoscale 
#'   landform on site productivity. Canadian Journal of Forest Research. 23:1100-1107.
#' 
#' Zevenbergen, L.W. & C.R. Thorne (1987). Quantitative Analysis of Land Surface 
#'   Topography. Earth Surface Processes and Landforms, 12:47-56.
#'
#' @examples 
#' \donttest{
#'   library(terra)
#'   elev <- rast(system.file("extdata/elev.tif", package="spatialEco"))
#'
#'   crv <- curvature(elev, type = "planform")
#'   mcnab.crv <- curvature(elev, type = "mcnab")
#'       plot(mcnab.crv, main="McNab's curvature") 
#' }
#'
#' @export curvature
curvature <- function(x, type = c("planform", "profile", "total", "mcnab", "bolstad"), ...) { 
  if(!inherits(x, "SpatRaster"))
    stop(deparse(substitute(x)), " must be a terra SpatRaster object")
    m <- matrix(1, nrow=3, ncol=3)
    type = type[1] 
	if(!any(c("planform", "profile", "total", "mcnab", "bolstad") %in% type)  )
      stop("Not a valid curvature option")	
	zt.crv <- function(m, method = type, res = terra::res(x)[1]) {
      p=(m[6]-m[4])/(2*res)
      q=(m[2]-m[8])/(2*res)
      r=(m[4]+m[6]-2*m[5])/(2*(res^2))
      s=(m[3]+m[7]-m[1]-m[9])/(4*(res^2))
      tx=(m[2]+m[8]-2*m[5])/(2*(res^2))
      if(type == "planform") {
        crv <- round( -(q^2*r-2*p*q*s+p^2*tx)/((p^2+q^2)*sqrt(1+p^2+q^2)),6) 
      } else if(type == "profile") {
        crv <- round( -(p^2*r+2*p*q*s+q^2*tx)/((p^2+q^2)*sqrt(1+p^2+q^2)^3),6 )
      } else if(type == "total") {
        crv <- round( -(q^2*r-2*p*q*s+p^2*tx)/((p^2+q^2)*sqrt(1+p^2+q^2)),6) + 
		        round( -(p^2*r+2*p*q*s+q^2*tx)/((p^2+q^2)*sqrt(1+p^2+q^2)^3),6 )  
	  }
	  return(crv)
	}  	
    if(type == "bolstad") {
      return( 10000 * ((x - terra::focal(x, w=m, fun=mean, ...)) / 1000 / 
	          (terra::res[1] + terra::res[1]/2) ) )  
    } else if(type == "mcnab") {
        #mcnab <- function(x) {
        #      ( (x[5] - x[1]) + (x[5] - x[2]) + (x[5] - x[3]) + (x[5] - x[4]) + 
        #        (x[5] - x[6]) + (x[5] - x[7]) + (x[5] - x[8]) + (x[5] - x[9]) ) / 8  }
      #x <- matrix(runif(9, 1100,1150), nrow=3,ncol=3)
      mcnab <- function(x) {
        m <- ceiling(length(x)/2) 
        sum(x[m] - x[-m], na.rm=TRUE) / length(x)-1 
      }
      tidx <- mask(terra::focal(x, w=m, fun=mcnab, ...), x) 
      return( tidx / as.numeric(terra::global(tidx, "max", na.rm=TRUE)) ) 
    } else {
	  return( terra::focal(x, w=m, fun = zt.crv, fillvalue = 0, ...) )
	}
}	

#' @title Dutilleul moving window bivariate raster correlation 
#' @description A bivarate raster correlation using Dutilleul's 
#'              modified t-test
#'       
#' @param x           A terra SpatRaster class object 
#' @param y           A terra SpatRaster class object, same dimensions as x
#' @param d           Distance for finding neighbors
#' @param sample      Apply sub-sampling options; c("none", "random", "hexagonal", "regular") 
#' @param p           If sample != "none", what proportion of population 
#'                    should be sampled
#' @param size        Fixed sample size (default NULL)               
#' 
#'
#' @description 
#' This function provides a bivariate moving window correlation using the modified  
#' t-test to account for spatial autocorrelation. Point based subsampling is provided 
#' for computation tractability. The hexagon sampling is recommended as it it good  
#' at capturing spatial process that includes nonstationarity and anistropy.    
#'
#' @return 
#' A terra SpatRaster or sf POINT class object with the following attributes:
#'   * corr - Correlation 
#'   * Fstat - The F-statistic calculated as degrees of freedom unscaled F-statistic
#'   * p.value - p-value for the test
#'   * moran.x - Moran's-I for x 
#'   * moran.y - Moran's-I for y  
#' @md
#' 
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'                                                                           
#' @references
#' Clifford, P., S. Richardson, D. Hemon (1989), Assessing the significance of the  
#'   correlationbetween two spatial processes. Biometrics 45:123-134.
#' 
#' Dutilleul, P. (1993), Modifying the t test for assessing the correlation between 
#'   two spatial processes. Biometrics 49:305-314. 
#' 
#' @examples
#' \donttest{
#'  p = c("sf", "sp", "terra", "gstat")
#'  if(any(!unlist(lapply(p, requireNamespace, quietly=TRUE)))) { 
#'    m = which(!unlist(lapply(p, requireNamespace, quietly=TRUE)))
#'    message("Can't run examples, please install ", paste(p[m], collapse = " "))
#'  } else {
#'    invisible(lapply(p, require, character.only=TRUE))
#'
#' data(meuse, package = "sp")
#' meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, 
#'                   agr = "constant") 
#' data(meuse.grid, package = "sp")                                      
#' meuse.grid <- st_as_sf(meuse.grid, coords = c("x", "y"), crs = 28992, 
#'                   agr = "constant") 
#' 
#' ref <- rast(ext(meuse.grid), resolution = 40)
#'   crs(ref) <- crs(meuse)
#' e <- ext(179407.8, 181087.9, 331134.4, 332332.1)
#'                                        
#' # GRID-1 log(copper): 
#' v1 <- variogram(log(copper) ~ 1, meuse) 
#'   x1 <- fit.variogram(v1, vgm(1, "Sph", 800, 1))           
#'   G1 <- krige(zinc ~ 1, meuse, meuse.grid, x1, nmax = 30)
#' G1 <- crop(rasterize(G1, ref, "var1.pred"),e)
#' names(G1) <- "copper"
#'  
#'  # GRID-2 log(elev):
#' v2 <- variogram(log(elev) ~ 1, meuse) 
#'   x2 <- fit.variogram(v2, vgm(1, "Sph", 800, 1))           
#'   G2 <- krige(zinc ~ 1, meuse, meuse.grid, x2, nmax = 30)
#' G2 <- crop(rasterize(G2, ref, "var1.pred"),e)
#' names(G2) <- "elev"
#' 
#' # Raster corrected correlation 
#' acor <- raster.modified.ttest(G1, G2)
#'   plot(acor)
#'  
#' # Sample-based corrected correlation
#' ( cor.hex <- raster.modified.ttest(G1, G2, sample = "hexagonal") )	 
#'   plot(cor.hex["corr"], pch=20)
#' }
#' }
#' @seealso \code{\link[SpatialPack]{modified.ttest}} for test details
#'
#' @export raster.modified.ttest
raster.modified.ttest <- function(x, y, d = "auto", sample = c("none", "random", "hexagonal", "regular"), 
								  p = 0.10, size = NULL) {	
  if(length(find.package("SpatialPack", quiet = TRUE)) == 0)								  
    stop("please install SpatialPack package before running this function")	
  if(missing(x))
    stop("x argument is missing")
  if(!inherits(x, "SpatRaster"))
    stop(deparse(substitute(x)), " must be a terra SpatRast object")	
  if(missing(y))
    stop("y argument is missing")	
  if(!inherits(y, "SpatRaster"))
    stop(deparse(substitute(y)), " must be a terra SpatRast object")
  if(!terra::ext(x) == terra::ext(y))
    stop("Extents do not match")
  if(!any(terra::res(x) == terra::res(y)))
    stop("Resolutions do not match")
  if(sample[1] == "none") {
    if(d == "auto"){
      d = sqrt(2*((terra::res(x)[1]*3)^2))
    } else {
      if(!is.numeric (d)) stop("Distance (d) must be numeric")
    }  
    pts <- sf::st_as_sf( terra::as.points( c(x,y) ) )
      names(pts)[1:2] <- c("x","y")
    nb <- spdep::dnearneigh(pts,0, d)  
    spatial.corr <- data.frame()
      for(i in 1:length(nb)) {
	    if(length(nb[[i]]) > 3) {
		  x.var <- pts$x[nb[[i]]]
		  y.var <- pts$y[nb[[i]]]
        sc <- SpatialPack::modified.ttest(x.var, y.var, 
	            sf::st_coordinates(pts[,1:2][nb[[i]],]), nclass = 1)		
        spatial.corr <- rbind(spatial.corr, round(data.frame(corr = sc$corr, 
		                      Fstat= (sc$dof * sc$Fstat),  
		                      p.value = sc$p.value, moran.x = sc$imoran[1], 
							  moran.y = sc$imoran[2]),5))
        } else {
          spatial.corr <- rbind(spatial.corr, rep(NA, 5))
        }		  							  
      }
	  v <- c("corr", "Fstat", "p.value", "moran.x", "moran.y")  
	  names(spatial.corr) <- v  
        pts <- cbind(pts, spatial.corr)
      s <- terra::rast(lapply(v, function(v) {terra::rasterize(pts, x, field=v)}))	 
	    names(s) <- v
		  terra::crs(s) <- terra::crs(x)
	} else {
	# "random", "hexagonal", "regular"
	  if(!is.null(size)) {
        n = size 
      } else {
        n = round(terra::ncell(x) * p, 0)
      } 
	  e <- as.vector(terra::ext(x))[c(1,3,2,4)] 
      e <- sf::st_as_sfc(sf::st_bbox(c(e[1], e[2], e[3], e[4])))
	  pts <- sf::st_as_sf(sf::st_sample(e, size=n, type=sample[1]))
         pts <- cbind(pts, terra::extract(c(x,y), terra::vect(pts))[,-1])
		   pts <- stats::na.omit(pts)
            sf::st_geometry(pts) <- "geometry"
		names(pts)[1:2] <- c("x","y")   
	    if(d == "auto"){
		  dm <- sf::st_distance(pts)
		    diag(dm) <- NA
              d = sqrt(2*((min(dm, na.rm=TRUE)*3)^2))
        } else {
          if(!is.numeric (d)) stop("Distance (d) must be numeric")
        }	   	   
	  nb <- spdep::dnearneigh(pts,0, d)
	  spatial.corr <- data.frame()	  
        for(i in 1:length(nb)) {
	      if(length(nb[[i]]) > 3) {
		  x.var <- pts$x[nb[[i]]]
		  y.var <- pts$y[nb[[i]]]
          sc <- SpatialPack::modified.ttest(x.var, y.var, 
	              sf::st_coordinates(pts[,1:2][nb[[i]],]), 
	              nclass = 1)		
          spatial.corr <- rbind(spatial.corr, round(data.frame(corr = sc$corr, 
		                        Fstat= (sc$dof * sc$Fstat),  
		                        p.value = sc$p.value, moran.x = sc$imoran[1], 
		  	  				    moran.y = sc$imoran[2]),5))
          } else {
            spatial.corr <- rbind(spatial.corr, rep(NA, 5))
          }		  
        }
	    names(spatial.corr) <- c("corr", "Fstat", "p.value", "moran.x", "moran.y")
          s <- cbind(pts, spatial.corr)
		    s <- s[,-which(names(s) %in% c("x","y"))] 
	  }
  return( s )  
}

#' @title Insert a row or column into a data.frame
#' @description Inserts a new row or column into a data.frame at a specified location
#' 
#' @param x        Existing data.frame
#' @param MARGIN   Insert a 1 = row or 2 = column
#' @param value    A vector of values equal to the length of MARGIN,
#'                 if nothing specified values with be NA
#' @param idx      Index position to insert row or column
#' @param name     Name of new column (not used for rows, MARGIN=1)
#'
#' @details 
#' Where there are methods to easily add a row/column to the end or beginning of a data.frame, 
#' it is not straight forward to insert data at a specific location within the data.frame. 
#' This function allows for inserting a vector at a specific location eg., between columns or 
#' rows 1 and 2 where row/column 2 is moved to the 3rd position and a new vector of values is 
#' inserted into the 2nd position.   
#'
#' @return A data.frame with the new row or column inserted
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com> 
#'
#' @examples
#' d <- data.frame(ID=1:10, y=runif(10))
#'
#' # insert row
#' insert(d, idx=2)
#' insert(d, value=c(20,0), idx=2)
#'
#' # insert column
#' insert(d, MARGIN=2, idx=2)
#' insert(d, MARGIN = 2, value = rep(0,10), idx=2, name="x")
#'
#' @export insert
insert <- function(x, MARGIN = 1, value = NULL, idx, name=NULL) {
  if(class(x)[1] != "data.frame")
    stop("x must be data.frame object")
  if(missing(idx))
    stop("idx argument must be supplied")
  idx = idx[1]
  if(MARGIN == 1) {
    message("Inserting row")
    if(is.null(value)) value = rep(NA, ncol(x))
	  if(length(value) != ncol(x))
	    stop("specified values not equal number of columns")
      x[seq(idx+1, nrow(x)+1),] <- x[seq(idx, nrow(x)),]
        x[idx,] <- value
  } else if(MARGIN == 2) {
    message("Inserting column")
    n <- names(x)
    if(is.null(value)) value = rep(NA, nrow(x))
	  if(length(value) != nrow(x))
	    stop("specified values not equal number of columns")
      x[,seq(idx,ncol(x)+1)] <- x[,seq(idx, ncol(x))]
        x[,idx] <- value
	    names(x)[-idx] <- n
      if(is.null(name)) name = "V1" 
    names(x)[idx] <- name
  }
  return(x)
}

#' @title Print effect size
#' @description print method for class "effect.size"
#'
#' @param x    Object of class effect.size
#' @param ...  Ignored
#'
#' @return
#' Prints the output data.frame contaning; effect size with upper and lower confidence 
#' and, mean and sd by group  
#'
#' @method print effect.size
#' @export
print.effect.size <- function(x, ...) {
  for(i in 1:nrow(x$effect.size)) {
    print(paste( paste0("Effect size for: ", rownames(x$effect.size)[i]), 
	      round(x$effect.size[,1][i],4), sep=" = ") )
	cat(paste0(paste(rep(" ", 4), collapse = ""), 
	    paste0( "Lower CI: ", round(x$effect.size[,2][i],4))))
    cat(paste0(paste(rep(" ", 4), collapse = ""), 
	    paste0( "Upper CI: ", round(x$effect.size[,3][i],3))))
	cat("", "\n")
	cat("", "\n")
  }
}  

#' @title Smooth Raster Time-series
#' @description Smooths pixel-level data in raster time-series and can impute 
#'              missing (NA) values.
#'
#' @param x            A terra SpatRaster with > 8 layers 
#' @param f            Smoothing parameter (see loess span argument)
#' @param smooth.data  (FALSE/TRUE) Smooth all of the data or just impute NA values 
#' @param ...          Additional arguments passed to terra::app (for 
#'                     writing results to disk)
#' 
#' @details  
#' This function uses a LOESS regression to smooth the time-series. If the data is 
#' smoothed, (using the smooth.data = TRUE argument) it will be entirely replaced by  
#' a loess estimate of the time-series (estimated distribution at the pixel-level). 
#' Alternately, with smooth.data = FALSE, the function can be used to impute missing 
#' pixel data (NA) in raster time-series (stacks/bricks).
#' The results can dramatically be effected by the choice of the smoothing 
#' parameter (f) so caution is warranted and the effect of this parameter tested.  
#' 
#' @return 
#' A terra SpatRaster containing imputed or smoothed data.  
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @examples
#' \donttest{
#' library(terra)
#'   random.raster <- function(rows=50, cols=50, l=20, min=0, max=1){ 
#'     do.call(c, replicate(l, rast(matrix(runif(rows * cols, min, max), 
#' 	        rows , cols))))
#'   }
#' r <- random.raster()
#' 
#' #### Smooth time-series using raster stack/brick 
#' r.smooth <- smooth.time.series(r, f = 0.4, smooth.data = TRUE)  
#' 
#' # extract pixel 100 for plotting
#' y <- as.numeric(r[100])
#' ys <- as.numeric(r.smooth[100])
#' 
#' # plot results	
#' plot(y, type="l")
#'   lines(ys, col="red")
#'     legend("bottomright", legend=c("original","smoothed"),
#'          lty=c(1,1), col=c("black","red"))	
#' }
#' @seealso \code{\link[stats]{loess}} for details on the loess regression  
#' @seealso \code{\link[terra]{app}} for details on additional (...) arguments 
#' @seealso \code{\link[spatialEco]{impute.loess}} for details on imputation model
#'  
#' @export smooth.time.series
smooth.time.series <- function(x, f = 0.80, smooth.data = FALSE, ...) { 
  if (!inherits(x, "SpatRaster")) 
    stop(deparse(substitute(x)), " must be a terra SpatRaster object")
  if(terra::nlyr(x) < 5)
    stop("Not enough observations (time-steps) for imputation or smoothing")  
  if(terra::nlyr(x) < 8)
      warning("function is intended for fitting a distribution 
	           in multi-temporal data\n      < 8 observations is questionable\n")	
  impute.loess <- function(y, x.length = NULL, s = f, 
                           sdata = smooth.data, na.rm, ...) {
         if (is.null(x.length)) {
            x.length = length(y)
         }
  	   if(length(y[!is.na(y)]) < 8) {
  	     warning("Fewer than 8 real-value observations, assigning NA")
  		   y <- rep(NA, x.length)
  	   } else {			   
           x <- 1:x.length
             p <- suppressWarnings( stats::loess(y ~ x, span = s, 
  		                            data.frame(x = x, y = y)) )
         if (sdata == TRUE) {
           y <- stats::predict(p, x)
         } else {
           na.idx <- which(is.na(y))
             if (length(na.idx) > 1) {
               y[na.idx] <- stats::predict(p, data.frame(x = na.idx))
             }
          } 
  	   }
     return(y)
   }   
  return( terra::app(x, fun = impute.loess, ...) ) 
}

#' @title Extract vertices for polygons or lines
#' @description Extracts [x,y] vertices from an sf line or polygon object
#'
#' @param x       An sf line or polygon class object
#' @param join    (TRUE/FALSE) Joint attributes from original object 
#'
#' @details 
#' This function returns the vertices of a line or polygon object, as opposed
#' to the polygon centroids or line start/stop coordinates 
#'
#' @return 
#' An sf POINT object of extrated line or polygon vertices
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @examples
#'
#' library(sf)
#' nc <- sf::st_read(system.file("shape/nc.shp", package="sf"))
#'   nc <- suppressWarnings(sf::st_cast(nc, "POLYGON"))
#'     nc <- nc[c(10,50),]
#'   
#' ( v <- extract.vertices(nc) )
#'   plot(st_geometry(nc))
#'     plot(st_geometry(v), pch=20, cex=2, col="red", add=TRUE)
#'
#' @export extract.vertices 
extract.vertices <- function(x, join = TRUE) {
  if(!inherits(x, c("sf", "sfc")))		
    stop(deparse(substitute(x)), " must be an sf object")
 if(!unique(as.character(sf::st_geometry_type(x))) %in% c("POLYGON", 
   "MULTIPOLYGON", "MULTILINESTRING", "LINESTRING"))
    stop(deparse(substitute(x)), " must be an sf POINT object")	
  if(join == TRUE) {
    xy <- ncp <- suppressWarnings(sf::st_cast(x, "POINT"))
	# xy <- as.data.frame(st_coordinates(x))[,c(1,2,4)]
    #   names(xy)[3] <- "LID"
    # xy <- split(xy, xy$L2)
    # xy <- lapply(1:nrow(x), function(i) {
    #   data.frame(xy[[i]],
    #     do.call("rbind", replicate(nrow(xy[[i]]), 
    #             sf::st_drop_geometry(x[i,]), 
    #   	   	  simplify = FALSE)))
    # })
    # xy <- do.call("rbind", xy)  
  } else {	
    xy <- as.data.frame(sf::st_coordinates(x))[,c(1,2,4)]
      names(xy)[3] <- "LID"  
    xy <- sf::st_as_sf(xy, coords = c("X", "Y"), 
                       crs = sf::st_crs(x), 
                       agr = "constant")
  } 
  return( xy )
}

#' @title Local Polynomial Regression
#' @description Calculates a Local Polynomial Regression for smoothing 
#'              or imputation of missing data.
#' 
#' @param y          Vector to smooth or impute NA values
#' @param x          Optional x covariate data (must match dimensions of y)
#' @param s          Smoothing parameter (larger equates to more smoothing)
#' @param impute     (FALSE/TRUE) Should NA values be inputed
#' @param na.only    (FALSE/TRUE) Should only NA values be change in y
#' @param ci         (FALSE/TRUE) Should confidence intervals be returned
#' @param ...        Additional arguments passed to loess 
#'
#' @details
#' This is a wrapper function for loess that simplifies data smoothing and imputation 
#' of missing values. The function allows for smoothing a vector, based on an index 
#' (derived automatically) or covariates. If the impute option is TRUE NA values are 
#' imputed, otherwise the returned vector will still have NA's present. If impute and 
#' na.only are both TRUE the vector is returned, without being smoothed but with imputed 
#' NA values filled in. The loess weight function is defined using the tri-cube weight 
#' function w(x) = (1-|x|^3)^3 where; x is the distance of a data point from the point 
#'  the curve being fitted.  
#'
#' @return If ci = FALSE, a vector of smoothed values,
#'         otherwise a list object with:
#' *  loess - A vector, same length of y, representing the smoothed or 
#'            inputed data 
#' *  lower.ci - Lower confidence interval 
#' *  upper.ci - Upper confidence interval
#' @md
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'                                                              
#' @examples 
#'  x <- seq(-20, 20, 0.1)
#'  y <- sin(x)/x + rnorm(length(x), sd=0.03)
#'  p <- which(y == "NaN")
#'  y <- y[-p]
#'  r <- poly.regression(y, ci=TRUE, s=0.30)
#'  
#'  plot(y,type="l", lwd=0.5, main="s = 0.10")
#'    y.polygon <- c((r$lower.ci)[1:length(y)], (r$upper.ci)[rev(1:length(y))])
#'    x.polygon <- c(1:length(y), rev(1:length(y)))
#'    polygon(x.polygon, y.polygon, col="#00009933", border=NA) 
#'       lines(r$loess, lwd=1.5, col="red")
#'  
#'  # Impute NA values, replacing only NA's
#'  y.na <- y
#'  y.na[c(100,200,300)] <- NA 
#'  p.y <- poly.regression(y.na, s=0.10, impute = TRUE, na.only = TRUE)
#'  y - p.y
#'  
#'  plot(p.y,type="l", lwd=1.5, col="blue", main="s = 0.10")
#'    lines(y, lwd=1.5, col="red")
#'
#' @seealso \code{\link[stats]{loess}} for loess ... model options  
#'
#' @export
poly.regression <- function(y, x = NULL, s = 0.75, impute = FALSE, 
                            na.only = FALSE, ci = FALSE, ...) {
  na.idx <- which(is.na(y))
    if(length(na.idx) > 0 )
      {
        y.na <- y 
	    y <- y[!is.na(y)]
      }
  if(is.null(x)) { 
    x <- 1:length(y)
    fmla <- stats::as.formula(y ~ x)
	r <- stats::loess(fmla, span = s, ...)
  } else {
    if(!is.data.frame(x)) x <- as.data.frame(x) 
      x.names <- names(x)
	  dat <- data.frame(y=y, x)
    fmla <- stats::as.formula(paste(paste("y", "~", sep=""), paste(x.names, collapse= "+")))
	r <- stats::loess(fmla, data=dat, span = s, ...)
  }
  if(ci == TRUE) { 
    r.pred <- stats::predict(r, se = TRUE)
      low.ci <- r.pred$fit - 1.96 * r.pred$se.fit
      up.ci <- r.pred$fit + 1.96 * r.pred$se.fit
  }
  if(impute == TRUE && length(na.idx) >= 1) {
    p <- stats::predict(r, data.frame(x=na.idx))
	  if(na.only) {
	    r <- y.na
	    r[na.idx] <- p
	  } else {
        r <- spatialEco::insert.values(r$fitted, p, na.idx)
	  }
  } else {		
    if(length(na.idx) > 0) { 
	  r <- spatialEco::insert.values(r$fitted, NA, na.idx)
	} else {
	  r <- r$fitted
    }	
  }  
  if(ci == TRUE) {
    return( list( loess = r, lower.ci = low.ci, upper.ci = up.ci) )
  } else {	
    return( r )
  }
}

#' @title Ecological similarity
#' @description Uses row imputation to identify "k" ecological similar observations 
#'
#' @param x          data.frame containing ecological measures 
#' @param k          Number of k nearest neighbors (kNN)
#' @param method     Method to compute multivariate distances c("mahalanobis", "raw", 
#'                   "euclidean", "ica")
#' @param frequency  Calculate frequency of each reference row (TRUE/FALSE) 
#' @param scale      Scale multivariate distances to standard range (TRUE/FALSE)
#' @param ID         Unique ID vector to use as reference ID's (rownames). Must be 
#'                   unique and same length as number of rows in x
#' 
#' @details 
#' This function uses row-based imputation to identify k similar neighbors for each 
#' observation. Has been used to identify offsets based on ecological similarity. 
#' 
#' @return 
#' data.frame with k similar targets and associated distances. If frequency = TRUE  the 
#' freq column represents the number of times a row (ID) was selected as a neighbor.
#' 
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @references
#' Evans, J.S., S.R. Schill, G.T. Raber (2015) A Systematic Framework for Spatial 
#'   Conservation Planning and Ecological Priority Design in St. Lucia, Eastern 
#'   Caribbean. Chapter 26 in Central American Biodiversity : Conservation, Ecology 
#'   and a Sustainable Future. F. Huettman (eds). Springer, NY.   
#'
#' @examples
#' \donttest{ 
#'  library(sf)
#'  data(pu)
#'  kNN <- similarity(st_drop_geometry(pu[2:ncol(pu)]), k = 4, 
#'                    frequency = TRUE, ID = pu$UNIT_ID)  
#'  p <- kNN$freq   
#'  clr <- c("#3288BD", "#99D594", "#E6F598", "#FEE08B", 
#'           "#FC8D59", "#D53E4F")   
#'  p <- ifelse(p <= 0, clr[1], 
#'         ifelse(p > 0 & p < 10, clr[2],
#'           ifelse(p >= 10 & p < 20, clr[3],
#'  	       ifelse(p >= 20 & p < 50, clr[4],
#'  	         ifelse(p >= 50 & p < 100, clr[5],
#'  	           ifelse(p >= 100, clr[6], NA))))))
#'  plot(st_geometry(pu), col=p, border=NA)
#'    legend("topleft", legend=c("None","<10","10-20",
#'           "20-50","50-100",">100"),
#'           fill=clr, cex=0.6, bty="n") 
#'    box()
#' }
#'  
#' @export similarity
similarity <- function(x, k=4, method="mahalanobis", frequency = TRUE, 
                      scale = TRUE, ID = NULL) {
  if(length(find.package("yaImpute", quiet = TRUE)) == 0)
    stop("please install yaImpute package before running this function")
  if(!class(x)[1] == "data.frame") stop( "x is not a data.frame")
    if(!is.null(x)) {  
      if(!length(unique(ID)) == nrow(x) ) stop("ID's are not unique")
	  rownames(x) <- ID
    }	  
    offsets <- yaImpute::yai(x=x, method=method, k=k)
      if( scale == TRUE) {
       dmax <- max(offsets$neiDstRefs)
         for (i in 1:dim(offsets$neiDstRefs)[2] ) {
           offsets$neiDstRefs[,i] <- offsets$neiDstRefs[,i] / dmax 
         }
      }
    imp.ids <- as.data.frame(offsets$neiIdsRefs) 
      for (i in 1:dim(imp.ids)[2] ) { 
        imp.ids[,i] <- as.numeric(as.character(imp.ids[,i])) 
      }
    KNN <- data.frame(row.names=as.character(rownames(offsets$neiIdsRefs)), 
	                  imp.ids, offsets$neiDstRefs)					 
      if(frequency == TRUE) {
        neighbors <- as.vector(as.matrix(KNN[,1:4])) 	  
        f <- vector()
        for(i in 1:nrow(KNN)) {  
          f <- append(f, length(neighbors[neighbors == as.numeric(rownames(KNN[i,]))]))
		}
	KNN <- data.frame(KNN, freq = f) 
   }
  return( KNN ) 
}

#' @title Annulus matrix
#' @description Creates a square matrix representing annulus position values of 1 
#'              and defined null 
#' 
#' @param scale        Number of rings (defines dimensions of matrix)
#' @param inner.scale  Number of inner rings to set to null.value 
#' @param outer.scale  Number of outer rings to set to null.value
#' @param null.value   Value to set inner and outer scale(s) to
#'
#' @details 
#' This function will return a matrix of 1 and defined null.value based on a specification
#' of the scale, inner scale and outer scale. The scale defines how many rings will be
#' represented in the matrix based on (2 * scale - 1). So, a scale of 3 will result in a
#' 5x5 matrix. The inner.scale and outer.scale arguments represent the > and < rings that
#' will be set to the defined null.value (see examples). The resulting matrix can be used
#' as the specified window in a focal function.      
#'
#' @return A matrix object with defined null.value and 1, representing retained rings  
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com> 
#'
#' @examples
#' annulus.matrix(5)                   # 5 concentric rings
#' annulus.matrix(5, 3)                # 5 concentric rings with the 3 inner set to 0
#' annulus.matrix(5, 3, null.value=NA) # 5 concentric rings with the 3 inner set to NA
#' annulus.matrix(5, 3, 5)             # 5 rings with 3 inner and 5 outer set to 0
#' annulus.matrix(9, 3, 7)             # 9 rings with 3 inner and 7 outer set to 0
#' 
#' @export annulus.matrix 
annulus.matrix <- function(scale = 3, inner.scale = 0, outer.scale = 0,  
                           null.value = 0) {
  if(scale <= inner.scale) stop("inner scale must be < than scale")
    if(outer.scale > scale) stop("outer scale must be <= than scale")	
  if(scale %% 2 == 0)
     stop("Outer scale must be an odd number") 
  if(inner.scale > 0) {	 
    if(inner.scale %% 2 == 0)
       stop("Inner scale must be an odd number")
  }	
  if(outer.scale > 0) {	 
    if(outer.scale %% 2 == 0)
       stop("Inner scale must be an odd number")
  }	  
  if(inner.scale > 0 & outer.scale > 0) {
    if(!(outer.scale >= inner.scale + 2))
	  stop("outer.scale must be at least two values larger than inner.scale")
  } 	 
    n <- 2 * scale - 1
      m <- diag(n)
      x <- pmax(abs(row(m) - scale), abs(col(m) - scale))
	  if(inner.scale > 0) x[x < inner.scale] <- null.value
      if(outer.scale > 0) x[x <= scale & x >= (outer.scale-1)] <- null.value
	x[x >= 1] <- 1
  return( x )  
}

#' @title Loess Bootstrap 
#' @description Bootstrap of a Local Polynomial Regression (loess) 
#' 
#' @param  y            Dependent variable
#' @param  x            Independent variable  
#' @param  nreps        Number of bootstrap replicates 
#' @param  confidence   Fraction of replicates contained in confidence 
#'                      region 
#' @param  ...          Additional arguments passed to loess function
#'
#' @details 
#' The function fits a loess curve and then calculates a symmetric nonparametric 
#' bootstrap with a confidence region. Fitted curves are evaluated at a fixed number 
#' of equally-spaced x values, regardless of the number of x values in the data. Some 
#' replicates do not include the values at the lower and upper end of the range of x   
#' values. If the number of such replicates is too large, it becomes impossible to 
#' construct a confidence region that includes a fraction "confidence" of the bootstrap 
#' replicates. In such cases, the left and/or right portion of the confidence region 
#' is truncated.
#'
#' @return list object containing  
#' * nreps        Number of bootstrap replicates 
#' * confidence   Confidence interval (region)
#' * span         alpha (span) parameter used loess fit
#' * degree       polynomial degree used in loess fit
#' * normalize    Normalized data (TRUE/FALSE)
#' * family       Family of statistic used in fit
#' * parametric   Parametric approximation (TRUE/FALSE) 
#' * surface      Surface fit, see loess.control
#' * data         data.frame of x,y used in model
#' * fit          data.frame including:
#'   1) x - Equally-spaced x index (see NOTES)         
#'   2) y.fit - loess fit
#'   3) up.lim - Upper confidence interval 
#'   4) low.lim - Lower confidence interval 
#'   5) stddev - Standard deviation of loess fit at each x value
#' @md
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @references
#' Cleveland, WS, (1979) Robust Locally Weighted Regression and Smoothing Plots Journal  
#'   of the American Statistical Association 74:829-836
#' 
#' Efron, B., and R. Tibshirani (1993) An Introduction to the Bootstrap Chapman and 
#'   Hall, New York
#' 
#' Hardle, W., (1989) Applied Nonparametric Regression Cambridge University Press, NY.
#' 
#' Tibshirani, R. (1988) Variance stabilization and the bootstrap. 
#'   Biometrika 75(3):433-44.
#'
#' @examples 
#'  n=1000
#'  x <- seq(0, 4, length.out=n)	 
#'  y <- sin(2*x)+ 0.5*x + rnorm(n, sd=0.5)
#'  sb <- loess.boot(x, y, nreps=99, confidence=0.90, span=0.40)
#'  plot(sb)
#'                      	
#' @export    	                               
loess.boot <- function(x, y, nreps=100, confidence=0.95, ...){
    dat <- stats::na.omit(data.frame(x=x,y=y))
      if(nrow(dat) == 0) stop ( "Error in dropping NA's")
        ndx <- order(dat$x)
          dat$x <- dat$x[ndx]
        dat$y <- dat$y[ndx]
      r <- range(dat$x, na.rm=TRUE)
    x.out <- seq(r[1], r[2], length.out=40)
    f <- stats::loess(y~x, data=dat, ...)
      y.fit <- stats::approx(f$x, stats::fitted(f), x.out,rule=2)$y
    len <- length(dat$x)
    mat <- matrix(0,nreps,length(x.out))
    for(i in seq(nreps)){
      ndx <- sample(len,replace=TRUE)
        x.repl <- x[ndx]
          y.repl <- y[ndx]
            f <- stats::loess(y.repl~x.repl, ...)
        mat[i,] <- stats::predict(f, newdata=x.out)
      }
    n.na <- apply(is.na(mat), 2, sum)  
      nx <- ncol(mat)
        up.lim <- rep(NA, nx)
      low.lim <- rep(NA, nx)
    stddev <- rep(NA, nx)
    for(i in 1:nx) {
        if(n.na[i] > nreps*(1.0-confidence)) {
            next
        }
        conf <- confidence*nreps/(nreps-n.na[i])
          pr <- 0.5*(1.0 - conf)
            up.lim[i] <- stats::quantile(mat[,i], 1.0-pr, na.rm=TRUE)
          low.lim[i] <- stats::quantile(mat[,i], pr, na.rm=TRUE)
        stddev[i] <- stats::sd(mat[,i], na.rm=TRUE)
      }
    ndx <- !is.na(up.lim)   
    fit <- data.frame(x=x.out[ndx], y.fit=y.fit[ndx], up.lim=up.lim[ndx],
                      low.lim=low.lim[ndx], stddev=stddev[ndx])
	fit.boot <- list(nreps=nreps, confidence=confidence,
                     span=f$pars$span, degree=f$pars$degree, 
				     normalize=f$pars$normalize, family=f$pars$family,
				     parametric=f$pars$parametric, surface=f$pars$surface,
                     data=dat, fit=fit)
    class( fit.boot ) <- "loess.boot"
  return( fit.boot )
}

#' @title Mean Angle
#' @description Calculates the mean angle of a vector
#' 
#' @param a        vector of angle values
#' @param angle    ("degree", "radians") to define angle in degrees or radians
#'
#' @details
#' The arithmetic mean is not correct for calculating the central tendency of
#' angles. This function is intended to return the mean angle for slope or aspect,
#' which could be used in a focal or zonal function.  
#'
#' @return A vector of mean angle 
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com> 
#'
#' @examples
#' library(terra)
#' mean_angle(c(180, 10))
#'   mean(c(180, 10))
#' mean_angle(c(90, 180, 70, 60))
#'   mean(c(90, 180, 70, 60))
#' mean_angle(c(90, 180, 270, 360))
#'   mean(c(90, 180, 270, 360))
#'
#' elev <- rast(system.file("extdata/elev.tif", package="spatialEco"))
#' asp <- terrain(elev, v="aspect")
#' s <- buffer(spatSample(asp, 20, as.points=TRUE, 
#'             na.rm=TRUE, values=FALSE), 5000)
#' 
#' plot(asp)
#'   plot(s, add=TRUE)
#' 
#' d <- extract(asp, s)
#' cat("Mean angles of aspect", "\n")
#'   tapply(d[,2], d[,1], mean_angle) 
#' cat("arithmetic means of aspect", "\n")
#'   tapply(d[,2], d[,1], mean, na.rm=TRUE) 
#' 
#' @export
mean_angle <- function(a, angle=c("degree", "radians")) {
  angle=angle[1]
  a <- a[!is.na(a)]
  if(length(a) < 1)
    stop("Vector appears to be all NA's")
  deg2rad <- function(x) { x * pi/180} 
  rad2deg <- function(x) { x * 180/pi }
  deg2vec <- function(x, ang = c("degree", "radians")) { 
    if(ang == "degree") {
	  a <- c(sin(deg2rad(x)), cos(deg2rad(x)))
	} else if(ang == "radians") {
	  a <- c(sin(x), cos(x))
	}
	return(a)
  }
  vec2deg <- function(x) {
    res <- rad2deg(atan2(x[1], x[2]))
    if (res < 0) { res <- 360 + res }
	return(res)
  }
  mean_vec <- function(x) {
    y <- lapply(x, deg2vec, ang=angle)
    Reduce(`+`, y)/length(y)
  }
  return( vec2deg(mean_vec(a)) ) 
}

#' @title Vector Ruggedness Measure (VRM)
#' @description Implementation of the Sappington et al., (2007) vector 
#'              ruggedness measure
#'
#' @param x          A terra SpatRaster class object
#' @param s          Scale of window. Must be odd number, can represent 2 dimensions 
#'                   (eg., s=c(3,5) would represent a 3 x 5 window)
#'
#' @details
#' This function measures terrain ruggedness by calculating the vector 
#' ruggedness measure
#'
#' @return A terra SpatRaster class object of the VRI
#' 
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'  
#' @references 
#' Sappington, J.M., K.M. Longshore, D.B. Thomson (2007). Quantifying Landscape 
#'   Ruggedness for Animal Habitat Analysis: A case Study Using Bighorn Sheep in 
#'   the Mojave Desert. Journal of Wildlife Management. 71(5):1419-1426
#'
#' @examples 
#' library(terra)
#' elev <- rast(system.file("extdata/elev.tif", package="spatialEco"))
#'    vrm3 <- vrm(elev) 
#'    vrm5 <- vrm(elev, s=5)
#'    plot(c(vrm3, vrm5))
#'
#' @export vrm
vrm <- function(x, s = 3) {
  if (!inherits(x, "SpatRaster")) 
	stop(deparse(substitute(x)), " must be a terra SpatRaster object")
	if(length(s) > 2) 
	  stop( "Specified window exceeds 2 dimensions") 
    if(any((s %% 2) == 0))
      stop( "Specified window must be odd number(s)") 
    if(length(s) == 1) s = rep(s,2)
  vrm.fun <- function(x, y, z) { 
    sqrt( (x^2) + (y^2) + (z^2) ) 
    }
  f = matrix(1,s[1],s[2])	
  scale.factor <- round(s[1] * s[2], 0)
  sa <- terra::terrain(x, v=c("slope", "aspect"), 
                       unit="radians", neighbors=8) 
  sin.slp <- terra::app(sa[["slope"]], fun=sin)                  # xyRaster 
    cos.slp <- terra::app(sa[["slope"]], fun=cos)                # zRaster 
      sin.asp <- terra::app(sa[["aspect"]], fun=sin) * sin.slp   # xRaster
        cos.asp <- terra::app(sa[["aspect"]], fun=cos) * sin.slp # yRaster  
          x.sum <- terra::focal(sin.asp, w = f, fun=sum) 
        y.sum <- terra::focal(cos.asp, w = f, fun=sum) 
      z.sum <- terra::focal(cos.slp, w = f, fun=sum) 
    r <- terra::lapp(c(x.sum, y.sum, z.sum), fun=vrm.fun)
  return( 1 - (r / scale.factor) )
}

#' @title Raster multidimensional scaling (MDS) 
#' @description 
#' Multidimensional scaling of raster values within an N x N focal window
#'                                                                       
#' @param r              A terra SpatRaster class object 
#' @param s              Window size (may be a vector of 1 or 2) of 
#'                       n x n dimension. 
#' @param window.median  (FALSE/TRUE) Return the median of the MDS 
#'                        matrix values. 
#' @param ...            Additional arguments passed to terra::focal    
#'
#' @details
#' An MDS focal function. If only one value provided for s, then a square matrix 
#' (window) will be used. If window.median = FALSE then the center value of the 
#' matrix is returned and not the median of the matrix   
#'
#' @return A terra SpatRaster class object 
#' 
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'                                                                           
#' @references
#' Quinn, G.P., & M.J. Keough (2002) Experimental design and data analysis 
#'   for biologists. Cambridge University Press. Ch. 18. Multidimensional 
#'   scaling and cluster analysis.
#' 
#' @examples 
#' \donttest{ 
#'  library(terra)
#'  r <- rast(system.file("ex/elev.tif", package="terra"))
#'    r <- r[[1]] / max(global(r, "max", na.rm=TRUE)[,1])
#'  
#'  diss <- raster.mds(r)
#'  diss.med <- raster.mds(r, window.median = TRUE)
#'
#' opar <- par(no.readonly=TRUE)
#'    par(mfrow=c(2,2))
#'    plot(r)
#'      title("Elevation")
#'    plot( focal(r, w = matrix(1, nrow=5, ncol=5), fun = var) )
#'      title("Variance")		 
#'      plot(diss)
#'        title("MDS")
#'      plot(diss.med)
#'        title("Median MDS")
#' par(opar)
#' }
#'
#' @export raster.mds  
raster.mds <- function(r, s = 5, window.median = FALSE, ...) {
  if(!inherits(r, "SpatRaster"))
	  stop(deparse(substitute(r)), " must be a terra SpatRast object")
  if( length(s) == 1) s = c(s[1],s[1])
  cmd.diss <- function(x, n = s, k = 1, med = window.median) {
    na.idx <- which(is.na(x))
    x <- x[!is.na(x)]
    if(length(x) < 2) {
      cmd <- NA	
	} else {
      if(stats::var(x) == 0) { 
        cmd <- 0
      } else {		
     cmd <- stats::cmdscale(stats::dist(x), k=1)
	 if(length(na.idx) > 0) cmd <- spatialEco::insert.values(cmd, NA, na.idx)
	   if(n[1] != n[2]) {
	     n=sort(n)
	     cmd <- matrix(cmd, nrow = n[2], ncol = n[1], byrow = TRUE)
		 post <- c(round(ncol(cmd)/2), round(nrow(cmd)/2)+1) 
	   } else {
	     cmd <- matrix(cmd, nrow = n, ncol = n, byrow = TRUE)
		 post <- c(round(ncol(cmd)/2)+1, round(nrow(cmd)/2)+1) 
	   }  
	     if( med == TRUE ) {
           cmd <- stats::median(cmd, na.rm = TRUE)
         } else {		
	       cmd <- cmd[,post[1]][post[2]]
	     }
	   }
     }	
    return(cmd)
  }  
  diss <- terra::focal(r, w = matrix(1, nrow=s[1], ncol=s[2]), 
	                    fun = cmd.diss, ...)						
} 

#' @title Topographic Position Index (tpi)
#' @description Calculates topographic position using mean deviations
#' 
#' @param x              A terra SpatRaster object
#' @param scale          focal window size (n-cell x n-cell for rectangle or 
#'                       distance for circle) 
#' @param win            Window type. Options are "rectangle" and "circle" 
#' @param normalize      Apply deviation correction that normalizes to local 
#'                       surface roughness 
#' @param zero.correct   Apply correction for zero values in matrix weights      
#' 
#' @return A terra SpatRaster object of tpi A terra SpatRaster object
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @references
#' De Reu, J., J. Bourgeois, M. Bats, A. Zwertvaegher, V. Gelorini, et al., (2014) 
#'   Application of the topographic position index to heterogeneous landscapes. 
#'   Geomorphology, 186:39-49.
#' 
#' @examples 
#' \donttest{
#' library(terra)
#' elev <- rast(system.file("extdata/elev.tif", package="spatialEco"))
#'
#' # calculate tpi and plot 
#'   tpi7 <- tpi(elev, scale=7) 
#'   tpi025 <- tpi(elev, win = "circle", scale=2500)
#'   tpi025.zc <- tpi(elev, win = "circle", scale=2500, 
#'                    zero.correct = TRUE)
#' 
#' opar <- par(no.readonly=TRUE)
#'     par(mfrow=c(2,2))
#'       plot(elev, main="original raster")
#'       plot(tpi7, main="tpi 7x7")
#'       plot(tpi025, main="tpi Circular window d=2500m")
#'	   plot(tpi025, main="tpi Circular window d=2500m, zero correct")
#' par(opar)
#' }
#'
#' @export tpi
tpi <- function(x, scale = 3, win = "rectangle", normalize = FALSE, 
                zero.correct = FALSE) {
  if (!inherits(x, "SpatRaster")) 
	stop(deparse(substitute(x)), " must be a terra SpatRaster object")

    if( win == "circle") {
      if( scale < terra::res(x)[1] * 2) 
        stop( "Scale is too small for a circular window")
        m <- terra::focalMat(x, scale, type="circle")
          m[m > 0] <- 1
      } else {
	     if(scale %% 2 == 0)
           stop("Scale for a rectangular window must be an odd number") 	  
         m <- matrix(1, nrow=scale, ncol=scale)
      }
	if( zero.correct ) {
	  tp <- x - terra::focal(x, w=m, fun = function(x, ...){ sum(x)/sum(m) })
    } else {	  
      tp <- x - terra::focal(x, w = m, fun = mean)
    }  
    if(normalize == TRUE) {
      if( zero.correct ) {
        tp.sd <- terra::focal(x, w=m, fun=function(x, ...)
			       { sqrt(sum(x-sum(x)/sum(m))^2/(sum(m)-1))})
      } else {	  
	    tp.sd <- terra::focal(x, w=m, fun=stats::sd)
	  }
	    tp <- tp / tp.sd 
    }
  return(tp)  
}

#' @name cor.data 
#' @description linear and nonlinear correlated data examples
#' @docType data
#'
#' @title Various correlation structures
#'
#' @description 
#' A list object with various linear and nonlinear correlation structures
#'
#' @format A list object with 4 elements containing data.frames:
#' \describe{
#'   \item{example 1}{two columns with nonlinear wave function relationship}
#'   \item{example 2}{two columns with simple nonlinear relationship}
#'   \item{example 3}{two columns with nonlinear multi-level wave function relationship}
#'   \item{example 4}{4 columns with first two having linear relationship}
#' }
#'
NULL

#' @title Print Loess bootstrap model
#' @description print method for class "loess.boot"
#'
#' @param x    Object of class loess.boot
#' @param ...  Ignored
#'
#' @return same as summary lowess.boot of data.frame including; 
#' * nreps        Number of bootstrap replicates 
#' * confidence   Confidence interval (region)
#' * span         alpha (span) parameter used loess fit
#' * degree       polynomial degree used in loess fit
#' * normalize    Normalized data (TRUE/FALSE)
#' * family       Family of statistic used in fit
#' * parametric   Parametric approximation (TRUE/FALSE) 
#' * surface      Surface fit, see loess.control
#' * data         data.frame of x,y used in model
#' * fit          data.frame including:
#'   1) x - Equally-spaced x index       
#'   2) y.fit - loess fit
#'   3) up.lim - Upper confidence interval 
#'   4) low.lim - Lower confidence interval 
#'   5) stddev - Standard deviation of loess fit at each x value
#' @md 
#' 
#' @method print loess.boot
#' @export
print.loess.boot <- function(x, ...) {
  cat("Number of permutations: ", x$nreps, "\n", sep="")
  cat("Loess alpha (span) parameter: ", x$span, "\n", sep="")
  cat("Polynomial degree: ", x$degree, "\n", sep="")
  cat("Data normalized: ", x$normalize, "\n", sep="")
  cat("Model distribution: ", x$family, "\n", sep="")
  cat("parametric model: ", x$parametric, "\n", sep="")
}

#' @title All pairwise combinations
#' @description Creates all pairwise combinations list for iteration
#'
#' @param x A numeric or character vector 
#' 
#' @details 
#' This returns a list of vector combinations starting with
#' pairwise, as the first nested list element, then in groups of 
#'  threes, fours, to length of the vector.  
#'
#' @return A list object with increasing all combination objects,
#'         the first list element are the pairwise comparisons
#'
#' @author Jeffrey S. Evans    <jeffrey_evans<at>tnc.org>
#'
#' @examples
#' classes <- paste0("class", 1:10)
#' 
#' all_pairwise(classes)[[1]]
#' 
#' #### How to use as an iterator
#' # dataframe with 4 cols, 100 rows
#' d <- as.data.frame(matrix(runif(100*4), 100, 4)) 
#'   names(d) <- paste0("class", 1:4) 
#' 
#' ( idx <- all_pairwise(colnames(d))[[1]] ) 
#' 
#' opar <- par(no.readonly=TRUE)
#'   par(mfrow=c(2,3))
#'     lapply(idx, function(i) {
#'       plot(d[,i[1]], d[,i[2]], main=paste0(i[1], " vs ", i[2]) )
#'     })	
#' par(opar)
#' 
#' @export
all_pairwise <- function(x) {
  if(!is.vector(x))
    stop(deparse(substitute(x)), " must be a vector object")  
  n <- length(x)
    idx <- lapply(2:n, function(y) {
      utils::combn(x, y, simplify = FALSE)
    })
  return(idx)
} 
 

#' @title Sieve raster data
#' @description Removes contiguous cells < specified query area
#'
#' @param x      An integer terra SpatRaster 
#' @param a      Query area to remove 
#' @param unit   The unit to use for area query options are c("m", "km", "ha")
#' 
#' @details 
#' A sieve can be used to establish a minimal mapping unit where
#' contiguous cells < specified query area are set to NA. These NA
#' values can then be filled using focal (majority, median, mean)
#'
#' @return 
#' A terra SpatRaster with cells < a set to NA
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @examples
#' \donttest{
#' library(terra)
#' elev <- rast(system.file("extdata/elev.tif", package="spatialEco"))
#' m <- matrix(c(100,200,1,200,300,2,300,400,3,400,
#'        500,4, 500,600,5), ncol=3, byrow=TRUE)
#'   x <- classify(elev, m)
#' 
#' # Sieve to a MMU of 60km
#' sv <- spatialEco::sieve(x, a = 60, unit = "km")
#'   plot(c(x, sv))
#' } 
#' @export
sieve <- function(x, a, unit = c("m", "km", "ha")) {
  if(missing(a))
    stop("Must define query area (a)")
  if (!inherits(x, "SpatRaster")) 
    stop(deparse(substitute(x)), " must be a terra SpatRaster object")		
  message("Sieve applied for a MMU < ", a, unit)
      rs <- terra::zonal(terra::cellSize(x, unit = unit[1]), 
	                     x, sum, as.raster = TRUE)					 
    rs <- terra::ifel(rs < a, NA, x)
  return( rs )
}  

#' @title Modified z-score for a raster
#' @description 
#' Calculates the modified z-score for raster values
#' 
#' @param x                A raster class object
#' @param p.value          Return p-value rather than z-score 
#'                         raster (FALSE/TRUE)
#' @param file.name        Name of raster written to disk
#' @param ...              Additional arguments passed to writeRaster
#'
#' @note 
#' Since this functions needs to operate on all of the raster values, 
#' it is not memory safe 
#'
#' @return raster class object or raster written to disk
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @examples 
#' \donttest{
#' library(terra)
#' r <- rast(nrows=500, ncols=500)
#'   r[] <- runif(ncell(r), 0, 1)
#'
#' # Modified z-score
#' ( z <- raster.Zscore(r) )
#'
#' # P-value
#' ( p <- raster.Zscore(r, p.value = TRUE) )
#' }	
#'
#' @export raster.Zscore
raster.Zscore <- function(x, p.value = FALSE, file.name = NULL, ...) {
  if (!inherits(x, "SpatRaster")) 
    stop(deparse(substitute(x)), " must be a terra SpatRaster object")
  r.vals <- x[][,1]
    na.idx <- which(!is.na(r.vals))
  if( p.value == TRUE ) {
    x[na.idx] <- stats::pnorm( outliers( stats::na.omit(r.vals) ) )
  } else {
    x[na.idx] <- outliers( stats::na.omit(r.vals) )  
  }
    if(!is.null(file.name)) { 
      terra::writeRaster(x, ...)
    } else {
      return( x )  
    }
} 

#' @title Distance-based subsampling
#'
#' @description Draws a minimum, and optional maximum constrained, distance sub-sampling 
#' 
#' @param x             A POLYGON or POINT sf object 
#' @param size          Subsample size 
#' @param d             Minimum sampling distance in meters
#' @param d.max         Maximum sampling distance in meters
#' @param replacement   (FALSE/TRUE) Subsample with replacement   
#'
#' @return A subsampled POLYGON or POINT sf object  
#'
#' @note This function provides a distance constrained subsample of existing point 
#'       or polygon data. Please note that units are in meters regardless of input
#'       CRS projection units (including lat/long).   
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com> 
#'
#' @examples      
#' \donttest{
#' if(require(sp, quietly = TRUE)) {
#' library(sf)
#'   data(meuse, package = "sp")
#'   meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, 
#'                     agr = "constant")
#' 
#' # Subsample with a 500m minimum sample spread 
#' sub.meuse <- subsample.distance(meuse, size = 10, d = 500)  
#'   plot(st_geometry(meuse), pch=19, main="min dist = 500")
#'     plot(st_geometry(sub.meuse), pch=19, col="red", add=TRUE) 
#'  
#' # Check distances	
#' dm <- st_distance(sub.meuse)
#'   diag(dm) <- NA
#' cat("\n", "Min distance for subsample", min(dm, na.rm=TRUE), "\n")  
#' cat("Max distance for subsample", max(dm, na.rm=TRUE), "\n")  
#' 
#' } else { 
#'   cat("Please install sp package to run example", "\n")
#' }
#' }
#' @export subsample.distance
subsample.distance <- function(x, size, d, d.max = NULL, 
                               replacement = FALSE) {						   
  gtypes = c("POLYGON", "POINT", "MULTIPOLYGON", "MULTIPOINT")			 
  if(missing(x)) 
    stop("Must define a spatial object")
  if(missing(d)) 
    stop("Must define minimum separation distance")
  if(!is.null(d.max)) {
    if(d.max <= d) 
	  stop("Maximum distance must be larger than minimum")
  } 
  if(!inherits(x, c("sf", "sfc")))
    stop(deparse(substitute(x)), " must be an sf object or coercible")	  
  if(any(unique(as.character(sf::st_geometry_type(x))) == gtypes[3:4]))
    stop("Function does not support multi-part geometry")  
  if(!any(unique(as.character(sf::st_geometry_type(x))) != gtypes[1:2]))
    stop(deparse(substitute(x)), " must be one of ", 
	     paste(gtypes, collopse=""))		     
  if( size >= nrow(x)) 
    stop("subsample size must be smaller than population")
  rs <- sample(1:nrow(x),1) 
    s <- x[rs,]
      if(replacement == FALSE) { x <- x[-rs,] }
        deval = TRUE	  
        for(i in 2:size) {	
          nsamp=0
          while(deval == TRUE) { 		  
            rs <- sample(1:nrow(x),1)
	          pts.dist <- units::drop_units(units::set_units(sf::st_distance(s, x[rs,]), "m"))
            if(is.null(d.max)) {
              deval <- any(pts.dist < d, na.rm = TRUE)
		    } else {
		      deval <- any(pts.dist < d, na.rm = TRUE) | any(pts.dist > d.max, na.rm = TRUE)
	        } 
            nsamp = nsamp + 1			
              if(nsamp == nrow(x)) break			
	      }  
        if(nsamp == nrow(x)) {
          message(paste0("Warning: sampling cannot converge at n=", size, " returning n=", nrow(s)))
            return(s)  
        }
	 deval = TRUE
	   s <- rbind(s, x[rs,])
	     if(replacement == FALSE) { x <- x[-rs,] } 
	} 	
  return(s)  
}

#' @title Canine-Human Age Equivalent
#' @description Calculates canines equivalent human age (for fun) 
#'
#' @param x  numeric vector, dog age
#' @return numeric vector, equivalent human age  
#'
#' @references
#' Wang, T., J. M, A.N. Hogan, S. Fong, K. Licon et al.  (2020) quantitative 
#'   translation of dog-to-human aging by conserved remodeling of epigenetic 
#'   networks. Cell Systems 11(2)176-185
#'     
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @examples
#' dat <- data.frame(DogAge = seq(0,18,0.25),
#'              HumanAge=chae(seq(0,18,0.25)))[-1,]
#' 
#' plot(dat$DogAge, dat$HumanAge, "l",
#'      main="Canine-Human Age Equivalence",
#' 	 ylab="Human Age", xlab="Dog Age")
#'   points( 15, chae(15), col="red", pch=19, cex=1.5)
#'   points( 10, chae(10), col="blue", pch=19, cex=1.5)
#'   points( 3, chae(3), col="black", pch=19, cex=1.5)
#' legend("bottomright", legend=c("Camas (15-YO)", "Kele (10-YO)", "Aster (3-YO)"), 
#'       pch=c(19,19,19), cex=c(1.5,1.5,1.5), 
#' 	     col=c("red","blue","black"))  
#' 
#' @export  
chae <- function(x) { round(16 * log(x) + 31, 2) }

#' @title Maximum extent of multiple rasters
#' @description returns a extent polygon representing maximum extent of
#'              input rasters
#'
#' @param x    terra SpatRaster class object
#' @param ...  additional SpatRaster class objects in same projection
#'
#' @details 
#' Creates a maximum extent polygon of all specified rasters
#'
#' @return An sf POLYGON class object representing maximum extents
#' 
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @examples
#' library(terra)
#' 
#' r1 <- rast(ext(61.87125, 76.64458, 23.90153, 37.27042))
#' r2 <- rast(ext(67.66625, 81.56847, 20.38458, 35.67347))
#' r3 <- rast(ext(72.64792,84.38125,5.91125,28.13347 ))
#' 
#' ( e <- max_extent(r1, r2, r3) )
#' plot(e, border=NA)
#'   plot(ext(r1), border="red", add=TRUE)
#'   plot(ext(r2), border="green", add=TRUE)
#'   plot(ext(r3), border="blue", add=TRUE)
#'   plot(e, border="black", add=TRUE)
#'			    
#'  sf::st_bbox(e) # full extent
#'
#' @export max_extent
max_extent <- function(x, ...) {
  if(length(list(...))){
    dots <- list(...)
	dots[[length(dots)+1]] <- x
  } else {
    dots <- list()
	dots[[1]] <- x
  }
  for(i in dots) { 
    if(!inherits(i, "SpatRaster"))		
      stop("One of the rasters is not a terra SpatRaster object")	
  } 
  e <- lapply(dots, function(i) {
    as.vector(terra::ext(i))[c(1,3,2,4)] 
  })
  if(length(dots) > 1) {  
    e <- do.call("rbind", lapply(e, function(b) {
      sf::st_as_sf(sf::st_as_sfc(sf::st_bbox(c(b[1], 
                   b[2], b[3], b[4]))))
    }))
  } else {
    e <- e[[1]]
    e <- sf::st_as_sf(sf::st_as_sfc(sf::st_bbox(c(e[1], 
                      e[2], e[3], e[4]))))
  }
  return(
    sf::st_as_sf(sf::st_as_sfc(sf::st_bbox(e)))
  )
}

#' @title Remove duplicate geometries
#' @description Removes duplicate geometries in a single-part feature class
#'
#' @param x           An sf POINT, POLYGON or LINESTRING object
#' @param threshold   A distance threshold indicating fuzzy duplication,
#'                    default i 0.00001
#'
#' @details
#' This function removes duplicate geometries based on order and not "non null" 
#' attribution or other factors, the first feature gets to stay. If one needs to 
#' know which points were removed sf::st_difference can be used between original 
#' data and results of the function.  
#'
#' @return sf object, of same feature class as x, with duplicate geometries removed
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#' 
#' @examples
#' library(sf)
#' 
#' # data with 10 duplicate obs
#' s <- data.frame(x = runif(100), y = runif(100))
#'   s <- data.frame(rbind(s, s[sample(1:nrow(s), 10),]) ) 
#'     s <- st_as_sf(s, coords = c("x", "y"))
#'       s$ID <- 1:nrow(s)
#' 
#' nrow(s) 
#' nrow( srmd <- remove_duplicates(s) )
#'
#' @export
remove_duplicates <- function(x, threshold=0.00001) {
  if(missing(x))
    stop("must supply x argument")
  if(!inherits(x, "sf"))		
    stop(deparse(substitute(x)), " must be an sf object")	
  if(!unique(as.character(sf::st_geometry_type(x))) %in% 
     c("POINT","POLYGON", "LINESTRING"))
    stop("Multipart geometry is not supported, must be 
	     POINT, POLYGON, or LINESTRING")	
  if(unique(as.character(sf::st_geometry_type(x))) == "POLYGON")
    warning("For POLYGON geometry, zero distance may indicate that a polygon
	  is entirely contained within another polygon (island effect)")		 
  d <- sf::st_is_within_distance(x, dist=threshold)
    dups <- unlist(mapply(function(x,y) x[x < y], d, seq_along(d)))
	  if(length(dups) > 0) {
	    message(length(dups), " duplicate observations removed")
          x <- x[-dups, ]
		return(x)
	  } else {
        message("No duplicate observations found")
      }		  
}

#' @title Hybrid K-means
#' @description 
#' Hybrid K-means clustering using hierarchical clustering to define 
#' cluster-centers 
#'
#' @param x        A data.frame or matrix with data to be clustered
#' @param k        Number of clusters
#' @param hmethod  The agglomeration method used in hclust
#' @param stat     The statistic to aggregate class centers (mean or median) 
#' @param ...      Additional arguments passed to \code{\link[stats]{kmeans}}
#'
#' @details 
#' This method uses hierarchical clustering to define the cluster-centers in the K-means 
#' clustering algorithm. This mitigates some of the know convergence issues in K-means.   
#' 
#'@note 
#' options for hmethod are: "ward.D", "ward.D2", "single", 
#' "complete", "average", mcquitty", "median", "centroid"
#'
#' @return 
#' returns an object of class "kmeans" which has a print and a fitted method
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @references 
#' Singh, H., & K. Kaur (2013) New Method for Finding Initial Cluster Centroids in 
#'   K-means Algorithm. International Journal of Computer Application. 74(6):27-30
#' 
#' Ward, J.H., (1963) Hierarchical grouping to optimize an objective function. Journal 
#'   of the American Statistical Association. 58:236-24
#'
#' @examples
#' x <- rbind(matrix(rnorm(100, sd = 0.3), ncol = 2),
#'            matrix(rnorm(100, mean = 1, sd = 0.3), ncol = 2))
#' 
#' # Compare k-means to hybrid k-means with k=4		   
#' km <- kmeans(x, 4)		   
#' hkm <- hybrid.kmeans(x,k=4)		   
#' 
#' opar <- par(no.readonly=TRUE)
#' par(mfrow=c(1,2))
#'   plot(x[,1],x[,2], col=km$cluster,pch=19, main="K-means")
#'   plot(x[,1],x[,2], col=hkm$cluster,pch=19, main="Hybrid K-means")
#' par(opar)
#'
#' @seealso \code{\link[stats]{kmeans}} for available ... arguments and function details  
#' @seealso \code{\link[stats]{hclust}} for details on hierarchical clustering 
#'  
#' @export hybrid.kmeans
hybrid.kmeans <- function(x, k = 2, hmethod = "ward.D", stat = mean, ...) {
 h.clust <- stats::hclust(stats::dist(x), method=hmethod)
   cut.clust <- stats::cutree(h.clust, k = k) 
   cluster.centers <- stats::aggregate(x, list(cut.clust), stat)[, -1]
 return( stats::kmeans(x, centers = cluster.centers, ...) )
}
 

#' @title Optimized sample variance 
#' @description 
#' Draws an optimal sample that minimizes or maximizes the sample variance 
#'
#' @param x       A vector to draw a sample from
#' @param n       Number of samples to draw
#' @param type    Type of sample variance optimization 
#'                c("maximized", "minimized")  
#'
#' @return 
#' A data.frame with "idx" representing the index of the original vector  
#' and "y" is the value of the sampled data
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @examples
#' library(sf)
#' if (require(sp, quietly = TRUE)) {
#'   data(meuse, package = "sp")
#'   meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, agr = "constant")
#'	
#'  n = 15
#'  # Draw n samples that maximize the variance of y
#'  ( max.sv <- optimized.sample.variance(meuse$zinc, 15) )
#'  
#'  # Draw n samples that minimize the variance of y
#'  ( min.sv <- optimized.sample.variance(meuse$zinc, 15, 
#'                type="minimized") )
#'  
#'  # Plot results
#'  plot(st_geometry(meuse), pch=19, col="grey")
#'    plot(st_geometry(meuse[max.sv$idx,]), col="red", add=TRUE, pch=19)
#'      plot(st_geometry(meuse[min.sv$idx,]), col="blue", add=TRUE, pch=19)
#'  	  box()
#'      legend("topleft", legend=c("population","maximized variance", 
#'             "minimized variance"), col=c("grey","red","blue"),  
#'             pch=c(19,19,19))  
#'  
#' } else { 
#'   cat("Please install sp package to run example", "\n")
#' }
#' @export optimized.sample.variance  
optimized.sample.variance <- function(x, n, type = "maximized") {
  if(!is.numeric(x)) stop("x is not a numeric vector")
    non.na.idx <- which(!is.na(x))
      if(length(non.na.idx) != length(x)) {
	    x <- stats::na.omit(x)
	    names(x) <- non.na.idx
	  } else {
        names(x) <- 1:length(x) 
      }	  
    y <- idx <- rep(NA, n)
    mean.y <- mean(x)
      for(i in 1:n){
	    if(type == "maximized") {  
          opt <- which.max((x - mean.y)^2)
	    } else if (type == "minimized") {
          opt <- which.min((x - mean.y)^2)		
        } else {
	      stop("Not a valid optimization option")
        }
        idx[i] <- as.numeric(names(opt))
        y[i] <- x[opt]
        x[opt] <- NA
      mean.y <- mean(y, na.rm=TRUE)
      }
    message("\n", paste0(type, " sample variance: "), stats::var(y), "\n")
  return( data.frame(idx=idx,y=y) )
}

#' @title Polygon perimeter
#' @description Calculates the perimeter length(s) for a polygon object
#'
#' @param x   sf POLYGON class object 
#'
#' @return A vector of polygon perimeters in projection units
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @examples 
#' library(sf)
#'   polys <- st_read(system.file("shape/nc.shp", package="sf"))
#'     polys <- suppressWarnings(st_cast(polys[c(10,100),], "POLYGON"))
#'  
#'  polyPerimeter(polys)
#'
#' @export
polyPerimeter <- function(x) {
  if (!inherits(x, "sf")) 
    stop(deparse(substitute(x)), " must be an sf POLYGON object")
  if(!unique(as.character(sf::st_geometry_type(x))) %in% c("POLYGON", "MULTIPOLYGON"))
    stop(deparse(substitute(x)), " must be an sf POLYGON object")		
  if(unique(as.character(sf::st_geometry_type(x))) %in% "MULTIPOLYGON")
    stop("Function does not support multi-part MLTIPOLYGON objects")		
  p <- suppressWarnings(sf::st_length(sf::st_cast(x, "LINESTRING")))
  return( units::drop_units(p) )
}  

#' @title Raster correlation
#' @description 
#' Performs a moving window correlation between two rasters
#'
#' @param x          A terra SpatRaster class object for x
#' @param y          A terra SpatRasterclass object for y
#' @param s          Scale of window. Can be a single value, two 
#'                   values for uneven window or a custom matrix. 
#'                   Must be odd number (eg., s=3, for 3x3 window or 
#'                   s=c(3,5) for 3 x 5 window)
#' @param type       Type of output, options are: "pearson", "spearman", 
#'                   "covariance"  
#'
#' @note 
#' The NA behavior is set to na.rm = TRUE to make default outputs
#' consistent between the terra and raster packages. 
#'
#' @return A terra SpatRaster class object
#' 
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'  
#' @examples 
#' \donttest{
#' library(terra)
#'  
#' r <- rast(system.file("ex/logo.tif", package="terra"))  
#'   x <- r[[1]]
#'   y <- r[[3]]
#'   
#'  r.cor <- rasterCorrelation(x, y, s = 5, type = "spearman")
#'    plot(r.cor)
#' }
#' 
#' @export rasterCorrelation
rasterCorrelation <- function(x, y, s = 3, type = "pearson") {
  if (!inherits(x, "SpatRaster")) 
    stop(deparse(substitute(x)), " must be a terra SpatRaster object")
  if (!inherits(y, "SpatRaster")) 
    stop(deparse(substitute(y)), " must be a terra SpatRaster object")
  if(!inherits(s, "matrix")) {  
    if(length(s) > 2) 
	  stop( "Specified window exceeds 2 dimensions")   
    if(length(s) == 1) { s = rep(s, 2) }
      s <- matrix(1,s[1],s[2])
  }
  tmpXY = x * y
    xBar <- terra::focal(x, w = s, fun = mean)
      yBar <- terra::focal(y, w = s, fun = mean)
        xyBar <- terra::focal(tmpXY, w = s, fun = mean)
  if(type == "covariance") {
    return( xyBar - (xBar * yBar) )
  } else if (type == "pearson") {
	  coVar <- xyBar - (xBar * yBar) 
      xStd <- terra::focal(x, w = s, fun=stats::sd)
      yStd <- terra::focal(y, w = s, fun=stats::sd)
      xyStd <-  xStd * yStd
      return( coVar / xyStd )
  } else if (type == "spearman") {
      coVar <- xyBar - (xBar * yBar)
      xStd <- terra::focal(x, w = s, fun=stats::sd)
      yStd <- terra::focal(y, w = s, fun=stats::sd)	
      return( coVar / (xStd * yStd) )
  } else {
	stop( "Only pearson, spearman correlation or covariance are supported")
  }
}

#' @title divergence
#' @description Kullback-Leibler Divergence (Cross-entropy) 
#'       
#' @param x     a vector of integer values, defining observed
#' @param y     a vector of integer values, defining estimates
#' @param type  Type of divergence statistic c("Kullback-Leibler", 
#'              "cross-entropy")
#' 
#' @return single value vector with divergence statistic
#' 
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'  
#' @examples
#' x <- round(runif(10,1,4),0)
#' y <- round(runif(10,1,4),0)
#'
#' divergence(x, y) 
#' divergence(x, y, type = "cross-entropy") 
#'
#' @export divergence
divergence <- function(x, y, type = c("Kullback-Leibler", "cross-entropy")) {
  type = type[1]
  if(!is.vector(x) | !is.vector(y))
    stop("x and y must be numeric of character vectors")
  if(any(type %in% c("Kullback-Leibler", "cross-entropy")==FALSE))
    stop("Not a valid option for statistic type")
  q <- table(x) / sum(table(x))  # observed or approximated
  p <- table(y) / sum(table(y))  # estimated or probability 
  classes <- intersect(names(q), names(p))
    p <- p[which(names(p) %in% classes)]
    q <- q[which(names(q) %in% classes)]
  if(type == "cross-entropy") {
    message("Cross-Entropy", "\n")
    return( -sum( q, log(p) ) ) 
  } else if(type == "Kullback-Leibler") {
    message("Kullback-Leibler", "\n")
    return( sum( p * log(p / q) ) )
  }
}  

#' @title Quadrats
#' @description Creates quadrat polygons for sampling or analysis  
#'
#' @param x    An sf POLYGONS object defining extent      
#' @param s    Radius defining single or range of sizes of quadrats    
#' @param n    Number of quadrats    
#' @param r    A rotation factor for random rotation, default is NULL
#' @param sp   (FALSE | TRUE) Output sp class object
#'               
#' @details 
#' The radius (s) parameter can be a single value or a range of values, 
#' representing a randomization range of resulting quadrat sizes. The
#' rotation (r) parameter can also be used to defined a fixed rotation or
#' random range of quadrat rotations. You can specify each of these parameters
#' using an explicit vector that will be sampled eg., seq(100,300,0.5)         
#' 
#' @return an sf POLYGONS object with rotated polygon(s)
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#' 
#' @examples
#' library(sf)
#' library(terra) 
#' 
#' # read meuse data and create convex hull 
#' if (require(sp, quietly = TRUE)) {
#'   data(meuse, package = "sp")
#'   meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, agr = "constant")
#'   e <- st_convex_hull(st_union(meuse))
#' 
#'  # Fixed size 250 and no rotation 
#'  s <- quadrats(e, s = 250, n = 10)
#'    plot(st_geometry(s))
#'  
#' \donttest{   
#' # Variable sizes 100-300 and rotation of 0-45 degrees
#' s <- quadrats(e, s = c(100,300), n = 10, r = c(0,45))
#'   plot(st_geometry(s))
#'  
#' # Variable sizes 100-300 and no rotation 
#' s <- quadrats(e, s = c(100,300), n = 10)
#'   plot(st_geometry(s))
#' }   
#'
#' } else { 
#'   cat("Please install sp package to run example", "\n")
#' }
#'  
#' @export quadrats
quadrats <- function(x, s = 250, n = 100, r = NULL, sp = FALSE) { 
  	if(!is.null(r)){
	  rrange = range(r)
	    if(min(rrange) < 0 | max(rrange) > 360)
	      stop("rotation parameter is out of 0-360 range")
	}	  
    quadrat <- list()
    for(i in 1:n) {
	  if(length(s) == 1) {
	    ss = s
	  } else if(length(s) == 2) {
        ss = sample(s[1]:s[2],1)
	  } else if(length(s) > 2) {
        ss = sample(s,1)	    
	  }
	  if(!is.null(r)){
	    if(length(r) == 1) {
	      rr = r
	    } else if(length(r) == 2) {
          rr = sample(r[1]:r[2],1)
	    } else if(length(rr) > 2) {
          rr = sample(rr,1)	    
	    }
	  }	 
       p <- sf::st_buffer(sf::st_sample(x, size=1, type="random"), ss)
	     p <- sf::st_as_sf(sf::st_as_sfc(sf::st_bbox(p)))
		     p$dist <- ss
	  if(!is.null(r)) {
        p <- rotate.polygon(p, angle = rr)
          p$angle <- rr		
            quadrat[[i]] <- p
      } else {
	    quadrat[[i]] <- p
	  }
    }
        quadrat <- do.call("rbind", quadrat)
	  sf::st_crs(quadrat) <- sf::st_crs(x)
	if(sp) p <- sf::as_Spatial(p)
  return(quadrat)   
}

#' @title Hexagons
#' @description Create hexagon polygons
#'                                                                                                                                                                              
#' @param x     sf class object indicating extent
#' @param res   Area of resulting hexagons
#' 
#' @details 
#' Based on extent of x, creates a hexagon mesh with size of hexagons defined by res argumnet 
#' 
#' @return sf POLYGONS object
#'                                                                 
#' @examples 
#' library(sf)
#' if(require(sp, quietly = TRUE)) {
#'   data(meuse, package = "sp")
#'   meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, 
#'                     agr = "constant")
#'
#' hex <- hexagons(meuse, res=300)   
#'   plot(st_geometry(hex))
#'     plot(st_geometry(meuse),pch=20,add=TRUE)
#' 
#' # subset hexagons to intersection with points
#' idx <- which(apply(st_intersects(hex, meuse, sparse=FALSE), 1, any))
#' hex.sub <- hex[idx,] 
#'   plot(st_geometry(hex.sub))
#'     plot(st_geometry(meuse),pch=20,add=TRUE)
#'
#' } else { 
#'   cat("Please install sp package to run example", "\n")
#' }
#'
#' @export      
hexagons <- function(x, res = 100) {
  if(!inherits(x, "sf"))		
    stop(deparse(substitute(x)), " must be an sf object")
  if(sf::st_is_longlat(x))
    stop("Data appears to be in Latitude/Longitude 
	  and needs to be projected")
  e <- sf::st_as_sfc(sf::st_bbox(x))
    res <- sf::st_as_sf(sf::st_make_grid(e, cellsize = res, 
                        square = FALSE))
        sf::st_geometry(res) <- "geometry"
      sf::st_crs(res) <- sf::st_crs(x) 						
  return(res)
} 

#' @title Impute loess
#' @description Imputes missing data or smooths using Loess regression 
#'
#' @param y       A vector to impute
#' @param s       Smoothing parameter () 
#' @param smooth  (FALSE/TRUE) Smooth data, else only replace NA's
#' 
#' @details  
#' Performs a local polynomial regression to smooth data or to 
#' impute NA values. The minimal number of non-NA observations to reliably
#' impute/smooth values is 6. There is not a reliably way to impute NA's
#' on the tails of the distributions so if the missing data is in the
#' first or last position of the vector it will remain NA. Please note
#' that smooth needs to be TRUE to return a smoothed vector, else only
#' NA's will be imputed.   
#'
#' @return 
#' A vector the same length as x with NA values filled or the data smoothed (or both).
#'
#' @author Jeffrey S. Evans  <jeffrey_evans<at>tnc.org>
#'
#' @examples 
#' data(cor.data)
#' d <- cor.data[[1]][,2]
#'   plot(d, type="l")
#'   lines(impute.loess(d, s=0.3, smooth=TRUE), lwd=2, col="red")
#'  
#' # add some NA's
#' d <- d[1:100]
#'   d[sample(30:70, 5)] <- NA 
#'   d
#'   
#' impute.loess(d, s=0.2)
#'   
#' @export impute.loess
impute.loess <- function(y, s = 0.2, smooth = FALSE) {
  x.length = length(y)
  if(length(y[!is.na(y)]) < 6) {
    warning("Fewer than 6 real-value observations, assigning NA")
      y <- rep(NA, x.length)
  } else {
    x <- 1:x.length
      p <- suppressWarnings(stats::loess(y ~ x, span = s, 
                            data.frame(x = x, y = y)))
    if (smooth == TRUE) {
      y <- stats::predict(p, x)
    } else {
      na.idx <- which(is.na(y))
        if(length(na.idx) > 1) {
          y[na.idx] <- stats::predict(p, data.frame(x = na.idx))
        }
    }
  }
  return(y)
}

#' @title Spatial cross correlation
#' @description Calculates univariate or bivariate spatial cross-correlation using  
#'              local Moran's-I (LISA), following Chen (2015)
#'
#' @param x               Vector of x response variables
#' @param y               Vector of y response variables, if not specified the 
#'                        univariate  statistic is returned
#' @param coords          A matrix of coordinates corresponding to (x,y), only 
#'                        used if w = NULL. Can also be an sp object with relevant 
#'                        x,y coordinate slot (ie., points or polygons)
#' @param w               Spatial neighbors/weights in matrix format. Dimensions 
#'                        must match (n(x),n(y)) and be symmetrical. If w is not defined 
#'                        then a default method is used.
#' @param type            c("LSCI","GSCI") Return Local Spatial Cross-correlation Index (LSCI) 
#'                        or Global Spatial cross-correlation Index (GSCI)
#' @param k               Number of simulations for calculating permutation distribution  
#'                        under the null hypothesis of no spatial autocorrelation
#' @param dist.function   ("inv.power", "neg.exponent", "none") If w = NULL, the default method 
#'                        for deriving spatial weights matrix, options are: inverse power 
#'                        or negative exponent, none is for use with a provided matrix    
#' @param scale.xy        (TRUE/FALSE) scale the x,y vectors, if FALSE it is assumed that  
#'                        they are already scaled following Chen (2015) 
#' @param scale.partial   (FALSE/TRUE) rescale partial spatial autocorrelation statistics
#' @param scale.matrix    (FALSE/TRUE) If a neighbor/distance matrix is passed, should it 
#'                         be scaled using (w/sum(w))
#' @param alpha = 0.05     confidence interval (default is 95 pct)
#' @param clust           (FALSE/TRUE) Return approximated lisa clusters
#' @param return.sims     (FALSE/TRUE) Return randomizations vector n = k
#'
#'
#' @details
#' In specifying a distance matrix, you can pass a coordinates matrix or spatial
#' object to coords or alternately, pass a distance or spatial weights matrix to
#' the w argument. If the w matrix represents spatial weights dist.function="none"
#' should be specified. Otherwise, w is assumed to represent distance and will be
#' converted to spatial weights using inv.power or neg.exponent. The w distances
#' can represent an alternate distance hypothesis (eg., road, stream, network distance)
#' Here are example argument usages for defining a matrix.      
#' * IF coords=x, w=NULL, dist.function= c("inv.power", "neg.exponent")
#'    A distance matrix is derived using the data passed to coords then 
#'    spatial weights derived using one of the dist.function options
#' * IF cords=NULL, w=x, dist.function= c("inv.power", "neg.exponent")
#'     It is expected that the distance matrix specified with w represent 
#'     some form of distance then the spatial weights are derived using 
#'     one of the dist.function options 
#' * IF cords=NULL, w=x, dist.function="none" 
#'     It is assumed that the matrix passed to w already represents 
#'     the spatial weights 
#'
#' @return 
#' When not simulated k=0, a list containing:
#' * I - Global autocorrelation statistic
#' * SCI - - A data.frame with two columns representing the xy and yx autocorrelation
#' * nsim - value of NULL to represent p values were derived from observed data (k=0)
#' * p - Probability based observations above/below confidence interval
#' * t.test - Probability based on t-test
#' * clusters - If "clust" argument TRUE, vector representing LISA clusters
#'
#' When simulated (k>0), a list containing: 
#'  * I - Global autocorrelation statistic
#'  * SCI - A data.frame with two columns representing the xy and yx autocorrelation
#'  * nsim - value representing number of simulations
#'  * global.p - p-value of global autocorrelation statistic
#'  * local.p - Probability based simulated data using successful rejection of t-test
#'  * range.p - Probability based on range of probabilities resulting from paired t-test
#'  * clusters - If "clust" argument TRUE, vector representing lisa clusters
#'
#' @references
#' Chen, Y.G. (2012) On the four types of weight functions for spatial contiguity 
#'    matrix. Letters in Spatial and Resource Sciences 5(2):65-72 
#' @references
#' Chen, Y.G. (2013) New approaches for calculating Moran’s index of spatial 
#'   autocorrelation. PLoS ONE 8(7):e68336
#' @references 
#' Chen, Y.G. (2015) A New Methodology of Spatial Cross-Correlation Analysis. 
#'   PLoS One 10(5):e0126158. doi:10.1371/journal.pone.0126158
#'
#' @examples
#' # replicate Chen (2015)
#'  data(chen)
#' ( r <- crossCorrelation(x=chen[["X"]], y=chen[["Y"]], w = chen[["M"]],  
#'                         clust=TRUE, type = "LSCI", k=0, 
#'                         dist.function = "inv.power") ) 
#'
#' \donttest{
#' library(sf)
#' library(spdep)
#'  
#'   if (require(sp, quietly = TRUE)) {
#'    data(meuse, package = "sp")
#'    meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, agr = "constant")
#'   } 
#' 
#' #### Using a default spatial weights matrix method (inverse power function)
#' ( I <- crossCorrelation(meuse$zinc, meuse$copper, 
#'              coords = st_coordinates(meuse)[,1:2], k=99) )
#'   meuse$lisa <- I$SCI[,"lsci.xy"]
#'     plot(meuse["lisa"], pch=20)
#' 
#' #### Providing a distance matrix
#' if (require(units, quietly = TRUE)) {
#'   Wij <- units::drop_units(st_distance(meuse))
#'  ( I <- crossCorrelation(meuse$zinc, meuse$copper, w = Wij, k=99) )
#' 
#' #### Providing an inverse power function weights matrix
#'   Wij <- 1 / Wij
#'     diag(Wij) <- 0 
#'       Wij <- Wij / sum(Wij) 
#'         diag(Wij) <- 0
#'  ( I <- crossCorrelation(meuse$zinc, meuse$copper, w = Wij, 
#'                          dist.function = "none", k=99) )
#' }
#' } 
#' 
#' @md 
#' @export crossCorrelation
crossCorrelation <- function(x, y = NULL, coords = NULL, w = NULL, type = c("LSCI", "GSCI"), 
                             k = 999, dist.function = c("inv.power", "neg.exponent", "none"), 
							 scale.xy = TRUE, scale.partial = FALSE, scale.matrix = FALSE, 
							 alpha = 0.05, clust = TRUE, return.sims = FALSE) {
	if(missing(x)) stop("x must be specified")
    if(is.null(y)) y = x						 
      if(length(y) != length(x)) stop("(X,Y) are not equal")
        if( length(which(is.na(x))) != 0 | length(which(is.na(y))) != 0) 
	      stop("NA's not permitted in (X,Y)")
            if( k == 0) message("Permutation is not being run, estimated p will be based on observed")
	          if(scale.xy == FALSE) warning("It is assumed that x,v vectors are already scaled") 		
    type = type[1]; dist.function = dist.function[1]		  
	  n <- length(x)
  if( is.null(w) ) {
    if( is.null(coords) ) stop("If no Wij matrix is provided, a coordinates matrix is required")
      w <- sp::spDists( coords ) 
  } else {
    if(!class(w)[1] == "matrix") stop("Spatial weights must be in matrix form")					   
      if(ncol(w) != length(x) | nrow(w) != length(x)) stop("Spatial weights matrix must be symmetrical and match x")
        w[which(is.na(w))] <- 0
  }  
    if( dist.function == "inv.power" ) {
      message("Calculating spatial weights matrix using inverse power function")
	    w <- 1 / w
          diag(w) <- 0 
        w <- w / sum(w) 
    } else if (dist.function == "neg.exponent") { 
      message("Calculating spatial weights matrix using negative exponent")    
	  diag(w) <- NA 
	  mu <- mean(w, na.rm=TRUE)
      for(i in 1:nrow(w)) {
        for(j in 1:nrow(w)) {
		 w[i,j] <- round(exp( (-2 * w[i,j]) / mu ),6)
        }
      }  
      diag(w) <- 0
    } else if (dist.function == "none") { 
      message("Wij matrix is being left raw")	
    } else {
      stop("Not a valid matrix option")
    }  
    if(scale.matrix) {
      if(sum(w) > 0) { w <- as.matrix(w / sum(w))}
    }   
  if( scale.xy ){  
    x <- ( x - mean(x) ) / ( stats::sd(x) * sqrt((length(x)-1)/length(x)) )
    y <- ( y - mean(y) ) / ( stats::sd(y) * sqrt((length(y)-1)/length(y)) )
  } 
  
  #### Local and global cross-correlation statistics function ####
  SCI <- function(x, y, W, type.cc, scale.cc) {
    if( type.cc == "LSCI" ) {
	# local spatial crosscorrelation index (chen's LSCIs)
	# The lsci.xy is empirically the same as Anselin's LISA
    lsci.xy = as.numeric( x*y%*%W )
    lsci.yx = as.numeric( y*x%*%W )
	  if(scale.cc) {
        lsci.xy <- (lsci.xy - min(lsci.xy)) * (1 - -1) / (max(lsci.xy) - min(lsci.xy)) + -1
        lsci.yx <- (lsci.yx - min(lsci.yx)) * (1 - -1) / (max(lsci.yx) - min(lsci.yx)) + -1
      }	
	sci =  data.frame(lsci.xy = lsci.xy, lsci.yx = lsci.yx)  
    } 
    if( type.cc == "GSCI" ) {
	# The global spatial crosscorrelation index (chen's SCI)
      gsci.xy = as.numeric( x*W%*%y ) #/ ( length(x) - 1 )
      gsci.yx = as.numeric( y*W%*%x ) #/ ( length(x) - 1 )
	    if(scale.cc) {
         gsci.xy <- (gsci.xy - min(gsci.xy)) * (1 - -1) / (max(gsci.xy) - min(gsci.xy)) + -1
  	     gsci.yx <- (gsci.yx - min(gsci.yx)) * (1 - -1) / (max(gsci.yx) - min(gsci.yx)) + -1
        }
      sci = data.frame(gsci.xy = gsci.xy, gsci.yx = gsci.yx)	  
    }
    return(sci)	
  } 
  global.i <- as.numeric(x%*%w%*%y) #/ (length(x) - 1) 
    if(type == "LSCI") { tstat = "lsci.xy" } else { tstat = "gsci.xy" }  
      sci.results <- SCI(x = x,  y = y, W = w, type.cc = type, scale.cc = scale.partial)   
      if(clust) {     
	    lisa.clust <- as.character( interaction(x > 0, w %*% y > 0) ) 
          lisa.clust <- gsub("TRUE", "High", lisa.clust)
            lisa.clust <- gsub("FALSE", "Low", lisa.clust)
	  }  
	  probs <- c(alpha / 2, 1 - alpha / 2)   
    if(k > 0) {
     message("\n Computing Permutation Distribution \n")
	 # Global Moran's-I p-value
       y.sim <- matrix(y[sample(1:n, size = n * k, replace = TRUE)], 
       		        nrow = n, ncol = k) 	
       isim <- apply(y.sim, MARGIN=2, function(j) t(x[sample(1:length(x))]) %*% w %*% j )
         ( global.p <- sum( abs(isim) > abs( global.i ) ) ) 	 

       # Local Moran's-I p-value	   
        y.sim <- matrix(y[sample(1:n, size = n * k, replace = TRUE)], 
                        nrow = n, ncol = k) 	
        isim <- apply(y.sim, MARGIN=2, function(j) {
             SCI(as.numeric(t(x[sample(1:length(x))])), j, W=w,type.cc=type,scale.cc=scale.partial)[,tstat] } )
		ttest.p <- round(apply(isim, 2, function(j) stats::t.test(sci.results[,tstat], y = j,
                          alternative = "two.sided", paired = TRUE, 
		 				  conf.level = 1-alpha)$p.value),6)
		p <- length( ttest.p[ttest.p > alpha] ) / length(ttest.p)							  
        p1 <- 2 * min(length(ttest.p[ttest.p > alpha]) / k, 
	                 length(ttest.p[ttest.p < alpha]) / k )	
	  results <- list(I=global.i, SCI=sci.results, nsim = k,  
                      global.p=global.p, local.p=p, range.p=p1)
                        if(clust) { results$clusters <- lisa.clust }					  
	                      if(return.sims) { results$simulated.I <- isim }
	    class(results) <- c("cross.cor","list") 
    } else {
     ttest.p <- round(stats::t.test(sci.results[,tstat], conf.level = 1-alpha)$p.value,6)
     #ci <- t.test(sci.results[,tstat], conf.level = 1-alpha)$conf.int
     ci <- stats::quantile(sci.results[,tstat], probs=probs)
     tstat <- sci.results[,tstat]
       p <- 2 * min(length(tstat[tstat >= ci[2]]) / length(tstat), 
	                length(tstat[tstat <= ci[1]]) / length(tstat) )
	  results <- list(I=global.i, SCI=sci.results, nsim = NULL,
                      p=p, t.test=ttest.p)
	    if(clust) { results$clusters <- lisa.clust }				  			  
	class(results) <- c("cross.cor","list")
    }
  return( invisible(results) )  
}

#' @title spectral separability
#' @description Calculates spectral separability by class  
#'           
#' @param x                   data.frame, matrix or vector of spectral values must,
#'                              match classes defined in y
#' @param y                   A vector or factor with grouping classes, must match
#'                              row wise values in x
#' @param jeffries.matusita   (TRUE/FALSE) Return J-M distance (default) else Bhattacharyya  
#'
#' @details 
#' Available statistics:
#' * Bhattacharyya distance (Bhattacharyya 1943; Harold 2003) measures the similarity
#'   of two discrete or continuous probability distributions.
#'
#' * Jeffries-Matusita (default) distance (Bruzzone et al., 2005; Swain et al., 1971)  
#'   is a scaled (0-2) version of Bhattacharyya. The J-M distance is asymptotic to 2, 
#'   where 2 suggest complete separability.
#' @md 
#'
#' @return 
#' A matrix of class-wise Jeffries-Matusita or Bhattacharyya distance separability values
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @references
#' Bhattacharyya, A. (1943) On a measure of divergence between two statistical 
#'   populations defined by their probability distributions'. Bulletin of the 
#'   Calcutta Mathematical Society 35:99-109
#' 
#' Bruzzone, L., F. Roli, S.B. Serpico (1995) An extension to multiclass cases of 
#'   the Jefferys-Matusita distance. IEEE Transactions on Pattern Analysis and  
#'   Machine Intelligence 33:1318-1321
#' 
#' Kailath, T., (1967) The Divergence and Bhattacharyya measures in signal  
#'   selection. IEEE Transactions on Communication Theory 15:52-60  
#' 
#' @examples 
#' require(MASS)                
#' # Create example data 
#' d <- 6                 # Number of bands
#' n.class <- 5           # Number of classes
#' n <- rep(1000, 5)                
#' mu <- round(matrix(rnorm(d*n.class, 128, 1), 
#'             ncol=n.class, byrow=TRUE), 0)
#' 
#' x <- matrix(double(), ncol=d, nrow=0)
#'   classes <- integer()
#'     for (i in 1:n.class) {
#'       f <- svd(matrix(rnorm(d^2), ncol=d))
#'       sigma <- t(f$v) %*% diag(rep(10, d)) %*% f$v
#'       x <- rbind(x, mvrnorm(n[i], mu[, i], sigma))
#'       classes <- c(classes, rep(i, n[i]))
#'     }
#' colnames(x) <- paste0("band", 1:6)
#' classes <- factor(classes, labels=c("water", "forest",
#'                   "shrub", "urban", "ag"))
#' 
#' # Separability for multi-band (multivariate) spectra 
#' spectral.separability(x, classes)
#'
#' # Separability for single-band (univariate) spectra 
#' spectral.separability(x[,1], classes)
#'
#' @export spectral.separability 
spectral.separability <- function(x, y, jeffries.matusita = TRUE) {
  if(!any(class(x)[1] == c("data.frame", "matrix", "numeric")))
    stop("x must be a matrix, data.frame or vector")
  if(!any(class(y)[1] == c("numeric", "character", "factor")))
    stop("y must be a vector")
  if(length(unique(y)) < 2) 
    stop("There are not multiple classes to test")  
  if(any(class(x)[1] == c("numeric")))
    x <- matrix(x, ncol=1)
  if(nrow(x) != length(y))
    stop("x and y do not match")	
  if(class(y)[1] == "factor") {
    cls <- levels(y) 
  } else {
    cls <- unique(y) 
  }  
  bhattacharyya <- function(m1, s1, m2, s2, jm = FALSE) {
    mahalanobis <- function(m1, m2, sigma) {
      m <- m1 - m2; m %*% solve(sigma, m)
    }
    d <- function(u) determinant(u, logarithm=TRUE)$modulus 
    s <- (s1 + s2)/2                           
    b <- mahalanobis(m1, m2, s)/8 + (d(s) - d(s1)/2 - d(s2)/2)/2
    if(jm)
      b <- sqrt(2*(1-exp(-b)))
    return( b ) 
  } 
  message("Calculating distances for ", length(cls), " classes and ", ncol(x), " spectra" )
  class.stats <- by(x, y, function(y) list(mean=apply(y, 2, mean), cov=stats::cov(y)))
  distances <- matrix(0.0, length(cls), length(cls))
    for (i in 2:length(cls)) {
      m1 <- class.stats[[i]]$mean
      s1 <- class.stats[[i]]$cov
        for (j in 1:(i-1)) {
          m2 <- class.stats[[j]]$mean
    	  s2 <- class.stats[[j]]$cov
          distances[i,j] <- distances[j,i] <- bhattacharyya(m1,s1,m2,s2,jm=jeffries.matusita)
        }
    }
	dimnames(distances) <- list(cls, cls) 
  return(distances)
}

#' @title Stratified random sample
#' @description Creates a stratified random sample of an sp class object
#'
#' @param x            An sf class object 
#' @param strata       Column in x with stratification factor
#' @param n            Number of random samples
#' @param reps         Number of replicates per strata
#' @param replace      (TRUE/FALSE) Sampling with replacement 
#'
#' @details 
#' If replace=FALSE features are removed from consideration in subsequent replicates.
#' Conversely, if replace=TRUE, a feature can be selected multiple times across 
#' replicates. Not applicable if rep=1.
#'
#' @return An sf class object containing random samples
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'  
#' @references
#' Hudak, A.T., N.L. Crookston, J.S. Evans, M.J. Falkowski, A.M.S. Smith, P. Gessler 
#'   and P. Morgan. (2006) Regression modelling and mapping of coniferous forest basal 
#'   area and tree density from discrete-return lidar and multispectral satellite data. 
#'   Canadian Journal of Remote Sensing 32: 126-138.
#'
#' @examples 
#' if(require(sp, quietly = TRUE)) {
#' library(sf)
#'   data(meuse, package = "sp")
#'   meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, 
#'                     agr = "constant")
#' 
#' # Create stratified variable using quartile breaks
#' x1 <- cut(meuse$cadmium, summary(meuse$cadmium)[-4], 
#'           include.lowest=TRUE)
#'   levels(x1) <- seq(1,nlevels(x1),1)
#' x2 <- cut(meuse$lead, summary(meuse$lead)[-4], 
#'           include.lowest=TRUE)
#'   levels(x2) <- seq(1,nlevels(x2),1) 
#' meuse$STRAT <- paste(x1, x2, sep='.')
#'
#' # Counts for each full strata (note; 2 strata have only 1 observation)
#' tapply(meuse$STRAT, meuse$STRAT, length)
#'    
#' # 2 replicates, 2 samples with replacement
#' ssample <- stratified.random(meuse, strata='STRAT', n=2, reps=2, 
#'                              replace=TRUE)
#'   tapply(ssample$STRAT, ssample$STRAT, length)
#' 
#' # 2 replicates, 2 samples no replacement
#' ssample.nr <- stratified.random(meuse, strata='STRAT', n=2, reps=2)
#'   tapply(ssample.nr$STRAT, ssample.nr$STRAT, length)
#' 
#' # n=1 and reps=10 for sequential numbering of samples 
#' ssample.ct <- stratified.random(meuse, strata='STRAT', n=1, reps=10, 
#'                                 replace=TRUE)
#'   tapply(ssample.ct$STRAT, ssample.ct$STRAT, length)
#' 
#' # Plot random samples colored by replacement
#' ssample$REP <- factor(ssample$REP)
#'   plot(ssample['REP'], pch=20)
#'
#' } else { 
#'   cat("Please install sp package to run example", "\n")
#' }
#' 
#' @export stratified.random
stratified.random <- function(x, strata, n = 10, reps = 1, replace = FALSE) {
  gtypes = c("POLYGON", "POINT", "LINESTRING", "MULTIPOLYGON", 
             "MULTIPOINT", "MULTILINESTRING")			 
  if(missing(x)) 
    stop("Must provide an input spatial object (x)")
  if(!inherits(x, c("sf", "sfc")))
    stop(deparse(substitute(x)), " must be an sf object or coercible")	  
  if(any(unique(as.character(sf::st_geometry_type(x))) == gtypes[4:6]))
    stop("Function does not support multi-part geometry")  
  if(!any(unique(as.character(sf::st_geometry_type(x))) != gtypes[1:3]))
    stop(deparse(substitute(x)), " must be one of ", 
	     paste(gtypes[1:3], collopse=""))		     
  spx <- sf::st_drop_geometry(x)
    if(!inherits(spx[,strata], "factor")) 
      spx[,strata] <- factor(spx[,strata])  
    if (nlevels(spx[,strata]) < 2) 
      stop("Not enough levels to stratify by (n<2)")
    spx$REP <- NA
    results <- list()	
      for(j in levels(spx[, strata])) {
        d <- spx[spx[,strata] == j,]
    	  d$rowname <- rownames(d)
          if(nrow(d) > n) {  	
            for (i in 1:reps) {	
              s <- lapply(1, function(ij) {
      	             d[sample(1:nrow(d), n),]})
      	        s[[1]]$REP <- i
             results[[paste(j,i,sep="_")]] <-s[[1]] 
            }
          } else {
    	    d$REP <- 1
    		results[[paste(j,i,sep="_")]] <- d
    	  }
      }
    results <- do.call(rbind, results)
	  if(!replace){
	    didx <- which(duplicated(results$rowname))
	    if(length(didx) > 0) results <- results[-didx,]
	  }
	  results <- stats::na.omit(results[,c("rowname","REP")])	
    results <- merge(x, results, by.y="rowname", by.x = 'row.names', 
	                 all.x = FALSE, all.y = TRUE)
  return(results)
} 

#' @title Kendall tau trend with continuity correction for time-series 
#' @description 
#' Calculates a nonparametric statistic for a monotonic trend based 
#' on the Kendall tau statistic and the Theil-Sen slope modification
#'
#' @param y            A vector representing a timeseries with >= 8 obs
#' @param tau          (FALSE/TRUE) return tau values
#' @param intercept    (FALSE/TRUE) return intercept values 
#' @param p.value      (FALSE/TRUE) return p.values
#' @param confidence   (FALSE/TRUE) return 95 pct confidence levels
#' @param method       Method for deriving tau and slope ("zhang", "yuepilon", "none")
#' @param threshold    The threshold for number of minimum observations in the time-series
#' @param ...          Not used
#'
#' @details 
#' This function implements Kendall's nonparametric test for a monotonic trend 
#' using the Theil-Sen (Theil 1950; Sen 1968; Siegel 1982) method to estimate 
#' the slope and related confidence intervals. Critical values are Z > 1.96 
#' representing a significant increasing trend and a Z < -1.96 a significant 
#' decreasing trend (p < 0.05). The null hypothesis can be rejected if Tau = 0. 
#' Autocorrelation in the time-series is addressed using a prewhitened linear trend 
#' following the Zhang et al., (2000) or  Yue & Pilon (2002) methods. If you do not
#' have autocorrelation in the data, the "none" or "yuepilon" method is recommended. 
#' Please note that changing the threshold to fewer than 6 observations (ideally 8) may 
#' prevent the function from failing but, will likely invalidate the statistic. 
#' A threshold of <=4 will yield all NA values. If method= "none" a modification of the
#' EnvStats::kendallTrendTest code is implemented.       
#'
#' @return Depending on arguments, a vector containing: 
#' * Theil-Sen slope, always returned 
#' * Kendall's tau two-sided test, if tau TRUE
#' * intercept for trend if intercept TRUE
#' * p value for trend fit if p.value TRUE
#' * lower confidence level at 95-pct if confidence TRUE
#' * upper confidence level at 95-pct if confidence TRUE
#' @md
#' 
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @references 
#' Theil, H. (1950) A rank invariant method for linear and polynomial regression 
#'   analysis. Nederl. Akad. Wetensch. Proc. Ser. A 53:386-392 (Part I),  
#'   53:521-525 (Part II), 53:1397-1412 (Part III).
#' @references 
#' Sen, P.K. (1968) Estimates of Regression Coefficient Based on Kendall's tau. 
#'   Journal of the American Statistical Association. 63(324):1379-1389.
#' @references 
#' Siegel, A.F. (1982) Robust Regression Using Repeated Medians. 
#'   Biometrika, 69(1):242-244
#' @references 
#' Yue, S., P. Pilon, B. Phinney and G. Cavadias, (2002) The influence of autocorrelation 
#'   on the ability to detect trend in hydrological series. 
#'   Hydrological Processes, 16: 1807-1829.
#' @references 
#' Zhang, X., Vincent, L.A., Hogg, W.D. and Niitsoo, A., (2000) Temperature 
#'   and Precipitation Trends in Canada during the 20th Century. 
#'   Atmosphere-Ocean 38(3): 395-429. 
#'
#' @examples
#' data(EuStockMarkets)
#' d <- as.vector(EuStockMarkets[,1])
#' kendall(d)
#' 
#' @seealso \code{\link[zyp]{zyp.trend.vector}} for model details
#'
#' @export kendall
kendall <- function(y, tau = TRUE, intercept = TRUE, p.value = TRUE, 
                    confidence = TRUE, method=c("zhang", "yuepilon", "none"),
					threshold = 6, ...) {
	if(any(method %in% c("zhang", "yuepilon"))) {
	  if(length(find.package("zyp", quiet = TRUE)) == 0)
        stop("please install zyp package before running this function")
	}    
    if(threshold < 6)
      warning("Setting the time-series threshold to fewer than 6 obs may invalidate 
	    the statistic and n <= 4 will always result in NA's") 
    if(length(y[!is.na(y)]) < threshold) 
      warning("The Kendall Tau should have at least 6 observations")	  

    out.names <- c("slope", "tau", "intercept", "p-value", "limits.LCL", "limits.UCL")[
	               which(c(TRUE, tau, intercept, p.value, rep(confidence,2)))]  
    idx <- 2
      if(tau == TRUE) { idx = append(idx, 5) }
        if(intercept == TRUE) { idx = append(idx, 11) }  
          if(p.value == TRUE) { idx = append(idx, 6) }
	        if(confidence == TRUE) { idx = append(idx, c(1,4)) }
  mk.trend <- function (y, x = seq(along = y), alternative = "two.sided", 
                        conf.level = 0.95, ...)  {
    out.names <- c("slope", "tau", "intercept", "p-value", "limits.LCL", "limits.UCL")	
    n <- length(y)
    alternative <- match.arg(alternative, c("two.sided", "greater", "less"))						
      vark <- function(x, y) {
          ties.x <- rle(sort(x))$lengths
          ties.y <- rle(sort(y))$lengths
          n <- length(x)
          t1 <- n * (n - 1) * (2 * n + 5)
          t2 <- sum(ties.x * (ties.x - 1) * (2 * ties.x + 5))
          t3 <- sum(ties.y * (ties.y - 1) * (2 * ties.y + 5))
          v1 <- (t1 - t2 - t3)/18
          if (n > 2) {
            t1 <- sum(ties.x * (ties.x - 1) * (ties.x - 2))
            t2 <- sum(ties.y * (ties.y - 1) * (ties.y - 2))
            v2 <- (t1 * t2)/(9 * n * (n - 1) * (n - 2))
          } else  {
          v2 <- 0
            t1 <- sum(ties.x * (ties.x - 1)) * sum(ties.y * (ties.y - 1))
            v3 <- t1/(2 * n * (n - 1))
            v1 + v2 + v3
        }
        return(v1)
      }						
   if(length(stats::na.omit(y)) < threshold) { 
     v <- c(NA,NA,NA,NA, NA, NA) 
	   names(v) <- out.names
    } else {
    index <- 2:n
    S <- sum(sapply(index, function(i, x, y) {
        sum(sign((x[i] - x[1:(i - 1)]) * (y[i] - y[1:(i - 1)])))}, x, y))
    tau <- (2 * S)/(n * (n - 1))
    slopes <- unlist(lapply(index, function(i, x, y) (y[i] -
                     y[1:(i - 1)])/(x[i] - x[1:(i - 1)]), x, y))
    slopes <- sort(slopes[is.finite(slopes)])
    slope <- stats::median(slopes)
    intercept <- stats::median(y) - slope * stats::median(x)
    estimate <- c(tau, slope, intercept)
    names(estimate) <- c("tau", "slope", "intercept")
    var.S <- vark(x, y)       
	stat <- S/sqrt(var.S)
        names(stat) <- "z"
      p.value <- switch(alternative, greater = 1 - stats::pnorm(stat),
          less = stats::pnorm(stat), two.sided = 2 * stats::pnorm(-abs(stat)))		
    N.prime <- length(slopes)
    type <- switch(alternative, two.sided = "two-sided",
        greater = "lower", less = "upper")
    alpha <- 1 - conf.level
    Z <- ifelse(type == "two-sided", stats::qnorm(1 - alpha/2),
                 stats::qnorm(conf.level))
    C.alpha <- Z * sqrt(var.S)
    M1 <- (N.prime - C.alpha)/2
    M2 <- (N.prime + C.alpha)/2
    limits <- switch(type, `two-sided` = stats::approx(1:N.prime,
        slopes, xout = c(M1, M2 + 1))$y, lower = c(stats::approx(1:N.prime,
        slopes, xout = M1)$y, Inf), upper = c(-Inf, stats::approx(1:N.prime,
        slopes, xout = M2 + 1)$y))
          names(limits) <- c("LCL", "UCL")
    v <-c(estimate[c(2,1,3)], p.value, limits) 
      names(v) <- out.names	  
    }
    return(v)
  }
  if(length(y[!is.na(y)]) < threshold) { 
    fit.results <- rep(NA,length(out.names))
  } else if(method[1] == "none") { 
    fit.results <- mk.trend(y)
  }  else {
    fit.results <- zyp::zyp.trend.vector(y, method=method[1])[idx]	
  }	
    names(fit.results) <- out.names  
  return(fit.results)
}	

#' @title Solar-radiation Aspect Index
#' @description 
#' Calculates the Roberts and Cooper (1989) Solar-radiation Aspect Index
#' 
#' @param x    A terra SpatRaster object
#' @param ...  Additional arguments passed to terra::app
#' 
#' @details
#' Roberts and Cooper (1989) rotates (transforms) the circular aspect to assign a 
#' value of zero to land oriented in a north-northeast direction, (typically the 
#' coolest and wettest orientation), and a value of one on the hotter, dryer 
#' south-southwesterly slopes. The result is a continuous variable between 0 - 1. 
#' The metric is defined as: trasp = ( 1 - cos((pi/180)(a-30) ) / 2 
#' where; a = aspect in degrees
#'
#' @return 
#' A terra SpatRaster object of Roberts and Cooper (1989) Solar-radiation Aspect Index
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#' 
#' @references 
#' Roberts. D.W., and Cooper, S.V. (1989). Concepts and techniques of vegetation mapping. 
#'   In Land Classifications Based on Vegetation: Applications for Resource Management.
#'   USDA Forest Service GTR INT-257, Ogden, UT, pp 90-96
#'
#' @examples 
#' library(terra)
#' elev <- rast(system.file("extdata/elev.tif", package="spatialEco"))
#'   s <- trasp(elev)
#'     plot(s)
#'     
#' @export trasp
trasp <- function(x, ...) {  
  if (!inherits(x, "SpatRaster")) 
	stop(deparse(substitute(x)), " must be a terra SpatRaster object")
    asp <- terra::terrain(x, v='aspect', unit='degrees') 
  return( terra::app(asp, fun=function(x) { (1 - cos( (3.142/180)  *(x - 30)) ) / 2 }, ... ) )
}  

#' @title Raster Percent Volume
#' @description Calculates a percent volume on a raster or based on a 
#'              systematic sample
#' 
#' @param x        A terra SpatRaster class object
#' @param p        percent raster-value volume
#' @param sample   (FALSE/TRUE) base volume on systematic point sample 
#' @param spct     sample percent, if sample (TRUE)
#' @param type     If sample=TRUE type of sample, options are "random" or "regular"
#'
#' @note 
#' Since this model needs to operate on all of the raster values, it is not memory safe 
#'
#' @return 
#' if sample (FALSE) binary raster object with 1 representing designated 
#' percent volume else, if sample (TRUE) n sf POINT object with points 
#' that represent the percent volume of the sub-sample
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @examples 
#' \donttest{
#' library(terra)
#'   r <- rast(ncols=100, nrows=100)
#'     r[] <- runif(ncell(r), 0, 1)
#'     r <- focal(r, w=focalMat(r, 6, "Gauss"))
#'   #r[sample(1:ncell(r)),10] <- NA
#'   
#'   # full raster percent volume 
#'   p30 <- raster.vol(r, p=0.30)
#'   p50 <- raster.vol(r, p=0.50)
#'   p80 <- raster.vol(r, p=0.80)
#'
#' opar <- par(no.readonly=TRUE)
#'     par(mfrow=c(2,2))
#'     plot(r, col=cm.colors(10), main="original raster")
#'     plot(p30, breaks=c(0,0.1,1), col=c("cyan","red"), legend=FALSE,
#'	      main="30% volume")
#'     plot(p50, breaks=c(0,0.1,1), col=c("cyan","red"), legend=FALSE,
#'	      main="50% volume")
#'     plot(p80, breaks=c(0,0.1,1), col=c("cyan","red"), legend=FALSE,
#'	      main="80% volume")
#' par(opar)  
#' }
#'
#' @export raster.vol    	
raster.vol <- function(x, p = 0.75, sample = FALSE, spct = 0.05,
                       type=c("random","regular")) {
  if (!inherits(x, "SpatRaster")) 
    stop(deparse(substitute(x)), " must be a terra SpatRaster object")					   
  if( sample == FALSE ) { 
	den <- x[][,1]
      z <- sort(den[!is.na(den)], decreasing=TRUE)
        y <- cumsum(as.numeric(z))
          i <- sum(y <= p * y[length(y)]) 
	        vol <- x
			  vol[] <- as.integer(den >= z[i])
    return( vol ) 
  } else {
    ss = round(terra::ncell(x) * spct, digits = 0)
    den <- sf::st_as_sf(terra::spatSample(x, size=ss, method=type[1],  
                        na.rm=TRUE, as.points=TRUE, values=TRUE))
      names(den)[1] <- "den" 						
      den$idx <- 1:nrow(den) 
	  den$cls = 0  
    #den <- data.frame(idx = 1:nrow(den), den = den[,1], cls = 0)
	  sum.p <- sum(den$den, na.rm = TRUE) * p
        den <- den[order(-den$den),] 
    
	i=0; j=0
      while(i <= sum.p) { 
        j=j+1
        if(!is.na(den$den[j])) {
          i = i + den$den[j]
          den$cls[j] <- 1
        } else {
          den$cls[j] <- NA
      }	
    }
    den <- den[order(den$idx),]
      return( den[den$cls == 1 ,] )	
  }
}

#' @title Kullback-Leibler divergence (relative entropy)
#'
#' @description Calculates the Kullback-Leibler divergence (relative entropy)
#' 
#' @param object  Matrix or dataframe object with >=2 columns
#' @param eps  Probabilities below this threshold are replaced by this 
#'             threshold for numerical stability.
#' @param overlap  Logical, do not determine the KL divergence for those 
#'                 pairs where for each point at least one of the densities 
#'                 has a value smaller than eps.
#'
#' @details
#' Calculates the Kullback-Leibler divergence (relative entropy) between 
#' unweighted theoretical component distributions. Divergence is calculated 
#' as: int [f(x) (log f(x) - log g(x)) dx] for distributions with densities 
#' f() and g(). 
#' @return pairwise Kullback-Leibler divergence index (matrix)   
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @references 
#' Kullback S., and R. A. Leibler (1951) On information and sufficiency.  
#'   The Annals of Mathematical Statistics 22(1):79-86
#'        
#' @examples 
#' x <- seq(-3, 3, length=200)
#' y <- cbind(n=dnorm(x), t=dt(x, df=10))
#'   matplot(x, y, type='l')
#'     kl.divergence(y)
#'    
#' # extract value for last column
#'   kl.divergence(y[,1:2])[3:3]
#'
#' @export  
kl.divergence <- function(object, eps = 10^-4, overlap = TRUE) {
    if (!is.numeric(object)) 
        stop("object must be a numeric matrix\n")
    z <- matrix(NA, nrow = ncol(object), ncol = ncol(object))
    colnames(z) <- rownames(z) <- colnames(object)
    w <- object < eps
    if (any(w)) 
        object[w] <- eps
    object <- sweep(object, 2, colSums(object), "/")
    for (k in seq_len(ncol(object) - 1)) {
      for (l in 2:ncol(object)) {
        ok <- (object[, k] > eps) & (object[, l] > eps)
          if (!overlap | any(ok)) {
              z[k, l] <- sum(object[, k] * (log(object[, k]) - log(object[, l])))
              z[l, k] <- sum(object[, l] * (log(object[, l]) - log(object[, k])))
          }
      }
    }
    diag(z) <- 0
    return(z)
} 

#' @title Sample transect
#' @description Creates random transects from points and generates 
#'              sample points along each transect
#' 
#' @param x           A sf point object
#' @param min.dist    Minimum length of transect(s)
#' @param max.dist    Maximum length of transect(s)
#' @param distance    A vector of distances, same length as x, used
#'                    to define transect distances (length)
#' @param azimuth     A vector of azimuths, same length as x, used
#'                    to define transect direction
#' @param id          A unique identification column in x
#' @param ...         Additional arguments passed to st_sample 
#'
#' @details
#' Function create lines and samples using random or defined direction 
#' and length transects and then creates a point sample along each transect. 
#' The characteristic of the sample points are defined by arguments passed 
#' to the sf::st_sample function. The distance and azimuth arguments allow
#' for specifying the exact length and direction for each points transect.   
#'
#' @return 
#' A list object contaning sf LINES and POINTS objects representing random transects 
#' and sample points along each transect. The "ID" column links the resulting data. 
#' 
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com> 
#'
#' @examples 
#' if(require(sp, quietly = TRUE)) {
#' library(sf)
#'   data(meuse, package = "sp")
#'   meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, 
#'                     agr = "constant")
#' meuse <- meuse[sample(1:nrow(meuse),10),]
#' 
#' transects <- sampleTransect(meuse, min.dist=200, max.dist=500, 
#'                             type="regular", size=20)
#'    plot(st_geometry(transects$transects))
#'      plot(st_geometry(meuse), pch=19, cex=2, add=TRUE)
#'        plot(st_geometry(transects$samples), 
#' 	        col="red", pch=19, add=TRUE)
#' 
#' } else { 
#'   cat("Please install sp package to run example", "\n")
#' }
#'
#' @export sampleTransect
sampleTransect <- function(x, min.dist, max.dist, distance = NULL,
                           azimuth = NULL, id = NULL, ...) {
  if (!inherits(x, "sf")) 
    stop(deparse(substitute(x)), " must be an sf POLYGON object")
  if(!is.null(distance)) {
    if(length(distance) != nrow(x))
	  stop("length of distance does not match x")
  }  
  if(!is.null(azimuth)) {
    if(length(azimuth) != nrow(x))
	  stop("length of azimuth does not match x")
  }  	
  if(!is.null(id)) {
    if(!names(x) %in% id) 
	  stop(id, " not present in ", deparse(substitute(x)))
    ids <- sf::st_drop_geometry(x[,id])[,1]	   
  }	else {
    ids <- 1:nrow(x)
  }
  dots <- as.list(match.call(expand.dots = TRUE)[-1])
    if (is.null(dots[["size"]]) & "size" %in% names(dots) == FALSE) 
    dots[["size"]] <- 10 
  if (is.null(dots[["type"]]) & "type" %in% names(dots) == FALSE) 
    dots[["type"]] <-  "random" 
  message(paste("Creating", dots$size, dots$type, "samples for each transect"))
  tlines <- list()   
  tpoints <- list()
    for(i in 1:nrow(x) ) {
	  p <- x[i,]
      if(!is.null(azimuth)) {      
	    az = azimuth[i]   
	  } else {
        az = stats::runif(1, 0, 360)
      }	 
      if(!is.null(azimuth)) {      
	    d = distance[i]  
	  } else {
        d = stats::runif(1, min.dist, max.dist)
      }	 	  
      samp.pt <- spatialEco::bearing.distance(
	                 sf::st_coordinates(p)[,1], 
	                 sf::st_coordinates(p)[,2], 
	                 distance = d, azimuth = az)
        l <- sf::st_as_sf(sf::st_sfc(sf::st_linestring(
               rbind(sf::st_coordinates(p)[,1:2], samp.pt))),
			     crs=sf::st_crs(x))
		  sf::st_geometry(l) <- "geometry"
		    l$ID <- ids[i]
	  dots[["x"]] <- l		
	  pts <- suppressWarnings(sf::st_as_sf(sf::st_cast(
	           do.call(sf::st_sample, dots),"POINT")))
		  sf::st_geometry(pts) <- "geometry"
		    pts$ID <- ids[i]			   
	  tpoints[[i]] <- pts 
	  tlines[[i]] <- l
    }	
  return( list( transects = do.call("rbind", tlines), 
          samples = do.call("rbind", tpoints) ) )	
}    

#' @title Insert Values
#' @description Inserts new values into a vector at 
#'              specified positions
#'
#' @param x      A vector to insert values 
#' @param value  Values to insert into x
#' @param index  Index position(s) to insert y values into x
#'
#' @details
#' This function inserts new values at specified positions in a vector. It 
#' does not replace existing values. If a single value is provided for y 
#' and l represents multiple positions y will be replicated for the length 
#' of l. In this way you can insert the same value at multiple locations.  
#'
#' @return 
#' A vector with values of y inserted into x and the position(s) 
#' defined by the index
#'  
#' @author Jeffrey S. Evans <sage_insights@@outlook.com>
#'
#' @examples
#' (x=1:10)
#'
#'  # Insert single value in one location
#'  insert.values(x, 100, 2) 
#'
#'  # Insert multiple values in multiple locations 
#'  insert.values(x, c(100,200), c(2,8)) 
#'
#'  # Insert single value in multiple locations 
#'  insert.values(x, NA, c(2,8))
#'
#' @export
insert.values <- function(x, value, index) {
  if(length(value) == 1) value = rep(value, length(index) )
  if(length(value) < length(index) ) 
    stop("length of replacement does not match index")
  z <- numeric(length(x) + length(index))
  z[index] <- value
  z[-index] <- x
  return(z)  
}

#' @title Statistical transformation for rasters
#' @description Transforms raster to a specified statistical transformation 
#' 
#' @param x       A terra SpatRaster class object
#' @param trans   Transformation method: "norm", "rstd", "std", "stretch", 
#'                 "nl", "slog", "sr" (please see notes)
#' @param smin    Minimum value for stretch 
#' @param smax    Maximum value for stretch
#' 
#' @details
#' Transformation option details:
#' * norm - (Normalization_ (0-1): if min(x) < 0 ( x - min(x) ) / ( max(x) - min(x) )
#' * rstd - (Row standardize) (0-1): if min(x) >= 0 x / max(x) This normalizes data 
#' *        with negative distributions
#' * std - (Standardize) (x - mean(x)) / sdv(x)
#' * stretch - (Stretch) ((x - min(x)) * max.stretch / (max(x) - min(x)) + min.stretch) 
#'              This will stretch values to the specified minimum and maximum values 
#'              (eg., 0-255 for 8-bit)
#' * nl - (Natural logarithms) if min(x) > 0 log(x)
#' * slog - (Signed log 10) (for skewed data): if min(x) >= 0 ifelse(abs(x) <= 1, 0, 
#'           sign(x)*log10(abs(x))) 
#' * sr - (Square-root) if min(x) >= 0 sqrt(x) 
#' @md
#'
#' @return A terra SpatRaster class object of specified transformation
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#' 
#' @examples 
#' \donttest{
#' library(terra)
#' r <- rast(nrows=500, ncols=500, xmin=571823, xmax=616763, 
#'             ymin=4423540, ymax=4453690)
#'   crs(r) <- "epsg:9001"
#' r[] <- runif(ncell(r), 1000, 2500)
#'
#'  # Positive values so, can apply any transformation    
#'  for( i in c("norm", "rstd", "std", "stretch", "nl", "slog", "sr")) {
#'    print( raster.transformation(r, trans = i) ) 
#'  }
#'
#'  # Negative values so, can't transform using "nl", "slog" or "sr"
#'  r[] <- runif(ncell(r), -1, 1)
#'    for( i in c("norm", "rstd", "std", "stretch", "nl", "slog", "sr")) {
#'	  try( print( raster.transformation(r, trans = i) ) ) 
#'    }
#' }
#'
#' @export raster.transformation
raster.transformation <- function(x, trans = "norm", smin=0, smax=255) {
  slog <- function(x) { ifelse(abs(x) <= 1, 0, sign(x)*log10(abs(x)))}
   if (!inherits(x, "SpatRaster")) 
    stop(deparse(substitute(x)), " must be a terra SpatRaster object")    
	  rmin <- terra::global(x, "min", na.rm = TRUE)[,1]
	  rmax <- terra::global(x, "max", na.rm = TRUE)[,1]
  	  rmean <- terra::global(x, "mean", na.rm = TRUE)[,1]
  	  rsd <- terra::global(x, "sd", na.rm = TRUE)[,1]	
  if( trans == "slog" && rmin < 0) {
    stop(" Minimum value < 0, cannot log transform")
  }
  if( trans == "nl" && rmin < 0) {
    stop(" Minimum value < 0, cannot log transform")
  }
  if( trans == "sr" && rmin < 0) {
    stop(" Minimum value < 0, cannot log transform")
  }
  if( trans == "norm" && rmin < 0) {
    message(" Min value < 0, running row standardization instead")
    return( x / rmax )
  }
  if( trans == "norm") {
    message("applying normalization transformation", "\n")
    return( ( x - rmin ) / ( rmax - rmin ) )
   } else if ( trans == "rstd") {
     message("applying row-standardization transformation", "\n")
     return( x / rmax )
      } else if ( trans == "std") {
	    message("applying standardization transformation", "\n")
	    return( (x - rmean) / rsd )
	     } else if ( trans == "stretch") {
		   message("applying stretch transformation", "\n")
		   return( (x - rmin) * smax / (rmax - rmin) + smin )
		    } else if ( trans == "nl") {
			  message("applying log transformation", "\n")
			  return(  terra::app(x, fun=log) )
			  } else if ( trans == "slog") {
			    message("applying signed-log10 transformation", "\n")
			    return(terra::app(x, fun=slog) )
			    } else if ( trans == "sr") {
				  message("applying square-root transformation", "\n")
			      return( terra::app(x, fun=sqrt) )		  
		          } else {
                    stop("Not a valid transformation") 
    }		
}

#' @title Logistic and Auto-logistic regression
#' @description Performs a logistic (binomial) or auto-logistic 
#'              (spatially lagged binomial) regression using maximum 
#'              likelihood or penalized maximum likelihood estimation. 
#'
#' @param ldata data.frame object containing variables
#' @param y Dependent variable (y) in ldata
#' @param x Independent variable(s) (x) in ldata  
#' @param penalty Apply regression penalty (TRUE/FALSE)
#' @param autologistic Add auto-logistic term (TRUE/FALSE)  
#' @param coords Geographic coordinates for auto-logistic model matrix 
#'               or sp object.
#' @param bw Distance bandwidth to calculate spatial lags (if empty neighbors 
#'           result, need to increase bandwidth). If not provided it will be 
#'           calculated automatically based on the minimum distance that includes 
#'           at least one neighbor.
#' @param type Neighbor weighting scheme (see autocov_dist)
#' @param style Type of neighbor matrix (Wij), default is mean of neighbors 
#' @param longlat Are coordinates (coords) in geographic, lat/long (TRUE/FALSE)
#' @param ... Additional arguments passed to lrm
#'
#' @details
#' It should be noted that the auto-logistic model (Besag 1972) is intended for 
#' exploratory analysis of spatial effects. Auto-logistic are know to underestimate 
#' the effect of environmental variables and tend to be unreliable (Dormann 2007).     
#' Wij matrix options under style argument - B is the basic binary coding, W is row 
#' standardized (sums over all links to n), C is globally standardized (sums over
#' all links to n), U is equal to C divided by the number of neighbours (sums over 
#' all links to unity) and S is variance-stabilizing. Spatially lagged y defined as:  
#' W(y)ij=sumj_(Wij yj)/ sumj_(Wij) where; Wij=1/Euclidean(i,j)
#' If the object passed to the function is an sp class there is no need to call the data 
#' slot directly via "object@data", just pass the object name.
#'
#' @return 
#' A list class object with the following components: 
#' * model - lrm model object (rms class)
#' * bandwidth - If AutoCov = TRUE returns the distance bandwidth used for the 
#'   auto-covariance function
#' * diagTable - data.frame of regression diagnostics
#' * coefTable - data.frame of regression coefficients (log-odds)
#' * Residuals - data.frame of residuals and standardized residuals
#' * AutoCov - If an auto-logistic model, AutoCov represents lagged 
#'             auto-covariance term
#' @md
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @references
#' Besag, J.E., (1972) Nearest-neighbour systems and the auto-logistic model for binary 
#'   data. Journal of the Royal Statistical Society, Series B Methodological 34:75-83
#' 
#' Dormann, C.F., (2007) Assessing the validity of autologistic regression. Ecological 
#'   Modelling 207:234-242   
#' 
#' Le Cessie, S., Van Houwelingen, J.C., (1992) Ridge estimators in logistic regression. 
#'   Applied Statistics 41:191-201
#' 
#' Shao, J., (1993) Linear model selection by cross-validation. JASA 88:486-494
#'
#' @examples 
#'  p = c("sf", "sp", "spdep", "rms")
#'  if(any(!unlist(lapply(p, requireNamespace, quietly=TRUE)))) { 
#'    m = which(!unlist(lapply(p, requireNamespace, quietly=TRUE)))
#'    message("Can't run examples, please install ", paste(p[m], collapse = " "))
#'  } else {
#'    invisible(lapply(p, require, character.only=TRUE))
#'  
#'  data(meuse, package = "sp")
#'  meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, 
#'                    agr = "constant")
#'    meuse$DepVar <- rbinom(nrow(meuse), 1, 0.5)
#'  
#'  #### Logistic model
#'  lmodel <- logistic.regression(meuse, y='DepVar', 
#'                    x=c('dist','cadmium','copper')) 
#'    lmodel$model
#'      lmodel$diagTable
#'        lmodel$coefTable
#'  
#'  #### Logistic model with factorial variable
#'  lmodel <- logistic.regression(meuse, y='DepVar', 
#'              x=c('dist','cadmium','copper', 'soil')) 
#'    lmodel$model
#'      lmodel$diagTable
#'        lmodel$coefTable
#'  
#'   ### Auto-logistic model using 'autocov_dist' in 'spdep' package
#'   lmodel <- logistic.regression(meuse, y='DepVar', 
#'               x=c('dist','cadmium','copper'), autologistic=TRUE, 
#'               coords=st_coordinates(meuse), bw=5000) 
#'     lmodel$model
#'       lmodel$diagTable
#'         lmodel$coefTable
#'     est <- predict(lmodel$model, type='fitted.ind')
#'   
#'   #### Add residuals, standardized residuals and estimated probabilities
#'   VarNames <- rownames(lmodel$model$var)[-1]
#'     meuse$AutoCov <- lmodel$AutoCov 
#'     meuse$Residual <- lmodel$Residuals[,1]
#'     meuse$StdResid <- lmodel$Residuals[,2]
#'     meuse$Probs <- predict(lmodel$model, 
#'                           sf::st_drop_geometry(meuse[,VarNames]),
#'  	                     type='fitted')  
#'   
#'  #### Plot fit and probabilities
#'  
#'  resid(lmodel$model, "partial", pl="loess") 
#'  
#'  # plot residuals
#'  resid(lmodel$model, "partial", pl=TRUE) 
#'   
#'  # global test of goodness of fit 
#'  resid(lmodel$model, "gof")
#'   
#'  # Approx. leave-out linear predictors
#'  lp1 <- resid(lmodel$model, "lp1")            
#'   
#'  # Approx leave-out-1 deviance            
#'  -2 * sum(meuse$DepVar * lp1 + log(1-plogis(lp1)))
#'   
#'  # plot estimated probabilities at points
#'  plot(meuse['Probs'], pch=20)
#'  
#' }
#' @seealso \code{\link[rms]{lrm}}
#' @seealso \code{\link[spdep]{autocov_dist}}
#'
#' @export
logistic.regression <- function(ldata, y, x, penalty = TRUE, autologistic = FALSE, 
                                coords = NULL, bw = NULL, type = "inverse", style = "W",  
                                longlat = FALSE, ...) {
	if(length(find.package("rms", quiet = TRUE)) == 0)
      stop("please install rms package before running this function")							
	if(inherits(ldata, "sf")) 
	  ldata <- sf::st_drop_geometry(ldata)
    if (is.na(match(y, names(ldata)))) 
      stop("Dependent variable not present in data")
	xNames <- intersect(x, names(ldata))
    if (length(xNames) < length(x)) 
        stop("Mismatch in Independent Variable Names")
    if (autologistic == TRUE) {
      if (is.null(coords)) { 
        stop("Need coordinates")
	  } else {
	    coords <- as.matrix(coords)	
	  }
      if (is.null(bw)) {       
	    k.nn <- spdep::knn2nb(spdep::knearneigh(coords))
        bw <- max(unlist(spdep::nbdists(k.nn, coords)))	
	  }	
      ldata$AutoCov <- spdep::autocov_dist(ldata[,y], xy = coords, nbs = bw, 
	                                       style = style, type = type)
      x <- append(x, "AutoCov")
    }
	form <- stats::as.formula(paste(y, paste(x, collapse = "+"), sep = "~"))
      fit <- rms::lrm(form, data = ldata, x = TRUE, y = TRUE, ...)
        bf <- rms::pentrace(fit, seq(0.2, 1, by = 0.05))
    if (penalty) {
      pen <- bf$penalty
    } else {
      pen <- 0
    }
    allPens <- bf$results.all[, 1]
    allAICs <- bf$results.all[, 3]
    for (i in 1:length(allPens)) {
     penValue <- allPens[i]
       if (penValue == pen) {
         aic <- allAICs[i]
       }
    }
    if (penalty) {
      fit <- stats::update(fit, penalty = bf$penalty)
    }
    res <- stats::residuals(fit)
    resSTD <- (res - mean(res))/sqrt(stats::var(res))
    allIndVars <- c("Intercept")
    allIndVars <- append(allIndVars, x)
    k <- length(fit$coefficients)
	  d <- matrix(0, k, 4)
	    d[, 1] <- fit$coefficients
          d[, 2] <- sqrt(diag(fit$var))
            d[, 3] <- d[, 1]/d[, 2]
              d[, 4] <- stats::pnorm(abs(d[, 3]), lower.tail = FALSE) * 2
    coefList <- list(Variable = allIndVars, Coef = d[, 1], StdError = d[, 2], 
	                 Wald = d[, 3], Prob = d[, 4])
	coefList$Variable <- names(fit$coefficients)
    coefFrame <- data.frame(coefList)
    diagFrame <- data.frame(Names = c(names(fit$stats), "PEN", "AIC"), 
	                        Value = c(as.vector(fit$stats), pen, aic))
    if (autologistic == TRUE) {
      return(list(model = fit, bandwidth=bw, diagTable = diagFrame, coefTable = coefFrame,  
             Residuals = data.frame(res = res, resSTD = resSTD), AutoCov = ldata$AutoCov))
    } else {
      return(list(model = fit, diagTable = diagFrame, coefTable = coefFrame, 
             Residuals = data.frame(res = res, resSTD = resSTD)))
    }
} 

#' @title Niche overlap (Warren's-I)
#' @description Similarity Statistic for Quantifying Niche Overlap using Warren's-I
#'
#' @param x    A matrix or SpatRaster raster class object
#' @param y    A matrix or SpatRaster raster class object
#'             with the same dimensions of x
#'
#' @details 
#' The overlap function computes the I similarity statistic (Warren et al. 2008)  
#' of two overlapping niche estimates. Similarity is based on the Hellenger distance. 
#' It is assumed that the input data share the same extent and cellsize and all values 
#' are positive.
#' 
#' The I similarity statistic sums the pair-wise differences between two
#' predictions to create a single value representing the similarity of the two
#' distributions. The I similarity statistic ranges from a value of 0, where
#' two distributions have no overlap, to 1 where two distributions are
#' identical (Warren et al., 2008). The function is based on code  
#' from Jeremy VanDerWal
#'
#' @return A vector (single value) representing the I similarity statistic
#'
#' @author Jeffrey Evans <sage_insights@@outlook.com> and Jeremy VanDerWal  
#'
#' @references 
#' Warren, D. L., R. E. Glor, M. Turelli, and D. Funk. (2008).
#'   Environmental Niche Equivalency versus Conservatism: Quantitative 
#'   Approaches to Niche Evolution. Evolution 62:2868-2883.
#'
#' @examples   
#' # add degree of separation in two matrices 
#' p1 <- abs(matrix(1:50,nr=50,nc=50) + 
#'          runif(n = 2500, min = -1, max = 1))
#' p2 <- abs(matrix(1:50,nr=50,nc=50) + 
#'          rnorm(n = 2500, mean = 1, sd = 1))
#'  
#' # High overlap/similarity 
#' ( I <- overlap(p1,p2) ) 
#' 
#' @export overlap 
overlap <- function(x, y){
  if(!inherits(x, c("SpatRaster", "matrix")))
    stop(deparse(substitute(x)), " ust be a SpatRaster or matrix object")
  if(!inherits(y, c("SpatRaster", "matrix")))
   stop(deparse(substitute(y)), " ust be a SpatRaster or matrix object")
  if (any(class(x) %in% "SpatRaster")) x <- terra::as.matrix(x)
  if (any(class(y) %in% "SpatRaster")) y <- terra::as.matrix(y)     
  if(length(which(dim(x) == dim(y))) != 2) 
    stop('matrix / raster objects must be of the same extent')
  if (min(c(x, y), na.rm = TRUE) < 0) 
    stop('all values must be positive') 	
  pos <- which(is.finite(x) & is.finite(y))
    H <- sqrt(sum((sqrt((x[pos] / sum(x[pos])))-sqrt((y[pos]/sum(y[pos]))))^2)) 
  # calculate original and corrected Warren-I statistic 	
  return ( list( original.I = (1 - 0.5 * H), 
           corrected.I = (1 - (H^2)/2) ) )  
}

#' @title Point estimate of Heat Load Index
#' @description Calculates the McCune & Keon (2002) Heat Load Index
#' 
#' @param alpha             Aspect in degrees
#' @param theta             Slope in degrees
#' @param latitude          A latitude representing the centrality of the data
#' @param direct            Boolean (FALSE/TRUE) Return direct incident radiation 
#'                          else HLI (default) 
#' @param scaled            Boolean (TRUE/FALSE) Apply arithmetic scale using EXP(h)          
#' @param force.hemisphere  If country is split at the equator, force southern 
#'                          or northern hemisphere equation c("southern", "northern")  
#' 
#' @details
#' Describes A southwest facing slope should have warmer temperatures than a 
#' southeast facing slope, even though the amount of solar radiation they receive 
#' is equivalent. The McCune and Keon (2002) method accounts for this by "folding" 
#' the aspect so that the highest values are southwest and the lowest values are  
#' northeast. Additionally, this method account for steepness of slope, which is 
#' not addressed in most other aspect rescaling equations. HLI values range 
#' from 0 (coolest) to 1 (hottest). 
#'
#' The equations follow McCune (2007) and support northern and southern hemisphere 
#' calculations. The folded aspect for northern hemispheres use (180 - (Aspect – 225) ) 
#' and for Southern hemisphere  ( 180 - ( Aspect – 315) ). If a country is split at the 
#' equator you can use the force.hemisphere argument to choose which equation to use. 
#' Valid values for this argument are "southern" and "northern" with the default "none".
#'
#' @return Vector of McCune & Keon (2002) Heat Load Index
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @references 
#' McCune, B., and D. Keon (2002) Equations for potential annual direct 
#'   incident radiation and heat load index. Journal of Vegetation 
#'   Science. 13:603-606.
#' 
#' McCune, B. (2007). Improved estimates of incident radiation and heat load 
#'   using non-parametric regression against topographic variables. Journal 
#'   of Vegetation Science 18:751-754. 
#' 
#' @examples 
#'
#' # Single point input
#' hli.pt(theta=180, alpha=30, latitude=40) 
#' 
#' # Multiple input, returns results from 
#' #   McCune, B., and D. Keon (2002)
#' # Raw -0.2551 -0.6280 0.0538 -0.6760 -1.1401 -0.2215
#' # arithmetic scale 0.7748 0.5337 1.0553 0.5086 0.3198 0.8013
#'
#' slp = c(0, 30, 30, 0, 30, 30)
#' asp =c(0, 0, 180, 0, 0, 180)
#' lat =c(40, 40, 40, 60, 60, 60)
#' hli.pt(theta = slp, alpha = asp, latitude = lat)
#'   
#' @export hli.pt
hli.pt <- function(alpha, theta, latitude, direct = FALSE, scaled = TRUE,    
  force.hemisphere = c("none", "southern", "northern")) {
  if (!inherits(alpha, "numeric")) 
    stop("x must be numeric")
  if (!inherits(alpha, "numeric")) 
    stop("x must be numeric") 	
  if (!inherits(latitude, "numeric")) 
    stop("x must be numeric") 	
  if (any(latitude < -180) | any(latitude > 180))
    stop("latitude is out of range")
  if(any(latitude < 0)) hemisphere = "southern" else hemisphere = "northern" 
  latitude = abs(latitude) * pi/180
    theta <- theta * pi/180 
      alpha <- alpha * pi/180   
    cl = cos(latitude)
  sl = sin(latitude) 
    if(hemisphere == "northern" | force.hemisphere[1] == "northern"){ 
	  message("Using folded aspect equation for Northern hemisphere") 	
	    # Folded Aspect Northern Hemisphere  (180 - (Aspect – 225) )
		#   180(deg)=3.141593(rad), 225=3.92699(rad)
		if(!direct) {
          folded.alpha <- abs(pi - abs(alpha - pi * 5/4)) 
        } else {
		  folded.alpha <- pi - abs(alpha - pi)
		}
	} else if(hemisphere == "southern" | force.hemisphere[1] == "southern") {
	  message("Using folded aspect equation for Southern hemisphere") 		
		# Folded Aspect Southern Hemisphere  ( 180 - ( Aspect – 315) )  
		#   180(deg)=3.141593(rad), 315=5.49779 
		folded.alpha <- abs(3.141593 - abs(alpha - 5.497791))
	}	
	  if(!direct) {
	    h <- -1.467 + 1.582 * cos(latitude) * cos(theta) -
                 1.5 * cos(folded.alpha) * sin(theta) * sin(latitude) -
                 0.262 * sin(latitude) * sin(theta) + 0.607 *
                 sin(folded.alpha) * sin(theta)
      } else {
        h <- -1.467 + 1.582 * cos(latitude) * cos(theta) - 1.5 * 
		     cos(folded.alpha) * sin(theta) * sin(latitude) - 0.262 * 
			 sin(latitude) * sin(theta) + 0.607 * sin(folded.alpha) * 
			 sin(theta)
		}				 	 
      if(scaled) h <- exp(h)			 
  return( ifelse(h > 1, 1, h) )
 } 

#' @title Inverse Distance Weighted smoothing
#' @description 
#' Distance weighted smoothing of a variable in a 
#'          spatial point object
#'
#' @param x   An sf POINT class object
#' @param y   Numeric data column in x to be smoothed
#' @param d   Distance constraint       
#' @param k   Maximum number of k-nearest neighbors within d    
#' 
#' @details
#' Smoothing is conducted with a weighted-mean where; weights represent inverse 
#' standardized distance lags Distance-based or neighbour-based smoothing can be 
#' specified by setting the desired neighbour smoothing method to a specified value 
#' then the other parameter to the potential maximum. For example; a constraint 
#' distance, including all neighbors within 1000 (d=1000) would require k to equal 
#' all of the potential neighbors (n-1 or k=nrow(x)-1).  
#'    
#' @return A vector, same length as nrow(x), of smoothed y values 
#'                                                                 
#' @examples 
#' 
#' library(sf)
#' if(require(sp, quietly = TRUE)) {
#'   data(meuse, package = "sp")
#'   meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, 
#'                     agr = "constant")
#'       
#'  # Calculate distance weighted mean on cadmium variable in meuse data   
#'    cadmium.idw <- idw.smoothing(meuse, 'cadmium', k=nrow(meuse), d = 1000)                
#'    meuse$cadmium.wm <- cadmium.idw
#'  
#'    opar <- par(no.readonly=TRUE)
#'      par(mfrow=c(2,1)) 
#'        plot(density(meuse$cadmium), main='Cadmium')
#'        plot(density(meuse$cadmium.wm), main='IDW Cadmium')
#'    par(opar)
#' 
#' plot(meuse[c("cadmium","cadmium.wm")], pch=20)   
#'
#' } else { 
#'   cat("Please install sp package to run example", "\n")
#' }
#'
#' @export
idw.smoothing <- function(x, y, d, k) {
    if(length(find.package("RANN", quiet = TRUE)) == 0)
      stop("please install RANN package before running this function")
    if (!inherits(x, "sf")) 
      stop(deparse(substitute(x)), " Must be an sf object")
    if (is.null(y)) 
      stop("y (value) column must be specified")
    if (!y %in% names(x)) 
      stop(deparse(substitute(y)), "does not exists")
    if (!inherits(sf::st_drop_geometry(x[, y])[,1], "numeric")) 
        stop("y must be numeric")
	ddata <- data.frame(ID = row.names(x), sf::st_coordinates(x)[,1:2])
      nearest <- RANN::nn2(ddata, query = ddata, k = k, treetype = c("bd"), 
	                       searchtype = c("radius"), radius = d)
        knn.id <- as.data.frame(nearest$nn.idx)
          names(knn.id) <- paste("NN", seq(1, ncol(knn.id), 1), sep = "")
        knn.id <- data.frame(ID = row.names(x), knn.id)
        knn.dist <- as.data.frame(nearest$nn.dists)
          names(knn.dist) <- paste("NND", seq(1, ncol(knn.dist), 1), sep = "")
      knn.dist <- data.frame(ID = row.names(x), knn.dist)
    wts <- rev(seq(1, d, 1))/d
	v <- as.numeric()
    for (i in 1:dim(knn.id)[1]) {
      knn.sub <- as.numeric(knn.id[i, ][2:k])
      lsub <- x[knn.sub, ]
      if (nrow(lsub) < 2) { wt <- 1 } else {
        dsub <- as.numeric(knn.dist[i, ][2:k])
          dsub[which(knn.sub == 0)] <- 0
            dsub <- dsub[dsub > 0]
          wt <- round(append(dsub, 1, after = 0), digits = 0)
        wt <- wts[wt]
      }
	  ysub <- sf::st_drop_geometry(lsub[,y])[,1]
        wm <- stats::weighted.mean(ysub, wt, na.rm = TRUE)
        wm[is.nan(wm)] <- NA
      v <- append(v, wm, after = length(v))
    }
    return(v)
} 

#' @name elev
#' @docType data
#'
#' @title Elevation raster
#'
#' @description elevation raster of Switzerland
#'
#' @format A raster RasterLayer class object:
#' \describe{
#'   \item{resoultion}{5 arc-minute 0.00833 (10000m)}
#'   \item{nrow}{264}
#'   \item{ncol}{564}
#'   \item{ncell}{148896}
#'   \item{xmin}{5.9}
#'   \item{xmax}{10.6}
#'   \item{ymin}{45.7}
#'   \item{ymax}{47.9}
#'   \item{proj4string}{+proj=longlat +ellps=WGS84}
#' }
#'
#' @source \url{https://diva-gis.org/data.html}
NULL

#' @title Loess with confidence intervals
#' @description Calculates a local polynomial regression fit 
#' with associated confidence intervals
#' 
#' @param y Dependent variable, vector 
#' @param x Independent variable, vector 
#' @param plot Plot the fit and confidence intervals
#' @param p Percent confidence intervals (default is 0.95)
#' @param ... Arguments passed to loess 
#' 
#' @return A list object with:
#' * loess Predicted values   
#' * se Estimated standard error for each predicted value
#' * lci Lower confidence interval 
#' * uci Upper confidence interval
#' * df Estimated degrees of freedom 
#' * rs Residual scale of residuals used in computing the 
#'            standard errors 
#' @md
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @references
#' W. S. Cleveland, E. Grosse and W. M. Shyu (1992) Local regression models. 
#'   Chapter 8 of Statistical Models in S eds J.M. Chambers and T.J. Hastie, 
#'   Wadsworth & Brooks/Cole. 
#' 
#' @examples 
#'  x <- seq(-20, 20, 0.1)
#'  y <- sin(x)/x + rnorm(length(x), sd=0.03)
#'  p <- which(y == "NaN")
#'    y <- y[-p]	
#'    x <- x[-p]
#'  
#' opar <- par(no.readonly=TRUE)
#'   par(mfrow=c(2,2))  
#'     lci <- loess.ci(y, x, plot=TRUE, span=0.10)
#'     lci <- loess.ci(y, x, plot=TRUE, span=0.30)
#'     lci <- loess.ci(y, x, plot=TRUE, span=0.50)
#'     lci <- loess.ci(y, x, plot=TRUE, span=0.80)
#' par(opar)
#'
#' @export
loess.ci <- function(y, x, p = 0.95, plot = FALSE, ...) {
  plx <- stats::predict(stats::loess(y ~ x, ...), se=TRUE)
  lci = plx$fit - stats::qt(p, plx$df) * plx$se.fit
  uci = plx$fit + stats::qt(p, plx$df) * plx$se.fit
    if(plot == TRUE) {
    oops <- options() 
      on.exit(options(oops)) 
      graphics::plot(x, y, type="n", main="Loess fit", 
	    sub=paste("confidence intervals at", p))
      graphics::polygon(c(x,rev(x)), c(lci, rev(uci)), col="grey86")
      graphics::points(x, y, pch=20, cex=0.70)
      graphics::lines(x, plx[["fit"]], lty=3)
    }
  return( list(loess=plx$fit, se=plx$se.fit, lci=lci, uci=uci,  
               df=plx$df, rs=plx$residual.scale ) )  
}    
 

#' @title Kendall tau trend with continuity correction for raster time-series 
#' @description Calculates a nonparametric statistic for a monotonic trend 
#'              based on the Kendall tau statistic and the Theil-Sen slope 
#'              modification
#'
#' @param x             A multiband terra SpatRaster object with at least 5 layers
#' @param tau           (FALSE/TRUE) return a raster with the pixel wise 
#'                       tau correlation values
#' @param intercept     (FALSE/TRUE) return a raster with the pixel 
#'                       wise intercept values 
#' @param p.value       (FALSE/TRUE) return a raster with the pixel 
#'                       wise p.values
#' @param confidence    (FALSE/TRUE) return a raster with the pixel 
#'                       wise 95 pct confidence levels
#' @param min.obs       The threshold of minimum number of observations (default 6)
#' @param method        Kendall method to use c("zhang", "yuepilon","none"), see kendall function
#' @param ...           Additional arguments passed to the terra app function 
#'
#' @details 
#' This function implements Kendall's nonparametric test for a monotonic trend 
#' using the Theil-Sen (Theil 1950; Sen 1968; Siegel 1982) method to estimate 
#' the slope and related confidence intervals.  
#'
#' @return Depending on arguments, a raster layer or rasterBrick object containing:
#'   * raster layer 1 - slope for trend, always returned
#'   * raster layer 2 - Kendall's tau two-sided test, reject null at 0, if tau TRUE
#'   * raster layer 3 - intercept for trend if intercept TRUE
#'   * raster layer 4 - p value for trend fit if p.value TRUE
#'   * raster layer 5 - lower confidence level at 95 pct, if confidence TRUE
#'   * raster layer 6 - upper confidence level at 95 pct, if confidence TRUE
#' @md
#' 
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @references 
#' Theil, H. (1950) A rank invariant method for linear and polynomial regression 
#'   analysis.  Nederl. Akad. Wetensch. Proc. Ser. A 53:386-392 (Part I), 
#'   53:521-525 (Part II), 53:1397-1412 (Part III).
#'  
#' Sen, P.K. (1968) Estimates of Regression Coefficient Based on Kendall's tau. 
#'   Journal of the American Statistical Association. 63(324):1379-1389.
#'  
#' Siegel, A.F. (1982) Robust Regression Using Repeated Medians. 
#'   Biometrika, 69(1):242-244
#'
#' @examples
#' \donttest{
#'  library(terra)
#'
#'  # note; nonsense example with n=9
#'  r <- c(rast(system.file("ex/logo.tif", package="terra")),
#'         rast(system.file("ex/logo.tif", package="terra")),
#'         rast(system.file("ex/logo.tif", package="terra"))) 
#'  
#'  # Calculate trend slope with p-value and confidence level(s)
#'  # ("slope","intercept", "p.value","z.value", "LCI","UCI","tau")
#'    k <- raster.kendall(r, method="none")
#'    plot(k)
#' }
#'
#' @seealso \code{\link[zyp]{zyp.trend.vector}} for model details
#' @seealso \code{\link[terra]{app}} for available ... arguments
#'
#' @export
raster.kendall <- function(x, intercept = TRUE, p.value = TRUE, 
                           confidence = TRUE, tau = TRUE, min.obs = 6,  
						   method=c("zhang", "yuepilon","none"), ...) {
  if(length(find.package("zyp", quiet = TRUE)) == 0)
    stop("please install zyp package before running this function")
  if (!inherits(x, "SpatRaster")) 
    stop(deparse(substitute(x)), " must be a terra SpatRaster object")
  if(terra::nlyr(x) > 150)
    warning("The Kendall Tau can become numerically unstable with large numbers of observations")   
  if(min.obs < 6)
    warning("Setting the time-series threshold (n) to fewer than 6 obs may invalidate 
	  the statistic and <= 4 will result in NA's") 
  if( terra::nlyr(x) < min.obs) 
    stop("Too few layers to calculate a trend")
  idx <- which(c(TRUE, tau, intercept, p.value, rep(confidence,2)))	
  out.names <- c("slope", "tau", "intercept", "p-value", "limits.LCL", "limits.UCL")[idx]
    trend.slope <- function(y, metrics=idx, method=method[1]) {
      kendall(y)[metrics]
    }
  k <- terra::app(x, fun=trend.slope, ...)
    names(k) <- out.names
  return( k )
}

#' @title Pseudo-absence random samples 
#' @description Generates pseudo-absence samples based on density estimate 
#'              of known locations 
#'
#' @param x         An sf POINT geometry object
#' @param n         Number of random samples to generate
#' @param window    Type of window (hull OR extent), overridden if mask provided
#' @param ref       Optional terra SpatRaster class raster. The resolution of the 
#'                  density estimate will match mask.        
#' @param s         Optional resolution passed to window argument. Caution should be 
#'                  used due to long processing times associated with high 
#'                  resolution. In contrast, coarse resolution can exclude 
#'                  known points.            
#' @param sigma     Bandwidth selection method for KDE, default is 'Scott'. 
#'                  Options are 'Scott', 'Stoyan', 'Diggle', 'likelihood', 
#'                  and 'geometry'
#' @param wts       Optional vector of weights corresponding to point pattern
#' @param KDE       Return KDE raster (TRUE/FALSE)
#' @param gradient  A scaling factor applied to the sigma parameter used to 
#'                  adjust the gradient decent of the density estimate. The 
#'                  default is 1, for no adjustment (downweight < 1 | upweight > 1)   
#' @param p         Minimum value for probability distribution (must be >  0)
#' @param edge      Apply Diggle edge correction (TRUE/FALSE)
#'
#' @details
#' The window type creates a convex hull by default or, optionally, uses the 
#' maximum extent (envelope). If a mask is provided the kde will represent 
#' areas defined by the mask and defines the area that pseudo absence data 
#' will be generated.
#' 
#' Available bandwidth selection methods are:
#'   * Scott (Scott 1992), Scott's Rule for Bandwidth Selection (1st order)
#'   * Diggle (Berman & Diggle 1989), Minimize the mean-square error via cross 
#'   * validation (2nd order)  
#'   * likelihood (Loader 1999), Maximum likelihood cross validation (2nd order)
#'   * geometry, Bandwidth is based on simple window geometry (1st order)
#'   * Stoyan (Stoyan & Stoyan 1995), Based on pair-correlation function (strong 2nd order)
#'   * User defined numeric distance bandwidth
#' @md
#'
#' @note
#' resulting bandwidth can vary widely by method. the 'diggle' method 
#' is intended for selecting bandwidth representing 2nd order spatial variation 
#' whereas the 'scott' method will represent 1st order trend. the 'geometry' approach 
#' will also represent 1st order trend. For large datasets, caution should be used with 
#' the 2nd order 'likelihood' approach, as it is slow and computationally expensive. 
#' finally, the 'stoyan' method will produce very strong 2nd order results.
#' 
#' @return 
#' A list class object with the following components:
#'   * sample  A sf POINT geometry object containing random samples
#'   * kde     A terra SpatRaster class of inverted Isotropic KDE estimates 
#'             used as sample weights (IF KDE = TRUE)
#'   * sigma   Selected bandwidth of KDE 
#' @md
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#' 
#' @references
#' Berman, M. and Diggle, P. (1989) Estimating weighted integrals of the second-order 
#'   intensity of a spatial point process. Journal of the Royal Statistical Society, 
#'   series B 51, 81-92. 
#' 
#' Fithian, W & T. Hastie (2013) Finite-sample equivalence in statistical models for 
#'   presence-only data. Annals of Applied Statistics 7(4): 1917-1939
#' 
#' Hengl, T., H. Sierdsema, A. Radovic, and A. Dilo (2009) Spatial prediction of species 
#'   distributions from occurrence-only records: combining point pattern analysis, 
#'   ENFA and regression-kriging. Ecological Modelling, 220(24):3499-3511  
#' 
#' Loader, C. (1999) Local Regression and Likelihood. Springer, New York. 
#' 
#' Scott, D.W. (1992) Multivariate Density Estimation. Theory, Practice and Visualization. 
#'   New York, Wiley. 
#' 
#' Stoyan, D. and Stoyan, H. (1995) Fractals, random shapes and point fields: methods of 
#'   geometrical statistics. John Wiley and Sons. 
#' 
#' Warton, D.i., and L.C. Shepherd (2010) Poisson Point Process Models Solve the Pseudo-Absence 
#'   Problem for Presence-only Data in Ecology. The Annals of Applied Statistics, 4(3):1383-1402
#'
#' @examples
#'  p = c("sf", "sp", "terra", "spatstat.data")
#'  if(any(!unlist(lapply(p, requireNamespace, quietly=TRUE)))) { 
#'    m = which(!unlist(lapply(p, requireNamespace, quietly=TRUE)))
#'    message("Can't run examples, please install ", paste(p[m], collapse = " "))
#'  } else {
#'    invisible(lapply(p, require, character.only=TRUE))
#'   
#' data(meuse, package = "sp")
#' meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, 
#'                   agr = "constant") 
#' 
#' # Using a raster mask   
#' r <- rast(ext(meuse), resolution=40, crs=crs(meuse))
#'   r[] <- rep(1,ncell(r))
#'   
#' pa <- pseudo.absence(meuse, n=100, window='hull', KDE=TRUE, ref = r, 
#'                      sigma='Diggle', s=50) 
#'   col.br <- colorRampPalette(c('blue','yellow'))
#'     plot(pa$kde, col=col.br(10))
#'       plot(st_geometry(meuse), pch=20, cex=1, add=TRUE)
#'         plot(st_geometry(pa$sample), col='red', pch=20, cex=1, add=TRUE)
#'           legend('top', legend=c('Presence', 'Pseudo-absence'), 
#'                  pch=c(20,20),col=c('black','red'), bg="white")
#' 
#' # With clustered data
#' data(bei, package = "spatstat.data")
#'   trees <- st_as_sf(bei)
#'     trees <- trees[-1,]
#' 
#' trees.abs <- pseudo.absence(trees, n=100, window='extent', KDE=TRUE)
#'   col.br <- colorRampPalette(c('blue','yellow'))
#'     plot(trees.abs$kde, col=col.br(10))
#'      plot(st_geometry(trees), pch=20, cex=0.50, add=TRUE)
#'         plot(st_geometry(trees.abs$sample), col='red', pch=20, cex=1, add=TRUE)
#'           legend('top', legend=c('Presence', 'Pseudo-absence'), 
#'                  pch=c(20,20),col=c('black','red'),bg="white")
#' }     
#' @export     
pseudo.absence <- function(x, n, window = "hull", ref = NULL, s = NULL, sigma = "Scott", 
                           wts = NULL, KDE = FALSE, gradient = 1, p = NULL, edge = FALSE) {

  if(!inherits(x, c("sf", "sfc")))	
    stop(deparse(substitute(x)), " must be a sf, or sfc object")
  if(unique(as.character(sf::st_geometry_type(x))) != "POINT")
      stop(deparse(substitute(x)), " must be single-part POINT geometry") 
  if (!is.null(ref)) {
    if(!inherits(ref, "SpatRaster")) 
      stop(deparse(substitute(ref)), " must be a terra SpatRaster object")
  }
  if (is.null(p)) p <- 1e-09
    a <- 10000    
    raster.as.im <- function(im) {
	  r <- terra::res(im)[1]
      orig <- as.numeric(sf::st_bbox(im)) + 0.5 * r
      dm <- dim(im)[2:1]
        xx <- unname(orig[1] + cumsum(c(0, rep(r[1], dm[1] - 1))))
        yy <- unname(orig[2] + cumsum(c(0, rep(r[1], dm[2] - 1))))
      return(spatstat.geom::im(matrix(terra::values(im), ncol = dm[1], 
             nrow = dm[2], byrow = TRUE)[dm[2]:1, ], 
		     xcol = xx, yrow = yy))
    }
	if (is.null(ref)) {
      if (window == "hull") {
        win <- spatstat.geom::convexhull.xy(sp::coordinates(x))
        win <- spatstat.geom::as.mask(win, eps = s)
        }
      if (window == "extent") {
        e <- as.vector(sf::st_bbox(x))
        win <- spatstat.geom::as.owin(c(e[1], e[3], e[2], e[4]))
        win <- spatstat.geom::as.mask(win, eps = s)
        }
    } else {
        win <- raster.as.im(ref)
        win <- spatstat.geom::as.mask(win, eps = terra::res(ref)[1])
    }	
    x.ppp <- suppressWarnings(
      spatstat.geom::as.ppp(sf::st_coordinates(x)[,1:2], win))
	  
    bw.Scott <- function(X) {
        stopifnot(spatstat.geom::is.ppp(X))
        n <- spatstat.geom::npoints(X)
        sdx <- sqrt(stats::var(X$x))
        sdy <- sqrt(stats::var(X$y))
        return(c(sdx, sdy) * n^(-1/6))
    }
    bw.Stoyan <- function(X, co = 0.15) {
        stopifnot(spatstat.geom::is.ppp(X))
        n <- spatstat.geom::npoints(X)
        W <- spatstat.geom::as.owin(X)
        a <- spatstat.geom::area.owin(W)
        stoyan <- co/sqrt(5 * n/a)
        return(stoyan)
    }
    bw.geometry <- function(X, f = 1/4) {
        X <- spatstat.geom::as.owin(X)
        g <- spatstat.explore::distcdf(X)
        r <- with(g, .x)
        Fr <- with(g, .y)
        iopt <- min(which(Fr >= f))
        return(r[iopt])
    }
    bw.likelihood <- function(X, srange = NULL, ns = 16) {
	  check.range <- function (x, fatal = TRUE) {
        xname <- deparse(substitute(x))
        if (is.numeric(x) && identical(x, range(x, na.rm = TRUE))) 
          return(TRUE)
        if (fatal) 
          stop(paste(xname, "should be a vector of length 2 giving (min, max)"))
        return(FALSE)
      }
        stopifnot(spatstat.geom::is.ppp(X))
        if (!is.null(srange)) 
            check.range(srange) else {
            nnd <- spatstat.geom::nndist(X)
            srange <- c(min(nnd[nnd > 0]), spatstat.geom::diameter(spatstat.geom::as.owin(X))/2)
        }
        sigma <- exp(seq(log(srange[1]), log(srange[2]), length = ns))
        cv <- numeric(ns)
        for (i in 1:ns) {
            si <- sigma[i]
            lamx <- spatstat.explore::density.ppp(X, sigma = si, at = "points", leaveoneout = TRUE)
            lam <- spatstat.explore::density.ppp(X, sigma = si)
            cv[i] <- sum(log(lamx)) - spatstat.geom::integral.im(lam)
        }
      result <- spatstat.explore::bw.optim(cv, sigma, iopt = which.max(cv), 
	                    criterion = "Likelihood Cross-Validation")
    return(result)
    }
    if (sigma == "Diggle") {
        bw <- spatstat.explore::bw.diggle(x.ppp)
      } else if(sigma == "Scott") { 
          bw <- bw.Scott(x.ppp)
        } else if(sigma == "Stoyan") {
            bw <- bw.Stoyan(x.ppp)
          } else if(sigma == "geometry") {
              bw <- bw.geometry(x.ppp)
            } else if(sigma == "likelihood") {
                bw <- bw.likelihood(x.ppp)
              } else if(is.numeric(sigma)) {
                  bw = sigma
                } else {
	              stop("Not a valid bandwidth option")
                } 	
    den <- terra::rast(spatstat.explore::density.ppp(x.ppp, weights = wts,  
                       sigma = bw, adjust = gradient, diggle = edge)) * a
        den <- 1 - (den/terra::global(den,"max",na.rm=TRUE)[,1])
          den[den <= p] <- p
    den.sp <- sf::st_as_sf(terra::as.points(den))
      names(den.sp)[1] <- "KDE"
        den.sp <- den.sp[sample(1:nrow(den.sp), size = n, prob = den.sp$KDE), ]
    if (KDE == TRUE) {
      return(list(sample = den.sp, kde = den, sigma = bw))
    } else {
      return(list(sample = den.sp, sigma = bw) )
    }
}

#' @title Rotate polygon
#' @description rotates polygon by specified angle
#'
#' @param p       A polygon object of sf or sp class
#' @param angle   Rotation angle in degrees
#' @param sp      (FALSE | TRUE) Output sp class object
#' @param anchor  Location to rotate polygon on options are "center", 
#'               "lower.left" and "upper.right" 
#'
#' @details 
#' The anchor is the location that the rotation is anchored to. The center
#' is the centroid where the lower.left and upper.right are based on the 
#' min or max of the coordinates respectively. 
#'
#' @return an sp or sf polygon object with rotated polygon
#' 
#' @examples
#' library(sf)
#'  
#' data(meuse, package = "sp")
#'   meuse <- st_as_sf(meuse, coords = c("x", "y"), 
#'                     crs = 28992, agr = "constant")
#'  
#'  e <- st_convex_hull(st_union(meuse))
#'    e30 <- rotate.polygon(e, angle=30)
#'  
#'  plot(e, main="rotated 30 degrees")
#'    plot(e30, add=TRUE)
#'  
#' @export rotate.polygon
rotate.polygon <- function(p, angle = 45, sp = FALSE,  
    anchor = c("center", "lower.left", "upper.right")) {
  if(class(p)[1] != "sf") p <- sf::st_as_sf(p)
    p.coords <- sf::st_coordinates(p)[,1:2]
  if(anchor[1] == "center") {
    p.center <- suppressWarnings(sf::st_coordinates(sf::st_centroid(p)))
  } else if(anchor[1] == "lower.left") {
    p.center <- c(min(sf::st_coordinates(p)[,1]),
                  min(sf::st_coordinates(p)[,2]))
  } else if(anchor[1] == "upper.right") { 
    p.center <- c(max(sf::st_coordinates(p)[,1]),
                  max(sf::st_coordinates(p)[,2])) 
  }  
  rotate.coords <- function(xy, a, center) {
    co <- cos(-a * pi / 180)
      si <- sin(-a * pi / 180)
        adj <- matrix(rep(center, nrow(xy)), ncol=2, byrow=TRUE)
      xy <- xy-adj
    cbind(co * xy[,1] - si * xy[,2],si * xy[,1] + co * xy[,2]) + adj
  }
      p.rotate <- rotate.coords(p.coords, a = angle, center = c(p.center[1], 
	                            p.center[2]))
	sf::st_geometry(p) <- sf::st_sfc(sf::st_polygon(list(p.rotate)))
    if(sp) p <- sf::as_Spatial(p)
  return( p )	
}

#' @title Buffer geographic data 
#' @description Buffers data in geographic (Latitude/Longitude) projection
#' 
#' @param x    A sf or sp vector class object
#' @param r    Buffer radius in meters
#' @param ...  Additional arguments passed to sf::st_buffer
#'
#' @details 
#' Projects (Latitude/Longitude) data in decimal-degree geographic projection 
#' using an on-the-fly azimuthal equidistant projection in meters centered on
#  each feature.  
#' 
#' @return an sp or sf polygon class object representing buffer for each feature
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @examples
#' library(sf)
#' e <- c(61.87125, 23.90153, 76.64458, 37.27042)
#'   names(e) <- c("xmin", "ymin", "xmax", "ymax")
#'   s <- st_as_sf(st_sample(st_as_sfc(st_bbox(e)), size=100, 
#'                  type = "regular"))
#' 	st_crs(s) <- st_crs(4326)
#'       s$id <- 1:nrow(s)
#' 
#' b <- geo.buffer(x=s, r=1000)
#'   plot(st_geometry(b[1,]))
#'      plot(st_geometry(s[1,]), pch=20,cex=2, add=TRUE)	 
#' 
#' @seealso \code{\link[sf]{st_buffer}} for st_buffer ... arguments
#' 
#' @export geo.buffer 
geo.buffer <- function(x, r, ...) {
  prj <- sf::st_crs("+proj=aeqd  +R=6371000 +lat_0=51 +lon_0=7")
  if(missing(x) | missing(r))
    stop("must supply x and r arguments")
  if(!inherits(x, "sf"))		
    stop(deparse(substitute(x)), " must be an sf POINT object")	
  if(unique(as.character(sf::st_geometry_type(x))) != "POINT")
    stop(deparse(substitute(x)), " must be an sf POINT object")			
  if(is.na(sf::st_crs(x)))
    stop(deparse(substitute(x)), " must have a defined projection")	  
  if(!sf::st_is_longlat(x))
    stop("Data appears to be projected and not Latitude/Longitude")
  b <- sf::st_buffer(sf::st_transform(x, prj), dist=r, ...) 
    b <- sf::st_transform(b, sf::st_crs(4326))
  return( b )	
}

#' @title Raster Entropy
#' @description Calculates entropy on integer raster (i.e., 8 bit 0-255)  
#'                                                                       
#' @param x            A terra SpatRaster object (requires integer raster)  
#' @param d            Size of matrix (window)
#' @param categorical  Is the data categorical or continuous (FALSE/TRUE)
#' @param global       Should the model use a global or local n to calculate 
#'                     entropy (FALSE/TRUE)
#' @param ...          Optional arguments passed terra focal function              
#'  
#' @details
#' Entropy calculated as: H = -sum(Pi*ln(Pi)) where; Pi, Proportion of one value 
#' to total values Pi=n(p)/m and m, Number of unique values. Expected range: 
#' 0 to log(m) H=0 if window contains the same value in all cells.
#' H increases with the number of different values in the window. The ellipsis
#' arguments can be used to write to disk using the filename argument. 
#'
#' @return terra SpatRaster class object            
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @references
#' Fuchs M., R. Hoffmann, F. Schwonke (2008) Change Detection with GRASS 
#'   GIS - Comparison of images taken by different sensor. 
#'
#' @examples 
#' library(terra)
#'   r <- rast(ncols=100, nrows=100)
#'     r[] <- round(runif(ncell(r), 1,8), digits=0)
#'
#' rEnt <- raster.entropy(r, d=5, categorical = TRUE, global = TRUE)
#'   opar <- par(no.readonly=TRUE)
#'     par(mfcol=c(2,1))
#'       plot(r)
#'         plot(rEnt)
#'   par(opar)
#'
#' # Maximum entropy is reached when all values are different, same as log(m)
#' #   for example; log( length( unique(x) ) ) 
#'
#' @export raster.entropy  
raster.entropy <- function(x, d = 5, categorical = FALSE, 
                           global = FALSE, ...) {
    if (!inherits(x, "SpatRaster")) 
	  stop(deparse(substitute(x)), " must be a terra SpatRaster object")	
	if(length(d) == 1) { 
	  d <- matrix(1, nrow=d, ncol=d, byrow=TRUE)
	} else if(length(d) == 2) { 
	  d <- matrix(1, nrow=d[1], ncol=d[2], byrow=TRUE)
	} else {
	  stop("Window matrix cannot have more than two dimensions")
	}
    if(global == TRUE) {
      k = sum(rep(terra::global(x, "max", na.rm=TRUE)[,1], nrow(d)*ncol(d))) 
    }		
	if(categorical == FALSE) {
      entropy <- function(x, n = NULL) {  
        x <- x[!is.na(x)]
        if (length(unique(x)) <= 1) { return(0) }
        return(-sum(prop.table(x) * log(prop.table(x))))
      }
    } else {
	  entropy <- function(x, n = NULL) {  
        x <- x[!is.na(x)]
        if (length(unique(x)) <= 1) { return(0) }		  
        if(!is.null(n)) { nv = n } else { nv <- sum(table(x)) }
        return(-sum((table(x) / nv) * (log(table(x) / nv))))
      }	
	}	
    if(global == TRUE) {
      e <- terra::focal(x, w = d, fun = function(x) { entropy(x, n = k) }, ...)
	} else {  
      e <- terra::focal(x, w = d, fun = function(x) { entropy(x) }, ...)
    }
  return( e )
} 

#' @title Autocorrelation Plot
#' @description Autocorrelation plot (Anselin 1996), following Chen (2015), 
#'              aka, Moran's-I plot (univariate or bivariate)
#' 
#' @param x               Vector of x response variables
#' @param y               Vector of y response variables
#' @param coords          A matrix of coordinates corresponding to [x,y]
#' @param type.ac         Type of autocorrelation plot ("xy", "yx") 
#' @param dist.function   ("inv.power", "neg.exponent") 
#' @param scale.xy        (TRUE/FALSE) scale the x,y vectors
#' @param scale.morans    (FALSE/TRUE) standardize the Moran's index to an 
#'                        expected [-1 to 1]? 
#' @param ...             Additional arguments passed to plot
#'
#' @details
#' The argument "type" controls the plot for x influencing y (type="xy") or y 
#' influencing x (type="yx"). If y is not defined then the statistic is univariate 
#' and only the "xy" plot will be available. The linear relationship between x and 
#' its spatial lag (Wx) is indicative of the spatial autoregressive process, underlying 
#' the spatial dependence. The statistic can be autocorrelation (univariate or 
#' cross-correlation (bivariate). The quadrants are the zero intercept for random 
#' autocorrelation and the red line represents the trend in autocorrelation. The quadrants 
#' in the plot indicate the type of spatial association/interaction (Anselin 1996). 
#' For example the upper-left quadrant represents negative associations of low values 
#' surrounded by high and the lower-right quadrant represents negative associations of
#' high values surrounded by low.  
#'
#' If y is not specified the univariate statistic for x is returned. the coords argument 
#' is only used if k = NULL. Can also be an sp object with relevant x,y coordinate slot 
#' (ie., points or polygons). If w = NULL, the default method for deriving spatial weights 
#' matrix, options are: inverse power or negative exponent. If scale.xy = FALSE it is 
#' assumed that they are already scaled following Chen (2015).         
#'
#' @return A plot of the scaled variable against its spatially lagged values. 
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @references 
#' Chen., Y. (2015) A New Methodology of Spatial Cross-Correlation Analysis. 
#'   PLoS One 10(5):e0126158. doi:10.1371/journal.pone.0126158
#'  
#' Anselin, L. (1996) The Moran scatterplot as an ESDA tool to assess local instability 
#'   in spatial association. pp. 111-125 in M. M. Fischer, H. J. Scholten and D. Unwin (eds) 
#'   Spatial analytical perspectives on GIS, London, Taylor and Francis
#' 
#' Anselin, L. (1995) Local indicators of spatial association, Geographical Analysis, 
#'   27:93-115
#'
#' @examples
#'  p = c("sf", "sp", "spdep")
#'  if(any(!unlist(lapply(p, requireNamespace, quietly=TRUE)))) { 
#'    m = which(!unlist(lapply(p, requireNamespace, quietly=TRUE)))
#'    message("Can't run examples, please install ", paste(p[m], collapse = " "))
#'  } else {
#'  invisible(lapply(p, require, character.only=TRUE))
#'  
#'  data(meuse, package = "sp")
#'  meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, agr = "constant") 
#' 
#' # Autocorrelation (univariate)  
#' morans.plot(meuse$zinc, coords = st_coordinates(meuse)[,1:2])   
#' 
#' # Cross-correlation of: x influencing y and y influencing x
#' opar <- par(no.readonly=TRUE)
#'   par(mfrow=c(1,2)) 
#'     morans.plot(x=meuse$zinc, y=meuse$copper, coords = st_coordinates(meuse)[,1:2], 
#'                 scale.morans = TRUE)
#'     morans.plot(x=meuse$zinc, y=meuse$copper, coords = st_coordinates(meuse)[,1:2],
#'                 scale.morans = TRUE, type.ac="yx") 
#' par(opar)
#' }                       
#' @export morans.plot
morans.plot <- function(x, y = NULL, coords = NULL, 
                        type.ac = c("xy", "yx"), 
                        dist.function = "inv.power",   
                        scale.xy = TRUE, scale.morans = FALSE, ...) { 
	if(missing(x)) stop("x must be specified")
      if(is.null(y)) {
	    type.lab = paste0("Autocorrelation of ", deparse(substitute(x)))
	    y = x	
        type.ac = "xy"
		message("Calculating autocorrelation for x")
	  } else {
	  	type.lab = paste0("Cross-correlation of ", deparse(substitute(x)), " and ", deparse(substitute(y)))
		message("Calculating cross-correlation for [x,y]")	 
      }	 
    if(length(y) != length(x)) stop("[X,Y] are not equal")
      if( length(which(is.na(x))) != 0 | length(which(is.na(y))) != 0) 
	    stop("NA's not permitted in [X,Y]")	
    dots <- as.list(match.call(expand.dots = TRUE)[-1])
	if ( !is.null(dots[["x"]]) ) dots[["x"]] <- NULL
		if ( !is.null(dots[["y"]]) ) dots[["y"]] <- NULL
			if ( !is.null(dots[["coords"]]) ) dots[["coords"]] <- NULL
				if ( !is.null(dots[["type.ac"]]) ) dots[["type.ac"]] <- NULL
					if ( !is.null(dots[["dist.function"]]) ) dots[["dist.function"]] <- NULL
						if ( !is.null(dots[["scale.xy"]]) ) dots[["scale.xy"]] <- NULL
							if ( !is.null(dots[["scale.morans"]]) ) dots[["scale.morans"]] <- NULL
	if(scale.xy == FALSE) warning("It is assumed that [x,y] vectors are already scaled")
      type.ac = type.ac[1]			  
    if( scale.xy ){
      x.scale <- as.numeric(scale(x))	
	  y.scale <- as.numeric(scale(y))
    }	    
    if( is.null(coords) ) stop("A coordinates matrix is required")
      w <- sp::spDists( coords ) 
    if( dist.function == "inv.power" ) {
      message("Calculating spatial weights matrix using inverse power function")
	    w <- 1 / w
          diag(w) <- 0 
        w <- w / sum(w) 
    } else if (dist.function == "neg.exponent") { 
      message("Calculating spatial weights matrix using negative exponent")    
	  diag(w) <- NA 
	  mu <- mean(w, na.rm=TRUE)
      for(i in 1:nrow(w)) {
        for(j in 1:nrow(w)) {
		 w[i,j] <- round(exp( (-2 * w[i,j]) / mu ),6)
        }
      }  
      diag(w) <- 0
    } else {
      stop("Not a valid matrix option")
    }
	
	# standardizes variable -1 to 1
    scale.partial <- function(p) {
      d <- (max(p) + min(p)) / 2
      return((p - d) / (max(p) - d))
    }
  
  plot.ac <- function(x.plot,y.plot,dots) {	  
	   if (is.null(dots[["pch"]]) & "pch" %in% names(dots) == FALSE) dots[["pch"]] <- 20
       if (is.null(dots[["cex"]]) & "cex" %in% names(dots) == FALSE) dots[["cex"]] <- 1
       dots[["x"]] <- x.plot
	   dots[["y"]] <- y.plot
  	do.call("plot", dots)
    }
	   
    if(type.ac[1] == "xy") {	
      if (is.null(dots[["xlab"]]) & "xlab" %in% names(dots) == FALSE) dots[["xlab"]] <- deparse(substitute(x))
	  if (is.null(dots[["ylab"]]) & "ylab" %in% names(dots) == FALSE) dots[["ylab"]] <- "nWy,xx^TWy"
	  if (is.null(dots[["main"]]) & "main" %in% names(dots) == FALSE) dots[["main"]] <- type.lab
	    # gsci.xy <- as.numeric(x*w%*%y) 
        gsci.xy <- as.numeric(x.scale*y.scale%*%w) 
		    if(scale.morans) { 
			  gsci.xy <- scale.partial(gsci.xy)
			  slag <- as.numeric(x.scale*y.scale%*%w)
			    slag <- scale.partial(slag)
            } else {
              slag <- as.numeric(x.scale*y.scale%*%w)
            }			
	      plot.ac(x.scale,gsci.xy,dots)
	        graphics::abline(v=0, lty=3)
            graphics::abline(h=0, lty=3)  
	        graphics::abline(stats::lm( slag ~ x.scale ), col="red")
		   
    } else if (type.ac[1] == "yx") {
	  type.lab <- paste0("Cross-correlation of ", deparse(substitute(y)), " and ", deparse(substitute(x)))
	  if (is.null(dots[["xlab"]]) & "xlab" %in% names(dots) == FALSE) dots[["xlab"]] <- deparse(substitute(y))
	  if (is.null(dots[["ylab"]]) & "ylab" %in% names(dots) == FALSE) dots[["ylab"]] <- "nWx,yy^TWx"
	  if (is.null(dots[["main"]]) & "main" %in% names(dots) == FALSE) dots[["main"]] <- type.lab
	   #gsci.xy <- as.numeric(y*w%*%x) 
	    gsci.yx <- as.numeric(y.scale*x.scale%*%w)
		    if(scale.morans) { 
			  gsci.yx <- scale.partial(gsci.yx)
			  slag <- as.numeric(x.scale*y.scale%*%w)
			    slag <- scale.partial(slag)
            } else {
              slag <- as.numeric(y.scale*x.scale%*%w)
            }	
	      plot.ac(y.scale,gsci.yx,dots)
	        graphics::abline(v=0, lty=3)
            graphics::abline(h=0, lty=3) 
            graphics::abline(stats::lm( slag ~ y.scale ), col="red")			   
    }
} 

#' @title Correlogram
#' @description Calculates and plots a correlogram
#' 
#' @param x         A sf POINT object
#' @param v         Test variable in x
#' @param dist      Distance of correlation lags, if latlong=TRUE units are 
#'                  great circle in kilometers
#' @param ns        Number of simulations to derive simulation envelope 
#' @param ...       Arguments passed to cor ('pearson', 'kendall' or 'spearman')
#' 
#' @return 
#' Plot of correlogram and a list object containing: 
#' * autocorrelation is a data.frame object with the following components
#' *   autocorrelation - Autocorrelation value for each distance lag 
#' *   dist - Value of distance lag
#' *   lci - Lower confidence interval (p=0.025)                
#' *   uci - Upper confidence interval (p=0.975)
#' * CorrPlot recordedplot object to recall plot
#' @md
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'                                                                   
#' @examples 
#' library(sf)
#' if(require(sp, quietly = TRUE)) {
#'   data(meuse, package = "sp")
#'   meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, 
#'                     agr = "constant")
#' }
#' 
#' zinc.cg <- correlogram(x = meuse, v = meuse$zinc, dist = 250, ns = 9)
#' 
#' @export correlogram  
correlogram <- function(x, v, dist = 5000, ns = 99, ...) {
  if (!inherits(x, c("SpatialPointsDataFrame", "sf")))		
    stop(deparse(substitute(x)), " x must be a sf or sp point object")
  if(inherits(x, c("SpatialPointsDataFrame", "SpatialPoints"))) {
     x <- sf::st_as_sf(x)
  }
	oops <- options() 
      on.exit(options(oops)) 
        options(scipen = 999)		
    w <- units::drop_units(sf::st_distance(x))
      aa <- ceiling(max(w)/dist)
      bw <- seq(0, aa * dist, dist)
      cors <- NULL
    for (i in 1:aa) {
        w1 <- ifelse(w > bw[i] & w <= bw[i + 1], 1, 0)
        w2 <- w1
        for (j in 1:dim(w1)[1]) {
            nu <- sum(w1[j, ])
            if (nu > 0) {
                w2[j, ] <- w1[j, ]/nu
            }
        }
        lag <- w2 %*% v
        cors <- c(cors, stats::cor(v, lag, ...))
    }
    if (length(which(is.na(cors))) > 0) {
      warning(paste(length(which(is.na(cors))), "Spatial lag empty and dropped", sep = " "))
      bw <- bw[-which(is.na(cors))]
     cors <- cors[-which(is.na(cors))]
    }
    mc <- matrix(NA, nrow = ns, ncol = length(cors))
    for (s in 1:ns) {
        x$rand <- sample(v, nrow(x), replace = FALSE)
        rcors <- NULL
        for (i in 1:aa) {
            w1 <- ifelse(w > bw[i] & w <= bw[i + 1], 1, 0)
            w2 <- w1
            for (j in 1:dim(w1)[1]) {
                nu <- sum(w1[j, ])
                if (nu > 0) {
                  w2[j, ] <- w1[j, ]/nu
                }
            }
            lag <- w2 %*% x$rand
            rcors <- c(rcors, stats::cor(x$rand, lag))
        }
        mc[s, ] <- rcors
    }
    bw <- bw[-1]
    cg <- data.frame(cbind(cors, bw))
    cg <- cbind(cg, t(apply(mc, 2, stats::quantile, probs = c(0.025, 0.975))))
    names(cg) <- c("autocorrelation", "dist", "lci", "uci")
    graphics::plot(cg$dist, cg$autocorrelation, type = "n", ylim = c(-1, 1), 
	               main = "Correlogram", xlab = "distance", 
				   ylab = "autocorrelation")
      graphics::polygon(c(rev(cg$dist), cg$dist), c(cg$uci, rev(cg$lci)), col = "blue")
        graphics::lines(cg$dist, cg$autocorrelation, type = "b", pch = 20)
  return(list(autocorrelation = cg, CorrPlot = grDevices::recordPlot()))
} 

#' @title optimalK
#' @description Find optimal k of k-Medoid partitions using 
#'              silhouette widths
#'
#' @param x           Numeric dataframe, matrix or vector  
#' @param nk          Number of clusters to test (2:nk)
#' @param plot        (TRUE / FALSE) Plot cluster silhouettes(TRUE/FALSE)
#' @param cluster     (TRUE / FALSE) Create cluster object with optimal k
#' @param clara       (FALSE / TRUE) Use clara model for large data
#' @param ...         Additional arguments passed to clara
#'
#' @return Object of class clust "pam" or "clara" with tested silhouette values 
#'
#' @author Jeffrey S. Evans  <jeffrey_evans<at>tnc.org>
#'
#' @references 
#' Theodoridis, S. & K. Koutroumbas(2006) Pattern Recognition 3rd ed.   
#'
#' @examples 
#' if (require(cluster, quietly = TRUE)) {
#'   x <- rbind(cbind(rnorm(10,0,0.5), rnorm(10,0,0.5)),
#'              cbind(rnorm(15,5,0.5), rnorm(15,5,0.5)))
#'
#'   clust <- optimal.k(x, 20, plot=TRUE, cluster=TRUE)
#'     plot(silhouette(clust$model), col = c('red', 'green'))
#'       plot(clust$model, which.plots=1, main='K-Medoid fit')
#' 
#' # Extract multivariate and univariate mediods (class centers)
#'   clust$model$medoids
#'     pam(x[,1], 1)$medoids  
#'
#' # join clusters to data
#'   x <- data.frame(x, k=clust$model$clustering) 
#'
#' } else { 
#'   cat("Please install cluster package to run example", "\n")
#' }
#'
#' @seealso \code{\link[cluster]{pam}} for details on Partitioning Around Medoids (PAM)  
#' @seealso \code{\link[cluster]{clara}} for details on Clustering Large Applications (clara) 
#'
#' @export  
optimal.k <- function(x, nk = 10, plot = TRUE, cluster = TRUE, clara = FALSE, ...) {
	if(length(find.package("cluster", quiet = TRUE)) == 0)
      stop("please install cluster package before running this function")
    asw <- numeric(nk)
      for (k in 2:nk) {
          if (clara == TRUE) {
            asw[k] <- cluster::clara(x, k, ...)$silinfo$avg.width
          } else {
            asw[k] <- cluster::pam(x, k, ...)$silinfo$avg.width
          }
          k.best <- which.max(asw)
      }
    message("Optimal-K", " = ", k.best)
    if (plot == TRUE) {
        graphics::plot(1:nk, asw, type = "s", main = "Clustering Optimization using K-Mediods",
                       xlab = "K (number of clusters)", ylab = "mean silhouette width")
        graphics::axis(1, k.best, paste("best", k.best, sep = "\n"), col = "red", col.axis = "red")
    }
    if (cluster == TRUE) {
        if (clara == TRUE) {
          k.solution <- cluster::clara(x, k.best, asw, ...)
        } else {
          k.solution <- cluster::pam(x, k.best, ...)
        }
	 return(list(model=k.solution, k=k.best, silhouettes=asw))
    }
  return(list(k=k.best, silhouettes=asw)) 	
} 

#' @title Raster Distance
#' @description Calculates the Euclidean distance of a defined raster class and
#'              all the other cells in a taster
#'
#' @param  x          A terra SpatRast or sf class object 
#' @param  y          Value(s) in x to to calculate distance to
#' @param  scale      (FALSE/TRUE) Perform a row standardization on results 
#'
#' @details 
#' This replicates the terra distance function but uses the Arya & Mount
#' Approximate Near Neighbor (ANN) C++ library for calculating distances. Where this
#' results in a notable increase in performance it is not memory safe, needing to read
#' in the entire raster and does not use the GeographicLib (Karney, 2013) spheroid 
#' distance method for geographic data.  
#'
#' @return A terra SpatRast raster representing distances
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com> 
#'
#' @references 
#' Arya S., Mount D. M., Netanyahu N. S., Silverman R. and Wu A. Y (1998), An 
#'   optimal algorithm for approximate nearest neighbor searching, Journal of 
#'   the ACM, 45, 891-923.
#'
#' @examples
#' \donttest{
#' library(sf)
#' library(terra)
#' 
#' # read, project and subset 10 polygons
#' nc <- suppressWarnings(st_cast(st_read(system.file("shape/nc.shp", 
#'          package="sf")), "POLYGON"))
#'   nc <- st_transform(nc, st_crs("ESRI:102008"))
#'     nc.sub <- nc[sample(1:nrow(nc),10),]
#' 
#' # create 1000m reference raster, rasterize subset polygons
#' ref <- rast(ext(nc), resolution=1000)
#'   rnc <- mask(rasterize(vect(nc.sub), field="CNTY_ID",
#'               ref, background=9999), vect(nc)) 
#'     crs(rnc) <- "ESRI:102008"  
#'   
#' # Calculate distance to class 1 in rnc raster, plot results
#' ids <- nc.sub$CNTY_ID 
#' rd <- rasterDistance(rnc, y=ids) 
#'   plot(rd)
#'     plot( st_geometry(nc.sub), add=TRUE)
#' 
#' }
#' @seealso \code{\link[terra]{distance}, \link[terra]{distance}}
#'
#' @import terra
#' @export rasterDistance 
rasterDistance <- function(x, y, scale = FALSE){
  if(length(find.package("RANN", quiet = TRUE)) == 0)
    stop("please install RANN package before running this function")
  if(missing(x))
    stop("x argument is missing")
  if(missing(y))
    stop("y argument is missing")	
  if(!any(y %in% unique(x[])[,1]))
    stop("values in ", deparse(substitute(y)), " are missing in ", deparse(substitute(x)))
  if(!inherits(x, "SpatRaster"))
	  stop(deparse(substitute(x)), " must be a terra SpatRast object")
  r <- terra::ifel(x %in% y, 1, NA)  
    idx <- which(r[] == 1)
      na.idx <- which(is.na(r[]))
  knn.dist <- RANN::nn2(terra::xyFromCell(r, idx),
                        terra::xyFromCell(r, na.idx), 
    			        k = 1)$nn.dists[,1]
	r <- terra::ifel(r == 1, 0, NA)					
      r[na.idx] <- knn.dist
	    r <- terra::mask(r, x)	
    if(scale) r <- r / terra::global(r, "max", na.rm=TRUE)[,1] 
  return(r)
}

#' @title separability
#' @description Calculates variety of two-class sample separability metrics 
#'           
#' @param x         X vector
#' @param y         Y vector
#' @param plot      plot separability (TRUE/FALSE)
#' @param cols      colors for plot (must be equal to number of classes)
#' @param clabs     labels for two classes
#' @param ...       additional arguments passes to plot
#'
#' @details 
#' Available statistics:
#' * M-Statistic (Kaufman & Remer 1994) - This is a measure of the difference of the 
#'   distributional peaks. A large M-statistic indicates good separation between the 
#'   two classes as within-class variance is minimized and between-class variance 
#'   maximized (M <1 poor, M >1 good).
#'    
#' * Bhattacharyya distance (Bhattacharyya 1943; Harold 2003) - Measures the similarity
#'   of two discrete or continuous probability distributions.   
#'    
#' * Jeffries-Matusita distance (Bruzzone et al., 2005; Swain et al., 1971) - The J-M 
#'   distance is a function of separability that directly relates to the probability of 
#'   how good a resultant classification will be. The J-M distance is asymptotic to v2, 
#'   where values of v2 suggest complete separability
#'    
#' * Divergence and transformed Divergence (Du et al., 2004) - Maximum likelihood approach. 
#'   Transformed divergence gives an exponentially decreasing weight to increasing distances
#'   between the classes.
#'
#' @md 
#'
#' @return A data.frame with the following separability metrics: 
#' *  B - Bhattacharryya distance statistic 
#' *  JM - Jeffries-Matusita distance statistic
#' *  M - M-Statistic
#' *  D - Divergence index
#' * TD - Transformed Divergence index
#' @md
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @references
#' Anderson, M. J., & Clements, A. (2000) Resolving environmental disputes: a  
#'   statistical method for choosing among competing cluster models. Ecological  
#'   Applications 10(5):1341-1355
#' 
#' Bhattacharyya, A. (1943) On a measure of divergence between two statistical 
#'   populations defined by their probability distributions'. Bulletin of the 
#'   Calcutta Mathematical Society 35:99-109
#' 
#' Bruzzone, L., F. Roli, S.B. Serpico (1995) An extension to multiclass cases of 
#'   the Jefferys-Matusita distance. IEEE Transactions on Pattern Analysis and  
#'   Machine Intelligence 33:1318-1321
#' 
#' Du, H., C.I. Chang, H. Ren, F.M. D'Amico, J. O. Jensen, J., (2004) New 
#'   Hyperspectral Discrimination Measure for Spectral Characterization. Optical 
#'   Engineering 43(8):1777-1786.
#' 
#' Kailath, T., (1967) The Divergence and Bhattacharyya measures in signal  
#'   selection. IEEE Transactions on Communication Theory 15:52-60  
#' 
#' Kaufman Y., and L. Remer (1994) Detection of forests using mid-IR reflectance: 
#'   An application for aerosol studies. IEEE T. Geosci.Remote. 32(3):672-683.
#' 
#' @examples 
#'    norm1 <- dnorm(seq(-20,20,length=5000),mean=0,sd=1) 
#'    norm2 <- dnorm(seq(-20,20,length=5000),mean=0.2,sd=2)                          
#'      separability(norm1, norm2) 
#'            
#'    s1 <- c(1362,1411,1457,1735,1621,1621,1791,1863,1863,1838)
#'    s2 <- c(1362,1411,1457,10030,1621,1621,1791,1863,1863,1838)
#'      separability(s1, s2, plot=TRUE) 
#'       
#' @export separability                            
separability <- function(x, y, plot = FALSE, cols = c("red", "blue"), 
                         clabs = c("Class1", "Class2"), ...) {
    if (length(cols) > 2) 
      stop("Too many colors")
    if (length(clabs) > 2) 
      stop("Too many class labels")
    trace.of.matrix <- function(SquareMatrix) {
      sum(diag(SquareMatrix))
    }
	
    x <- as.matrix(x)
    y <- as.matrix(y)
    
	mdif <- mean(x) - mean(y)
    p <- (stats::cov(x) + stats::cov(y))/2
    bh.distance <- 0.125 * t(mdif) * p^(-1) * mdif + 0.5 * log(det(p)/sqrt(det(stats::cov(x)) * 
	               det(stats::cov(y))))
    
	m <- (abs(mean(x) - mean(y)))/(stats::sd(x) + stats::sd(y))
    jm.distance <- 2 * (1 - exp(-bh.distance)) # bound 0 - 1414
    
	dt1 <- 1/2 * trace.of.matrix((stats::cov(x) - stats::cov(y)) *  
	       (stats::cov(y)^(-1) - stats::cov(x)^(-1)))
    dt2 <- 1/2 * trace.of.matrix((stats::cov(x)^(-1) + stats::cov(y)^(-1)) * 
	                            (mean(x) - mean(y)) * t(mean(x) - mean(y)))
    divergence <- dt1 + dt2
    transformed.divergence <- 2 * (1 - exp(-(divergence/8)))
    
	if (plot == TRUE) {
        color1 <- as.vector(grDevices::col2rgb(cols[1])/255)
        color2 <- as.vector(grDevices::col2rgb(cols[2])/255)
        d1 <- stats::density(x)
        d2 <- stats::density(y)
        graphics::plot(d1, type = "n", ylim = c(min(c(d1$y, d2$y)), max(c(d1$y, d2$y))),  
		               xlim = c(min(c(d1$x, d2$x)), max(c(d1$x, d2$x))), ...)
        graphics::polygon(d1, col = grDevices::rgb(color1[1], color1[2], color1[3], 1/4))
        graphics::polygon(d2, col = grDevices::rgb(color2[1], color2[2], color2[3], 1/4))
        graphics::abline(v = mean(x), lty = 1, col = "black")
        graphics::abline(v = mean(y), lty = 2, col = "black")
        graphics::legend("topright", legend = clabs, fill = c(grDevices::rgb(color1[1], color1[2], color1[3], 1/4), 
		                 grDevices::rgb(color2[1], color2[2], color2[3], 1/4)))
    }
    return(data.frame(B = bh.distance, JM = jm.distance, M = m, mdif = abs(mdif), 
	                  D = divergence, TD = transformed.divergence))
} 

#' @title Class breaks
#' @description Finds class breaks in a distribution
#'
#' @param x     A vector to find breaks for
#' @param n     Number of breaks
#' @param type  Statistic used to find breaks c("equal", "quantile", "std", "geometric")
#'
#' @details 
#' The robust std method uses sqrt(sum(x^2)/(n-1)) to center the data before deriving "pretty" breaks.
#'
#' @return 
#' A vector containing class break values the length is n+1 to allow for 
#' specification of ranges
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#' 
#' @examples
#'  y <- rnbinom(100, 10, 0.5)
#'    classBreaks(y, 10)  
#'    classBreaks(y, 10, type="quantile")
#'  
#' opar <- par(no.readonly=TRUE)
#'    par(mfrow=c(2,2))
#'      d <- density(y)
#'      plot(d, type="n", main="Equal Area breaks")
#'        polygon(d, col="cyan")
#'        abline(v=classBreaks(y, 10)) 
#'      plot(d, type="n", main="Quantile breaks")
#'        polygon(d, col="cyan")
#'        abline(v=classBreaks(y, 10, type="quantile"))
#'      plot(d, type="n", main="Robust Standard Deviation breaks")
#'        polygon(d, col="cyan")
#'        abline(v=classBreaks(y, 10, type="std"))
#'      plot(d, type="n", main="Geometric interval breaks")
#'        polygon(d, col="cyan")
#'        abline(v=classBreaks(y, 10, type="geometric"))
#'  par(opar)
#' 	
#'  ( y.breaks <- classBreaks(y, 10) )   	
#'  cut(y, y.breaks, include.lowest = TRUE, labels = 1:10)
#'
#' @export
classBreaks <- function(x, n, type = c("equal", "quantile", "std", "geometric") ) {
  if(type[1] == "equal") { 
    return( seq(min(x), max(x), length.out=(n+1)) )
  } else if(type[1] == "quantile") {	
    return( c(stats::quantile(x=x, probs=seq(0,1,1/10))) )
  } else if(type[1] == "std") {
    svar <- scale(x)
    return( c((pretty(x = svar, n = n) * attr(svar, "scaled:scale")) + 
               attr(svar, "scaled:center")) )
  } else if(type[1] == "geometric") {
    breaks <- c(min(x), max(x))
      r <- exp((log(max(x)) - log(min(x)))/n)
        i.min <- breaks[1]
          for (i in 1:(n - 1)) {
            breaks <- c(breaks, i.min * r)
              i.min <- i.min * r
            breaks <- sort(breaks)
            }
    return(breaks)
  } else {
    stop("Not a valid statistic")
  }
}

#' @title date sequence
#' @description creates date sequence given start and stop dates
#'
#' @param start      Start date in "yyyy/mm/dd" character format         
#' @param end        End date in "yyyy/mm/dd" character format 
#' @param step       Time step, options are c("day", "week", "month", "quarter", 
#'                                            "year", "minute") 
#' @param rm.leap    Remove extra days in leap years 
#'
#' @return 
#' A date vector of class POSIXct for minute and Date for other options 
#'     
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @examples
#' # monthly steps 1990/01/01 - 2019/12/31
#' d <- date_seq("1990/01/01", "2019/12/31", step="month")
#'
#' # daily steps 1990/01/01 - 2019/12/31
#' d <- date_seq("1990/01/01", "2019/12/31", step="day")
#'
#' # daily steps 1990/01/01 - 2019/12/31 with leap days removed
#' d <- date_seq("1990/01/01", "2019/12/31", step="day", rm.leap=TRUE)
#'
#' # daily step 2008/12/29 - 2008/12/31, 2008 is leap year
#' d <- date_seq("2008/12/29", "2008/12/31")
#'
#' # minutes step 2008/12/29 - 2008/12/31, 2008 is leap year
#' d <- date_seq("2008/12/29", "2008/12/31", step="minute")
#'
#' @export date_seq 
date_seq <- function(start, end, step = c("day", "week", "month", "quarter", "year", "minute"),
                     rm.leap = FALSE){
  is.leap <- function(year) {
    return(ifelse((year %%4 == 0 & year %%100 != 0) | year %%400 == 0, TRUE, FALSE))
  }
  if(step[1] == "minute"){
    if(rm.leap) 
	  message("The rm.leap argument is not honored for minute sequences")
    d = seq(as.Date(start), as.Date(end), "days")
	if(length(d) > 10) 
	  message("Using minutes, your vector is n=", length(d)*1440)
    d <- seq.POSIXt(min(as.POSIXct(d)), (min(as.POSIXct(d)) + 
                   (length(d)*1440)*60), by = "1 min")
  } else {
    d = seq(as.Date(start), as.Date(end), step[1])
    if(rm.leap)
      d = d[!(format(d,"%m") == "02" & format(d, "%d") == "29"),drop = FALSE]  
  }  
  y <- unique(as.numeric(format(d,"%Y")))
  leap.idx <- which(sapply(y, is.leap))
  if(length(leap.idx) > 0)
    message("The following are leap years ", y[leap.idx], "\n" ,sep="\t")
  return(d)
}

#' @title is.empty
#' @description evaluates empty elements in a vector
#'  
#' @param x           A vector to evaluate elements
#' @param all.na      (FALSE / TRUE) Return a TRUE if all elements are NA
#' @param na.empty    (TRUE / FALSE) Return TRUE if element is NA
#' @param trim        (TRUE / FALSE) Trim empty strings
#'
#' @details
#' This function evaluates if an element in a vector is empty the na.empty argument 
#' allows for evaluating NA values (TRUE if NA) and all.na returns a TRUE if all elements 
#' are NA. The trim argument trims a character string to account for the fact that c(" ") 
#' is not empty but, a vector with c("") is empty. Using trim = TRUE will force both 
#' to return TRUE   
#'
#' @return A Boolean indicating empty elements in a vector, if all.na = FALSE
#' a TRUE/FALSE value will be returned for each element in the vector 
#'  
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @examples
#' is.empty( c("") )
#' is.empty( c(" ") )
#' is.empty( c(" "), trim=FALSE )
#' 
#' is.empty( c("",NA,1) )
#' is.empty( c("",NA,1), na.empty=FALSE)
#' 
#' is.empty( c(NA,NA,NA) )
#' is.empty( c(NA,NA,NA), all.na=TRUE )
#' is.empty( c(NA,2,NA), all.na=TRUE )
#' 
#' any( is.empty( c("",2,3) ) )
#' any( is.empty( c(1,2,3) ) )
#'
#' @export is.empty
is.empty <- function (x, all.na = FALSE, 
                      na.empty = TRUE, 
					  trim = TRUE) {
  if(trim) x <- gsub("^\\s+|\\s+$", "", x)
  if (!is.null(x)) {
    if(length(x) == 0) { 
      z <- TRUE
	} else {
    if (is.character(x)) {
      z <- nchar(x) == 0
	  }
    }
  }
  if(na.empty) z[is.na(z)] <- TRUE 
    if(all.na){ 
	  if(all(is.na(x))){ 
	    z <- TRUE
	  } else {
	    z <- FALSE
	  }
	}  	  
  return(unname(z))  
}

#' @title Cohen's-d effect size
#' @description 
#' Cohen's-d effect size with pooled sd for a control and experimental group 
#' 
#' @param y           A character or factor vector 
#' @param x           A numeric vector, same length as y
#' @param pooled      Pooled or population standard deviation (TRUE/FALSE)
#' @param conf.level  Specified confidence interval. Default is 0.95
#'
#' @return 
#' An effect.size class object with x, y and a data.frame with columns for effect 
#' size, lower confidence interval, lower confidence interval. The row names of 
#' the data frame represent the levels in y
#'
#' @note 
#' This implementation will iterate through each class in y and treating a given class 
#' as the experimental group and all other classes as a control case. Each class had d 
#' and the confidence interval derived. A negative d indicate directionality with same 
#' magnitude. The expected range for d is 0 - 3 
#' d is derived; ( mean(experimental group) - mean(control group) ) / sigma(p)
#' pooled standard deviation is derived; 
#' sqrt(  ( (Ne - 1) * sigma(e)^2 + (Nc - 1) * sigma(c)^2 ) / (Ne + Nc - 2) )   
#'   where; Ne, Nc = n of experimental and control groups.
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'         
#' @references  
#' Cohen, J., (1988) Statistical Power Analysis for the Behavioral Sciences 
#'   (second ed.). Lawrence Erlbaum Associates.
#' @references  
#' Cohen, J (1992) A power primer. Psychological Bulletin 112(1):155-159
#'                                                          
#' @examples
#'  ( es <- effect.size(iris$Species, iris$Sepal.Length) )
#'    plot(es)
#'
#' @export effect.size
effect.size <- function(y, x, pooled = TRUE, conf.level = 0.95) {
  effectsize <- vector()
  low.ci <- vector()
  up.ci <- vector()
  if(!inherits(y, "factor")) y <- as.factor(y)
    for(i in levels(y)){
      y.i <- ifelse( y == i, i, "control")
	    g.means <- tapply(x, y.i, mean)
	    g.sd <- tapply(x, y.i, stats::sd)
      n <- table(y)
	  if(pooled) {
        sd.pooled <- sqrt( ((n[1] - 1) * g.sd[1]^2 + (n[2] - 1) * g.sd[2]^2) / 
	                     (n[1] + n[2] - 2) )
	  } else {
        sd.pooled <- stats::sd(x)
      }
      delta.m <- g.means[i] - g.means["control"]  	
	    es <- delta.m / sd.pooled
		effectsize <- append(effectsize, es)
        deg.f = n[1] + n[2] - 2
          Sd <- sqrt(((n[1] + n[2])/(n[1] * n[2]) + 0.5 * es ^ 2 / deg.f) * 
		            ((n[1] + n[2]) / deg.f))
            Z <- -stats::qt((1 - conf.level) / 2, deg.f)
          conf.int <- c(es - Z * Sd, es + Z * Sd)
        low.ci <- append(low.ci, conf.int[1])
      up.ci <- append(up.ci, conf.int[2])	
	}
    effectsize <- list( effect.size = 
	                      data.frame(row.names=unique(y), 
	                      effect.size = abs(effectsize), lower.ci = low.ci,   
	                      upper.ci = up.ci), 
						  y = y, x = x, CI = conf.level)
    class(effectsize) <- c("effect.size", "data.frame")
  return( effectsize )
}

#' @title Random raster
#' @description Create a random raster or raster stack using specified 
#'              distribution
#'
#' @param r            Optional existing terra raster defining nrow/ncol                
#' @param n.row        Number of rows             
#' @param n.col        Number of columns
#' @param n.layers     Number of layers in resulting raster stack 
#' @param x            A vector of values to sample if distribution is "sample"
#' @param min          Minimum value of raster
#' @param max          Maximum value of raster
#' @param mean         Mean of centered distribution 
#' @param sd           Standard deviation of centered distribution
#' @param p            p-value for binominal distribution 
#' @param s            sigma value for Gaussian distribution
#' @param mask         (TRUE/FALSE) If r is provided, mask results to r
#' @param distribution Available distributions, c("random", "normal", 
#'                     "seq", "binominal", "gaussian", "sample")
#'
#' @details 
#' Options for distributions are; random, normal, seq, binominal, gaussian and sample raster(s)
#'
#' @return terra SpatRaster object with random rasters
#' 
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @examples
#' library(terra)
#' 
#' # Using existing raster to create random binominal  
#' r <- rast(system.file("ex/elev.tif", package="terra"))    
#' ( rr <- random.raster(r, n.layers = 3, distribution="binominal") )
#'   plot(c(r,rr)) 
#' 
#' # default; random, nrows=50, ncols=50, n.layers=5
#' ( rr <- random.raster() )
#' 
#' # specified; binominal, nrows=20, ncols=20, nlayers=5
#' ( rr <- random.raster(n.layer=5, n.col=20, n.row=20,  
#'                      distribution="binominal") )
#' 
#' # specified; gaussian, nrows=50, ncols=50, nlayers=1
#' ( rr <- random.raster(n.col=50, n.row=50, s=8,  
#'                       distribution="gaussian") )
#'    plot(rr)
#'  
#' # specified; sample, nrows=50, ncols=50, nlayers=1
#' ( rr <- random.raster(n.layer=1, x=c(2,6,10,15), 
#'                      distribution="sample" ) )
#'   freq(rr)
#'
#' @export random.raster 
random.raster <- function(r=NULL, n.row = 50, n.col = 50, n.layers = 1, x = seq(1,10), 
                          min = 0, max = 1, mean = 0, sd = 1, p = 0.5, s = 1.5, 
						  mask=TRUE, distribution = c("random", "normal", "seq", 
						                   "binominal", "gaussian")){
  d <- distribution[1]
    if(!is.null(r)) {	
      if(inherits(r, "SpatRaster")) {  
        i = terra::nrow(r)
        j = terra::ncol(r)
      }
    } else if(is.null(r)) {
      i = n.col
  	j = n.row
  	if(missing(min)) min = 0
  	  if(missing(max)) max = 1
  	  if(missing(sd)) sd = 1
  	if(missing(mean)) mean = 0
    }
    if(d == "gaussian") {
      if( i != j ) {  
  	  stop("rows/columns must be equal for Gaussian symmetrical kernel")
      }
    }
    if(missing(p)) p = 0.5
    n = i*j 
    dist.fun <- function(.n, .min = min, .max = max, .mean = mean, 
                         .x = x, .sd = sd, .p = p, .s = s, .d = d) { 
      if( d == "seq") {
        v <- 1:.n  
      } else if(d == "normal") {
        v <- stats::rnorm(stats::runif(.n, min=.min, max=.max),
                          mean=.mean, sd=.sd)
      } else if(d == "random") {
        v <- stats::runif(.n, min=.min, max=.max)			 
      } else if(d == "binominal") {
        v <- stats::rbinom(.n, 1, .p)
      } else if(d == "sample") {
        v <- sample(x, .n, replace=TRUE)	  
      } else {
        stop("Not an available distribution")
      }
  	return(v)
    }   
  message(paste0("Calculating ", d, " distribution with ", 
          i," x ", j," x ", n.layers, " dimensions"))
  if(d == "gaussian") {		  
    rr <- do.call(c, replicate(n.layers, terra::rast( 
                    gaussian.kernel(sigma=s, s=max(c(i,j)))))) 		  
  } else {		  
    rr <- do.call(c, replicate(n.layers, 
	                terra::rast(matrix(dist.fun(.n=n), i, j))))
  }
  if(!is.null(r)) {
    terra::ext(rr) <- terra::ext(r)
    # terra::res(rr) <- terra::res(r)
    terra::crs(rr) <- terra::crs(r)
	if(mask) rr <- terra::mask(rr, r)
  }  
  return(rr) 
}

#' @title Class comparison between two nominal rasters
#' @description Creates a labeled cross tabulation between two nominal rasters 
#'       
#' @param x          A terra SpatRaster class object   
#' @param y          A terra SpatRaster class object to compare to x
#' @param values     Expected values in both rasters
#' @param labs       Labels associated with values argument
#' @param pct        (TRUE/FALSE) return proportions rather than counts
#' @param ...        Additional arguments 
#'
#' @details  
#' This function returns a cross tabulation between two nominal rasters. 
#' Arguments allow for labeling the results and returning proportions 
#' rather than counts. It also accounts for asymmetrical classes between
#' the two rasters
#'
#' @return a table with the cross tabulated counts 
#' 
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'                                                                           
#' @references 
#' Pontius Jr, R.G., Shusas, E., McEachern, M. (2004). Detecting important categorical land changes 
#    while accounting for persistence. Agriculture, Ecosystems & Environment 101(2):251-268.
#'
#' @examples
#' library(terra)
#'  
#' e <- ext(179407.8, 181087.9, 331134.4, 332332.1)
#' lulc2010 <- rast(e, resolution=20)
#'   lulc2010[] <- sample(1:5, ncell(lulc2010), replace=TRUE)
#' lulc2020 <- rast(e, resolution=20)
#'   lulc2020[] <- sample(1:5, ncell(lulc2020), replace=TRUE)
#'  
#'  ( v = sort(unique(c(lulc2010[], lulc2020[]))) )
#'  l = c("water","urban","forest",
#'        "ag","barren")
#' 
#' cross.tab(lulc2010, lulc2020) 
#' cross.tab(lulc2010, lulc2020, values = v, labs = l)
#' cross.tab(lulc2010, lulc2020, values = v, labs = l, pct=TRUE)
#' 
#' # Create asymmetrical classes 
#' na.idx <- which(!is.na(lulc2010[]))
#' lulc2020[na.idx] <- sample(c(1,2,4,5), length(na.idx), replace=TRUE)
#' cross.tab(lulc2010, lulc2020, values = v, labs = l, pct=TRUE)
#' 
#'
#' @export
cross.tab <- function(x, y, values = NULL, labs = NULL,  
                      pct = FALSE, ...) {
  if(!inherits(x, "SpatRaster"))
    stop(deparse(substitute(x)), " Must be a terra SpatRaster object") 
  if(!inherits(y, "SpatRaster"))
    stop(deparse(substitute(y)), " Must be a terra SpatRaster object") 
  if(!is.null(labs)){
    if(length(labs) != length(values))
	  stop("length of labs and values must match")
  }	
  xname = deparse(substitute(x))
  yname = deparse(substitute(y))
  ct <- terra::crosstab(c(x, y), long = FALSE, ...) 
  if(!is.null(labs)) {
    colnames(ct) <- labs[which(values %in% colnames(ct))] 
    rownames(ct) <- labs[which(values %in% rownames(ct))] 
  }
    if(pct) ct <- round(apply(ct, 1, function(x) x / sum(x) ),5) 
	  l <- list( rownames(ct), colnames(ct))
        names(l) <- c(yname, xname)	  
      dimnames(ct) <- l 
  return(ct)
}  

#' @title Time to event
#' @description Returns the time (sum to position) to a specified value 
#'       
#' @param x          A vector, representing time-series, to evaluate 
#' @param y          Threshold value tor return position for
#' @param dir        Direction of evaluation c("LR", "RL")
#' @param int        FALSE | TRUE - Evaluate as integer (rounds to 0 decimal places)
#' @param up.to      FALSE | TRUE - Return value before event
#' @param na.action  c("fail", "ignore"), if "fail" function will return error with NA's
#'                   with "ignore" NA values will be included in count to event
#'
#' @details
#' The time to event represents the sum of positions, in the vector,
#' until the specified value is found ie., (0,0,1) would be 3 or, 
#' 2 with up.to=TRUE. The int argument allows for rounding a continuous  
#' variable. Since it may be difficult to find an exact match to a floating 
#' point value rounding mitigates the problem. If you want a specific rounding 
#' value (eg., 1 decimal place) you can apply it to x first then pass it to 
#' the function. The up.to argument will stop one value before the specified value 
#' of (y) regardless of integer or float. For NA handling, na.action defines the
#' function behavior, causing it to fail or count NAs. Note that it makes no
#' sense to actually remove NAs as it will make the run uninterpretable.    
#'
#' @return A vector value representing the time to event 
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com> 
#'
#' @examples
#' # Binomial instance
#' time_to_event(c(0,0,0,0,1,0,0,0,1,0))
#' time_to_event(c(0,0,0,0,1,0,0,0,1,0), up.to = TRUE)
#' time_to_event(c(0,0,0,0,1,0,0,0,1,0), dir="RL")
#' time_to_event(c(NA,0,0,0,1,0,0,0,1,0), na.action="ignore")
#' 
#' # Continuous threshold instance
#' ( x <- runif(100, 0,7) ) 
#' time_to_event(x, y = 5, int=TRUE)
#' 
#' # raster example
#' library(terra)
#'
#' # Binomial instance
#' r <- do.call(c, replicate(20,terra::rast(matrix(sample(
#'              c(0,1), 1000, replace=TRUE), 100, 100))))             
#'   ( t2e <- app(r, fun=time_to_event) )
#' 
#' # Continuous threshold instance
#' r <- do.call(c, replicate(20,terra::rast(matrix(
#'               runif(1000,0,7), 100, 100))))
#'   ( t2e <- app(r, fun=time_to_event, y=5) )
#'
#' @export
time_to_event <- function(x, y = 1, dir = c("LR", "RL"), int = FALSE,
                          up.to = FALSE, na.action=c("fail","ignore")) {
	if(length(which(x >= y)) < 1)
	  stop("y does not exist in data")  
	if(any(is.na(x))) {
	  if(na.action[1] == "fail")
	    stop("NA's present in data")
	  if(y == 0) { 
        x[is.na(x)] <- 1	
	  } else {
	    s <- stats::na.omit(unique(x[x != y]))
		x[is.na(x)] <- sample(s[s < y], 1)
	  }
	}	
    if(dir[1] == "RL") x <- rev(x)
	  if(int) x <- round(x,0)
        x.idx <- rle(x)$values
        e.idx <- rle(x)$lengths 
    if(x[1] == y) {
      if(up.to) e = 0 else e = 1
    } else {
	  e <- sum(e.idx[1:(which(x.idx >= y)[1]-1)])
    } 
  if(up.to) return(e) else return( e + 1 )
}

#' @title Remove or return polygon holes
#' @description Removes or returns all holes (null geometry) in sf polygon 
#' class objects 
#'
#' @param x           sf POLYGON or MULTIPOLYGON object
#' @param only.holes  Delete holes (FALSE) or returns only holes (FALSE)
#'
#' @details 
#' A hole is considered a polygon within a polygon (island) representing null 
#' geometry. If you want to return only holes, no longer NULL, use keep = TRUE. 
#' To delete holes use default only.holes = FALSE. Single part features will be 
#' returned regardless of input.  
#'
#' @return sf POLYGON object
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#' 
#' @examples
#' library(sf)
#' 
#' p <- sf::st_as_sf(sf::st_sfc(
#'   sf::st_polygon(list(
#'   cbind(c(2,4,4,1,2),c(2,3,5,4,2)),
#'   cbind(c(2.33, 2.05, 3.25, 3.25, 2.33), 
#'       c(3.00, 3.56, 3.95, 3.46, 3.00)))),
#'   sf::st_polygon(list(	
#'     cbind(c(5,4,2,5),c(2,3,2,2)))),
#'   sf::st_polygon(list(  
#'     cbind(c(4,4,5,10,4),c(5,3,2,5,5)),
#'     cbind(c(5,6,6,5,5),c(4,4,3,3,4)) 
#' ))))
#'   p$ID <- 1:3
#' 
#' rh <- remove.holes(p)
#' kh <- remove.holes(p, only.holes=TRUE)
#' 
#' opar <- par(no.readonly=TRUE)
#'    par(mfrow=c(2,2))
#'      plot(st_geometry(p), main="Original with holes")
#'      plot(st_geometry(rh), main="holes removed only.holes=FALSE")
#' 	 plot(st_geometry(kh), main="return holes only.holes=TRUE")
#' par(opar)
#' 
#' @export remove.holes
remove.holes <- function(x, only.holes = FALSE) {
  if(!inherits(x, c("sf", "sfc")))		
    stop(deparse(substitute(x)), " must be an sf or sfc object")
 if(!unique(as.character(sf::st_geometry_type(x))) %in% c("POLYGON", "MULTIPOLYGON"))
    stop(deparse(substitute(x)), " must be aPOLYGON object")		
  if(!only.holes) {
   l <- lapply(sf::st_geometry(x), function(i) i[1])
      l <- lapply(l, function(x) {
	         if(length(x) > 0)
             x[[1]]		   
	       })
      l[sapply(l, is.null)] <- NULL
    l <- lapply(l, function(x) sf::st_polygon(list(x)) ) 
	h <- sf::st_as_sf(sf::st_sfc(l))
	  if(all(class(x) == "sf" | ncol(x) > 1)){
	    h <- cbind(h, sf::st_drop_geometry(x))   
	  }
  } else if(only.holes) {
    l <- lapply(sf::st_geometry(x), function(i) i[-1])
      l <- lapply(l, function(x) {
	         if(length(x) > 0)
             x[[1]]		   
	       })
        l[sapply(l, is.null)] <- NULL
	  l <- lapply(l, function(x) sf::st_polygon(list(x)) )
	h <- sf::st_as_sf(sf::st_sfc(l))
	  h$ID <- 1:nrow(h)
  }
    sf::st_geometry(h) <- "geometry"
  return( h )
}

#' @title Query AWS-OLI
#' @description Query of Amazon AWS OLI-Landsat 8 cloud service
#'
#' @param ...  not used
#'    
#' @details 
#' Given the changes to AWS Registry of Open Data, using the data  
#' digest is no longer reliable. Plese use the AWS Data Exchange 
#' or a STAC API for data acccess 
#'
#' @export oli.aws
oli.aws <- function(...) {
  .Deprecated("oli.aws", package="spatialEco", 
    msg="Given the changes to AWS Registry of Open Data, using the data  
         digest is no longer reliable")
 message("Plese use the AWS Data Exchange or a STAC API for data acccess")	
}

#' @title Nth values
#' @description Returns the Nth highest or lowest values in a vector
#' 
#' @param x           Numeric vector
#' @param N           Number of (Nth) values returned
#' @param smallest    (FALSE/TRUE) Return the highest, else smallest values
#'
#' @details 
#' This function returns n lowest or highest elements in a vector
#'
#' @return Numeric vector of Nth values 
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com> 
#'
#' @examples
#' nth.values(1:20, N=3, smallest = TRUE)                 
#' nth.values(1:20, N=3)
#' 
#' @export nth.values
nth.values <- function(x, N=2, smallest = FALSE) {
  if(!is.numeric(x))
    stop("x must be a numeric vector")
  if(N >= length(x))
    stop("N cannot be larger than length-1 of the vector")
  if(smallest) {
    x[order(x)[1:N]]
  } else {
    sort(x[order(x, decreasing = T)[1:N]])
  }
}

#' @title Confidence interval for mean or median
#' @description Calculates confidence interval for the mean or median of a distribution with 
#'              unknown population variance
#' 
#' @param x             Vector to calculate confidence interval for
#' @param cl            Percent confidence level (default = 0.95)
#' @param stat          Statistic (mean or median)
#' @param std.error     Return standard error (TRUE/FALSE)
#'
#' @return data.frame contaning: 
#'   * lci - Lower confidence interval value
#'   * uci - Upper confidence interval value
#'   * mean - If stat = "mean", mean value of distribution
#'   * mean - Value of the mean or median
#'   * conf.level - Confidence level used for confidence interval  
#'   * std.error - If std.error = TRUE standard error of distribution
#' @md
#'   
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @examples 
#'  x <- runif(100)
#'  cr <- conf.interval(x, cl = 0.97) 
#'  print(cr)
#'
#'  d <- density(x)
#'  plot(d, type="n", main = "PDF with mean and 0.97 confidence interval")
#'    polygon(d, col="cyan3")
#'    abline(v=mean(x, na.rm = TRUE), lty = 2)
#'    segments( x0=cr[["lci"]], y0=mean(d$y), x1=cr[["uci"]], 
#'              y1=mean(d$y), lwd = 2.5, 
#'              col = "black")
#'  	legend("topright", legend = c("mean", "CI"), 
#'  	       lty = c(2,1), lwd = c(1,2.5)) 
#'
#' @export
conf.interval <- function(x, cl = 0.95, stat = "mean", std.error = TRUE) {
  se <- function(x) { sqrt(stats::var(x, na.rm = TRUE) / length(stats::na.omit(x))) }
  x <- x[!is.na(x)]
  if( stat == "mean") {
    e = stats::qt(cl, df = length(x)-1) * se(x)
    cie <- mean(x) + c(-e, e)
	cie <- list("lci"=cie[1], "uci"=cie[2], "mean"=mean(x), "conf.level"=cl)
  } else {
  cie <- stats::wilcox.test(x, conf.int = TRUE, conf.level = cl)
  cie <- list("lci"=cie$conf.int[1], "uci"=cie$conf.int[2], "median"=cie$estimate,
             "conf.level"=cl)
  }
  if( std.error ) { 
    cie[["std error"]] <- se(x)
  }	  
  return(cie)
}

#' @title Percent area sample
#' @description Creates a point sample of polygons where n is based 
#'              on percent area
#'
#' @param x         An sf POLYGON object
#' @param pct       Percent of area sampled
#' @param join      {FALSE/TRUE} Join polygon attributed to point sample
#' @param sf        Scaling factor (default is meters to acres conversion factor)
#' @param stype     Sampling type ('random', 'regular', 'nonaligned', 'hexagonal')
#' @param ...       Additional arguments passed to spsample
#'
#' @details 
#' This function results in an adaptive sample based on the area of 
#' each polygon. The default scaling factor (sf) converts meters to
#' acres. You can set sf=1 to stay in the native projection units
#'
#' @return An sf POINT object
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @examples 
#' library(sf)
#' nc <- st_read(system.file("shape/nc.shp", package="sf"))
#'   nc <- suppressWarnings(st_cast(nc[c(10,100),], "POLYGON"))
#'   
#'  ( ars <- parea.sample(nc, pct=0.001, join = TRUE, stype='random') ) 
#'      plot(st_geometry(nc))
#'         plot(st_geometry(ars), pch=19, add=TRUE)  
#'
#' @export
parea.sample <- function(x, pct = 0.1, join = FALSE, sf = 4046.86,    
                         stype = "random", ...) {
  if (!inherits(x, "sf")) 
    stop(deparse(substitute(x)), " must be an sf POLYGON object")
  if(!unique(as.character(sf::st_geometry_type(x))) %in% c("POLYGON", "MULTIPOLYGON"))
    stop(deparse(substitute(x)), " must be an sf POLYGON object")		
  ns <- unlist(lapply(1:nrow(x), function(i) {
    round( (units::drop_units(sf::st_area(x[i,])) / sf) * pct, 0) 
  })) 
  s <- sf::st_as_sf(sf::st_sample(x, size=ns, type=stype, 
                    by_polygon=TRUE, ...))
    if(join == TRUE) {
      s <- suppressWarnings(sf::st_intersection(s, x))
    }
  return(s)
} 

#' @title Heat Load Index
#' @description Calculates the McCune & Keon (2002) Heat Load Index
#' 
#' @param x                terra SpatRaster class object
#' @param check            (TRUE/FALSE) check for projection integrity and
#'                          calculate central latitude for non-geographic
#'                          projections 
#' @param force.hemisphere  If country is split at the equator, force southern 
#'                          or northern hemisphere equation c("southern", "northern")  
#' 
#' @details
#' Describes A southwest facing slope should have warmer temperatures than a 
#' southeast facing slope, even though the amount of solar radiation they receive 
#' is equivalent. The McCune and Keon (2002) method accounts for this by "folding" 
#' the aspect so that the highest values are southwest and the lowest values are  
#' northeast. Additionally, this method account for steepness of slope, which is 
#' not addressed in most other aspect rescaling equations. HLI values range 
#' from 0 (coolest) to 1 (hottest). 
#' 
#' The equations follow McCune (2007) and support northern and southern hemisphere 
#' calculations. The folded aspect for northern hemispheres use (180 - (Aspect – 225) ) 
#' and for Southern hemisphere  ( 180 - ( Aspect – 315) ). If a country is split at the 
#' equator you can use the force.hemisphere argument to choose which equation to use. 
#' Valid values for this argument are "southern" and "northern" with the default "none".     
#'
#' @return terra SpatRaster class object of McCune & Keon (2002) Heat Load Index
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @references 
#' McCune, B., and D. Keon (2002) Equations for potential annual direct 
#'   incident radiation and heat load index. Journal of Vegetation 
#'   Science. 13:603-606.
#' 
#' McCune, B. (2007). Improved estimates of incident radiation and heat load 
#'   using non-parametric regression against topographic variables. Journal 
#'   of Vegetation Science 18:751-754. 
#' 
#' @examples 
#'   library(terra)
#'   elev <- rast(system.file("extdata/elev.tif", package="spatialEco"))
#'   heat.load <- hli(elev)
#'     plot(heat.load, main="Heat Load Index") 
#'    
#' @export hli
hli <- function(x, check = TRUE, force.hemisphere = c("none", "southern", "northern")) {
  if (!inherits(x, "SpatRaster")) 
    stop(deparse(substitute(x)), " must be a terra SpatRaster class object") 
  if(is.na(sf::st_crs(terra::crs(x))))	
    stop("Projection must be defined")	
  if(check) {
	if(!sf::st_is_longlat(x)) {
	  e <- sf::st_as_sf(terra::as.polygons(terra::ext(x)))
	    sf::st_crs(e) <- sf::st_crs(terra::crs(x))
	      l <- sf::st_bbox(sf::st_transform(e, 4326))[2]
    } else {
      l <- terra::ext(x)[3] 
    }	  
  }  
  if(l < 0) hemisphere = "southern" else hemisphere = "northern"  
    l = abs(l) * 0.017453293	
      cl = cos(l)
        sl = sin(l)
      tmp1 <- terra::terrain(x, v="slope", unit="degrees") * 0.017453293
    tmp2 <- terra::terrain(x, v="aspect", unit="degrees") * 0.017453293
	if(hemisphere == "northern" | force.hemisphere[1] == "northern"){ 
	  message("Using folded aspect equation for Northern hemisphere") 	
	    # Folded Aspect Northern Hemisphere  (180 - (Aspect – 225) )
		#   180(deg)=3.141593(rad), 225=3.92699(rad)
        tmp3 <- terra::app(tmp2, fun=function(x) { abs(3.141593 - abs(x - 3.926991)) } ) 
	} else if(hemisphere == "southern" | force.hemisphere[1] == "southern") {
	  message("Using folded aspect equation for Southern hemisphere") 		
		# Folded Aspect Southern Hemisphere  ( 180 - ( Aspect – 315) )  
		#   180(deg)=3.141593(rad), 315=5.49779 
		tmp3 <- terra::app(tmp2, fun=function(x) { abs(3.141593 - abs(x - 5.497791)) } ) 
	}	
          tmp4 <- terra::app(tmp1, fun = cos)
        tmp5 <- terra::app(tmp1, fun = sin)
      tmp6 <- terra::app(tmp3, fun = cos)
    tmp7 <- terra::app(tmp3, fun = sin)
	h <- exp( -1.467 +  1.582 * cl * tmp4  - 1.5 * tmp6 * tmp5 * sl - 0.262 * 
             sl * tmp5  + 0.607 * tmp7 * tmp5)
	if(terra::global(h, "max", na.rm=TRUE)[,1] > 1){
	  h <- ( h / terra::global(h, "max", na.rm=TRUE)[,1] ) 
    }		  
  return( h )
 } 

#' @title Shannon's Diversity (Entropy) Index
#' @description Calculates Shannon's Diversity Index and Shannon's Evenness Index
#'
#' @param x        data.frame object containing counts or proportions 
#' @param counts   Are data counts (TRUE) or relative proportions (FALSE) 
#' @param ens      Calculate effective number of species (TRUE/FALSE)
#' @param margin   Calculate diversity for rows or columns. c("row", "col")    
#'
#' @details 
#' The expected for H is 0-3+ where a value of 2 has been suggested as medium-high diversity, 
#' for evenness is 0-1 with 0 signifying no evenness and 1, complete evenness. 
#'
#' @return 
#' data.frame with "H" (Shannon's diversity) and "evenness" (Shannon's 
#' evenness where H / max( sum(x) ) ) and ESN 
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @references 
#' Shannon, C. E. and W. Weaver (1948) A mathematical theory of communication. The Bell 
#'   System Technical Journal, 27:379-423.
#' 
#' Simpson, E. H. (1949) Measurement of diversity. Nature 163:688
#' 
#' Roth, D. S., I. Perfecto, and B. Rathcke (1994) The effects of management systems on 
#'   ground-foraging ant diversity in Costa Rica. Ecological Applications 4(3):423-436.
#'
#' @examples
#' # Using Costa Rican ant diversity data from Roth et al. (1994)
#' data(ants)
#'   
#' # Calculate diversity for each covertype ("col") 
#' shannons(ants[,2:ncol(ants)], ens = TRUE, counts = FALSE, margin = "col")
#'
#' # Calculate diversity for each species ("row") 
#' ant.div <- shannons(ants[,2:ncol(ants)], ens = TRUE, counts = FALSE, 
#'                     margin = "row")
#'   row.names(ant.div) <- ants[,1]
#'   ant.div
#'	
#' @export
shannons <- function(x, counts = TRUE, ens = FALSE, margin = "row") {
  if( margin == "row") { m = 1 } else { m = 2 }
    s <- function(x) { -1 * sum( x[!is.na(x) > 0] * log(x[!is.na(x) > 0]) ) } 
    e <- function(x) { -1 * sum( (1 / x[!is.na(x) > 0]) * log((1 / x[!is.na(x) > 0])) ) } 
  if( counts ) {
    x.sum <- apply(x, MARGIN = m, sum, na.rm=TRUE)  
    x <- sweep(x, MARGIN = m, x.sum, "/")
  }   
    H <- apply(x, MARGIN = m, FUN = s)
	n.spp <- apply(x, MARGIN = m, FUN = function(x) {length(x[x > 0])} )
    H <- data.frame(H=H, evenness = ( H / log(n.spp) ) )
  if( ens ) {
    H <- data.frame(H, ENS = round(exp ( H[,1] ),0) ) 
  }
  return ( H ) 	
}

#' @title Summarizing effect size
#' @description Summary method for class "effect.size".
#'
#' @param object  Object of class effect.size
#' @param ... Ignored
#'
#' @return Prints the output data.frame contaning; effect size with upper and lower confidence 
#' and, mean and sd by group  
#'
#' @method summary effect.size
#' @export
summary.effect.size <- function(object, ...) {
  cat("Mean by groups in y: ", "\n")
    print( tapply(object$x, object$y, mean) )
	cat("", "\n") 
  cat("Standard deviation by groups in y: ", "\n")
    print( tapply(object$x, object$y, stats::sd) )
	cat("", "\n") 
  for(i in 1:nrow(object$effect.size)) {
    print(paste( paste0("Effect size for: ", rownames(object$effect.size)[i]), 
	      round(object$effect.size[,1][i],4), sep=" = ") )
	cat(paste0(paste(rep(" ", 4), collapse = ""), 
	    paste0( "Lower CI: ", round(object$effect.size[,2][i],4))))
    cat(paste0(paste(rep(" ", 4), collapse = ""), 
	    paste0( "Upper CI: ", round(object$effect.size[,3][i],3))))
	cat("", "\n")
	cat("", "\n")
  }
}

#' @title Provide URL's for Copernicus Global Land Service datasets
#' @description Returns URL's of a product/version/resolution 
#'   
#' @param ...  not used
#'    
#' @details 
#' Given the changes to ESA-Copenricus data distribution, using digests is
#' no longer reliable. Plese use the Sentinel Hub or a STAC API for data 
#' acccess 
#'
#' @export cgls_urls
cgls_urls <- function(...) {
  .Deprecated("cgls_urls", package="spatialEco", 
    msg="Given the changes to ESA-Copenricus data distribution, using the data  
         digest is no longer reliable")
 message("Plese use the Sentinel Hub or a STAC API for data acccess")	
}

#' @title trend.line
#' @description Calculated specified trend line of x,y
#'
#' @param x      Vector of x
#' @param y      Vector of y
#' @param type   Trend line types are: 'linear', 'exponential', 
#'               'logarithmic', 'polynomial'
#' @param plot   plot results (TRUE/FALSE)
#' @param ...    Additional arguments passed to plot 
#'
#' @return A list class object with the following components:
#' * for type = 'linear'  x is slope and y is intercept
#' * for type = 'exponential', 'logarithmic', or 'polynomial'
#'       x is original x variable and y is vector of fit 
#'       regression line
#' @md
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @examples 
#' x <- 1:10
#' y <- jitter(x^2)
#'
#' opar <- par(no.readonly=TRUE)
#'   par(mfcol=c(2,2))
#'     trend.line(x,y,type='linear',plot=TRUE,pch=20,main='Linear')
#'     trend.line(x,y,type='exponential',plot=TRUE,pch=20,main='Exponential')
#'     trend.line(x,y,type='logarithmic',plot=TRUE,pch=20,main='Logarithmic')
#'     trend.line(x,y,type='polynomial',plot=TRUE,pch=20,main='Polynomial')
#'  par(opar)
#' 
#' @export  
trend.line <- function(x, y, type = "linear", plot = TRUE, ...) {
    if (type == "linear") {
        fit <- stats::glm(y ~ x)
        trend <- c(as.numeric(stats::coef(fit)[1]), as.numeric(stats::coef(fit)[2]))
    }
    if (type == "exponential") {
        exp.f <- function(x, a, b) {
            a * exp(b * x)
        }
        fit <- stats::nls(y ~ exp.f(x, a, b), start = c(a = 1, b = 1))
        trend <- exp.f(x, a = stats::coef(fit)[1], b = stats::coef(fit)[2])
    }
    if (type == "logarithmic") {
        log.f <- function(x, a, b) {
            a * log(x) + b
        }
        fit <- stats::nls(y ~ log.f(x, a, b), start = c(a = 1, b = 1))
        trend <- log.f(x, a = stats::coef(fit)[1], b = stats::coef(fit)[2])
    }
    if (type == "polynomial") {
        poly.f <- function(x, a, b, d) {
            (a * x^2) + (b * x) + d
        }
        fit <- stats::nls(y ~ poly.f(x, a, b, d), start = c(a = 1, b = 1, d = 1))
        trend <- poly.f(x, a = stats::coef(fit)[1], b = stats::coef(fit)[2], d = stats::coef(fit)[3])
    }
    if (plot == TRUE) {
        graphics::plot(x, y, ...)
        if (type == "linear") {
            graphics::abline(a = trend[1], b = trend[2], col = "black", lwd = 2)
        } else {
            graphics::lines(x = x, y = trend, col = "black", lwd = 2)
        }
    }
    if (type == "linear") 
        return(list(x = trend[1], y = trend[2])) else return(list(x = x, y = trend))
} 

#' @title Gaussian smoothing of raster
#' @description Applies a Gaussian smoothing kernel to smooth raster. 
#' 
#' @param x         A terra SpatRaster raster object
#' @param s         Standard deviation (sigma) of kernel (default is 2)
#' @param n         Size of the focal matrix, single value (default is 
#'                  5 for 5x5 window) 
#' @param scale     (FALSE/TRUE) Scale sigma to the resolution of the raster
#' @param type      The statistic to use in the smoothing operator; 
#'                  "mean", "median", "sd", "convolution"
#' @param ...       Additional arguments passed to terra::focal 
#'
#' @details
#' This applies a Gaussian Kernel smoother. The convolution option performs
#' a Gaussian decomposition whereas the other options use the kernel
#' as weights for the given statistic. 
#' 
#' @return A terra SpatRaster class object of the local distributional moment
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @examples 
#' library(terra)
#' elev <- rast(system.file("extdata/elev.tif", package="spatialEco"))
#' 
#' # Gaussian smoothing with sigma = 2 and 7x7 window
#' g1 <- raster.gaussian.smooth(elev, s = 2, n = 7)
#'     plot(c(elev,g1))
#'
#' # g1 <- raster.gaussian.smooth(elev, s = 2, n = 7, type = "convolution")
#' 
#' @export
raster.gaussian.smooth <- function(x, s = 2, n = 5, scale = FALSE, 
             type = c("mean", "median", "sd", "convolution"), ...) {   
  if (!inherits(x, "SpatRaster")) 
    stop(deparse(substitute(x)), " must be a terra SpatRaster object")   
  if(scale) s = s * terra::res(x)[1]
  type=type[1]
    message("Using ", type, " for decomposition")
    raster.convolution <- function(x, std = s, nw = n) {
	  message("Note; convolution operations can be very slow")
       convolution <- function(p, K = NULL, sdv = std, m = nw) {
              if(inherits(p, "matrix")) 
    		    p <- as.vector(p)
                if(length(is.na(p)) > 0)
                  p[is.na(p)] <- mean(p, na.rm=TRUE)
                  n <- floor((sqrt(1 + 8 * length(p)) - 1)/2)-1
              if(is.null(K)) {
                K <- gaussian.kernel(sigma = sdv, s = m)
                K <- K / sum(K)
    		   }	  
               X <- as.matrix(Matrix::bandSparse(length(p), 
                              k = seq(-(length(K)-1),0,1), 
                              diag = t(replicate(length(p), rev(K))), 
            		          symm = FALSE))
                  lvc <- X %*% as.matrix(p, ncol=1)
          	lvc <- lvc[ceiling(length(lvc)/2)]
              return(as.numeric(lvc[1]))
            }	
    	  cv <- terra::focal(x, w = nw, fun = convolution)
        return(cv)
      }		 
      if(type == "convolution") {
        g <- raster.convolution(x, std = s, nw = n)
	  } else {
	    gm <- gaussian.kernel(sigma = s, s = n)
        g <- terra::focal(x, w = gm, fun=eval(parse(text = type)), ...)
      }
  return( g )
}  

#' @title Print poly_trend
#' @description print method for class "poly.trend"
#'
#' @param x    Object of class poly.trend
#' @param ...  Ignored
#'
#' @return Prints trend model summary, order and trend confidence intervals
#' 
#' @method print poly.trend
#' @export
print.poly.trend <- function(x, ...) {
  cat("Polynomial trend model fit using a", x$order, 
     "order polynomial",  "\n")
    print(summary(x$model))
  cat("", "\n")
  cat("Summary of trend confidence intervals",  "\n")
	print(summary(x$trend[,2:3]))
}  

#' @title shift
#' @description Shift a vector by specified positive or negative lag
#'       
#' @param x     A vector
#' @param lag   Number of lagged offsets, default is 1
#' @param pad   Value to fill the lagged offset with, default is NA
#' 
#' @return 
#' A vector, length equal to x, with offset length filled with pad values
#' 
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @examples
#' x <- 1:10
#' 
#' shift(x, 1)     # shift positive (from beginning of vector) by 1
#' shift(x, -1)    # shift negative (from end of vector) by 1
#' shift(x, 5, 0)  # Shift by 5 and fill (pad) with 0
#'
#' @export shift
shift <- function(x, lag=1, pad = NA) {
  if(!is.na(pad)) 
    message(paste0("Padding shifted values with - ", pad))
  n <- length(x)
  y <- rep(pad, length(x))
    if (lag < 0) {
	  message(paste0("Shifting ", abs(lag), " positions from end of vector")) 
      y[1:(n-abs(lag))] <- x[(abs(lag)+1):n]
    } else if (lag > 0) {
	  message(paste0("Shifting ", lag, " positions from beginning of vector"))
      y[(lag+1):length(x)] <- x[1:(length(x)-lag)]
    }
  return(y)
}

#' @title Erase points
#' @description Removes points intersecting a polygon feature class
#'    
#' @param y       A sf POINT object   
#' @param x       A sf POLYGON object  
#' @param inside  (TRUE/FALSE) Remove points inside polygon, else outside polygon
#'
#' @details  
#' Used to erase points that intersect polygon(s). The default of inside=TRUE
#' erases points inside the polygons however, if inside=FALSE then
#' the function results in an intersection where points that
#' intersect the polygon are retained. 
#'
#' @return An sf POINT object
#' 
#' @author Jeffrey S. Evans    <jeffrey_evans<at>tnc.org>
#'
#' @examples 
#'
#' library(sf)
#'   
#' if (require(sp, quietly = TRUE)) {
#'   data(meuse, package = "sp")
#'   meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, agr = "constant")
#' 
#'   s <- st_as_sf(st_sample(st_as_sfc(st_bbox(meuse)), size=1000, 
#'                  type = "regular"))
#'     s$id <- 1:nrow(s)
#'   b <- st_buffer(s[sample(1:nrow(s),5),], dist=300)
#'     b$id <- 1:nrow(b)
#'   
#' # Erase points based on polygons
#' in.erase <- erase.point(s, b)
#' out.erase <- erase.point(s, b, inside = FALSE)
#' 
#'  opar <- par(no.readonly=TRUE)
#'  par(mfrow=c(2,2))
#'    plot(st_geometry(s), pch=20, main="original data")
#'      plot(st_geometry(b),add=TRUE)
#'    plot(st_geometry(in.erase), pch=20, main="erased data")
#'      plot(st_geometry(b),add=TRUE)
#'    plot(st_geometry(out.erase), pch=20,  
#'         main="erased data using inside=FALSE")
#'      plot(st_geometry(b),add=TRUE)
#'  par(opar)
#' 
#' } else { 
#'   cat("Please install sp package to run example", "\n")
#' }
#'
#' @export erase.point
erase.point <- function(y, x, inside = TRUE) {
  if(!inherits(y, "sf"))		
    stop(deparse(substitute(y)), " must be an sf POINT object")	
  if(unique(as.character(sf::st_geometry_type(y))) != "POINT")
    stop(deparse(substitute(y)), " must be an sf POINT object")		
  if(!inherits(x, "sf"))		
    stop(deparse(substitute(x)), " must be an sf POLYGON object")
  if(unique(as.character(sf::st_geometry_type(x))) != "POLYGON")
    stop(deparse(substitute(x)), " must be an sf POLYGON object")		
  idx <- sf::st_intersects(y, x, sparse = FALSE)   
  if(inside == TRUE) { 
    y <- y[-which(apply(idx, 1, any)),]
  } else {
    y <- y[which(apply(idx, 1, any)),]  
  }
  return(y)
}

#' @title Invert raster
#' @description Inverts (flip) the values of a raster
#' 
#' @param x  A terra SpatRaster object
#' 
#' @details
#' Inverts raster values using the formula: (((x - max(x)) * -1) + min(x)
#'
#' @return 
#' A terra SpatRaster object with inverted (flipped) raster values
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#' 
#' @examples 
#' library(terra)
#' r <- rast(nrows=500, ncols=500, xmin=571823, xmax=616763, 
#'             ymin=4423540, ymax=4453690)
#'  crs(r) <- "epsg:9001"
#' r[] <- runif(ncell(r), 1000, 2500)
#' r <- focal(r, focalMat(r, 150, "Gauss") )
#'
#' r.inv <- raster.invert(r)
#'
#' opar <- par(no.readonly=TRUE)
#'     par(mfrow=c(1,2))
#'       plot(r, main="original raster")
#'       plot(r.inv, main="inverted raster") 
#' par(opar)   
#'
#' @export raster.invert
raster.invert <- function(x) {  
  if (!inherits(x, "SpatRaster")) 
    stop(deparse(substitute(x)), " must be a terra SpatRaster object")
      rmax <- terra::global(x, "max", na.rm = TRUE)[,1]
	rmin <- terra::global(x, "min", na.rm = TRUE)[,1]
  return( ((x - rmax) * -1) + rmin )
}  

#' @title Diurnal Anisotropic Heat Index
#' @description 
#' Simple approximation of the anisotropic diurnal heat (Ha) distribution 
#'
#' @param x          An elevation raster of class terra SpatRaster
#' @param amax       The Alpha Max (amax) parameter in degrees defined 
#'                   as: minimum = 0, maximum = 360 with the default = 202.500  
#'
#' @details 
#' The Diurnal Anisotropic Heat Index is based on this equation.  
#' Ha = cos(amax - a) * arctan(b)
#'   Where; amax defines the aspect with the maximum total heat  
#'   surplus, a is the aspect and b is the slope angle. Please
#'   note that all parameters are converted to radians.  
#'
#' @return terra SpatRaster class object Diurnal Anisotropic Heat Index
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com> 
#'
#' @references
#'  Boehner, J., and Antonic, O. (2009) Land-surface parameters specific to 
#'    topo-climatology. In: Hengl, T., & Reuter, H. (Eds.), Geomorphometry -
#'    Concepts, Software, Applications. Developments in Soil Science, 
#'    33:195-226 

#' @examples
#' library(terra)
#' elev <- rast(system.file("extdata/elev.tif", package="spatialEco"))
#' Ha <- dahi(elev)
#'   plot(Ha, col=grey(0:100/100), smooth=TRUE)
#'
#' @export dahi
dahi <- function(x, amax = 202.500) {
  if(!inherits(x, "SpatRaster"))
    stop(deparse(substitute(x)), " must be a terra SpatRaster object")
	amax = amax * (pi/180)
    tr <- terra::terrain(x, v=c("slope", "aspect"), unit="radians")
  return( cos(amax - tr[[2]]) * atan(tr[[1]]) )
}

#' @title Plot effect size 
#' @description Plot function for effect.size object 
#'
#' @param  x      A effect.size object
#' @param  ...    Additional arguments passed to plot
#'
#' @return Plot of effect size object with group effect sizes and 95% confidence  
#' 
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'                    
#' @method plot effect.size 
#' @export    	                               
plot.effect.size <- function(x, ...) {
  dots <- as.list(match.call(expand.dots = TRUE)[-1])
  dots[["x"]] <- as.numeric(as.factor(rownames(x$effect.size)))
  dots[["y"]] <- x$effect.size[,1]
  dots[["xaxt"]] <- "n"
  dots[["ylim"]] <- c(min(x$effect.size), max(x$effect.size))  
	if (is.null(dots[["pch"]]) & "pch" %in% names(dots) == FALSE) dots[["pch"]] <- 20
    if (is.null(dots[["cex"]]) & "cex" %in% names(dots) == FALSE) dots[["cex"]] <- 1
	if (is.null(dots[["xlab"]]) & "xlab" %in% names(dots) == FALSE) dots[["xlab"]] <-  "class"
	if (is.null(dots[["ylab"]]) & "ylab" %in% names(dots) == FALSE) dots[["ylab"]] <-  "effect size"
	if (is.null(dots[["main"]]) & "main" %in% names(dots) == FALSE) dots[["main"]] <- paste("Effect size with", x$CI, "CI",sep=" ") 
	  do.call("plot", dots)
	    graphics::axis(side=1, at = as.numeric(as.factor(rownames(x$effect.size))), 
	                   labels = rownames(x$effect.size))
        for(i in 1:nrow(x$effect.size)) {
	      graphics::arrows(i, x$effect.size[,2][i], i, 
	  	                   x$effect.size[,3][i], 
	  			           length = 0.05, angle = 90, 
	  			           code = 3)
	    }
}	

#' @title Plot Loess Bootstrap 
#' @description Plot function for loess.boot object 
#'
#' @param  x      A loess.boot object
#' @param  ...    Additional arguments passed to plot
#'
#' @return plot of lowess bootstrap
#' 
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @examples 
#'  n=1000
#'  x <- seq(0, 4, length.out=n)	 
#'  y <- sin(2*x)+ 0.5*x + rnorm(n, sd=0.5)
#'  sb <- loess.boot(x, y, nreps = 99, confidence = 0.90, span = 0.40)
#'  plot(sb)
#'                    
#' @method plot loess.boot 
#' @export    	                               
plot.loess.boot <- function(x, ...) {
  oops <- options() 
    on.exit(options(oops))
  dots <- as.list(match.call(expand.dots = TRUE)[-1])
  dots[["x"]] <- x$data$x
  dots[["y"]] <- x$data$y
	if (is.null(dots[["pch"]]) & "pch" %in% names(dots) == FALSE) dots[["pch"]] <-  20
	if (is.null(dots[["cex"]]) & "cex" %in% names(dots) == FALSE) dots[["cex"]] <-  0.55
	if (is.null(dots[["xlab"]]) & "xlab" %in% names(dots) == FALSE) dots[["xlab"]] <-  "x"
	if (is.null(dots[["ylab"]]) & "ylab" %in% names(dots) == FALSE) dots[["ylab"]] <-  "loess fit"
	if (is.null(dots[["main"]]) & "main" %in% names(dots) == FALSE) dots[["main"]] <-  paste0("Loess bootstrap n = ", x$nreps)   
	if (is.null(dots[["sub"]]) & "sub" %in% names(dots) == FALSE) dots[["sub"]] <-  paste0("Confidence region - ", x$confidence)   
	do.call("plot", dots)
	  graphics::polygon(c(x$fit$x, rev(x$fit$x)), c(x$fit$up.lim, rev(x$fit$low.lim)),  
	            col=grDevices::rgb(0.75, 0.75, 0.75, 0.5))				
      graphics::lines(stats::spline(x$fit$x, x$fit$y.fit), lwd=0.75, lty=2) 
}	

#' @title Square buffer
#' @description Creates a square buffer of a feature class
#' 
#' @param x    An sf object
#' @param a    Numeric single or vector indicating buffer distance(s)
#' @param ...  Additional arguments passed to st_buffer
#'
#' @details  Function creates a square buffer of feature class.
#'
#' @return A single feature sf class polygon object
#' 
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @examples
#' library(sf)
#' xy <- st_as_sf(data.frame(x = c(1,3,6,7),
#'                y = c(3,2,7,8), z = c(38,23,12,12),
#'                area = c(32,23,45,67)),
#' 			      coords = c("x", "y"), 
#' 			      agr = "constant") 
#' 
#' # With fixed buffer
#' sb <- squareBuffer(xy, 32)
#'   plot(st_geometry(sb))
#'     plot(st_geometry(xy), pch=20, add=TRUE)
#'   
#' # With variable buffer
#' sb.var <- squareBuffer(xy, xy$area)
#'   plot(st_geometry(sb.var))
#'     plot(st_geometry(xy), pch=20, add=TRUE)
#'   
#' @export				 					   
squareBuffer <- function(x, a, ...) {
  if(!inherits(x, c("sf", "sfc")))		
    stop(deparse(substitute(x)), " must be an sf class object")	
  if(missing(a))
    stop("buffer distance argument is missing")  
  a <- sqrt(a)/2
    dots <- as.list(match.call(expand.dots = TRUE)[-1])
	if(any(names(dots) == "a"))
	  dots <- dots[-which(names(dots) %in% "a")]
      dots$x <- x
      dots$dist <- a
      dots$nQuadSegs <- 1
      dots$endCapStyle <- "SQUARE"   
  return(do.call(sf::st_buffer, dots)) 
}

#' @title Concordance test for binomial models
#' @description Performs a concordance/disconcordance (C-statistic) test on 
#'              binomial models.  
#'
#' @param y vector of binomial response variable used in model
#' @param p estimated probabilities from fit binomial model 
#'
#' @details 
#' Test of binomial regression for the hypothesis that probabilities of all 
#' positives [1], are greater than the probabilities of the nulls [0]. The 
#' concordance would be 100% for a perfect model where, disconcordance is the 
#' inverse of concordance, representing the null. The C-statistic has been  
#' show to be comparable to the area under an ROC
#'
#' Results are: concordance - percent of positives that are greater than 
#' probabilities of nulls. discordance - concordance inverse of concordance 
#' representing the null class, tied - number of tied probabilities and 
#' pairs - number of pairs compared
#'
#' @return list object with: concordance, discordance, tied and pairs
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @references 
#' Austin, P.C. & E.W. Steyerberg (2012) Interpreting the concordance statistic of a 
#'   logistic regression model: relation to the variance and odds ratio of a continuous 
#'   explanatory variable. BMC Medical Research Methodology, 12:82 
#' @references 
#' Harrell, F.E. (2001) Regression modelling strategies. Springer, New York, NY. 
#' @references 
#' Royston, P. & D.G. Altman (2010) Visualizing and assessing discrimination in the 
#'   logistic  regression model. Statistics in Medicine 29(24):2508-2520
#'
#' @examples
#' data(mtcars)
#' dat <- subset(mtcars, select=c(mpg, am, vs))
#' glm.reg <- glm(vs ~ mpg, data = dat, family = binomial)
#' concordance(dat$vs, predict(glm.reg, type = "response")) 
#' 
#' @export
concordance <- function (y, p){
  if(is.factor(y)) y <- as.numeric(as.character(y)) 
    fit <- data.frame(response = y,  p = p)
      ones <- fit[fit$response == 1, ] 
      zeros <- fit[fit$response == 0, ] 
      totalPairs <- nrow (ones) * nrow (zeros) 
    conc <- sum(c(vapply (ones$p, function(x) {((x > zeros$p))}, 
                FUN.VALUE=logical(nrow(zeros)))))
    return(list("concordance" = (conc / totalPairs), 
          "discordance" = ((totalPairs - conc) / totalPairs),
          "tied" = (1 - (conc / totalPairs) - ((totalPairs - conc) / totalPairs)), 
		  "pairs" = totalPairs))
}

#' @title built index
#' @description Remote sensing built-up index
#'
#' @param red            Red band (0.636 - 0.673mm), landsat 5&7 band 3, OLI 
#'                       (landsat 8) band 4
#' @param green          Green band (0.53 - 0.59mm), landsat 5&7 band 3, OLI 
#'                       (landsat 8) band 3
#' @param nir            Near infrared band (0.851 - 0.879mm) landsat 5&7 band 4, 
#'                       OLI (landsat 8) band 5 
#' @param swir1          short-wave infrared band 1 (1.566 - 1.651mm), landsat 5&7 
#'                       band 5, OLI (landsat 8) band 6
#' @param swir2          short-wave infrared band 2 (2.11 - 2.29mm), landsat 5&7 
#'                       band 7, OLI (landsat 8) band 7
#' @param L              The L factor for the savi index 
#' @param method         Method to use for index options are "Bouhennache", "Zha", "Xu"
#'
#' @details 
#' This function calculates the built-up index. Three methods are available:
#' * Bouhennache is a new method that uses a larger portion of the VIR/NIR 
#'     following OLI bands (((b3+b4+b7)-b6)/3) / (((b3+b4+b7)+b6)/3)   
#' * Zha is the original band ratio method using TM5 ndbi = (b5 - b4) / (b5 + b4)
#' * Xu is a modification to eliminate noise using ETM+7 
#'    (ndbi-((savi-nndwi)/2) / (ndbi+((savi-nndwi)/2) 
#' 
#' Generally water has the highest values where built-up areas will occur in the
#' mid portion of the distribution. Since Bouhennache et al (2018) index exploits 
#' a larger portion of the visible (Vis) and infra red spectrum, vegetation will 
#' occur as the  lowest values and barren will exhibit greater values than the 
#' vegetation and lower values than the built-up areas.    
#'
#' Band wavelength (nanometers) designations for landsat
#' TM4, TM5 and ETM+7
#' * band-2 0.52-0.60 (green) 
#' * band-3 0.63-0.69 (red) 
#' * band-4 0.76-0.90 (NIR)
#' * band-5 1.55-1.75 (SWIR 1) 
#' * band-7 2.09-2.35 (SWIR 2) 
#' 
#' OLI (Landsat 8)
#' * band-3 0.53-0.59 (green) 
#' * band-4 0.64-0.67 (red) 
#' * band-5 0.85-0.88 (NIR)
#' * band-6 1.57-1.65 (SWIR 1) 
#' * band-7 2.11-2.29 (SWIR 2) 
#' @md
#'
#' @return A terra raster object of the built index
#'
#' @references 
#' Bouhennache, R., T. Bouden, A. Taleb-Ahmed & A. Chaddad(2018) A new spectral index 
#'   for the extraction of built-up land features from Landsat 8 satellite imagery, 
#'   Geocarto International 34(14):1531-1551
#' @references 
#' Xu H. (2008) A new index for delineating built-up land features in satellite imagery. 
#'  International Journal Remote Sensing 29(14):4269-4276. 
#' @references 
#' Zha G.Y., J. Gao, & S. Ni (2003) Use of normalized difference built-up index in 
#'   automatically mapping urban areas from TM imagery. International Journal of 
#'   Remote Sensing 24(3):583-594
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com> 
#'
#' @examples
#' \donttest{
#' library(terra)
#'  lsat <- rast(system.file("/extdata/Landsat_TM5.tif", package="spatialEco"))
#'    plotRGB(lsat, r=3, g=2, b=1, scale=1.0, stretch="lin")
#' 			   
#'  # Using Bouhennache et al., (2018) method (needs green, red, swir1 and swir2) 
#'  ( bouh <- built.index(red = lsat[[3]], green = lsat[[2]], swir1 = lsat[[5]], 
#'                       swir2 = lsat[[6]]) )
#'     plotRGB(lsat, r=3, g=2, b=1, scale=1, stretch="lin")
#'       plot(bouh, legend=FALSE, col=rev(terrain.colors(100, alpha=0.35)), 
#' 	       add=TRUE )
#' 
#'  # Using simple Zha et al., (2003) method (needs nir and swir1)
#'  ( zha <- built.index(nir = lsat[[4]], swir1 = lsat[[5]], method = "Zha") )
#'    plotRGB(lsat, r=3, g=2, b=1, scale=1, stretch="lin")
#'      plot(zha, legend=FALSE, col=rev(terrain.colors(100, alpha=0.35)), add=TRUE )
#' 
#'  # Using Xu (2008) normalized modification of Zha (needs green, red, nir and swir1)
#'  ( xu <- built.index(green= lsat[[2]], red = lsat[[3]], nir = lsat[[4]], 
#'                      swir1 = lsat[[5]], , method = "Xu") )
#'    plotRGB(lsat, r=3, g=2, b=1, scale=1, stretch="lin")
#'      plot(xu, legend=FALSE, col=rev(terrain.colors(100, alpha=0.35)), add=TRUE ) 
#' }
#' 
#' @export built.index
built.index <- function(green, red, nir, swir1, swir2, L = 0.5, 
                        method = c("Bouhennache", "Zha", "Xu")) { 
  if(method[1] == "Zha") {
    if(missing(nir) | missing(swir1))
      stop("must specify swir1 and nir bands")
  if(!any(c(inherits(nir, "SpatRaster"),inherits(swir1, "SpatRaster"))))
    stop("Rasters must be terra::SpatRaster class")	  
    bi <- (swir1 - nir) / (swir1 + nir)
  } else if(method[1] == "Xu") {
    if(missing(green) | missing(red) | missing(swir1))
      stop("Must specify green, red and swir1 bands") 
  if(!any(c(inherits(green, "SpatRaster"),inherits(red, "SpatRaster"),
     inherits(swir1, "SpatRaster"))))
    stop("Rasters must be terra::SpatRaster class") 	  
    ibi <- function(g, r, nr, s1, l = 0.5) { 
	  savi <- ((nr - r)*(1+l)) / (nr + r + l) 
      mndwi <- (g - s1) / (g - s1) 
      ndbi <- (swir1 - nir) / (swir1 + nir)	  
	  return( (ndbi - ((savi + mndwi) / 2)) / 
	          (ndbi + ((savi + mndwi) / 2)) ) 
	}
	bi <- ibi(g=green, r=red, nr= nir, s1=swir1,  l=L)
  } else if(method[1] == "Bouhennache") {
    if(missing(green) | missing(red) | missing(swir1) | missing(swir2))
      stop("Must specify green, red and swir1 bands")  
    if(!any(c(inherits(green, "SpatRaster"),inherits(red, "SpatRaster"),
       inherits(swir1, "SpatRaster"),inherits(swir2, "SpatRaster"))))
    stop("Rasters must be terra::SpatRaster class") 	  
    blfei <- function(g, r, s1, s2) {
       return( (( (g + r + s2) / 3) - s1) /
              ((( g + r + s2 ) / 3) + s1) )
    }
    bi <- blfei(g=green, r=red, s1=swir1, s2=swir2) 
  }
  return(bi)
}  

#' @name pu
#' @docType data 
#'
#' @title Biodiversity Planning Units
#'
#' @description Subset of biodiversity planning units for Haiti ecoregional spatial reserve plan 
#'
#' @format A sp SpatialPolygonsDataFrame with 5919 rows and 46 variables:
#' \describe{
#'   \item{UNIT_ID}{Unique planning unit ID}
#'   \item{DR_Dr_A}{Biodiversity target}
#'   \item{DR_Dr_L}{Biodiversity target}
#'   \item{Ht_Dr_A}{Biodiversity target}
#'   \item{Ht_Dr_L}{Biodiversity target}
#'   \item{DR_Ms_A}{Biodiversity target}
#'   \item{DR_Ms_L}{Biodiversity target}
#'   \item{Ht_Ms_L}{Biodiversity target}
#'   \item{DR_LM_M}{Biodiversity target}
#'   \item{H_LM_M_L}{Biodiversity target}
#'   \item{H_LM_R_L}{Biodiversity target}
#'   \item{DR_LM_R_L}{Biodiversity target}
#'   \item{DR_Rn_L}{Biodiversity target}
#'   \item{DR_LM_R_S}{Biodiversity target}
#'   \item{DR_Rn_S}{Biodiversity target}
#'   \item{DR_Ms_S}{Biodiversity target}
#'   \item{Ht_Ms_A}{Biodiversity target}
#'   \item{DR_Ms_E}{Biodiversity target}
#'   \item{DR_Ms_I}{Biodiversity target}
#'   \item{DR_Rn_E}{Biodiversity target}
#'   \item{DR_Rn_I}{Biodiversity target}
#'   \item{H_LM_R_E}{Biodiversity target}
#'   \item{Ht_Ms_E}{Biodiversity target}
#'   \item{Ht_Rn_E}{Biodiversity target}
#'   \item{DR_Rn_A}{Biodiversity target}
#'   \item{Ht_Rn_A}{Biodiversity target}
#'   \item{Ht_Rn_I}{Biodiversity target}
#'   \item{Ht_Dr_E}{Biodiversity target}
#'   \item{Ht_Ms_S}{Biodiversity target}
#'   \item{Ht_Dr_S}{Biodiversity target}
#'   \item{Ht_Rn_L}{Biodiversity target}
#'   \item{Ht_Th_A}{Biodiversity target}
#'   \item{Ht_Th_L}{Biodiversity target}
#'   \item{Ht_Th_S}{Biodiversity target}
#'   \item{Ht_Dr_U}{Biodiversity target}
#'   \item{Ht_Dr_I}{Biodiversity target}
#'   \item{Ht_Ms_I}{Biodiversity target}
#'   \item{H_LM_M_A}{Biodiversity target}
#'   \item{H_LM_M_E}{Biodiversity target}
#'   \item{H_LM_R_A}{Biodiversity target}
#'   \item{H_LM_M_S}{Biodiversity target}
#'   \item{H_LM_R_I}{Biodiversity target}
#'   \item{H_LM_R_S}{Biodiversity target}
#'   \item{Ht_Rn_S}{Biodiversity target}
#'   \item{Ht_Ms_U}{Biodiversity target}
#'   \item{Ht_Rn_U}{Biodiversity target}
#' }
#'
#' @references 
#' Evans, J.S., S.R. Schill, G.T. Raber (2015) A Systematic Framework for Spatial Conservation Planning and Ecological Priority Design in St. Lucia, Eastern Caribbean. Chapter 26 in Central American Biodiversity : Conservation, Ecology and a Sustainable Future. F. Huettman (eds). Springer, NY.
#'
#' @source "The Nature Conservancy" 
NULL

#' @title Polynomial trend
#' @description Fits a polynomial trend using specified order
#'
#' @param x        Vector of x
#' @param y        Vector of y
#' @param degree   Polynomial order (default 3)
#' @param ci       +/- confidence interval (default 0.95) 
#' @param plot     Plot results (TRUE/FALSE)
#' @param ...      Additional arguments passed to plot 
#'
#' @details 
#' A fit using a lm(y ~ x + I(X^2) + I(X^3)) form will be correlated which,  
#' can cause problems. The function avoids undue correlation using orthogonal 
#' polynomials
#'
#' @return A poly.trend class (list) containing  
#' * trend        data.frame of fit polynomial and upper/lower confidence intervals 
#' * model        Class lm model object fit with poly term       
#' * prameterCI   Intercept confidence intervals of Nth order polynomials    
#' * order        Specified polynomial order
#' @md
#' 
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com> 
#'
#' @examples
#'  set.seed(42)
#'  x <- seq(from=0, to=20, by=0.1)
#'  y <- (500 + 0.4 * (x-10)^3)
#'  noise <- y + rnorm(length(x), mean=10, sd=80) 
#'  
#'  p <- poly_trend(x, noise, degree = 3, ci = 0.95,
#'                  main="3rd degree polynomial")
#'  
#'  dev.new(height=6, width=12)
#'    layout(matrix(c(1,2), 1, 2, byrow = TRUE))
#'    p <- poly_trend(x, noise, degree = 3, 
#'                    main="3rd degree polynomial")
#'    p <- poly_trend(x, noise, degree = 6, 
#'                    main="6th degree polynomial")
#' 
#' cat("Confidence intervals for", "1 -", p$order, "polynomials",  "\n")
#'   p$prameterCI
#'
#' @export poly_trend
poly_trend <- function(x, y, degree, ci = 0.95, plot=TRUE, ...) {
  poly.f <- function(x, a, b, d) {
    (a * x^2) + (b * x) + d
  }
  fit <- stats::lm(y ~ stats::poly(x,degree))
    conf <- stats::confint(fit, level=ci)
      trend <- stats::predict(fit, data.frame(x=x),
                       interval='confidence',
                       level=ci)
  if (plot == TRUE) { 
    #dots <- as.list(match.call(expand.dots = TRUE)[-1])
	dots <- list(...)
	if(any(names(dots) %in% c("degree", "ci"))) {
	  dots <- dots[-which(names(dots) %in% c("degree", "ci"))]
	}  
      dots[["x"]] <- x
      dots[["y"]] <- y
	if(is.null(dots[["pch"]]) & "pch" %in% names(dots) == FALSE) dots[["pch"]] <- 20
	if(is.null(dots[["xlab"]]) & "xlab" %in% names(dots) == FALSE) dots[["xlab"]] <- "x"
	if(is.null(dots[["ylab"]]) & "ylab" %in% names(dots) == FALSE) dots[["ylab"]] <- "y"
	if(is.null(dots[["main"]]) & "main" %in% names(dots) == FALSE) dots[["main"]] <- "polynomial trend"  
	if(is.null(dots[["col"]]) & "col" %in% names(dots) == FALSE) dots[["col"]] <- "deepskyblue4"
	  do.call("plot", dots)
         graphics::lines(x, trend[,1], col='black', lty=1, lwd=2)
           graphics::lines(x, trend[,2], col='black', lty=3, lwd=2)
           graphics::lines(x, trend[,3], col='black', lty=3, lwd=2)
      graphics::legend("bottomright", c("trend", "upper-lower confidence"), 
	  				 lwd=c(2,2), lty=c(1,3), col=c('black', 'black'))
  }
      p <- list(trend=trend, model=fit, prameterCI=conf, order=degree)
    class(p) <- "poly.trend"
  return(p)
}

#' @title DAYMET Tile ID's
#' @description Returns a vector of DAYMET tile id's within a specified extent
#'
#' @param ... ignored
#'
#' @return 
#' Vector of DAYMET tile IDS or if sp = TRUE a sp class SpatialPolygonsDataFrame 
#' 
#' @note 
#' Function accepts sp, raster or extent class object or bounding coordinates. 
#' All input must be in the same projection as the tile index SpatialPolygonsDataFrame. 
#' The library includes the DAYMAT tile index "DAYMET_tiles" which can be add using 
#' data(), see examples. 
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'                                                                           
#' @export daymet.tiles
daymet.tiles <- function(...) {
 message("Because THREDDS has moved to a NetCDF Markup Language (NcML) query service, 
          this function is currently depreciated")
 message("You can access the North America NetCDF subset tool here:
         https://thredds.daac.ornl.gov/thredds/ncss/grid/daymet-v3-agg/na.ncml/dataset.html")	
  .Deprecated("daymet.tiles", package="spatialEco", 
    msg="Function is deprecated due to significant changes to the ORNL DAAC THREDDS server")	  
}

#' @title Fuzzy Sum
#' @description Calculates the fuzzy sum of a vector
#' 
#' @param x  Vector of values to apply fuzzy sum 
#' 
#' @details 
#' The fuzzy sum is an increasing linear combination of values. 
#' This can be used to sum probabilities or results of multiple 
#' density functions. 
#'
#' @return Value of fuzzy sum
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com> 
#'
#' @examples 
#' p = c(0.8,0.76,0.87)
#'   fuzzySum(p)
#'   sum(p)
#' 
#' p = c(0.3,0.2,0.1)
#'   fuzzySum(p)
#'   sum(p)  
#'
#' @export  
fuzzySum <- function(x) { return( 1 - prod( (1 - x) ) ) }

#' @title Surface Area Ratio
#' @description Calculates the Berry (2002) Surface Area Ratio based on slope
#' 
#' @param x      A terra SpatRaster object
#' @param s      cell resolution (default is NULL and not needed if projection 
#'               is in planar units)
#' @param scale  (TRUE/FALSE) Scale (row standardize) results 
#' 
#' @details  
#' SAR is calculated as: resolution^2 * cos( (degrees(slope) * (pi / 180)) )
#'
#' @return A terra SpatRaster class object of the Surface Area Ratio
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#' 
#' @references 
#' Berry, J.K. (2002). Use surface area for realistic calculations. Geoworld 15(9):20-1.
#'
#' @examples 
#'  library(terra)
#'  elev <- rast(system.file("extdata/elev.tif", package="spatialEco"))
#'  ( surface.ratio <- sar(elev) )
#'    plot(surface.ratio)
#'     
#' @export
sar <- function(x, s = NULL, scale = TRUE) {  
  if (!inherits(x, "SpatRaster")) 
	stop(deparse(substitute(x)), " must be a terra SpatRaster object")
  if (length(grep("longlat", terra::crs(x))) > 0 && is.null(s))
    stop("Projection is geographic, must define cell size argument in planar units")
    if( is.null(s) ) {
      s = terra::res(x)[1] ^2 
	} else {
      if(!is.numeric(s))
	    stop("s must be numeric")
    }	 
        saf <- function(x, cs = s) { cs * cos(x) } 
      slp <- terra::terrain(x, v='slope', unit='degrees') * (pi / 180)
	s <- terra::app(slp, fun=saf)
  if(!scale) {
    return( s )
  } else {  
    return( s / terra::global(s, "max", na.rm=TRUE)[,1] )
  }
}  

#' @title Winsorize transformation 
#' @description Removes extreme outliers using a winsorization transformation
#' 
#' @param x           A numeric vector
#' @param min.value   A fixed lower bounds, all values lower than this will be 
#'                    replaced by this value. The default is set to the 5th-quantile 
#'                    of x.
#' @param max.value   A fixed upper bounds, all values higher than this will be replaced 
#'                    by this value. The default is set to the 95th-quantile of x.
#' @param p           A numeric vector of 2 representing the probabilities used in the 
#'                    quantile function. 
#' @param na.rm       (FALSE/TRUE) should NAs be omitted? 
#' 
#' @details
#' Winsorization is the transformation of a distribution by limiting extreme values 
#' to reduce the effect of spurious outliers. This is done by shrinking outlying 
#' observations to the border of the main part of the distribution. 
#' 
#' @return 
#' A transformed vector the same length as x, unless na.rm is TRUE, then x is length 
#' minus number of NA's
#' 
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#' 
#' @references 
#' Dixon, W.J. (1960) Simplified Estimation from Censored Normal Samples. Annals of Mathematical 
#'   Statistics. 31(2):385-391 
#' 
#' @examples
#' set.seed(1234)     
#' x <- rnorm(100)     
#' x[1] <- x[1] * 10  
#' winsorize(x)       
#' 
#' plot(x, type="l", main="Winsorization transformation")
#'   lines(winsorize(x), col="red", lwd=2)
#'     legend("bottomright", legend=c("Original distribution","With outliers removed"),
#'	        lty=c(1,1), col=c("black","red"))
#' 
#' # Behavior with NA value(s)
#' x[4] <- NA
#' winsorize(x)             # returns x with original NA's 
#' winsorize(x, na.rm=TRUE) # removes NA's 
#' 
#' @export winsorize
winsorize <- function(x, min.value = NULL, max.value = NULL,  
                      p = c(0.05, 0.95), na.rm = FALSE) {			  
    if(na.rm == TRUE) { 
      x <- x[!is.na(x)]
    } else {
	  na.idx <- which(is.na(x) == TRUE)
	    if(length(na.idx) > 0) x <- x[-na.idx]
    }	 
	if (is.null(min.value) || is.null(max.value)) {
      xq <- stats::quantile(x = x, probs = p, na.rm = TRUE)
        if (is.null(min.value)) min.value <- xq[1]
          if (is.null(max.value)) max.value <- xq[2]
    }
      x[x < min.value] <- min.value
      x[x > max.value] <- max.value
	if(na.rm == FALSE) { x[na.idx] <- NA }
  return(x)
}

#' @title Raster local deviation from the global trend 
#' @description 
#' Calculates the local deviation from the raster, a specified global statistic 
#' or a polynomial trend of the raster.  
#' 
#' @param x         A terra SpatRaster object
#' @param type      The global statistic to represent the local deviation  
#'                  options are: "trend", "min", "max", "mean", "median"
#' @param degree    The polynomial degree if type is trend, default is 1st order. 
#' @param s         Size of matrix (focal window), not used with type="trend"
#' @param global    Use single global value for deviation or cell-level values 
#'                 (FALSE/TRUE). Argument is ignored for type="trend"
#' 
#' @details
#' The deviation from the trend is derived as [y-hat - y] where; y-hat is the 
#' Nth-order polynomial. Whereas the deviation from a global statistic is [y - y-hat] 
#' where; y-hat is the local (focal) statistic. The global = TRUE argument allows 
#' one to evaluate the local deviation from the global statistic [stat(x) - y-hat] 
#' where; stat(x) is the global value of the specified statistic and y-hat is the 
#' specified focal statistic.  
#'
#' @return 
#' A SpatRaster class object representing local deviation from the raster or the 
#' specified global statistic
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @references
#'  Magee, Lonnie (1998). Nonlocal Behavior in Polynomial Regressions. The American 
#'   Statistician. American Statistical Association. 52(1):20-22
#' 
#'  Fan, J. (1996). Local Polynomial Modelling and Its Applications: From linear  
#'    regression to nonlinear regression. Monographs on Statistics and Applied 
#'    Probability. Chapman and  Hall/CRC. ISBN 0-412-98321-4
#' 
#' @examples 
#' library(terra)
#' elev <- rast(system.file("extdata/elev.tif", package="spatialEco"))
#' 
#' # local deviation from first-order trend, global mean and raw value
#' r.dev.trend <- raster.deviation(elev, type="trend", degree=1) 
#' r.dev.mean <- raster.deviation(elev, type="mean", s=5)
#' r.gdev.mean <- raster.deviation(elev, type="mean", s=5, global=TRUE)
#' 
#' opar <- par(no.readonly=TRUE)
#' par(mfrow=c(2,2))
#'   plot(elev, main="original")
#'   plot(r.dev.trend, main="dev from trend")
#'   plot(r.dev.mean, main="dev of mean from raw values")
#'   plot(r.gdev.mean, main="local dev from global mean")
#' par(opar)
#'
#' @export raster.deviation
raster.deviation <- function(x, type = c("trend", "min", "max", "mean", "median"), 
                             s = 3, degree = 1, global = FALSE) {  
  if (!inherits(x, "SpatRaster")) 
	stop(deparse(substitute(x)), " must be a terra SpatRaster object")
  if( type[1] != "trend") {
     if( length(s) == 1) s = c(s[1],s[1])
       m <- matrix(1, nrow=s, ncol=s)
	   if( global == FALSE) {   
	     return( x - terra::focal(x, w = m, fun =   match.fun(type[1]), 
		                          na.rm = TRUE) )
	   } else {
	     return( terra::global(x, eval(parse(text=type[1])), na.rm = TRUE)[,1] - 
	             terra::focal(x, w = m, fun = match.fun(type[1]), 
			                  na.rm = TRUE) )
	  }  
    }	
  if( type[1] == "trend" ) {
    r <- data.frame(y=stats::na.omit(as.vector(x)),
	                terra::crds(x, df=TRUE, 
					na.rm=TRUE))
      names(r) <- c("y", "xcoord", "ycoord")	
     # 1st and 2nd order polynomial equations
      if( degree == 1 ) {   
        p.1 <- stats::as.formula(y ~ xcoord + ycoord) 
          poly.mdl <- stats::lm(p.1, data=r)
      } else if(degree > 1 ) {	
        #f.2 <- stats::as.formula(y ~ xcoord+ycoord+I(xcoord*xcoord)+I(ycoord*ycoord)+I(xcoord*ycoord))
        #poly.mdl <- stats::lm(f.2, data=r)
	    poly.mdl <- stats::lm(y ~ stats::poly(xcoord, degree) + 
		                      poly(ycoord, degree), data=r)
	  }	
      r <- data.frame(r, trend = stats::predict(poly.mdl, newdata=r)) 
      rsf <- sf::st_as_sf(r, coords = c("xcoord", "ycoord"), 
	                      agr = "constant")
      rstat <- terra::rasterize(terra::vect(rsf), x, field="trend")
      message("polynomial confidence intervals")	  
	    stats::confint(poly.mdl, level=0.95)
     return( rstat - x ) 
  } 
}  

#' @title Senescence weighted Vegetation Index (swvi) 
#' @description Modified Soil-adjusted Vegetation Index (MSAVI) or Modified Triangular 
#'              Vegetation Index 2 (MTVI) weighted by the Normalized difference senescent 
#'              vegetation index (NDSVI)
#'
#' @param red            SpatRaster, Red band (0.636 - 0.673mm), landsat 5&7 band 3, OLI 
#'                       (landsat 8) band 4
#' @param nir            SpatRaster, Near infrared band (0.851 - 0.879mm) landsat 5&7 band 4, 
#'                       OLI (landsat 8) band 5 
#' @param swir           SpatRaster, short-wave infrared band 1 (1.566 - 1.651mm), landsat 5&7 
#'                       band 5, OLI (landsat 8) band 6
#' @param mtvi           (FALSE | TRUE) Use Modified Triangular Vegetation Index 2  
#'                       instead of MSAVI
#' @param green          Green band if MTVI = TRUE
#' @param senescence     The critical value, in NDSVI, representing senescent vegetation 
#' @param threshold      Threshold value for defining NA based on < p
#' @param weight.factor  Apply partial weights (w * weight.factor) to the NDSVI weights 
#' @param ...            Additional arguments passed to terra::lapp function
#'
#' @details
#' The intent of this index is to correct the MSAVI or MTVI index for bias associated 
#' with senescent vegetation. This is done by: 
#'   1 deriving the NDSVI
#'   2 applying a threshold to limit NDSVI to values associated with senescent vegetation 
#'   3 converting the index to inverted weights (-1*(NDSVI/sum(NDSVI))) 
#'   4 applying weights to MSAVI or MTVI 
#' 
#' The MSAVI formula follows the modification proposed by Qi et al. (1994), 
#' often referred to as MSAVI2. MSAVI index reduces soil noise and increases 
#' the dynamic range of the vegetation signal. The implemented modified version 
#' (MSAVI2) is based on an inductive method that does not use a constant L value, in 
#' separating soil effects, an highlights healthy vegetation. The MTVI(2) index follows 
#' Haboudane et al., (2004) and represents the area of a hypothetical triangle in spectral 
#' space that connects (1) green peak reflectance, (2) minimum chlorophyll absorption, and 
#' (3) the NIR shoulder. When chlorophyll absorption causes a decrease of red reflectance, 
#' and leaf tissue abundance causes an increase in NIR reflectance, the total area of the 
#' triangle increases. It is good for estimating green LAI, but its sensitivity to chlorophyll 
#' increases with an increase in canopy density. The modified version of the index accounts 
#' for the background signature of soils while preserving sensitivity to LAI  and resistance 
#' to the influence of chlorophyll. 
#'
#' The Normalized difference senescent vegetation index (NDSVI) follows methods from 
#' Qi et a., (2000). The senescence is used to threshold the NDSVI. Values less then this value 
#' will be NA. The threshold argument is used to apply a threshold to MSAVI. The default is NULL 
#' but if specified all values (MSAVI <= threshold) will be NA. Applying a weight.factor can be 
#' used to change the influence of the weights on MSAVI. 
#' @md
#' 
#' @return A terra SpatRaster class object of the weighted MSAVI metric 
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com> 
#'
#' @references 
#' Haboudane, D., et al. (2004) Hyperspectral Vegetation Indices and Novel Algorithms 
#'   for Predicting Green LAI of Crop Canopies: Modeling and Validation in the Context 
#'   of Precision Agriculture. Remote Sensing of Environment 90:337-352.
#'  
#' Qi J., Chehbouni A., Huete A.R., Kerr Y.H., (1994). Modified Soil Adjusted Vegetation 
#'   Index (MSAVI). Remote Sens Environ 48:119-126.
#' 
#' Qi J., Kerr Y., Chehbouni A., (1994). External factor consideration in vegetation 
#'   index development. Proc. of Physical Measurements and Signatures in Remote Sensing, 
#'   ISPRS, 723-730.
#'  
#' Qi, J., Marsett, R., Moran, M.S., Goodrich, D.C., Heilman, P., Kerr, Y.H., Dedieu, 
#'   G., Chehbouni, A., Zhang, X.X. (2000). Spatial and temporal dynamics of vegetation
#    in the San Pedro River basin area. Agricultural and Forest Meteorology. 105:55-68. 
#'
#' @examples
#' \donttest{
#' library(terra)
#' lsat <- rast(system.file("/extdata/Landsat_TM5.tif", package="spatialEco"))
#'  
#' # Using Modified Soil-adjusted Vegetation Index (MSAVI)
#' ( wmsavi <- swvi(red = lsat[[3]], nir = lsat[[4]], swir = lsat[[5]]) )
#'     plotRGB(lsat, r=6,g=5,b=2, scale=1, stretch="lin")
#'       plot(wmsavi, legend=FALSE, col=rev(terrain.colors(100, alpha=0.35)), add=TRUE )
#' 
#' # Using Modified Triangular Vegetation Index 2 (MTVI) 
#' ( wmtvi <- swvi(red = lsat[[3]], nir = lsat[[4]], swir = lsat[[5]],
#'                           green = lsat[[3]], mtvi = TRUE) )
#'   plotRGB(lsat, r=6,g=5,b=2, scale=1, stretch="lin")
#'     plot(wmtvi, legend=FALSE, col=rev(terrain.colors(100, alpha=0.35)), add=TRUE )
#' }
#' 
#' @export swvi
swvi <- function(red, nir, swir, green = NULL, mtvi = FALSE, 
                 senescence = 0, threshold = NULL, 
                 weight.factor = NULL, ...) {
  if(missing(red) | missing(nir) | missing(swir))
    stop("Must specify red, nir and swir1 bands")
  if(mtvi) { 
    if(is.null(green)) 
      stop("Must specify green band") 
    if (!inherits(green, "SpatRaster")) 
	stop("green band must be a terra SpatRaster object")	  
  } 	
  if (!inherits(red, "SpatRaster")) 
	stop("red band must be a terra SpatRaster object")
  if (!inherits(nir, "SpatRaster")) 
	stop("NIR band must be a terra SpatRaster object")
  if (!inherits(swir, "SpatRaster")) 
	stop("SWIR band must be a terra SpatRaster object")

  f.msavi <- function(nir, red) {
    return( (2 * nir + 1 - sqrt( (2 * nir + 1)^2 - 8 * (nir - red) )) / 2 )
  }
  f.mtvi <- function(nir, red, green) {
    return( 1.5 * (1.2*(nir - green) - 2.5*(red - green)) /
	        sqrt( (2*nir+1)^2 - (6 * nir - 5 * sqrt(red) - 0.5) ) )
  }
  ndsvi <- function(nir, swir) { return( (swir - nir) / (swir + nir) ) }  
    wf <- function(x, y) {
      if(!is.null(weight.factor)) { 
	    y <- y / weight.factor 
	  }
      return( x * y )
    }
  w <- ndsvi(nir, swir)
    w[w < senescence] <- NA
    # w <- (w / terra::global(w, "sum", na.rm=TRUE)[,1]) * -1
  if(mtvi == TRUE) {
    d <- c(f.mtvi(nir, red, green), w)  
    return( terra::lapp(d, fun = wf, ...) )
  } else {
    d <- c(f.msavi(nir, red), w)
	return( terra::lapp(d, fun=wf, ...) )
  }  
}

#' @title Outliers 
#' @description Identify outliers using modified Z-score
#'
#' @param x  A numeric vector
#' @param s  Scaling factor for mad statistic 
#'
#' @return value for the modified Z-score
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @references
#' Iglewicz, B. & D.C. Hoaglin (1993) How to Detect and Handle Outliers, 
#'   American Society for Quality Control, Milwaukee, WI.
#'
#' @examples 
#'  # Create data with 3 outliers
#'     x <- seq(0.1, 5, length=100) 
#'     x[98:100] <- c(100, 55, 250)
#'  
#'  # Calculate Z score
#'      Z <- outliers(x) 
#'  
#'  # Show number of extreme outliers using Z-score
#'      length(Z[Z > 9.9])
#'  
#'  # Remove extreme outliers 
#'      x <- x[-which(Z > 9.9)]
#'
#' @export  
outliers <- function(x, s = 1.4826) {
 e <- (length(x) - 1) / sqrt(length(x)) 
 mad <- function (x, center=stats::median(x), constant=s,
                  low=FALSE, high=FALSE) {
  n <- length(x)
  constant * if ((low || high) && n%%2 == 0) {
      if (low && high) 
          stop("'low' and 'high' cannot be both TRUE")
      n2 <- n%/%2 + as.integer(high)
      sort(abs(x - center), partial = n2)[n2]
    }
   else stats::median(abs(x - center))
  }                         
   return( ( (0.6745 * (x - stats::median(x))) / mad(x) ) )
}

#' @title Topographic distance
#' @description 
#' Calculates topographic corrected distance for a line object
#'
#' @param x     sf LINESTRING object
#' @param r     terra SpatRaster class elevation raster
#' @param echo  (FALSE/TRUE) print progress to screen
#'
#' @details
#' This function corrects straight-line (euclidean) distances for topographic-slope effect. 
#'
#' @return 
#' Vector of corrected topographic distances same length as nrow(x)
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @examples 
#'  library(sf)
#'  library(terra)
#'  
#'  # create example data
#'  elev <- rast(system.file("extdata/elev.tif", package="spatialEco"))
#'    names(elev) <- "elev"
#' 
#'  lns <- lapply(1:5, function(i) {
#'    p <- st_combine(st_as_sf(spatSample(elev, size=2, as.points=TRUE)))
#'    st_as_sf(st_cast(p, "LINESTRING")) }) 
#'  lns <- do.call(rbind, lns) 
#'   
#'   plot(elev)
#'     plot(st_geometry(lns), add=TRUE)
#'       
#'  # Calculate topographical distance  
#'  ( tdist <- topo.distance(lns, elev) )
#'  ( lgt <- as.numeric(st_length(lns)) ) 
#'  
#'  # Increase in corrected distance
#'  tdist - lgt
#'  
#'  # Percent increase in corrected distance
#'  ((tdist - lgt) / lgt) * 100
#'
#' @export topo.distance
topo.distance <- function(x, r, echo = FALSE) {
  att <- attr(class(x), "package")
  if( if(length(att) > 0) { att == "sp" } else { FALSE } )
    x <- sf::st_as_sf(x)
  if(sf::st_geometry_type(x, by_geometry = FALSE) != "LINESTRING")
    stop("x must be a LINESTRING object")
  if (inherits(r, "RasterLayer"))
    r <- terra::rast(r)	
  if(!inherits(r, "SpatRaster"))	
    stop("r must be a terra or raster object")	
  step.dist <- function(x) {
    d <- vector()
      for(i in 1:(nrow(x)-1)){
        d <- append(d, as.numeric(sf::st_distance(x[i,], x[i+1,])))
      }
    return( d <- append(d, NA) )	
  }
  line.dist <- vector()
    for(i in 1:nrow(x)) {
	  if(echo) message("Calculating corrected distance for: ", i, " of ", nrow(x), "\n")
	  pts <- sf::st_cast(sf::st_line_sample(x[i,], 
	           density=1/terra::res(r)[1]), "POINT")		   
	    pts <- sf::st_as_sf(pts)
          pts$elev <- terra::extract(r, terra::vect(pts))[,2]
		  d <- c(rep(as.numeric(sf::st_distance(pts[1,],pts[2,])), 
		         nrow(pts)-1), NA)
		  z <- pts$elev
          n <- length(z) - 1
      rise <- abs( z[2:(n+1)] - z[1:n] )
	  d <- sum( d[!is.na(d)] + ( d[!is.na(d)] * 
	             (rise / d[!is.na(d)]) ), na.rm = TRUE)
      sl.length <- as.numeric(sf::st_length(x[i,]))
      if(sl.length > d) { 
	    line.dist[i] <- sl.length 
	  } else { 
	    line.dist[i] <- d 
	  }
    }
  return(line.dist)
}

#' @title Terrain Ruggedness Index
#' @description Implementation of the Riley et al (1999) Terrain Ruggedness Index
#'
#' @param r      A terra SpatRaster class object
#' @param s      Scale of window. Must be odd number, can represent 2 
#'               dimensions (eg., s=c(3,5) would represent a 3 x 5 window)
#' @param exact  Calculate (TRUE/FALSE) the exact TRI or an algebraic 
#'               approximation. 
#' @param ...    Additional arguments passed to terra::focal or terra::app
#'
#' @details
#' The algebraic approximation is considerably faster. However, because 
#' inclusion of the center cell, the larger the scale the larger the divergence 
#' of the minimum value. Resuls are driven by local variations so, fixed thresholds
#' are not very reliable. However there are some reccomended breaks (eg., Riley et al., 1999). 
#' 
#' Riley et al., (1999) ranges for classifying Topographic Ruggedness Index:
#'   * 0-80 - level terrain surface.
#'   * 81-116 - nearly level surface.
#'   * 117-161 - slightly rugged surface.
#'   * 162-239 - intermediately rugged surface.
#'   * 240-497 - moderately rugged surface.
#'   * 498-958 - highly rugged surface.
#'   * gt 959 - extremely rugged surface. 
#' @md 
#'
#' @return A terra SpatRaster class object of the TRI
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'  
#' @references 
#' Riley, S.J., S.D. DeGloria and R. Elliot (1999) A terrain 
#'   ruggedness index that quantifies topographic heterogeneity, 
#'   Intermountain Journal of Sciences 5(1-4):23-27.
#'
#' @examples 
#' \donttest{
#' library(terra)
#' elev <- rast(system.file("extdata/elev.tif", package="spatialEco"))
#'   ( tri.ext <- tri(elev) )
#'   ( tri.app <- tri(elev, exact = FALSE) )
#'   plot(c(tri.ext, tri.app))
#' }
#'
#' @export
tri <- function(r, s = 3, exact = TRUE, ...) {
  if (!inherits(r, "SpatRaster")) 
	stop(deparse(substitute(x)), " must be a terra SpatRaster object")
  if(length(s) > 2) stop( "Specified window exceeds 2 dimensions")   
    if(length(s) == 1) s = rep(s,2)
  r.sqrt <- function(x) {
    x <- x[x >= 0]
    if(length(x) >= 1) { 
      return(sqrt(x)) 
    } else{
      return(NA)  
    }
  }	
  tri.calc <- function(x) {
    xc <- x[ceiling(length(x)/2)]
    x <- x[-ceiling(length(x)/2)]
    x <- x[!is.na(x)]
      if(!is.na(xc) & length(x) > 0) {
        x.dev <- vector()
        for(i in 1:length(x)) { x.dev <- append(x.dev, (xc - x[i])^2) }
  	  return( sqrt(sum(x.dev)) )
      } else {
      return( NA )  
    }
  }	  
  if(exact == TRUE) {  
    return( terra::focal(r, w=matrix(1,s[1],s[2]), fun=tri.calc, ...) )
  } else {
    sx <- terra::focal(r, w = matrix(1,s[1],s[2]), sum)
      e2 <- r * r
      st <- terra::focal(e2, w = matrix(1,s[1],s[2]), sum)
    r2 <- st + (s[1]*s[2]) * e2 - 2 * r * sx
  return( terra::app(r2, fun= r.sqrt, ...) )
  }
}

#' @name ants
#' @docType data
#'
#' @title Ant Biodiversity Data
#'
#' @description Roth et al., (1994) Costa Rican ant diversity data
#'
#' @format A data.frame with 82 rows (species) and 5 columns (covertypes):
#' \describe{
#'   \item{species}{Ant species (family)}
#'   \item{Primary.Forest}{Primary forest type}
#'   \item{Abandoned.cacao.plantations}{Abandoned cacao plantations type}
#'   \item{Productive.cacao.plantations}{Active cacao plantations type}
#'   \item{Banana.plantations}{Active banana plantations type}
#' }
#'
#' @references Roth, D. S., I. Perfecto, and B. Rathcke (1994) The effects of management systems on 
#'             ground-foraging ant diversity in Costa Rica. Ecological Applications 4(3):423-436.
#'
NULL

#' @title Raster moments
#' @description Calculates focal statistical moments of a raster  
#' 
#' @param x         A terra SpatRaster object
#' @param type      The global statistic to represent the local deviation  
#'                  options are: "min", "min", "mean", "median", "var, "sd", 
#'                  "mad", "kurt", "skew", "quantile"  
#' @param s         Size of matrix (focal window), can be single value or two 
#'                  values defining the [x,y] dimensions of the focal matrix
#' @param p         if type="quantile", the returned percentile. 
#' @param ...       Additional arguments passed to terra::focal 
#' 
#' @details
#' This is a simple wrapper for the terra focal function, returning local statistical moments 
#'
#' @return A terra SpatRaster object representing the local distributional moment
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @examples 
#' \donttest{
#' library(terra)
#' r <- rast(nrows=500, ncols=500, xmin=571823, xmax=616763, 
#'             ymin=4423540, ymax=4453690)
#'   crs(r) <- "epsg:9001"
#' r[] <- runif(ncell(r), 1000, 2500)
#' 
#' # Calculate 10th percentile for 3x3 window
#' r.p10 <- raster.moments(r, type="quantile", p=0.10) 
#' }
#'
#' @export raster.moments
raster.moments <- function(x, type = "mean", s = 3, p = 0.75, ...) {  
   if (!inherits(x, "SpatRaster")) 
	  stop(deparse(substitute(x)), " must be a terra SpatRaster object")
  if( length(s) == 1) s = c(s[1],s[1])
    m <- matrix(1, nrow=s, ncol=s)
	if(type == "quantile") {
	  pct <- function(x, q = p) { stats::quantile(x, probs=q, na.rm=TRUE) } 
	  return( terra::focal(x, w = m, fun = pct, ...) )
	} else {
	  return( terra::focal(x, w = m, fun = match.fun(type), na.rm = TRUE, ...) )
    }	
}  

#' @title Parse bits
#' @description Returns specified bit value based on integer input
#' 
#' @param x      Integer value
#' @param bit    A single or vector of bits to return             
#' @param depth  The depth (length) of the bit range, default is 8
#' @param order  c("reverse", "none") sort order for the bits
#'
#' @description
#' Data such as MODIS the QC band are stored in bits. This function returns the 
#' value(s) for specified bit. For example, the MODIS QC flag are bits 0-1 with 
#' the bit value 00 representing the "LST produced, good quality" flag. When 
#' exported from HDF the QC bands are often in an 8 bit integer range (0-255). 
#' With this function you can parse the values for each bit to assign the 
#' flag values. 
#'
#' @return a vector or data.frame of parsed interger value(s) associated with input bit
#' 
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com> 
#'
#' @examples 
#' # Return value for bit 5 for integer value 100
#' parse.bits(100, 5)
#'  
#' # Return value(s) for bits 0 and 1 for integer value 100
#' parse.bits(100, c(0,1))
#'
#' # Return value(s) for bits 0 and 1 for integer values 0-255
#' for(i in 0:255) { print(parse.bits(i, c(0,1))) }
#'  
#' \donttest{
#' #### Applied Example using Harmonized Landsat Sentinel-2 QC 
#'
#' # Create dummy data and qc band
#' library(terra)
#' r <- rast(nrow=100, ncol=100)
#'   r[] <- round(runif(ncell(r), 0,1)) 
#' qc <- rast(nrow=100, ncol=100)
#'   qc[] <- round(runif(ncell(qc), 64,234)) 
#' 
#' # Calculate bit values from QC table
#' ( qc_bits <- data.frame(int=0:255, 
#' 	cloud = unlist(lapply(0:255, FUN=parse.bits, bit=1)),
#' 	shadow = unlist(lapply(0:255, FUN=parse.bits, bit=3)),
#' 	acloud = unlist(lapply(0:255, FUN=parse.bits, bit=2)),
#' 	cirrus = unlist(lapply(0:255, FUN=parse.bits, bit=0)),
#' 	aerosol = unlist(lapply(0:255, FUN=parse.bits, bit=c(7,6)))) )
#' 		
#' # Query the results to create a vector of integer values indicating what to mask 
#' #  cloud is bit 1 and shadow bit 3	
#' m <- sort(unique(qc_bits[c(which(qc_bits$cloud == 1),
#'                            which(qc_bits$shadow == 1)
#' 						   ),]$int))
#' 
#' # Apply queried integer values to mask image with QA band
#' qc[qc %in% m] <- NA
#' r <- mask(r, qc)
#' }
#'
#' @export parse.bits
parse.bits <- function(x, bit, depth=8, order = c("reverse", "none") ) {
  position <- depth - bit
  if(order[1] == "reverse") sort.order = rev(1:depth) else sort.order = 1:depth  
    b <- as.integer(intToBits(x)[1:depth])[depth:1][position]
    if(length(bit) > 1) b <- paste0(b, collapse=" ")
  return(b)
}

#' @title Background sample
#'
#' @description Creates a point sample that can be used as 
#'              a NULL for SDM's and other modeling approaches. 
#'
#' @param x      A sf class polygon defining sample region    
#' @param p      Size of sample
#' @param known  An sf POINT class of known locations with same CSR as x
#' @param d      Threshold distance for known proximity 
#' @param type   Type of sample c("systematic", "random", "hexagon", "nonaligned")
#'
#' @details 
#' This function creates a background point sample based on an extent 
#' or polygon sampling region. The known argument can be used with d 
#' to remove sample points based on distance-based proximity to existing  
#' locations (eg., known species locations). The size (p) of the resulting 
#' sample will be dependent on the known locations and the influence of 
#' the distance threshold (d). As such, if the know and d arguments are
#' provided the exact value provided in p will not be returned. 
#'
#' @return A sf POINT feature class or data.frame with x,y coordinates
#' 
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com> 
#'
#' @examples
#' library(sf)
#' 
#' # define study area
#' sa <- suppressWarnings(st_cast(st_read(
#'         system.file("shape/nc.shp", 
#'         package="sf")), "POLYGON"))
#'   sa <- sa[10,]
#' 
#' # create "known" locations  
#' locs <- st_sample(sa, 50)
#'   st_crs(locs) <- st_crs(sa)
#' 
#' # systematic sample using extent polygon
#' e <- st_as_sf(st_as_sfc(st_bbox(sa)))
#'   st_crs(e) <- st_crs(sa)
#' s <- background(e, p=1000, known=locs, d=1000)
#'   plot(st_geometry(s), pch=20)
#'     plot(st_geometry(locs), pch=20, col="red", add=TRUE)
#' 
#' # systematic sample using irregular polygon
#' s <- background(sa, p=1000, known=locs, d=1000)
#'   plot(st_geometry(sa)) 
#'     plot(st_geometry(s), pch=20, add=TRUE)
#'       plot(st_geometry(locs), pch=20, col="red", add=TRUE)
#' 
#' # random sample using irregular polygon
#' s <- background(sa, p=500, known=locs, 
#'                 d=1000, type="random")
#'   plot(st_geometry(sa)) 
#'     plot(st_geometry(s), pch=20, add=TRUE)
#'       plot(st_geometry(locs), pch=20, col="red", add=TRUE)
#' 
#' @export
background <- function(x, p=1000, known=NULL, d=NULL, 
                type=c("regular", "random", "hexagon", "nonaligned")) {
    if(is.null(d)) 
      stop("distance (d) must be defined")  
    if(missing(x))
      stop("extent or x argument must be defined")	 
	if(inherits(x, c("sf", "sfc"))) {   
	  if(as.character(unique(sf::st_geometry_type(x))) != "POLYGON")
        stop(deparse(substitute(x)), " x must be an sf POLYGON object")	
    }
  if(length(find.package("lwgeom", quiet = TRUE)) == 0)
      stop("please install lwgeom package before running this function")
  if(!is.null(known)){
    if(!inherits(known, c("sf", "sfc")))  
      stop(deparse(substitute(known)), " must be an sf POINT object")
    if(as.character(unique(sf::st_geometry_type(known))) != "POINT")
      stop(deparse(substitute(known)), " must be an sf POINT object")
	if(sf::st_crs(x) != sf::st_crs(known))
      stop("CSR of known and x do not match")	  
  }
  s <- sf::st_as_sf(sf::st_sample(x=x, size=p, type=type[1]))
    if(!is.null(known)) {
      idx <- unique(unlist(sf::st_is_within_distance(known, s,  
                     dist = d,sparse = TRUE)))
        s <- s[-idx,] 
    }
    if(!is.na(sf::st_crs(x))) {
      sf::st_crs(s) <- sf::st_crs(x) 
    }	  
  return(s)
}

#' @title Raster change between two nominal rasters
#' @description 
#' Compares two categorical rasters with a variety of statistical options
#'       
#' @param x            A terra SpatRaster 
#' @param y            A terra SpatRaster for comparison to x   
#' @param s            Integer or matrix for defining Kernel, 
#'                     must be odd but not necessarily square
#' @param stat         Statistic to use in comparison, please see details for 
#'                     options. 
#' @param ...          Additional arguments passed to terra::focalPairs
#'
#' @details
#' This function provides a various statistics for comparing two classified maps. 
#' Valid options are: 
#'   * kappa - Cohen's Kappa 
#'   * t.test - Two-tailed paired t-test 
#'   * cor - Persons Correlation 
#'   * entropy - Delta entropy 
#'   * cross-entropy - Cross-entropy loss function 
#'   * divergence - Kullback-Leibler divergence (relative entropy) 
#'
#' Kappa and t-test values < 0 are reported as 0. For a weighted kappa, a matrix must 
#' be provided that correspond to the pairwise weights for all values in both rasters. 
#' Delta entropy is derived by calculating Shannon's on each focal window then 
#' differencing  them (e(x) - e(y)). The s argument can be a single scalar, defining
#' a symmetrical kernel, two scalers defining the dimensions of the kernel eg., c(3,5)
#' or a matrix defining the kernel say, resulting from terra::focalMat   
#'
#' @return 
#' A terra SpatRaster layer containing one of the following layers:
#'   * kappa - Kappa or Weighted Kappa statistic (if stat = "kappa")
#'   * correlation - Paired t.test statistic  (if stat = "cor")
#'   * entropy - Local entropy  (if stat = "entropy")
#'   * divergence - Kullback-Leibler divergence (if stat = "divergence")
#'   * cross.entropy - Local Cross-entropy (if stat = "cross.entropy")
#'   * t.test - Paired t.test statistic  (if stat = "t.test")
#'   * p.value - p-value of the paired t.test statistic (if stat = "t.test") 
#' 
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @references
#' Cohen, J. (1960). A coefficient of agreement for nominal scales. Educational  
#'   and Psychological Measurement, 20:37-46 
#' 
#' McHugh M.L. (2012) Interrater reliability: the kappa statistic. 
#'   Biochemia medica, 22(3):276–282. 
#' 
#'  Kullback, S., R.A. Leibler (1951). On information and sufficiency. Annals of 
#'    Mathematical Statistics. 22(1):79–86
#' 
#' @examples
#' \donttest{
#'  library(sf) 
#'  library(terra) 
#'   
#'  e <- ext(179407.8, 181087.9, 331134.4, 332332.1)
#'  r1 <- rast(e, resolution=20)
#'    r1[] <- sample(1:5, ncell(r1), replace=TRUE)
#'  r2 <- rast(e, resolution=20)
#'    r2[] <- sample(1:5, ncell(r2), replace=TRUE)
#'  	  
#'  d = 5 # kernel    
#'  ( r.kappa <- raster.change(r1, r2, s = d) )   
#'  ( r.ttest <- raster.change(r1, r2, s = d, stat="t.test") )
#'  ( r.ent <- raster.change(r1, r2, s = d, stat="entropy") )   
#'  ( r.cor <- raster.change(r1, r2, s = d, stat="cor") )
#'  ( r.ce <- raster.change(r1, r2, s = d, stat = "cross-entropy") )
#'  ( r.kl <- raster.change(r1, r2, s = d, stat = "divergence") )	
#'        
#'    opar <- par(no.readonly=TRUE)
#'    par(mfrow=c(3,2))
#'      plot(r.kappa, main="Kappa")
#'      plot(r.ttest[[1]], main="Paired t-test")
#'      plot(r.ent, main="Delta Entropy")
#'      plot(r.cor, main="Rank Correlation")
#'      plot(r.kl, main="Kullback-Leibler")
#'      plot(r.ce, main="cross-entropy")
#'    par(opar) 
#' }
#' 
#' @md
#' @export raster.change
raster.change <- function(x, y, s = 3, stat = c("kappa", "t.test",    
                          "cor", "entropy", "cross-entropy", 
					      "divergence"), ...) {
  stat = stat[1]
  if (!inherits(x, "SpatRaster"))
    stop(deparse(substitute(x)), " Must be a terra SpatRaster object")
  if (!inherits(y, "SpatRaster"))
    stop(deparse(substitute(y)), " Must be a terra SpatRaster object")
  if(!any((dim(x)[1:2] == dim(y)[1:2]))) 
    stop("Rasters dimensions do not match") 
  if(!terra::ext(x) ==  terra::ext(y)) 
    stop("Rasters extents do not match") 
  if(stat == "wkappa") 
    stop("Sorry; weighted kappa is not yet implemented")  
  if(!any( (stat %in% c("kappa", "wkappa", "t.test", "cor",  
            "entropy", "cross-entropy", "divergence"))) )
    stop("Not a valid option for evaluation statistic")
  if(!inherits(s, "matrix")) {
    if(any(round(s) %% 2 == 0))
      stop("Dimensions for kernel must be an odd number")
    if(length(s) < 2) s = c(s,s)
    s <- matrix(1, s[1], s[2])
  } else {
    if(any(dim(s) %% 2 == 0))
      stop("Dimensions for kernel must be an odd number")
  }	  
  cohens <- function(x, y) {
    x <- factor(x)
    y <- factor(y)
      lvl <- unique(c(levels(x), levels(y)))
        x <- factor(x, levels = lvl)
          y <- factor(y, levels = lvl)
            x <- table(x, y)
            d <- diag(x)
          n <- sum(x)
        nc <- ncol(x)
      colFreqs <- colSums(x) / n
    rowFreqs <- rowSums(x) / n
    return( (sum(d)/n - crossprod(colFreqs, rowFreqs)[1])/ 
            (1 - crossprod(colFreqs, rowFreqs)[1]) )
  }

  divergence <- function(x, y, type = c("Kullback-Leibler", "cross-entropy")) {
    type = type[1]
    if(!is.vector(x) | !is.vector(y))
      stop("x and y must be numeric of character vectors")
    if(any(type %in% c("Kullback-Leibler", "cross-entropy")==FALSE))
      stop("Not a valid option for statistic type")
    q <- table(x) / sum(table(x))  # observed or approximated
    p <- table(y) / sum(table(y))  # estimated or probability 
    classes <- intersect(names(q), names(p))
      p <- p[which(names(p) %in% classes)]
      q <- q[which(names(q) %in% classes)]
    if(type == "cross-entropy") {
      return( -sum( q, log(p) ) ) 
    } else if(type == "Kullback-Leibler") {
      return( sum( p * log(p / q) ) )
    }
  }  
	
  ent <- function(x,y) {
    return((-sum(prop.table(x) * log(prop.table(x)))) -  
   (-sum(prop.table(y) * log(prop.table(y)))))
  }   
  
  ttest <- function(x,y) {
    tt <- tryCatch(stats::t.test(x,y), 
	     error = function(e) {})
	if(is.null(tt)) {
      tt <- c(NA,NA) 
	} else {	
      tt <- c(tt$statistic, tt$p.value)
    }    
    return(tt)
  }
  
  if(stat == "kappa") {	   
    r <- terra::focalPairs(c(x,y), w=s, function(x, y) { cohens(x, y) }, ...)
  }						 
  if(stat == "divergence") {
    r <- terra::focalPairs(c(x,y), w=s, function(x, y) { 
                         divergence(x, y, type = "Kullback-Leibler") }, ...)  
   }
  if(stat == "cross-entropy") {
    r <- terra::focalPairs(c(x,y), w=s, function(x, y) { divergence(x, y,
  				         type = "cross-entropy") }, ...)  
   }   
  if(stat == "cor") {
    r <- terra::focalPairs(c(x,y), w=s, function(x, y){ stats::cor(x, y, 
  					     method = "spearman") }, ...) 
  }				    
  if(stat == "entropy") {
    r <- terra::focalPairs(c(x,y), w=s, function(x, y) { ent(x, y) }, ...)
  }  
  if(stat == "t.test") {
    r <- terra::focalPairs(c(x,y), w=s, function(x, y){ ttest(x,y) }, ...)
      names(r) <- c("t", "p.value")					
  }
return(r)  
}
  

#' @title Spatial K nearest neighbor
#' @description 
#' Find K nearest neighbors for two spatial objects
#'
#' @param y          Spatial sf object or coordinates matrix 
#' @param x          Spatial points or polygons object or coordinates matrix
#' @param k          Number of neighbors
#' @param d          Optional search radius
#' @param ids        Optional column of ID's in x
#' @param weights.y  A vector or matrix representing covariates of y
#' @param weights.x  A vector or matrix representing covariates of x
#' @param indexes    (FALSE/TRUE) Return row indexes of x neighbors
#'
#' @details
#' Finds nearest neighbor in x based on y and returns rownames, index and distance,
#' If ids is NULL, rownames of x are returned. If coordinate matrix provided, 
#' columns need to be ordered [X,Y]. If a radius for d is specified than a maximum 
#' search radius is imposed. If no neighbor is found, a neighbor is not returned  
#'
#' You can specify weights to act as covariates for x and y. The vectors or matrices
#' must match row dimensions with x and y as well as columns matching between weights.
#' In other words, the covariates must match and be numeric.   
#'
#' @return 
#' A data.frame with row indexes (optional), rownames, ids (optional) and 
#' distance of k
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com> 
#'
#' @examples 
#' \donttest{
#' if(require(sp, quietly = TRUE)) {
#' library(sf)
#'   data(meuse, package = "sp")
#'   meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, 
#'                     agr = "constant")
#'  
#' # create reference and target obs
#' idx <- sample(1:nrow(meuse), 10) 
#'   pts <- meuse[idx,]
#'     meuse <- meuse[-idx,]
#'     meuse$IDS <- 1:nrow(meuse)
#'  
#' # Find 2 neighbors in meuse
#' ( nn <- knn(pts, meuse, k=2, ids = "IDS", indexes = TRUE) )
#'   plot( st_geometry(pts), pch=19, main="KNN")
#'     plot(st_geometry(meuse[nn[,1],]), pch=19, col="red", add=TRUE)
#' 
#' # Using covariates (weights)
#' wx = as.matrix(st_drop_geometry(meuse[,1:3]))
#' wy = as.matrix(st_drop_geometry(pts[,1:3]))
#' 
#' ( nn <- knn(pts, meuse, k=2, ids = "IDS", indexes = TRUE,
#'             weights.y=wy, weights.x=wx) )
#'   plot(st_geometry(pts), pch=19, main="KNN")
#'     plot(st_geometry(meuse[nn[,1],]), pch=19, col="red")
#' 	  
#' # Using coordinate matrices
#' y <- st_coordinates(pts)[,1:2]
#' x <- st_coordinates(meuse)[,1:2]
#' knn(y, x, k=2)
#'
#' } else { 
#'   cat("Please install sp package to run example", "\n")
#' }
#' }
#'	  
#' @seealso \code{\link[RANN]{nn2}} for details on search algorithm 
#' @export knn
knn <- function(y, x, k = 1, d = NULL, ids = NULL, 
                weights.y = NULL, weights.x = NULL,
                indexes = FALSE) {
  if(length(find.package("RANN", quiet = TRUE)) == 0)
      stop("please install RANN package before running this function")

  gtypes = c("POLYGON", "POINT")
  mtypes = c("MULTIPOLYGON", "MULTIPOINT") 

  if(!is.null(ids)) { 
    if(!ids %in% names(x)) 
      stop("ids do not exist in data")
    nidx <- which( names(x) %in% ids )
  }

  if(inherits(x, "matrix")) {
    if(ncol(x) > 2)
      stop("coordinate matrix has too many columns")
    xmat <- x
    x <- as.data.frame(x)		
  } else {
    xgeom = unique(as.character(sf::st_geometry_type(x)))  
    if(!inherits(x, c("sf", "sfc")))
      stop(deparse(substitute(x)), " must be an sf class object")	  
    if(any(xgeom == mtypes))
      stop(deparse(substitute(x)), " is multipart geometry and not supported ")
    if(!any(xgeom != gtypes))
      stop(deparse(substitute(x)), " must be one of ", 
  	  paste(gtypes, collopse=""))
    if(xgeom == gtypes[1]) {
	  warning("x has polygon geometry using centroid coordinates")
      xmat <- sf::st_coordinates(sf::st_centroid(x))[,1:2]
    } else {
      xmat <- sf::st_coordinates(x)[,1:2]
    }
  }
  
  if(inherits(y, "matrix")) {
    if(ncol(y) > 2)
      stop("coordinate matrix has too many columns")
    ymat <- y
    y <- as.data.frame(y)	
  } else {
    ygeom = unique(as.character(sf::st_geometry_type(y)))	
    if(!inherits(y, c("sf", "sfc")))
      stop(deparse(substitute(y)), " must be an sf class object")	  
    if(any(ygeom == mtypes))
      stop(deparse(substitute(y)), " is multipart geometry and not supported ")
    if(!any(ygeom != gtypes))
      stop(deparse(substitute(y)), " must be one of ", 
  	  paste(gtypes, collopse=""))
    if(ygeom == gtypes[1]) {
	  warning("y has polygon geometry using centroid coordinates")
      ymat <- sf::st_coordinates(sf::st_centroid(y))[,1:2]
    } else {
      ymat <- sf::st_coordinates(y)[,1:2]
    }	  
  }

  if(any(!is.null(weights.x), !is.null(weights.y))) {	
    if(any(is.null(weights.x), is.null(weights.y)))
      stop("Both x and y weights must be defined and represent the same covariates")
    if(is.vector(weights.x)) {	
	  weights.x <- matrix(weights.x, byrow=TRUE)
    }	
    if(is.vector(weights.x)) {	
	  weights.x <- matrix(weights.x, byrow=TRUE)
    }
    if(any(apply(xmat,2,is.numeric) == FALSE))
	  stop("weights must be numeric")
    if(any(apply(ymat,2,is.numeric) == FALSE))
	  stop("weights must be numeric")	
    if(nrow(weights.x) != nrow(xmat) )
	  stop("Row dimensions of x weights do not match")
	if(nrow(weights.y) != nrow(ymat) )
	  stop("Row dimensions of x weights do not match")
  	if(ncol(weights.y) != ncol(weights.y) )
	  stop("Weights of x and y must represent the same covariates")	
    xmat <- cbind(xmat, as.matrix(weights.x))
    ymat <- cbind(ymat, as.matrix(weights.y))	
  }  
  
  fun.args <- list(data = xmat, 
                   query = ymat,
                   treetype = "kd",
                   k = k)
    if(!is.null(d)) {
      if(!is.numeric(d)) 
	    stop("Distance must be numeric")
      fun.args[["radius"]] <-  d
      fun.args[["searchtype"]] <- "radius"     
    } 
  nn <- do.call(RANN::nn2, fun.args) 
  
  xrows <- list() 
    for(i in 1:ncol(nn$nn.idx)) {
      if(!is.null(ids)) {
        xrows[[i]] <- as.data.frame(x[nn$nn.idx[,i],])[,nidx]
      } else {
        xrows[[i]] <- row.names(x)[nn$nn.idx[,i]]
      }	
    }	
   xrows <- as.data.frame(do.call("cbind", xrows))
     names(xrows) <- paste0("ids", 1:ncol(xrows))
   xdist <- as.data.frame(nn$nn.dists)
     names(xdist) <- paste0("dist", 1:ncol(xdist))
  if( indexes ) {
    xidx <- as.data.frame(nn$nn.idx)
      names(xidx) <- paste0("idx", 1:ncol(xdist))
    return( data.frame(xidx, xrows, xdist) )	  
  } else { 
    return( data.frame(xrows, xdist) )
  }  
}

#' @title Weighted centroid
#' @description Creates centroid of [x,y] coordinates with optional
#'              weights field 
#'
#' @param x         sf POINT class object                     
#' @param p         Weights column in x 
#' @param spatial   (TRUE/FALSE) Output sf POINT object    
#' 
#' @details
#' The weighted centroid is calculated as:
#' [Xw]=[X]*[p], [Yw]=[Y]*[p], [sXw]=SUM[Xw], [sYw]=SUM[Yw], [sP]=SUM[p]
#'    wX=[sXw]/[sP], wY=[sYw]/[sP]    
#'    where; X=X coordinate(S), Y=Y coordinate(S), p=WEIGHT 
#'    
#' @return 
#' An x,y coordinate or sf POINT object representing the weighted or unweighted 
#' coordinate centroid      
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'                                                                  
#' @examples 
#' p = c("sf", "sp")
#' if(any(!unlist(lapply(p, requireNamespace, quietly=TRUE)))) { 
#'   m = which(!unlist(lapply(p, requireNamespace, quietly=TRUE)))
#'   message("Can't run examples, please install ", paste(p[m], collapse = " "))
#' } else {
#' invisible(lapply(p, require, character.only=TRUE))
#' 
#' data(meuse, package = "sp")
#' meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, 
#'                   agr = "constant")
#' 
#' wt.copper <- wt.centroid(meuse, p='copper') 
#' wt.zinc <- wt.centroid(meuse,  p='zinc') 
#' 
#' plot(st_geometry(meuse), pch=20, cex=0.75, 
#'      main='Weighted centroid(s)')
#'   plot(st_geometry(wt.copper), pch=19, col='red', 
#'        cex=1.5, add=TRUE)  
#'   plot(st_geometry(wt.zinc), pch=19, col='blue', 
#'        cex=1.5, add=TRUE)
#' legend('topleft', legend=c('all','copper', 'zinc'), 
#'        pch=c(20,19,19),col=c('black','red','blue'))
#' }
#' @export wt.centroid                                                                                   
wt.centroid <- function(x, p = NULL, spatial = TRUE) {
  gtypes = c("POLYGON", "MULTIPOLYGON", "POINT", "MULTIPOINT")
  if (!inherits(x, c("sf", "sfc"))) 
    stop(deparse(substitute(x)), " must be an sf class object or coercible")
  if(!any(unique(as.character(sf::st_geometry_type(x))) != gtypes[c(1,3)]))
    stop(deparse(substitute(x)), " must be POINT or POLYGON geometry ")
  if(any(unique(as.character(sf::st_geometry_type(x))) == gtypes[c(2,4)]))
    stop("Function cannot accept multi-part geometry, please use st_cast 
	      to create single-part geometry")	    
  if(is.na(sf::st_crs(x))) 
    stop("Projection is not defined, lat/long not acceptable input")
  if(sf::st_is_longlat(x))
    stop("Projection must be in distance units, not lat/long") 
  #if(any(class(x) %in% c("tbl_df", "tbl"))) {
  #  x <- as.data.frame(x)
  #}
  if(any(names(x) %in% p)){
    p <- as.data.frame(sf::st_drop_geometry(x[,p]))[,1]
  } else {
    stop(p, " does not appear to be a column in ", deparse(substitute(x)))
  }
    if(!is.numeric(p))
	  stop(deparse(substitute(p)), " must be numeric")
    Xw <- sum(sf::st_coordinates(x)[,1] * p)
      Yw <- sum(sf::st_coordinates(x)[, 2] * p)
        wX <- Xw/sum(p)
          wY <- Yw/sum(p)
    xy <- data.frame(ID=1, X=wX, Y=wY)
      if (spatial) {
        xy <- sf::st_as_sf(xy, coords = c("X", "Y"), 
		        crs = sf::st_crs(x), agr = "constant")
	  }  
  return(xy)
} 

#' @title Spherical Variance or Standard Deviation of Surface 
#' @description Derives the spherical standard deviation of a raster surface  
#'    
#' @param r         A terra SpatRaster class object
#' @param d         Size of focal window or a matrix to use in focal function
#' @param variance  (FALSE|TRUE) Output spherical variance rather than standard deviation
#' @param ...       Additional arguments passed to terra:app (can write raster to disk here)
#'
#' @details
#' Surface variability using spherical variance/standard deviation. 
#' The variation can be assessed using the spherical standard deviation of the normal 
#' direction within a local neighborhood. This is found by expressing the normal 
#' directions on the surfaces cells in terms of their displacements in a Cartesian (x,y,z) 
#' coordinate system. Averaging the x-coordinates, y-coordinates, and z-coordinates 
#' separately gives a vector (xb, yb, zb) pointing in the direction of the average 
#' normal. This vector will be shorter when there is more variation of the normals and 
#' it will be longest--equal to unity--when there is no variation. Its squared length 
#' is (by the Pythagorean theorem) given by: R^2 = xb^2 + yb^2 + zb^2
#'    where; x = cos(aspect) * sin(slope) and xb = nXn focal mean of x 
#'           y = sin(aspect) * sin(slope) and  yb = nXn focal mean of y
#'           z = cos(slope) and zb = nXn focal mean of z 	   
#' 
#' The slope and aspect values are expected to be in radians. 
#' The value of (1 - R^2), which will lie between 0 and 1, is the spherical variance. 
#' and it's square root can be considered the spherical standard deviation.
#'
#' @return A terra SpatRaster class object of the spherical standard deviation   
#'
#' @author Jeffrey S. Evans <jeffrey_evans<at>tnc.org>
#'
#' @examples
#' \donttest{
#'  library(terra)
#'  elev <- rast(system.file("extdata/elev.tif", package="spatialEco"))
#'  
#'  ssd <- spherical.sd(elev, d=5)
#'  
#'  slope <- terrain(elev, v='slope')
#'  aspect <- terrain(elev, v='aspect')
#'  hill <- shade(slope, aspect, 40, 270)
#'  plot(hill, col=grey(0:100/100), legend=FALSE, 
#'       main='terrain spherical standard deviation')
#'    plot(ssd, col=rainbow(25, alpha=0.35), add=TRUE)
#' }
#'  
#' @seealso \code{\link[terra]{app}} for details on ... arguments
#' 
#' @export
spherical.sd <- function(r, d, variance = FALSE, ...) {
  if(!inherits(r, "SpatRaster"))	
    stop("r must be a terra or raster object")	
    if(class(d)[1] != "matrix") { d = matrix(1,d,d) }
  s <- terra::terrain(r, v='slope', unit='radians') 
    a <- terra::terrain(r, v='aspect', unit='radians')
      x <- terra::lapp(c(a,s), fun=function(x,y) {cos(x) * sin(y) } )
	    y <- terra::lapp(c(a,s), fun=function(x,y) {sin(x) * sin(y) } )
          z <- terra::app(s, fun=cos)
        xb = terra::focal(x, d, fun=mean)
      yb = terra::focal(y, d, fun=mean)
    zb = terra::focal(z, d, fun=mean)
  if(variance == TRUE) {
    return(
	  terra::lapp( c(xb, yb, zb), fun=function(x,y,z) { 1 - (x^2 + y^2 + z^2) }, ... )
    )	  
  } else {
    return(
      terra::lapp( c(xb, yb, zb), fun=function(x,y,z) { sqrt( 1 - (x^2 + y^2 + z^2) ) }, ... )
	)  
  }  
}

#' @title Cosine Similarity Index 
#' @description Calculates the cosine similarity and angular similarity on  
#'              two vectors or a matrix
#' 
#' @param x          A vector or matrix object 
#' @param y          If x is a vector, then a vector object 
#' @param angular    Booliean (TRUE/FALSE) return angular similarity
#'
#' @details 
#' The cosine similarity index is a measure of similarity between two 
#' vectors of an inner product space. This index is bested suited for high-dimensional 
#' positive variable space. One useful application of the index is to measure separability 
#' of clusters derived from algorithmic approaches (e.g., k-means). It is a good common 
#' practice to center the data before calculating the index. It should be noted that the 
#' cosine similarity index is mathematically, and often numerically, equivalent to the 
#' Pearson's correlation coefficient   
#'
#' The cosine similarity index is derived: 
#' s(xy) = x * y / ||x|| * ||y||, where the expected is 1.0 (perfect similarity)  
#' to -1.0 (perfect dissimilarity). A normalized angle between the vectors can 
#' be used as a bounded similarity function within [0,1] 
#' angular similarity  = 1 - (cos(s)^-1/pi)
#'
#' @return 
#' If x is a matrix, a list object with: similarity and angular.similarity matrices or,  
#' if x and y are vectors, a vector of similarity and angular.similarity   
#'
#' @author Jeffrey S. Evans  <jeffrey_evans@@tnc.org>
#'
#' @examples 
#' # Compare two vectors (centered using scale)
#' x=runif(100)
#' y=runif(100)^2
#' csi(as.vector(scale(x)),as.vector(scale(y)))
#'   
#' # Compare columns (vectors) in a matrix (centered using scale)
#' x <- matrix(round(runif(100),0),nrow=20,ncol=5)
#' ( s <- csi(scale(x)) )
#'     
#' # Compare vector (x) to each column in a matrix (y)
#' y <- matrix(round(runif(500),3),nrow=100,ncol=5)
#' x=runif(100) 
#' csi(as.vector(scale(x)),scale(y))
#'
#' @export   
csi <- function (x, y = NULL, angular = TRUE) {
    if (is.matrix(x) && is.null(y)) {
      s = array(0, c(ncol(x), ncol(x)))
      f = colnames(x)
      dimnames(s) = list(f, f)
        for (i in 2:ncol(x)) {
          for (j in 1:(i - 1)) { s[i, j] = csi(x[, i], x[, j])[1] }
        }
      s = s + t(s)
	  acs  <- 1 - (cos(s)^ -1 / pi)
	  diag(s) = 1
      diag(acs) = 1
	  if(angular) {
        return(list(similarity = s, angular.similarity = acs))
      } else {
        return(list(similarity=s))
	  }      
    }
    else if (is.vector(x) && is.vector(y)) {
      cs <- crossprod(x, y) / sqrt(crossprod(x) * crossprod(y))
	  acs <- 1 - (cos(cs)^ -1 / pi)
	  if(angular) {
        s <- c(cs,acs)
	      names(s) <- c("similarity", "angular.similarity")
      } else {
        s <- c(cs,acs)
	      names(s) <- c("similarity")
	  }
	  return(s)
    }
    else if (is.vector(x) && is.matrix(y)) {
      s = vector(mode = "numeric", length = ncol(y))
      names(s) = colnames(y)
        for (i in 1:ncol(y)) { s[i] = csi(x, y[, i])[1] }
	  acs  <- 1 - (cos(s)^ -1 / pi)
	  if(angular) {
	    s <- rbind(s,acs)
	      rownames(s) <-c("similarity", "angular.similarity")
      } else {
	    s <- s
		  rownames(s) <- c("similarity")
	  }
	  return(s)
    } else {
      stop("Error: function supports two vectors or single matrix")
    }
  }

#' @title Remove extension
#' @description Removes file extension (and path) from string
#'
#' @param x A character vector representing a file with extension 
#'
#' @return 
#' The file name with extension and file path stripped off
#'
#' @examples 
#' rm.ext("C:/path/file.txt")
#'
#' @export rm.ext 
rm.ext <- function(x) {
  sub("([^.]+)\\.[[:alnum:]]+$", "\\1", basename(x))
}

#' @title Spatial Select
#'
#' @description 
#' Performs a spatial select (feature subset) between a polygon(s) and other 
#' feature class
#' 
#' @param x           An sp or sf polygon(s) object that defines the spatial query
#' @param y           A sp or sf feature class that will be subset by the query of x
#' @param distance    A proximity distance of features to select (within distance)
#' @param predicate   Spatial predicate for intersection
#' @param neighbors   If predicate = "contingency" type of neighbors options are 
#'                    c("queen", "rook")
#'
#' @details
#' Performs a spatial select of features based on an overlay of a polygon (x),   
#' which can represent multiple features, and a polygon, point or line feature 
#' classes (y). User can specify a partial or complete intersection, using within 
#' argument, or within a distance, using distance argument, predicated on the  
#' query polygon. This function is similar to ArcGIS/Pro spatial select. Please note
#' that for point to point neighbor selections use the knn function. 
#' Valid spatial predicates include: intersect, touches, covers, contains, proximity 
#' and contingency. 
#' See DE-9IM topology model for detailed information on following data predicates. 
#'   * intersection  Create a spatial intersection between two features
#'   * intersect     Boolean evaluation of features intersecting   
#'   * contains      Boolean evaluation of x containing y
#'   * covers        Boolean evaluation of x covering y
#'   * touches       Boolean evaluation of x touching y
#'   * proximity     Evaluation of distance-based proximity of x to y (x and y can be the same)
#'   * contingency   Evaluation of polygon contingency (eg., 1st, 2nd order)
#' @md
#'
#' @return 
#' An sf object representing a subset of y based on the spatial query of x or, 
#' if predicate = contingency a sparse matrix representing neighbor indexes
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com> 
#'                                                                    
#' @examples 
#' if(require(sp, quietly = TRUE)) {
#' library(sf)
#'   data(meuse, package = "sp")
#'   meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, 
#'                     agr = "constant")
#' 
#' spolys <- hexagons(meuse, res=100)
#'   spolys$ID <- 1:nrow(spolys)
#'     p <- st_as_sf(st_sample(spolys, 500))
#' 	  p$PTID <- 1:nrow(p) 
#' 	  sf::st_geometry(p) <- "geometry"
#' 
#'   plot(st_geometry(spolys), main="all data")
#'     plot(st_geometry(p), pch=20, add=TRUE)
#' 	
#' sub.int <- spatial.select(p, spolys, predicate = "intersect")
#'   plot(st_geometry(sub.int), main="intersects")
#'     plot(st_geometry(p), pch=20, add=TRUE)	
#' 
#' sub.prox <- spatial.select(p, spolys, distance=100, predicate = "proximity")	
#'   plot(st_geometry(sub.int), main="intersects")
#'     plot(st_geometry(p), pch=20, add=TRUE)
#' 
#' # For rook or queen polygon contingency 	
#' plot( spolys <- sf::st_make_grid(sf::st_sfc(sf::st_point(c(0,0)), 
#'                  sf::st_point(c(3,3))), n = c(3,3)) )
#'   
#' spatial.select(x=spolys, predicate = "contingency")
#' spatial.select(spolys, predicate = "contingency", neighbors = "rook") 
#' 
#' } else { 
#'   cat("Please install sp package to run example", "\n")
#' }
#'
#' @seealso \code{\link[sf]{st_intersection}} for details on intersection predicate
#' @seealso \code{\link[sf]{st_intersects}} for details on intersect predicate
#' @seealso \code{\link[sf]{st_contains}} for details on contain predicate
#' @seealso \code{\link[sf]{st_covers}} for details on covers predicate
#' @seealso \code{\link[sf]{st_touches}} for details on touches predicate 
#' @seealso \code{\link[sf]{st_is_within_distance}} for details on proximity predicate
#' @seealso \url{https://en.wikipedia.org/wiki/DE-9IM} for details on DE-9IM topology model
#'
#' @export spatial.select	
spatial.select <- function(x, y = NULL, distance = NULL, 
                           predicate = c("intersection", "intersect",   
                           "contains", "covers", "touches", "proximity",
						   "contingency"), neighbors = c("queen", "rook")) {
  predicate = predicate[1]
  gtypes = c("POLYGON", "MULTIPOLYGON", "POINT", "MULTIPOINT", 
             "LINESTRING", "MULTILINESTRING")
  if(!is.null(y)) {
    if(any(methods::is(y, "Spatial"))){
      y <- sf::st_as_sf(y)
	  message("coercing y to sf")
	}  
    if(!inherits(y, c("sf", "sfc")))
      stop(deparse(substitute(y)), " must be an sf object or coercible")	  
    if(!any(unique(as.character(sf::st_geometry_type(y))) != gtypes))
      stop(deparse(substitute(y)), " must be one of ", 
	       paste(gtypes, collopse=""))		     
  } else {
    if(predicate != "contingency")
      stop("The only predicate that supports self realization is contingency")
  }
  if(any(methods::is(x, "Spatial"))){
    x <- sf::st_as_sf(x)
	message("coercing y to sf")
  }	
  if(!inherits(x, c("sf", "sfc")))
    stop(deparse(substitute(x)), " must be an sf object or coercible")
  if(!any(unique(as.character(sf::st_geometry_type(x))) != gtypes))
    stop(deparse(substitute(c)), " must be one of ", 
	     paste(gtypes[1:2], collopse=""))
		 
  if(predicate == "intersection") {
    y <- suppressWarnings(sf::st_intersection(x,y))
	  return(y)
  } else if(predicate == "intersect") {
    idx <- suppressWarnings(sort(unique(unlist(sf::st_intersects(x,y)))))	   
  } else if(predicate == "contains") {
    idx <- suppressWarnings(sort(unique(unlist(sf::st_contains(x,y)))))		 
  } else if(predicate == "covers") {
    idx <- suppressWarnings(sort(unique(unlist(sf::st_covers(x,y)))))		
  } else if(predicate == "touches") {
    idx <- suppressWarnings(sort(unique(unlist(sf::st_touches(x,y)))))	
  } else if(predicate == "contingency") {
    if(neighbors[1] == "rook"){ adj = "F***1****"} else {adj = "F***T****"}
          cont <- function(a, b = a, adj) sf::st_relate(a, b, pattern = adj)
  	    y <- cont(x, adj = adj) 
      return(y)
  } else if(predicate == "proximity") {
    if(is.null(distance)) 
      stop("For proximity predicate distance must be defined")
        idx <- sort(unique(unlist(sf::st_is_within_distance(x, y, dist = distance))))	 
  } else {
    stop("Not a valid spatial relationship predicate")
  }
  if(length(idx) < 1)
    stop(paste0("No results match ", predicate,  " predicate"))	
  return(y[idx,])
}

#' @title Leaf Area Index
#' @description Remote sensing measure of LAI (leaf area per ground-unit area)
#'
#' @param ndvi     NDVI in floating point standard scale range (-1 to 1)
#' @param method   Method to use for index options c("Jonckheere", "Chen")
#'
#' @details 
#' This function calculates the Leaf Area Index (LAI) representing the amount of leaf area 
#' per unit of ground area. This is an important parameter for understanding the structure 
#' and function of vegetation, as it affects processes such as photosynthesis, transpiration, 
#' and carbon cycling. These two approaches are based on the empirical relationship between 
#' NDVI and LAI, which has been observed in many studies, and it is a widely used method for 
#' estimating LAI from remote sensing data. The formulas are derived from the fact that vegetation 
#' with higher LAI tends to have higher reflectance in the near-infrared (NIR) band and lower 
#' reflectance in the red band, resulting in higher NDVI values. But still, the exact relationship 
#' between NDVI and LAI can vary depending on factors such as vegetation type, canopy structure, 
#' and environmental conditions.
#'
#' @return A terra SpatRaster object with derived LAI vaues
#'
#' @references 
#' Jonckheere, I., Fleck, S., Nackaerts, K., Muys, B., Coppin, P. (2004). A comparison of two 
#'   methods to retrieve the leaf area index (LAI) from SPOT-4 HRVIR data. International 
#'   Journal of Remote Sensing, 25(21):4407–4425.
#'  
#' Chen, J. M., Liu, R., & Ju, W. (2014). A simple and effective method for estimating 
#'   leaf area index from Landsat imagery. Remote Sensing of Environment, 152:538–548.
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com> 
#'
#' @examples
#' \donttest{
#' library(terra)
#' lsat <- rast(system.file("/extdata/Landsat_TM5.tif", package="spatialEco"))
#'    plotRGB(lsat, r=3, g=2, b=1, scale=1.0, stretch="lin")
#' 	
#'   ndvi <-  ( lsat[[4]] - lsat[[3]] ) / (lsat[[4]] + lsat[[3]]) 
#' 		   
#'  # Using Jonckheere et al., (2004) method
#'  lai01 <- lai(ndvi)
#'    plot(lai01)
#'
#' }
#' 
#' @export lai
lai <- function(ndvi, method = c("Jonckheere", "Chen")) { 
  if(missing(ndvi))
    stop("must specify NDVI")
  if(terra::nlyr(ndvi) > 1)
    stop("Expecting single band for NDVI")
  if (!inherits(ndvi, "SpatRaster")) 
	stop(deparse(substitute(ndvi)), " must be a terra SpatRaster object")		
  ndvi <- terra::ifel(ndvi < 0, 0, ndvi)
  if(tolower(method[1]) == "jonckheere") {
    LA <- (ndvi - 0.2) / 0.3	  
  } else if(tolower(method[1]) == "chen") {
    LA = (1 - exp(-0.69 * ndvi)) / 0.69
  }   
  return(LA * 10)
}  

#' @title Dispersion (H-prime) 
#' @description Calculates the dispersion ("rarity") of targets associated 
#'              with planning units
#'
#' @param  x data.frame object of target values 
#'
#' @details  
#' The dispersion index (H-prime) is calculated H = sum( sqrt(p) / sqrt(a) ) 
#' where; P = (sum of target in planning unit / sum of target across all 
#' planning units) and a = (count of planning units containing 
#' target / number of planning units)  
#'
#' @return 
#' data.frame with columns H values for each target, H , sH, sHmax 
#'  
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @references
#' Evans, J.S., S.R. Schill, G.T. Raber (2015) A Systematic Framework for Spatial 
#'   Conservation Planning and Ecological Priority Design in St. Lucia, Eastern Caribbean. 
#'   Chapter 26 in Central American Biodiversity : Conservation, Ecology and a Sustainable 
#'   Future. F. Huettman (eds). Springer, NY.   
#'
#' @examples
#' \donttest{ 
#'  library(sf)
#'    data(pu)
#'   
#'  d <- dispersion(st_drop_geometry(pu[,2:ncol(pu)]))  
#'    p <- d[,"H"]
#'  clr <- c("#3288BD", "#99D594", "#E6F598", "#FEE08B", 
#'           "#FC8D59", "#D53E4F")      
#'  clrs <- ifelse(p < 0.5524462, clr[1], 
#'            ifelse(p >= 0.5524462 & p < 1.223523, clr[2],
#'              ifelse(p >= 1.223523 & p < 2.465613, clr[3],
#'  	          ifelse(p >= 2.465613 & p < 4.76429, clr[4],
#'  	            ifelse(p >= 4.76429 & p < 8.817699, clr[5],
#'  	              ifelse(p >= 8.817699, clr[6], NA))))))
#'  plot(st_geometry(pu), col=clrs, border=NA)
#'    legend("bottomleft", legend=rev(c("Very Rare","Rare","Moderately Rare",
#'           "Somewhat Common","Common","Over Dispersed")),
#'           fill=clr, cex=0.6, bty="n") 
#'    box()
#' }
#'	
#' @export
dispersion <- function (x) {
  if (!inherits(x, "data.frame")) 
    stop("x is not a data.frame or matrix object")
  H <- as.data.frame(array(0, dim=c( dim(x)[1], 0 )))
   rownames(H) <- rownames(x)
    tcounts <- apply(x, MARGIN=1, function(x){ length(x[x > 0]) } )
    total <- apply(x, MARGIN=2, sum, na.rm = TRUE) 
       for (n in 1:ncol(x) ) {
          s = total[n]
          p <- (x[,n] / s)
          a <- length(x[x[,n] > 0 ,]) / length(x[,n])
          r <- sqrt(ifelse(p >= 0, p, 0)) / sqrt(a)   
          H <- cbind(H, r)
       }
    names(H) <- names(x)
	Hm <- rowSums(H)
	maxH <- apply(H, MARGIN=1, max)
	return ( data.frame(row.names = rownames(x), H, H = Hm, sH = Hm / max(Hm), 
	         sHmax = maxH / max(maxH) ) )
}  

#' @title Gaussian Kernel
#' @description Creates a Gaussian Kernel of specified size and sigma
#'
#' @param sigma  sigma (standard deviation) of kernel (defaults 2)
#' @param s      scale defining the number of rows and columns for kernel (default 5)
#'
#' @return Symmetrical (NxN) matrix of a Gaussian distribution
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'  
#' @examples 
#' opar <- par()
#'   par(mfrow=c(2,2))
#'   persp(gaussian.kernel(sigma=1, s=27), theta = 135, 
#'         phi = 30, col = "grey", ltheta = -120, shade = 0.6, 
#'         border=NA )
#'   persp(gaussian.kernel(sigma=2, s=27), theta = 135, phi = 30,
#'         col = "grey", ltheta = -120, shade = 0.6, border=NA )	
#'   persp(gaussian.kernel(sigma=3, s=27), theta = 135, phi = 30,
#'         col = "grey", ltheta = -120, shade = 0.6, border=NA )	
#'   persp(gaussian.kernel(sigma=4, s=27), theta = 135, phi = 30,
#'         col = "grey", ltheta = -120, shade = 0.6, border=NA )	
#'  par(opar) 			
#' @export
gaussian.kernel <- function(sigma=2, s=5) {
    m <- matrix(ncol=s, nrow=s)
     mcol <- rep(1:s, s)
     mrow <- rep(1:s, each=s)
       x <- mcol - ceiling(s/2)
       y <- mrow - ceiling(s/2)
     m[cbind(mrow, mcol)] <- 1/(2*pi*sigma^2) * exp(-(x^2+y^2)/(2*sigma^2))
   m / sum(m)
}

#' @name chen
#' @docType data
#'
#' @title Cross-correlation data from Chen (2015) 
#'
#' @description Urbanization and economic development data from Chen (2015)
#' compiled from, National Bureau of Statistics of China 
#'
#' @format A list object with 3 elements:
#' \describe{
#'   \item{X}{per capita GRP(yuan)}
#'   \item{Y}{Level of urbanization percent}
#'   \item{M}{Railway Distance (km) matrix of 29 Chinese regions}
#' }
#'
#' @references
#' Chen, Y.G. (2012) On the four types of weight functions for spatial contiguity 
#'    matrix. Letters in Spatial and Resource Sciences 5(2):65-72 
#' @references
#' Chen, Y.G. (2013) New approaches for calculating Moran’s index of spatial 
#'   autocorrelation. PLoS ONE 8(7):e68336
#' @references 
#' Chen, Y.G. (2015) A New Methodology of Spatial Cross-Correlation Analysis. 
#'   PLoS One 10(5):e0126158. doi:10.1371/journal.pone.0126158
#'
#' @source \url{https://journals.plos.org/plosone/article?id=10.1371/journal.pone.0126158}
NULL

#' @title Summary of spatial cross correlation
#' @description summary method for class "cross.cor"
#'
#' @param object    Object of class cross.cor
#' @param ...       Ignored
#'
#'
#' @return 
#' When not simulated k=0, prints functions list object containing:
#' * I - Global autocorrelation statistic
#' * SCI - - A data.frame with two columns representing the xy and yx autocorrelation
#' * nsim - value of NULL to represent p values were derived from observed data (k=0)
#' * p - Probability based observations above/below confidence interval
#' * t.test - Probability based on t-test
#'
#' When simulated (k>0), prints functions list object containing: 
#'  * I - Global autocorrelation statistic
#'  * SCI - A data.frame with two columns representing the xy and yx autocorrelation
#'  * nsim - value representing number of simulations
#'  * global.p - p-value of global autocorrelation statistic
#'  * local.p - Probability based simulated data using successful rejection of t-test
#'  * range.p - Probability based on range of probabilities resulting from paired t-test
#' @md
#' 
#' @method summary cross.cor
#'
#' @export
summary.cross.cor <- function(object, ...) {
  if(!is.null(object$nsim)) {
  cat("Moran's-I under randomization assumptions...", "\n")
    cat("  First-order Moran's-I: ", object$I, "\n")
  	cat("  First-order p-value: ", object$global.p, "\n")
	cat("", "\n") 
  cat("Chen's SCI under randomization assumptions...", "\n")
	cat("\n", "Summary statistics of local partial cross-correlation [xy]", "\n")
      print( summary(object$SCI[,1]) )
        cat("\n", "", "\n")  
  	cat("  p-value based on 2-tailed t-test: ", object$local.p, "\n")	
    cat("  p-value based on 2-tailed t-test observations above/below CI: ", object$range.p, "\n")	
    if( exists(object$clusters) )
	  cat("\n", "Counts of cluster types")
	    print(table(object$cluster))
  } else {
  cat("Moran's-I...", "\n")
      cat("  First-order Moran's-I: ", object$I, "\n")
  	  cat("  First-order p-value: ", object$global.p, "\n")
	  cat("", "\n") 
  cat("Chen's SCI under randomization assumptions...", "\n")
	cat("\n", "Summary statistics of local partial cross-correlation [xy]", "\n")
      print( summary(object$SCI[,1]) )
	    cat("\n", "", "\n") 
  	cat("    non-simulated second-order p-value based on 2-tailed t-test: ", object$t.test, "\n")
	cat("    p-value based on 2-tailed t-test observations above/below CI: ", object$p, "\n")	
    if( exists(object$clusters) )
	  cat("\n", "Counts of cluster types")
	    print(table(object$cluster))
  }
}

#' @title Raster Downscale
#' @description Downscales a raster to a higher resolution raster using 
#'              a robust regression
#' 
#' @param x            A terra SpatRaster object representing independent variable(s) 
#' @param y            A terra SpatRaster object representing dependent variable
#' @param full.res     (FALSE/TRUE) Use full resolution of x (see notes)  
#' @param scatter      (FALSE/TRUE) Optional scatter plot  
#' @param residuals    (FALSE/TRUE) Output raster residual error raster, 
#'                     at same resolution as y 
#' @param se           (FALSE/TRUE) Output standard error raster, using prediction or 
#'                     confidence interval  
#' @param p            The confidence/prediction interval (default is 95%)  
#' @param uncertainty  Output uncertainty raster(s) of confidence or prediction interval, 
#'                     at same resolution as y. Options are c("none", "prediction", "confidence")  
#'
#' @details
#' This function uses a robust regression, fit using an M-estimation with Tukey's biweight 
#' initialized by a specific S-estimator, to downscale a raster based on higher-resolution
#' or more detailed raster data specified as covariate(s). You can optionally output residual 
#' error, standard error and/or uncertainty rasters. However, please note that when choosing 
#' the type of uncertainty, using a confidence interval (uncertainty around the mean predictions) 
#' when you should be using the prediction interval (uncertainty around a single values) will 
#' greatly underestimate the uncertainty in a given predicted value (Bruce & Bruce 2017). 
#' The full.res = TRUE option uses the x data to sample y rather than y to sample x. THis makes 
#' the problem much more computationally and memory extensive and should be used with caution. 
#' There is also the question of pseudo-replication (sample redundancy) in the dependent variable.
#' Statistically speaking one would expect to capture the sample variation of x by sampling at the
#' frequency of y thus supporting the downscaling estimate. Note that if uncertainty is not defined
#' the prediction interval for standard error defaults to "confidence" else is the same output as
#' uncertainty (eg., prediction or confidence).    
#'
#' @return 
#' A list object containing:  
#'   * downscale - downscaled terra SpatRaster object
#'   * model - MASS rlm model object 
#'   * MSE - Mean Square Error
#'   * AIC - Akaike information criterion
#'   * parm.ci - Parameter confidence intervals
#'   * residuals - If residuals = TRUE, a SpatRaster of the residual error
#'   * uncertainty - If pred.int = TRUE, SpatRaster's of the lower/upper prediction intervals
#'   * std.error - If se = TRUE, SpatRaster's of the standard error 
#' @md
#' 
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @references
#' Bruce, P., & A. Bruce. (2017). Practical Statistics for Data Scientists. O’Reilly Media.
#'
#' @examples 
#' \dontrun{
#' if (require(geodata, quietly = TRUE)) {
#' library(terra)
#' library(geodata)
#'
#' # Download example data (requires geodata package)
#' elev <- elevation_30s(country="SWZ", path=tempdir())
#' slp <- terrain(elev, v="slope")
#' tmax <- worldclim_country(country="SWZ", var="tmax", path=tempdir())
#'   tmax <- crop(tmax[[1]], ext(elev))
#' 
#' # Downscale temperature
#' x=c(elev,slp)
#'   names(x) <- c("elev","slope")
#' y=tmax
#'   names(y) <- c("tmax")
#' 
#' tmax.ds <- raster.downscale(x, y, scatter=TRUE, residuals = TRUE,
#'                             uncertainty = "prediction", se = TRUE)
#' 	
#'   # plot prediction and parameters	
#'   opar <- par(no.readonly=TRUE)
#'     par(mfrow=c(2,2))
#'       plot(tmax, main="Temp max")
#'       plot(x[[1]], main="elevation")
#'       plot(x[[2]], main="slope")
#'       plot(tmax.ds$downscale, main="Downscaled Temp max")
#'   par(opar)
#' 
#'   # Plot residual error and raw prediction +/- intervals
#'   opar <- par(no.readonly=TRUE)
#'     par(mfrow=c(2,2))
#'       plot(tmax.ds$std.error, main="Standard Error")
#'       plot(tmax.ds$residuals, main="residuals")
#'       plot(tmax.ds$uncertainty[[1]], 
#' 	       main="lower prediction interval")
#'       plot(tmax.ds$uncertainty[[2]], 
#' 	       main="upper prediction interval")
#'   par(opar)
#'   
#'   # plot prediction uncertainty
#'   opar <- par(no.readonly=TRUE)
#'     par(mfrow=c(2,1))
#'       plot(tmax.ds$downscale - tmax.ds$uncertainty[[1]], 
#' 	       main="lower prediction interval")
#'       plot(tmax.ds$downscale - tmax.ds$uncertainty[[2]], 
#' 	       main="upper prediction interval")  
#'   par(opar)  
#'
#' } else { 
#'   cat("Please install geodata package to run example", "\n")
#' }
#' }
#' @export raster.downscale
raster.downscale <- function(x, y, scatter = FALSE, full.res = FALSE, 
                             residuals = FALSE, se = FALSE, p = 0.95,  
							 uncertainty = c("none", "prediction", "confidence")) {
	uncertainty = uncertainty[1]
    if (!inherits(x, "SpatRaster")) 
	  stop(deparse(substitute(x)), " must be a terra SpatRaster object")	
    if (!inherits(y, "SpatRaster")) 
	  stop(deparse(substitute(y)), " must be a terra SpatRaster object")
    if (terra::nlyr(y) > 1)  {
      warning("Dependent variable must be a single response, 
	    only the first raster will be used")	
          y <- y[[1]]
    }	  
	if(full.res == FALSE) {
	  sub.samp.sp <- terra::as.points(y, na.rm=TRUE)
	    sub.samp <- data.frame(sub.samp.sp, terra::extract(x, sub.samp.sp) )
	      sub.samp <- sub.samp[,-which(names(sub.samp) %in% "ID")]
	        names(sub.samp) <- c("y", names(x))
			  na.idx <- unique(which(is.na(sub.samp), arr.ind = TRUE)[,1])
			  if(length(na.idx) > 0)
	            sub.samp <- sub.samp[-na.idx,]
	} else {
	  message("Population is being used and may cause memory issues")
	  sub.samp.sp <- terra::as.points(x, na.rm=TRUE)
	    sub.samp <- data.frame(terra::extract(y, sub.samp.sp), sub.samp.sp)
	      sub.samp <- sub.samp[,-which(names(sub.samp) %in% "ID")]
	        names(sub.samp) <- c("y", names(x))
			  na.idx <- unique(which(is.na(sub.samp), arr.ind = TRUE)[,1])
			  if(length(na.idx) > 0)
	            sub.samp <- sub.samp[-na.idx,]
    }		
	if(length(names(sub.samp)[-1]) > 1){ 
      xnames <- paste(names(sub.samp)[-1], collapse = "+") 
	} else {
	  xnames <- names(sub.samp)[2]
	}
    rrr <- MASS::rlm(stats::as.formula(paste(names(sub.samp)[1], xnames, sep=" ~ ")), 
                     data=sub.samp, scale.est="Huber", psi=MASS::psi.hampel, init="lts")
    if(scatter == TRUE) {
	  oops <- options() 
        on.exit(options(oops)) 
      n = terra::nlyr(x)
	  if(n > 1) {
	    graphics::par(mfrow=c(n,n/2))   
	    for(i in 2:(n+1)) {
	      graphics::plot(sub.samp[,i], sub.samp[,1], pch=20, cex=0.50,
		                 col="grey", xlab=names(sub.samp)[i], ylab="y") 
		  graphics::abline(stats::coefficients(rrr)[c(1,i)], col="red") 
		}					 
	  } else {
	    graphics::plot(sub.samp[,2], sub.samp[,1], pch=20, cex=0.50,
		               col="grey", xlab=names(sub.samp)[2], ylab="y") 
		  graphics::abline(stats::coefficients(rrr)[c(1,2)], col="red") 	  
	  }
	}  
     r <- terra::predict(x, rrr, na.rm=TRUE)
       results <- list(downscale = r, model = rrr, 
                    MSE = round(mean(rrr$residuals), digits=4), 
                    AIC = round(stats::AIC(rrr), digits=4),
                    parm.ci=stats::confint.default(object = rrr, level = 0.95))
  if(residuals == TRUE) {
    sub.samp.sp <- sub.samp.sp[-na.idx,]
    sub.samp.sp$resd <- rrr$residuals 
      if(full.res) {
	    results$residuals <- terra::rasterize(sub.samp.sp, x[[1]], field="resd")
	  } else {
	    results$residuals <- terra::rasterize(sub.samp.sp, y, field="resd")
	  }
  }
  if(uncertainty != "none"){
    if(!uncertainty %in% c("prediction", "confidence")) {
     warning(paste0(uncertainty, " is not a valid option, uncertainty will not be calculated"))
	} else {	
	  pred.uncert <- function(model, data) {
        v <- suppressWarnings(stats::predict(model, data, se.fit=FALSE,  
                              interval=uncertainty,level=p))
	      cbind(lci=as.vector(v[,"lwr"]), uci=as.vector(v[,"upr"]) )
      }
    results$uncertainty <- terra::predict(x, rrr, fun=pred.uncert) 
    }	
  }
  if(se == TRUE) {
    if(uncertainty == "none")
	  uncertainty = "confidence"
	  message("Calculating standard error using ", uncertainty, " interval")
    pred.se <- function(model, data) {
      suppressWarnings(stats::predict(model, data, se.fit=TRUE,  
                      interval=uncertainty,level=p)$se.fit)
    }
  results$std.error <- terra::predict(x, rrr, fun=pred.se) 
  }    
  return( results )		
}

#' @title Collinearity test
#' @description Test for linear or nonlinear collinearity/correlation in data 
#'
#' @param x            A data.frame or matrix containing continuous data     
#' @param p            The correlation cutoff (default is 0.85)
#' @param nonlinear    A boolean flag for calculating nonlinear correlations 
#'                     (FALSE/TRUE)
#' @param p.value      If nonlinear is TRUE, the p value to accept as the 
#'                     significance of the correlation
#'
#' @author Jeffrey S. Evans  <jeffrey_evans<at>tnc.org>
#'
#' @details  
#' Evaluation of the pairwise linear correlated variables to remove is 
#' accomplished through calculating the mean correlations of each variable 
#' and selecting the variable with higher mean. If nonlinear = TRUE, pairwise
#' nonlinear correlations are evaluated by fitting y as a semi-parametrically 
#' estimated function of x using a generalized additive model and testing 
#' whether or not that functional estimate is constant, which would indicate 
#' no relationship between y and x thus, avoiding potentially arbitrary decisions
#' regarding the order in a polynomial regression. 
#'  
#' @return Messages and a vector of correlated variables 
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @examples 
#' data(cor.data)
#'
#' # Evaluate linear correlations on linear dataCollinearity between
#' head( dat <- cor.data[[4]] ) 
#' pairs(dat, pch=20)
#'   ( cor.vars <- collinear( dat ) )
#' 
#' # Remove identified variable(s)
#' head( dat[,-which(names(dat) %in% cor.vars)] )
#'
#' \donttest{
#' # Evaluate linear correlations on nonlinear data
#' #   using nonlinear correlation function
#' plot(cor.data[[1]], pch=20) 
#'   collinear(cor.data[[1]], p=0.80, nonlinear = TRUE ) 	
#' }	       
#'
#' @export collinear
collinear <- function (x, p = 0.85, nonlinear = FALSE, p.value = 0.001) {
  if(!class(x)[1] == "matrix" && !class(x)[1] == "data.frame")
    stop("x does not appear to be a matrix or data.frame object") 
  if(any(sapply(x, is.numeric) == FALSE)) 
    stop("x contains non-numeric data") 
    if(is.null(colnames(x))) { 
      colnames(x) <- paste0("X", 1:ncol(x))
    }
  sd.check <- unlist(suppressWarnings(lapply(x, stats::sd)))
    sd.check <- which(sd.check == 0)
    if(length(sd.check) > 0) {
	  cn <- names(sd.check)
	  warning("removing columns: ", paste(cn, collapse=", "), 
	          ", due to zero variance") 
	  x <- x[,-which(names(x) %in% cn)] 
	}
  if(nonlinear == TRUE) {
    if(length(find.package("mgcv", quiet = TRUE)) == 0)
      stop("please install mgcv package for nonlinear option")  
    nlcor <- function(x, y, pv = 0.05) {
      g <- mgcv::gam(y ~ s(x))
        g.summ <- summary(g)
        if(g.summ$p.table[4] > pv) {
          nl.corr = 0
        } else {
          nl.corr = ( g$null.deviance - g$deviance ) / g$null.deviance  
        }
      return(nl.corr)
    }
    perm <- expand.grid(colnames(x), colnames(x))
      # perm <- perm[!duplicated(t(apply(perm, 1, sort))), ]
        perm <- perm[-which(perm[,1] == perm[,2]),]
	deletecol <- rep(FALSE,length(colnames(x)))
  	  names(deletecol) <- colnames(x)			
      for (i in 1:nrow(perm)) {
        x.name = as.vector(perm[,1])[i]
		y.name = as.vector(perm[,2])[i]
		message("evaluating ", x.name, " and ", y.name, "\n") 
        nl.cor <- nlcor(x[,grep(paste0("^", y.name, "$"), colnames(x))], 
		                x[,grep(paste0("^", x.name, "$"), colnames(x))], 
						pv = p.value )
		  if(nl.cor > p) {
            deletecol[grep(y.name, names(deletecol))] <- TRUE
            message("Nonlinear correlation between ", x.name, " and ", y.name,
                 " = ", nl.cor, "\n")
            message("   recommend dropping ", y.name, "\n", "\n")
          } 
        }
    deletecol <- unique(names(deletecol[deletecol==TRUE]))
      if( length(deletecol) < 1 )
        message("No nonlinear correlations found")	 
  } else if(nonlinear == FALSE) {
    x <- stats::cor(x)  
      diag(x) <- 0
    if (!isTRUE(all.equal(x, t(x)))) 
      stop("correlation matrix is not symmetric")  	
    if (!any(x[!is.na(x)] > p))
      stop("\r All correlations are <=", p, "\n")
    if (dim(x)[1] < 2) 
      stop("There is only one variable")
        x2 <- abs(x)
          originalOrder <- 1:ncol(x2)
            averageCorr <- function(x) mean(x, na.rm = TRUE)
              tmp <- x2
                diag(tmp) <- NA
                  maxAbsCorOrder <- order(apply(tmp, 2, averageCorr), 
  				                        decreasing = TRUE)
                  x2 <- x2[maxAbsCorOrder, maxAbsCorOrder]
                newOrder <- originalOrder[maxAbsCorOrder]
              rm(tmp)
        diag(x2) <- NA
    combine.vars <- expand.grid(colnames(x), colnames(x))
      combine.vars <- combine.vars[!duplicated(t(apply(combine.vars, 1, sort))), ]
        combine.vars <- combine.vars[-which(combine.vars[,1] == combine.vars[,2]),]
	deletecol <- rep(FALSE,length(colnames(x2)))
  	  names(deletecol) <- colnames(x)		
      for (i in 1:nrow(x2)) {
        i.name = rownames(x2)[i]
        if( deletecol[grep(i.name, names(deletecol))][1] == TRUE ) {
          next
        }
        for(j in (1:ncol(x2))[-i]){
          idx.names = c(i.name,colnames(x2)[j])
          if( deletecol[grep(colnames(x2)[j], names(deletecol))][1] == TRUE ) {
            next
          }  	
            if(x2[i, j] > p) {
              rc1 <- mean(x2[i, ], na.rm = TRUE)
              cc2 <- mean(x2[-j, ], na.rm = TRUE)
      	      deletecol[grep(idx.names[which.max(c(rc1,cc2))], names(deletecol))] <- TRUE
                message("Collinearity between ", idx.names[1], " and ", 
                  idx.names[2], " correlation = ", round(x2[i,j], 4), "\n")
                message("  Correlation means: ", round(rc1, 3), " vs ", round(cc2, 3), "\n")	
                message("   recommend dropping ", idx.names[which.max(c(rc1,cc2))], "\n", "\n")			
              } 
          } 
      }
	  deletecol <- unique(names(deletecol[deletecol==TRUE]))
    }
  return(deletecol)
}

#' @title Surface Relief Ratio
#' @description Calculates the Pike (1971) Surface Relief Ratio
#' 
#' @param x    A terra SpatRaster object
#' @param s    Focal window size
#' @param ...  Additional arguments passed to terra::lapp
#' 
#' @details 
#' Describes rugosity in continuous raster surface within a specified window. 
#' The implementation of SRR can be shown as: (mean(x) - min(x)) / (max(x) - min(x)) 
#'
#' @return A terra SpatRaster object of Pike's (1971) Surface Relief Ratio
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#' 
#' @examples 
#' \donttest{
#' library(terra)
#' elev <- rast(system.file("extdata/elev.tif", package="spatialEco"))
#'   r.srr <- srr(elev, s=5)
#'     plot(r.srr, main="Surface Relief Ratio") 
#'  }     
#' @export srr
srr <- function(x, s = 5, ...) {  
  if (!inherits(x, "SpatRaster")) 
	stop(deparse(substitute(x)), " must be a terra SpatRaster object")
  m <- matrix(1, nrow=s, ncol=s)
  rmin <- terra::focal(x, w=m, fun=min)
    rmax <- terra::focal(x, w=m, fun=max)
	  rmean <- terra::focal(x, w=m, fun=mean)
  return( terra::lapp(c(rmean,rmin,rmax), 
    fun=function(x,y,z) { (x - y ) / ( z - y ) }, ... ) )
} 

#' @title Sample annulus
#' @description Creates sample points based on annulus with defined 
#' inner and outer radius
#' 
#' @param x      An sf POINT class object
#' @param r1     Numeric value defining inner radius of annulus 
#'               (in projection units)
#' @param r2     Numeric value defining outer radius of annulus 
#'               (in projection units)
#' @param size   Number of samples
#' @param ...    Additional arguments passed to sf::st_sample
#'
#' @details
#' Function can be used for distance based sampling which is a sampling method 
#' that can be used to capture spatially lagged variation.
#'
#' @return sf POINTS object
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com> 
#'
#' @examples
#'  library(sf)
#'  if(require(sp, quietly = TRUE)) {
#'    data(meuse, package = "sp")
#'    meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, 
#'                      agr = "constant")
#' 
#'  xy <- meuse[2,]
#'  rs100 <- sample.annulus(xy, r1=50, r2=100, size = 50)
#'  rs200 <- sample.annulus(xy, r1=100, r2=200, size = 50)
#'  
#'  plot(st_geometry(rs200), pch=20, col="red")
#'    plot(st_geometry(rs100), pch=20, col="blue", add=TRUE)
#'    plot(st_geometry(xy), pch=20, cex=2, col="black", add=TRUE)
#'  legend("topright", legend=c("50-100m", "100-200m", "source"), 
#'         pch=c(20,20,20), col=c("blue","red","black"))
#' 
#' \donttest{
#' # Run on multiple points
#' rs100 <- sample.annulus(meuse[1:3,], r1=50, r2=100, 
#'                         size = 50)
#' rs200 <- sample.annulus(meuse[1:3,], r1=50, r2=200, 
#'                         size = 50)
#' plot(st_geometry(rs200), pch=20, col="red")
#'   plot(st_geometry(rs100), pch=20, col="blue", add=TRUE)
#'     plot(st_geometry(meuse[1:3,]), pch=20, cex=2, col="black", add=TRUE)
#'  legend("topright", legend=c("50-100m", "100-200m", "source"), 
#'         pch=c(20,20,20), col=c("blue","red","black"))
#' }
#' } else { 
#'   cat("Please install sp package to run example", "\n")
#' }
#'
#' @export sample.annulus
sample.annulus <- function(x, r1, r2, size = 10, ...) {
  if(!inherits(x, c("sf", "sfc")))		
    stop(deparse(substitute(x)), " must be an sf or sfc object")
 if(!unique(as.character(sf::st_geometry_type(x))) %in% c("POINT", "MULTIPOINT"))
    stop(deparse(substitute(x)), " must be an sf POLYGON object")		
  if(unique(as.character(sf::st_geometry_type(x))) %in% "MULTIPOINT")
    stop("Function does not support multi-part MULTIPOINT objects")		
  if(r1 >= r2) stop("inner radius (r1) must be smaller than outer (r2)")
    dots <- as.list(match.call(expand.dots = TRUE)[-1])
  if (is.null(dots[["type"]]) & "type" %in% names(dots) == FALSE) dots[["type"]] <-  "random"
    if (is.null(dots[["size"]]) & "size" %in% names(dots) == FALSE) dots[["size"]] <- 10
  c1 <- sf::st_buffer(x, dist = r1)
  c2 <- sf::st_buffer(x, dist = r2)
  annulus <- sf::st_difference(c2, c1)
	dots[["x"]] <- annulus
	s <- sf::st_as_sf(do.call(sf::st_sample, dots))
	  sf::st_geometry(s) <- "geometry"
	  s$SID = 1:length(s)
	  s$PID = rownames(x[1,])
  return( s )
}

#' @title Partial and Semi-partial correlation
#' @description Calculates a partial or semi-partial correlation  
#'              with parametric and nonparametric options
#'
#' @param x          A vector, data.frame or matrix with 3 columns
#' @param y          A vector same length as x 
#' @param z          A vector same length as x
#' @param method     Type of correlation: "partial" or "semipartial" 
#' @param statistic  Correlation statistic, options are: "kendall", 
#'                   "pearson", "spearman" 
#'
#' @details
#' Partial and semipartial correlations show the association between two 
#' variables when one or more peripheral variables are controlled 
#' to hold them constant. 
#'
#' Suppose we have three variables, X, Y, and Z. Partial correlation holds 
#' constant one variable when computing the relations two others. Suppose we 
#' want to know the correlation between X and Y holding Z constant for both 
#' X and Y. That would be the partial correlation between X and Y controlling 
#' for Z. Semipartial correlation holds Z constant for either X or Y, but not 
#' both, so if we wanted to control X for Z, we could compute the semipartial 
#' correlation between X and Y holding Z constant for X.
#'
#' @return data.frame containing:
#' * correlation - correlation coefficient
#' * p.value - p-value of correlation
#' * test.statistic - test statistic
#' * n - sample size
#' * Method - indicating partial or semipartial correlation
#' * Statistic - the correlation statistic used
#' @md
#' 
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @examples
#' air.flow = stackloss[,1]
#' water.temperature = stackloss[,2]
#' acid = stackloss[,3]
#' 
#' # Partial using Kendall (nonparametric) correlation
#' partial.cor(air.flow, water.temperature, acid)
#' 
#' scholar <- data.frame(
#'   HSGPA=c(3.0, 3.2, 2.8, 2.5, 3.2, 3.8, 3.9, 3.8, 3.5, 3.1), 
#' 	 FGPA=c(2.8, 3.0, 2.8, 2.2, 3.3, 3.3, 3.5, 3.7, 3.4, 2.9),
#'   SATV =c(500, 550, 450, 400, 600, 650, 700, 550, 650, 550)) 
#' 
#' # Standard Pearson's correlations between HSGPA and FGPA  
#' cor(scholar[,1], scholar[,2])
#'
#' # Partial correlation using Pearson (parametric) between HSGPA 
#' #   and FGPA, controlling for SATV
#' partial.cor(scholar, statistic="pearson")
#' 
#' # Semipartial using Pearson (parametric) correlation 
#' partial.cor(x=scholar[,2], y=scholar[,1], z=scholar[,3], 
#'             method="semipartial", statistic="pearson")
#'
#' @export partial.cor 
partial.cor <- function (x, y, z, method = c("partial", "semipartial"),
                         statistic = c("kendall", "pearson", "spearman")) {
  method <- match.arg(method)
  statistic <- match.arg(statistic)  
  if(any(class(x)[1] == c("data.frame", "matrix")) ) {
    if(!missing(y))
	  stop("if x, y and z are defined, x must be a vector")
    if(dim(x)[2] > 3)
	  stop("Only 3 variables are supported for partial correlation")
    xyz <- stats::na.omit(as.matrix(x))
  } else {
    if(missing(y) | missing(z))
	  stop("Must have x, y and z vectors defined for partial correlation")
    e = sapply(list(x[!is.na(x)],y[!is.na(y)],z[!is.na(z)]), length)
      e = range(e) / mean(e)
    if(!isTRUE(all.equal(e[1], e[2])))
      stop("x, y, and z vectors must be same length, check for NA's")    
    xyz <- stats::na.omit(cbind(x, y, z))
  }
  if (!(is.numeric(xyz) || is.logical(xyz))) 
    stop("x must be numeric")
  stopifnot(is.atomic(xyz))   
  pcor.fun <- function (x, method) {
      n <- dim(x)[1]
      gp <- dim(x)[2] - 2
      cvx <- stats::cov(x, method = method)
      if (det(cvx) < .Machine$double.eps) {
        warning("The inverse of variance-covariance matrix is being calculated 
		         using the Moore-Penrose generalized matrix inverse due to a 
				 determinant of zero in the covariance")
      icvx <- MASS::ginv(cvx)
      } else { 
  	  icvx <- solve(cvx)
  	}
      pcor <- -stats::cov2cor(icvx)
        diag(pcor) <- 1
      if (method == "kendall") {
        statistic <- pcor/sqrt(2 * (2 * (n - gp) + 5)/(9 * (n - gp) * (n - 1 - gp)))
          p.value <- 2 * stats::pnorm(-abs(statistic))
      } else {
        statistic <- pcor * sqrt((n - 2 - gp) / (1 - pcor^2))
          p.value <- 2 * stats::pt(-abs(statistic), (n - 2 - gp))
      }
        diag(statistic) <- 0
      diag(p.value) <- 0
    return( list(estimate = pcor, p.value = p.value, 
	             statistic = statistic, n = n) )
  }
  spcor.fun <- function (x, method) {
    x <- as.matrix(x)
      n <- dim(x)[1]
        gp <- dim(x)[2] - 2
          cvx <- stats::cov(x, method = method)
    if(det(cvx) < .Machine$double.eps) {
      warning("The inverse of variance-covariance matrix is being calculated 
	          using the Moore-Penrose generalized matrix inverse due to a 
	  		  determinant of zero in the covariance")      
	  icvx <- MASS::ginv(cvx)
    } else { 
  	  icvx <- solve(cvx)
  	}
    spcor <- -stats::cov2cor(icvx) / sqrt(diag(cvx)) / sqrt(abs(diag(icvx) - 
	          t(t(icvx^2) / diag(icvx))))
        diag(spcor) <- 1
      if (method == "kendall") {
        statistic <- spcor/sqrt(2 * (2 * (n - gp) + 5)/(9 * (n - gp) * (n - 1 - gp)))
          p.value <- 2 * stats::pnorm(-abs(statistic))
      } else {
        statistic <- spcor * sqrt((n - 2 - gp)/(1 - spcor^2))
          p.value <- 2 * stats::pt(-abs(statistic), (n - 2 - gp))
      }
          diag(statistic) <- 0
        diag(p.value) <- 0
    return( list(estimate = spcor, p.value = p.value, 
  	             statistic = statistic, n = n,  
                 method = method) )
  }
  if(method == "partial") {	
    pcor <- pcor.fun(xyz, method = statistic)
  } else if(method == "semipartial") { 
    pcor <- spcor.fun(xyz, method = statistic) 
  } 
  return(data.frame(correlation = pcor$est[1, 2], 
                    p.value = pcor$p.value[1, 2], 
	                test.statistic = pcor$statistic[1, 2], 
					n = pcor$n, Method = method, 
                    Statistic = statistic) )
}

#' @title Inhomogeneous O-ring 
#' @description Calculates the inhomogeneous O-ring point pattern statistic 
#'             (Wiegand & Maloney 2004) 
#'    
#' @param x              spatstat ppp object
#' @param inhomogeneous  (FALSE/TRUE) Run homogeneous (pcf) or inhomogeneous 
#'                       (pcfinhom)  
#' @param ...            additional arguments passed to pcf or pcfinhom 
#'
#' @details
#' The function K(r) is the expected number of points in a circle of radius r centered 
#' at an arbitrary point (which is not counted), divided by the intensity l of the pattern. 
#' The alternative pair correlation function g(r), which arises if the circles of 
#' Ripley's K-function are replaced by rings, gives the expected number of points at 
#' distance r from an arbitrary point, divided by the intensity of the pattern. Of special 
#' interest is to determine whether a pattern is random, clumped, or regular. 
#'
#' Using rings instead of circles has the advantage that one can isolate specific 
#' distance classes, whereas the cumulative K-function confounds effects at larger 
#' distances with effects at shorter distances. Note that the K-function and the O-ring 
#' statistic respond to slightly different biological questions. The accumulative 
#' K-function can detect aggregation or dispersion up to a given distance r and is 
#' therefore appropriate if the process in question (e.g., the negative effect of 
#' competition) may work only up to a certain distance, whereas the O-ring statistic 
#' can detect aggregation or dispersion at a given distance r. The O-ring statistic 
#' has the additional advantage that it is a probability density function (or a 
#' conditioned probability spectrum) with the interpretation of a neighborhood 
#' density, which is more intuitive than an accumulative measure.
#'
#' @return plot of o-ring and data.frame with plot labels and descriptions  
#' 
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'    
#' @references
#' Wiegand T., and K. A. Moloney (2004) Rings, circles and null-models for point 
#' pattern analysis in ecology. Oikos 104:209-229
#'
#' @examples
#' if (require(spatstat.explore, quietly = TRUE)) {
#' data(lansing)
#'   x  <- spatstat.geom::unmark(split(lansing)$maple)
#'   o.ring(x)
#'
#' } else { 
#'   cat("Please install spatstat.explore package to run example", "\n")
#' }
#'
#' @export
o.ring <- function(x, inhomogeneous = FALSE, ...) {
  if( inhomogeneous ) { 
      g <- spatstat.explore::pcfinhom(x, ...)
    } else {
	  g <- spatstat.explore::pcf(x, ...)
	}
      lambda <- summary(x)$intensity
    O <- spatstat.explore::eval.fv(lambda * g)
  graphics::plot(O, ylab = "O-ring(r)", main = "O-ring")
}

#' @title Dissection
#' @description Calculates the Evans (1972) Martonne's modified 
#'              dissection 
#' 
#' @param x    A terra SpatRaster class object
#' @param s    Focal window size
#' @param ...  Additional arguments passed to terra::lapp
#' 
#' @details 
#' Dissection is calculated as: 
#' ( z(s) - min(z(s)) ) / ( max(z(s)) - min(z(s)) ) 
#'
#' @return A SpatRaster class object of Martonne's modified dissection  
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#' 
#' @examples
#' \donttest{ 
#' library(terra)
#' elev <- rast(system.file("extdata/elev.tif", package="spatialEco"))
#'   d <- dissection(elev, s=3)
#'     plot(d, main="dissection") 
#' }     
#'
#' @export dissection
dissection <- function(x, s=5, ...) {  
  if (!inherits(x, "SpatRaster")) 
    stop(deparse(substitute(x)), " must be a terra SpatRaster object")		
    if( length(s) == 1) s = c(s[1],s[1])
       m <- matrix(1, nrow=s, ncol=s)
    rmin <- terra::focal(x, w=m, fun=min)
    rmax <- terra::focal(x, w=m, fun=max)
    return( terra::lapp(c(x,rmin,rmax), fun = function(x,y,z) { (x - y ) / 
	        ( z - y ) }, ... )   )
}  

#' @title Average Nearest Neighbor Index (NNI)
#' @description Calculates the NNI as a measure of clustering or dispersal 
#'
#' @param x An sf point object
#' @param win Type of window 'hull' or 'extent'
#'
#' @details
#' The nearest neighbor index is expressed as the ratio of the observed distance 
#' divided by the expected distance. The expected distance is the average distance 
#' between neighbors in a hypothetical random distribution. If the index is less than 1, 
#' the pattern exhibits clustering; if the index is greater than 1, the trend is toward 
#' dispersion or competition. The Nearest Neighbor Index is calculated as:
#'   * Mean Nearest Neighbor Distance (observed) D(nn) = sum(min(Dij)/N) 
#'   * Mean Random Distance (expected) D(e) = 0.5 SQRT(A/N)
#'   * Nearest Neighbor Index NNI = D(nn)/D(e)
#'       Where; D=neighbor distance, A=Area
#' @md
#'
#' @return 
#' list object containing NNI = nearest neighbor index, z.score = Z 
#' Score value, p = p value, expected.mean.distance = Expected mean 
#' distance, observed.mean.distance = Observed meand distance.
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @references 
#' Clark, P.J., and F.C. Evans (1954) Distance to nearest neighbour as a measure 
#'   of spatial relationships in populations. Ecology 35:445-453
#'  
#' Cressie, N (1991) Statistics for spatial data. Wiley & Sons, New York.
#'
#' @examples 
#'
#' p = c("sf", "sp")
#'   if(any(!unlist(lapply(p, requireNamespace, quietly=TRUE)))) { 
#'     m = which(!unlist(lapply(p, requireNamespace, quietly=TRUE)))
#'     message("Can't run examples, please install ", paste(p[m], collapse = " "))
#'   } else {
#'   invisible(lapply(p, require, character.only=TRUE))
#' 
#'   data(meuse, package = "sp")
#'   meuse <- sf::st_as_sf(meuse, coords = c("x", "y"),  
#'                         crs = 28992, agr = "constant")
#'   nni(meuse)
#'   }
#'
#' @export
nni <- function(x, win = c("hull", "extent")) {
  if(!inherits(x, "sf"))		
    stop(deparse(substitute(x)), " must be an sf POINT object")	
  if(unique(as.character(sf::st_geometry_type(x))) != "POINT")
    stop(deparse(substitute(x)), " must be an sf POINT object")		
    if (win[1] == "hull") {
      w <- spatstat.geom::convexhull.xy( sf::st_coordinates(x)[,1:2] )
    }
    if (win[1] == "extent") {
      e <- as.vector(sf::st_bbox(x))
      w <- spatstat.geom::as.owin(c(e[1], e[3], e[2], e[4]))
    }
    x <- spatstat.geom::as.ppp(sf::st_coordinates(x)[,1:2], w)
    A <- spatstat.geom::area.owin(w)
      obsMeanDist <- sum(spatstat.geom::nndist(x))/x$n
      expMeanDist <- 0.5 * sqrt(A / x$n)
      se <- 0.26136 / ((x$n**2.0 / A)**0.5)
      nni <- obsMeanDist / expMeanDist
      z <- (obsMeanDist - expMeanDist) / se
    return(list(NNI = nni, z.score = z, p = 2*stats::pnorm(-abs(z)),  
	       expected.mean.distance = expMeanDist,
		   observed.mean.distance = obsMeanDist))
}

#' @title spatialEco news
#' @description Displays release notes
#' @param ...  not used
#' @return Shows package NEWS file
#' @export
spatialEcoNews <- function(...) {
    newsfile <- file.path(system.file(package="spatialEco"), "NEWS")
    file.show(newsfile)
}

#' @title Breeding density areas (aka, core habitat areas)
#' @description Calculates breeding density areas base on population counts and 
#'              spatial point density.
#' 
#' @param x        sf POINT object
#' @param pop      Population count/density column in x 
#' @param p        Target percent of population 
#' @param bw       Bandwidth distance for the kernel estimate (default 8500) 
#' @param b        Buffer distance (default 8500)     
#' @param self     (TRUE/FALSE) Should source observations be included in 
#'                 density (default TRUE)
#'
#' @details
#' The breeding density areas model identifies the Nth-percent population exhibiting 
#' the highest spatial density and counts/frequency. It then buffers these points by 
#' a specified distance to produce breeding area polygons. If you would like to recreate 
#' the results in Doherty et al., (2010), then define bw = 6400m and b[if p < 0.75 
#' b = 6400m, | p >= 0.75 b = 8500m]  
#'
#' @return A list object with:
#' \itemize{ 
#' \item pop.pts   sf POINT object with points identified within the specified p
#' \item pop.area  sf POLYGON object of buffered points specified by parameter b
#' \item bandwidth Specified distance bandwidth used in identifying neighbor counts 
#' \item buffer    Specified buffer distance used in buffering points for pop.area  
#' \item p         Specified population percent
#' }
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @references
#' Doherty, K.E., J.D. Tack, J.S. Evans, D.E. Naugle (2010) Mapping breeding densities of 
#'   greater  sage-grouse: A tool for range-wide conservation planning.  
#'   Bureau of Land Management. Number L10PG00911
#'                                                              
#' @examples 
#' library(sf)
#' 
#' n=1500
#' bb <- rbind(c(-1281299,-761876.5),c(1915337,2566433.5))
#' bb.mat <- round(cbind(c(bb[1,1], bb[1,2], bb[1,2], bb[1,1]),
#'                   c(bb[2,1], bb[2,1], bb[2,2], bb[2,2])),2)
#'  bbp <- st_sfc(st_polygon(list(rbind(bb.mat, bb.mat[1,]))))
#'    pop <- st_as_sf(st_sample(bbp, n, type = "random"))
#'	  st_geometry(pop) <- "geometry"
#'      pop$ID <- 1:nrow(pop)
#'	  pop$counts <- round(runif(nrow(pop), 1,250),0)
#'    
#'     bd75 <- breeding.density(pop, pop='counts', p=0.75, b=8500, bw=6400)	 
#'       plot(st_geometry(bd75$pop.area), border = NA,  
#'	        main='75% breeding density areas', col="grey")
#'          plot(st_geometry(pop), pch=20, col='black', add=TRUE)
#'          plot(st_geometry(bd75$pop.pts), pch=20, col='red', add=TRUE)
#'       legend("bottomright", legend=c("selected areas","selected sites", "all sites"),
#'              bg="white", fill=c("grey","red", "black"), pt.cex = 2) 
#'
#' @export
breeding.density <- function(x, pop, p = 0.75, bw = 6400, 
                             b = 8500, self = TRUE) {
  if (!inherits(x, c("SpatialPointsDataFrame", "SpatialPoints", "sf", "sfc", "matrix")))		
    stop(deparse(substitute(x)), " x must be a spatial (sp, df) or matrix object")
  if(inherits(x, c("SpatialPointsDataFrame", "SpatialPoints"))) {
     x <- sf::st_as_sf(x)
  }
  if(as.character(unique(sf::st_geometry_type(x))) != "POINT")
    stop(deparse(substitute(x)), " x must be an sf POINT object")
  if(is.na(match(pop, names(x)))) 
    stop("Count/density field not present in data")
    point.density <- function(x, bw, self = TRUE) {
        d <- sf::st_distance(x, x)
		  if(inherits(d, "units")) 
		    d <- units::drop_units(d)
        if (self == TRUE) {
            diag(d) <- 1
        } else {
            diag(d) <- NA
        }
        den <- vector()
        for (i in 1:nrow(d)) {
          di <- stats::na.omit(d[i, ])
          den <- append(den, length(di[di <= bw]))
        }
      den
    }
    pn <- which(names(x) == pop)
    pop.counts <- sf::st_drop_geometry(x[,pn]) 
	pop.n <- sum(pop.counts) * p
    d <- (point.density(x, bw = bw, self = self)) * pop.counts
    pop.den <- data.frame(sf::st_coordinates(x)[,1:2], pop.counts, d)
      names(pop.den) <- c("x", "y", "pop", "pden")
    pop.den <- pop.den[order(-pop.den$pden), ]
      i <- 0
      j <- 0
    pts <- as.data.frame(array(0, dim = c(0, 4)))
      while (i <= pop.n) {
          j <- j + 1
          i <- i + pop.den[, "pop"][j]
          pts <- rbind(pts, pop.den[j, ])
      }
    pts <- sf::st_as_sf(pts, coords = c("x", "y"), agr = "constant")    
	  if(!is.na(sf::st_crs(x))) {
	    sf::st_crs(pts) <- sf::st_crs(x) 
	  }	 		
	pop.buff <- sf::st_buffer(pts, dist=b,
                           nQuadSegs = 30,
                           endCapStyle = "ROUND",
                           joinStyle = "ROUND",
                           mitreLimit = 1)
  return( list(pop.pts = pts, pop.area = pop.buff, bandwidth = bw, 
          buffer = b, p = p) )	
} 

#' @title Local minimum and maximum
#' @description 
#' Calculates the local minimums and maximums in a numeric vector, 
#' indicating inflection points in the distribution.  
#' 
#' @param x A numeric vector 
#' @param dev Deviation statistic (mean or median)
#' @param plot plot the minimum and maximum values with the 
#'             distribution (TRUE/FALSE)
#' @param add.points Should all points of x be added to 
#'                   plot (TRUE/FALSE)
#' @param ... Arguments passed to plot
#' 
#' @details  
#' Useful function for identifying inflection or enveloping points in 
#' a distribution
#'
#' @return A list object with:  
#' * minima - minimum local values of x
#' * maxima - maximum local values of x
#' * mindev - Absolute deviation of minimum from specified deviation 
#'          statistic (dev argument) 
#' * maxdev - Absolute deviation of maximum from specified deviation 
#'          statistic (dev argument) 
#' @md
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @examples                                                  
#' x <- rnorm(100,mean=1500,sd=800) 
#' ( lmm <- local.min.max(x, dev=mean, add.points=TRUE, 
#'                        main="Local Minima and Maxima") )
#'
#' # return only local minimum values
#'    local.min.max(x)$minima 
#'                                            
#' @export
local.min.max <- function(x, dev=mean, plot=TRUE, add.points=FALSE,  ...) {
  x <- stats::na.omit(x)
    r <- rle(x) 
  minima <- which(rep(x=diff(sign(diff(c(-Inf, r$values, -Inf)))) == 2, times=r$lengths))
  maxima <- which(rep(x=diff(sign(diff(c(-Inf, r$values, -Inf)))) == -2, times=r$lengths)) 
    if (plot == TRUE) {				 
      plot(x,type="l", ...)
        graphics::points(x[minima]~minima,pch=19,col="blue") 
          graphics::points(x[maxima]~maxima,pch=19,col="red")
		    graphics::abline(h=dev(x, na.rm=TRUE), col="grey")
		if (add.points == TRUE) graphics::points(x, col="grey")
	  graphics::legend("topleft", legend=c("Minima","Maxima"), pch=c(19,19), 
	                   col=c("blue","red"), bg="white")
    }
	return( list(minima=x[minima], maxima=x[maxima],
				 devmin=abs(dev(x) - x[minima]), 
				 devmax=abs(dev(x) - x[maxima])) )
 }	
 

#' @title Hierarchical Slope Position
#' @description 
#' Calculates a hierarchical scale decomposition of topographic 
#' position index  
#'                                                                       
#' @param x            A terra SpatRaster class object 
#' @param min.scale    Minimum scale (window size)
#' @param max.scale    Maximum scale (window size)
#' @param inc          Increment to increase scales
#' @param win          Window type, options are "rectangle" or "circle"
#' @param normalize    Normalize results to 0-1 scale (FALSE | TRUE)          
#'    
#' @details
#' if win  = "circle" units are distance, if win = "rectangle" units 
#' are number of cells 
#'   
#' @return  terra SpatRaster class object of slope position 
#'     
#' @references
#' Murphy M.A., J.S. Evans, and A.S. Storfer (2010) Quantify Bufo boreas 
#'   connectivity in Yellowstone National Park with landscape genetics. 
#'   Ecology 91:252-261
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'    
#' @examples
#' \donttest{ 
#'   library(terra)
#'   elev <- rast(system.file("extdata/elev.tif", package="spatialEco"))
#'   hsp27 <- hsp(elev, 3, 27, 4, normalize = TRUE)
#'   plot(hsp27)
#'  }
#'
#' @export  
hsp <- function(x, min.scale = 3, max.scale = 27, inc = 4, win = "rectangle",
                normalize = FALSE) { 
  if (!inherits(x, "SpatRaster")) 
    stop(deparse(substitute(x)), " must be a terra SpatRaster class object") 
  scales = rev(seq(from=min.scale, to=max.scale, by=inc)) 
    for(s in scales) {
	  if( win == "circle") {
	    if( min.scale < terra::res(x)[1] * 2) 
		  stop( "Minimum resolution is too small for a circular window")
			m <- terra::focalMat(x, s, type=c('circle'))
              m[m > 0] <- 1  
          } else { 	  
        m <- matrix(1, nrow=s, ncol=s)
	  }
	message("Calculating scale:", s, "\n")
        scale.r <- x - terra::focal(x, w=m, fun=mean)
	  if( s == max(scales) ) {
        scale.r.norm <- 100 * ( (scale.r - 
		     terra::global(scale.r, "mean", na.rm=TRUE)[,1] / 
             terra::global(scale.r, "sd", na.rm=TRUE)[,1] ) )
	    } else {
	    scale.r.norm <-  scale.r.norm + 100 * ( (scale.r - 
		     terra::global(scale.r, "mean", na.rm=TRUE)[,1] / 
             terra::global(scale.r, "sd", na.rm=TRUE)[,1] ) )   
	  }   			 
    }
  if(normalize == TRUE) {  
 	scale.r.norm <- (scale.r.norm - terra::global(scale.r.norm, "min", na.rm=TRUE)[,1]) /
                    (terra::global(scale.r.norm, "max", na.rm=TRUE)[,1] - 
					 terra::global(scale.r.norm, "min", na.rm=TRUE)[,1])				 			 
  }
  return(scale.r.norm)  
}

#' @title Trigonometric  transformation of a slope and aspect interaction 
#' @description The Trigonometric  Stage (1978) 
#'              [slope * cos(aspect)] or [slope * sin(aspect)] 
#'
#' @param slope       slope values in degrees, radians or percent 
#' @param aspect      aspect values in degrees or radians
#' @param type        Type of transformation, options are: "cos", "sin" 
#' @param slp.units   Units of slope values, options are: "degrees", 
#'                    "radians" or "percent"
#' @param asp.units   Units of aspect values, options are: 
#'                   "degrees" or "radians"
#'
#' @details 
#' An a priori assumption of a maximum in the NW quadrant (45 azimuth)
#' and a minimum in the SW quadrant can be replaced by an empirically
#' determined location of the optimum without repeated calculations of
#' the regression fit. In addition it is argued that expressions for
#' the effects of aspect should always be considered as terms involving
#' an interaction with slope (Stage, 1976)
#' 
#' For slopes from 0% - 100%, The functions are linearized and
#' bounded from -1 to 1. Greater than 100% slopes are treated
#' out of the -1 to 1 range.
#' 
#' An alternative for slopes with values approaching infinity is
#' to take the square root of slope/100 to reduce the range of
#' values.By default this model test all values greater than 100%
#' to 101% and flat areas (-1) to nodata.
#'
#' @return A vector of the modeled value
#'
#' @author Jeffrey S. Evans  <jeffrey_evans@@tnc.org>
#'
#' @references 
#' Stage, A. R. 1976. An Expression of the Effects of Aspect, Slope, 
#'   and Habitat Type on Tree Growth. Forest Science 22(3):457-460.
#'  
#' @examples 
#' library(terra)
#' sa.trans(slope = 48.146, aspect = 360.000)
#'
#' # Example of creating slope*cos(aspect) raster
#' elev <- rast(system.file("extdata/elev.tif", package="spatialEco"))
#' sa <- terra::terrain(elev, v=c("slope", "aspect"), unit="degrees")
#' scosa <- terra::lapp(c(sa[[1]], sa[[2]]), fun = sa.trans)
#'
#' @export
sa.trans <- function(slope, aspect, type = "cos", slp.units = "degrees", 
                       asp.units = "degrees") {			  
  switch(slp.units, 
    degrees = { slope <- ( slope / 0.572957795786 ) * 0.01 },
    radians = { slope <- ((slope * (180 / pi)) / 0.572957795786 ) * 0.01 },
    percent = { slope <- slope * 0.01 })
  if(asp.units == "degrees") { aspect <- aspect * (pi / 180) }
	slope[slope > 1] <- 1.001
    slope[slope <= 0] <- NA
    trans <- as.vector(t(apply(data.frame(slope, aspect), 1, 
	           function (x){ return ( x[1] * cos(x[2]) ) } )))
    return(trans)
  }

#' @title Sobel-Feldman operator
#' @description An isotropic image gradient operator using a 3x3 window
#' 
#' @param x            A raster class object
#' @param method       Type of operator ("intensity", "direction", "edge")
#' @param ...          Additional arguments passed to raster::overlay or, 
#'                     if method="edge", raster::focal (if you want a file 
#'                      written to disk use filename = "" argument)
#'
#' @details
#' The Sobel-Feldmanh operator is a discrete differentiation operator, deriving an 
#' approximation of the gradient of the intensity function. abrupt discontinuity 
#' in the gradient function represents edges, making this a common approach for edge 
#' detection. The Sobel-Feldman operator is based on convolving the image with a small, 
#' separable, and integer matrix in the horizontal and vertical directions. The operator 
#' uses two 3x3 kernels which are convolved with the original image to calculate 
#' approximations of the derivatives - one for horizontal changes, and one for vertical. 
#' Where x is defined here as increasing in the right-direction, and y as increasing in 
#' the down-direction. At each pixel in the raster, the resulting gradient can be combined 
#' to give the gradient intensity, using: SQRT( Gx^2 Gy^2 ). This can be expanded into the 
#' gradient direction using atan(Gx/Gy)
#'
#' @return A raster class object or raster written to disk 
#' 
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'                                                                           
#' @references
#' Sobel, I., & G. Feldman, (1969) A 3x3 Isotropic Gradient Operator for Image Processing, 
#'   presented at the Stanford Artificial Intelligence Project (SAIL). 
#' 
#' @examples
#' library(terra)
#' 
#' r <- rast(system.file("ex/logo.tif", package="terra"))  
#'   s.int <- sobal(r[[1]])
#'   s.dir <- sobal(r[[1]], method = "direction")
#'   s.edge <- sobal(r[[1]], method = "edge")
#'
#' opar <- par(no.readonly=TRUE)
#' par(mfrow=c(2,2))
#'   plot(r[[1]])
#'   plot(s.int, main="intensity") 
#'   plot(s.dir, main="direction") 
#'   plot(s.edge, main="edge")
#' par(opar)   
#'
#' @export sobal
sobal <- function(x, method = "intensity", ...) {
  if (!inherits(x, "SpatRaster")) 
    stop(deparse(substitute(x)), " must be a terra SpatRaster object")
  sobal.y <- matrix( c(-1, 0, 1, -2, 0, 2, -1, 0, 1), nrow=3, ncol=3)
  sobal.x <- matrix( c(-1, -2, -1, 0, 0, 0, 1, 2, 1), nrow=3, ncol=3)
    if (method == "direction") {
	  Gx = terra::focal(x, w=sobal.x, "sum")
      Gy = terra::focal(x, w=sobal.y, "sum")
      return( terra::lapp(c(Gy, Gx), fun=atan2) )
    } else if (method == "intensity") {
	  Gx = terra::focal(x, w=sobal.x, "sum")
      Gy = terra::focal(x, w=sobal.y, "sum")
      return( terra::lapp(c(Gx, Gy), fun=function(x, y) { sqrt( x^2 + y^2) }) )
    } else {
	  return( terra::focal(x, w=(sobal.y / 4), "sum") )
	}
}

#' @title Bounding box polygon
#'
#' @description Creates a polygon from a vector or raster extent
#' 
#' @param x    An sf or terra object or vector of bounding coordinates 
#'
#' @details 
#' If not a spatial object, expected order of input for x is: xmin, ymin, 
#' xmax, ymax. Where; xmin, ymin and the coordinates of top left corner of the 
#' bounding box and xmax, ymax represent the bottom right corner. The maximum 
#' value of xmax is width of the extent while maximum value of ymax is the height 
#' of the extent.
#' 
#' @return A single feature sf class polygon object
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @examples
#' if(require(sp, quietly = TRUE)) {
#' library(terra)
#' library(sf)
#'   data(meuse, package = "sp")
#'   meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, 
#'                     agr = "constant")
#' 
#' # raster (terra)
#' r <- rast(ext(meuse))
#'   r[] <- runif(ncell(r))
#'  crs(r) <- "epsg:28992"
#' e <- bbox_poly(r)
#' 
#' plot(r)
#'   plot(st_geometry(e), border="red", add=TRUE)
#' 
#' # extent vector
#' e <- bbox_poly(c(178605, 329714, 181390, 333611)) 
#'   plot(e)
#' 
#' # vector bounding box
#' e <- bbox_poly(meuse)
#' 
#' plot(st_geometry(meuse), pch=20)
#'   plot(st_geometry(e), add=TRUE)
#' 
#' } else { 
#'   cat("Please install sp package to run this example", "\n")
#' }
#'
#' @export bbox_poly
bbox_poly <- function(x) {
  if(inherits(x, "SpatRaster")) {
    e <- as.vector(terra::ext(x))[c(1,3,2,4)] 
      e <- sf::st_as_sfc(sf::st_bbox(c(e[1], e[2], e[3], e[4])))
	    p <- terra::crs(x)
		if(! p == "") 
	      sf::st_crs(e) <- sf::st_crs(p) 
  } else if(inherits(x, c("sf", "sfc"))) {
     e <- sf::st_as_sfc(sf::st_bbox(x))
       sf::st_crs(e) <- sf::st_crs(x)   
  } else if(inherits(x, "numeric")) {
    names(x) <- c("xmin", "ymin", "xmax", "ymax")
    e <- sf::st_as_sfc(sf::st_bbox(c(x[1], x[2], x[3], x[4])))
  }
  return(sf::st_as_sf(e))
}

#' @title DAYMET point values
#' @description Downloads DAYMET climate variables for specified point and time-period 
#' 
#' @param lat          latitude of point (decimal degrees WGS84)
#' @param long         longitude pf point (decimal degrees WGS84)
#' @param start.year   First year of data
#' @param end.year     Last year of data
#' @param site         Unique identification value that is appended to data
#' @param files       (TRUE/FALSE) Write file to disk
#' @param echo        (TRUE/FALSE) Echo progress
#'
#' @details 
#' data is available for Long -131.0 W and -53.0 W; lat 52.0 N and 14.5 N
#' Function uses the Single Pixel Extraction tool and returns year, yday, 
#'       dayl(s), prcp (mm/day), srad (W/m^2), swe (kg/m^2), tmax (deg c), 
#'       tmin (deg c), vp (Pa)
#' Metadata for DAYMET single pixel extraction: 
#' \url{ https://daymet.ornl.gov/files/UserGuides/current/readme_singlepointextraction.pdf }
#' 
#' @return A data.frame with geographic coordinate point-level climate results 
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'         
#' @examples
#' \donttest{
#' ( d <- daymet.point(lat = 36.0133, long = -84.2625, start.year = 2013, 
#'                     end.year=2014, site = "1", files = FALSE, echo = FALSE) )
#'}
#'
#' @export
daymet.point <- function (lat, long, start.year, end.year, site=NULL, 
                          files = FALSE, echo = FALSE) {
	if(length(find.package("RCurl", quiet = TRUE)) == 0)
      stop("please install RCurl package before running this function")
    if(missing(lat)) stop("Please define lat") 
    if(missing(long)) stop("Please define long") 
	if(missing(start.year)) stop("Please define start year") 
    if(missing(end.year)) stop("Please define end year") 
    if (start.year < 1980) stop("Data not available prior to 1980")
	if (start.year > 2015) stop("Data not available after 2015")
    year.range = paste(seq(start.year, end.year, by = 1), collapse = ",")
    download.url = sprintf("https://daymet.ornl.gov/data/send/saveData?lat=%s&lon=%s&measuredParams=tmax,tmin,dayl,prcp,srad,swe,vp&year=%s", 
                           lat, long, year.range)
	if (echo == TRUE) {
      message(paste("Downloading DAYMET data for: ", site, " at ", 
          lat, "/", long, " latitude/longitude !\n", sep = ""))
    }
    x <- try( RCurl::getURL(download.url, ssl.verifypeer = FALSE), silent=TRUE)
	  if (!inherits(x, "try-error")) {
        dat <- utils::read.csv(textConnection(x), skip = 7)
		  names(dat) <- c("year", "julian", "dayl", "prcp", "srad", "swe", 
		                  "tmax", "tmin", "vp")
		if(!is.null(site)) dat <- data.frame(site = site, dat)
      } else {
	    stop("There was an error downloading this file")
	  }
    dat <- utils::read.csv(textConnection(x), skip = 7)
	  names(dat) <- c("year", "julian", "dayl", "prcp", "srad", "swe", 
		              "tmax", "tmin", "vp")
	    if(!is.null(site)) dat <- data.frame(site = site, dat)
      
	if( files == TRUE ) {
	  if(is.null(site)) site = paste( paste("lat", lat, sep="."), 
	                                  paste("long", long, sep="."), sep=".") 
      utils::write.csv( paste(site, "_", start.year, "_", end.year, ".csv", sep = ""),
	             row.names = FALSE)
    }
  return(dat)
}

#' @title raster combine
#' @description Combines rasters into all unique combinations of inputs
#'
#' @param x         raster stack/brick or SpatialPixelsDataFrame object
#'
#' @details
#' A single ratified raster object is returned with the summary table
#' as the raster attribute table, this is most similar to the ESRI 
#' format resulting from their combine function. 
#'     
#' Please note that this is not a memory safe function that utilizes
#' out of memory in the manner that the terra package does.
#'
#' @return 
#' A ratified (factor) terra SpatRaster representing unique combinations.
#' 
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com>
#'
#' @examples
#' library(terra)
#'
#' # Create example data (with a few NA's introduced)
#'  r1 <- rast(nrows=100, ncol=100)
#'    names(r1) <- "LC1"
#'    r1[] <- round(runif(ncell(r1), 1,4),0)
#'      r1[c(8,10,50,100)] <- NA
#'  r2 <- rast(nrows=100, ncol=100)
#'    names(r2) <- "LC2"
#'    r2[] <- round(runif(ncell(r2), 2,6),0)
#'      r2[c(10,50,100)] <- NA   
#'  r3 <- rast(nrows=100, ncol=100)
#'    names(r3) <- "LC3"
#'    r3[] <- round(runif(ncell(r3), 2,6),0)
#'      r3[c(10,50,100)] <- NA   
#'  r <- c(r1,r2,r3)  
#'    names(r) <- c("LC1","LC2","LC3")
#' 
#'  # Combine rasters with a multilayer stack
#'  cr <- combine(r)
#'    head(cr$summary)
#'    plot(cr$combine)
#'
#' # or, from separate layers
#'  cr <- combine(c(r1,r3))
#'
#' @export combine
combine <- function(x) {
  if(!inherits(x, "SpatRaster"))
    stop(deparse(substitute(x)), " must be a terra SpatRaster object")
  if(terra::nlyr(x) < 2)
    stop(deparse(substitute(x)), " needs > 1 layers to combine")  
  is.int <- function(x){
    e <- all.equal(x, as.integer(x), 
          check.attributes = FALSE)
    if(e == TRUE){ 
      return(TRUE) 
    } else { 
      return(FALSE) 
    }
  }	
  r <-  terra::as.data.frame(x, cells=TRUE) 
    if(any(apply(r, 2, is.int) == FALSE))
	  stop("All rasters must be integer, 
	        no floating point data is allowed")
    combine <- apply(r[,-1], 1, function(x) paste(x, collapse = "_"))
      r$value=as.numeric(factor(combine))
	    s <- as.data.frame(table(r[,-1]))
          s <- s[s$Freq != 0,]
		    names(s)[which(names(s) == "Freq")] <- "count"
    r.combine <- x[[1]]
      r.combine[as.numeric(r[,"cell"])] <- as.numeric(r[,"value"]) 
      r.combine[which(!1:terra::ncell(x) %in% as.numeric(r[,"cell"]))] <- NA
   return(list(combine=r.combine, summary=s))	
}

#' @title Bearing and Distance
#' @description Calculates a new point [X,Y] based on defined 
#'              bearing and distance
#' 
#' @param x             x coordinate
#' @param y             y coordinate
#' @param distance      Distance to new point (in same units as x,y)
#' @param azimuth       Azimuth to new point
#' @param EastOfNorth   Specified surveying convention
#'
#' @details East of north is a surveying convention and defaults to true. 
#'
#' @return a new point representing location of baring and distance 
#'
#' @author Jeffrey S. Evans  <sage_insights@@outlook.com> 
#'
#' @examples 
#'  pt <- cbind( x=480933, y=4479433)
#'  bearing.distance(pt[1], pt[2], 1000, 40)
#'
#' @export bearing.distance
bearing.distance <- function(x, y, distance, azimuth, EastOfNorth = TRUE) { 
  radians <- function(a) { a * (pi / 180) }
   if(EastOfNorth) {
      pt <- cbind( X = (x + distance * sin(radians(azimuth))), 
                   Y = (y + distance * cos(radians(azimuth)))) 
   } else {
      pt <- cbind( X = (x + distance * cos(radians(azimuth))), 
                   Y = (y + distance * sin(radians(azimuth))))       
  }
  return(pt)
}

#' @title Probability density plot by group
#'
#' @param y Numeric vector (density variable)
#' @param x Numeric, character or factorial vector of grouping 
#'          variable (must be same length as y)
#' @param col Optional line colors (see par, col)
#' @param lty Optional line types (see par, lty)
#' @param lwd Optional line widths (see par, lwd)
#' @param lx Position of legend (x coordinate or 'topright', 'topleft', 
#'           'bottomright', 'bottomleft') 
#' @param ly Position of legend (y coordinate)
#' @param ... Additional arguments passed to plot
#'
#' @description Creates a probability density plot of y for 
#'              each group of x 
#'
#' @return Plot of grouped PDF's
#'
#' @author Jeffrey S. Evans  <jeffrey_evans<at>tnc.org>
#'
#' @references
#' Simonoff, J. S. (1996). Smoothing Methods in Statistics. Springer-Verlag, New York.
#'
#' @examples 
#' y=dnorm(runif(100))
#' x=rep(c(1,2,3), length.out=length(y)) 
#' group.pdf(x=as.factor(x), y=y, main='Probability Density of y by group(x)', 
#' ylab='PDF', xlab='Y', lty=c(1,2,3))
#'
#' @export
group.pdf <- function(x, y, col = NULL, lty = NULL, lwd = NULL, 
                     lx = "topleft", ly = NULL, ...) {
  oops <- options() 
    on.exit(options(oops)) 
    if (!is.numeric(y)) 
        stop("y MUST BE NUMERIC")
    if (length(x) != length(y)) 
        stop("x AND y HAVE UNEQUAL LENGTHS")
    if (!is.factor(x)) 
        x <- as.factor(x)
    if (is.null(col)) 
        col <- seq(1:nlevels(x))
    if (is.null(lty)) 
        lty <- 1
    if (is.null(lwd)) 
        lwd <- 1.5
    ydat <- split(y, x)
    cden <- lapply(ydat, stats::density)
    xlim <- range(cden[[1]]$x, cden[[2]]$x)
    ylim <- range(cden[[1]]$y, cden[[2]]$y)
    graphics::plot(cden[[1]], type = "n", xlim = xlim, ylim = ylim, ...)
    for (i in 1:nlevels(x)) {
        graphics::lines(cden[[i]], col = col[i], lty = (if (length(lty) > 1) 
            lty[i] else lty), lwd = (if (length(lwd) > 1) 
            lwd[i] else lwd))
    }
    graphics::legend(lx, ly, names(cden), col = col, lty = lty, lwd = lwd)
} 

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/remove.holes.R
\name{remove.holes}
\alias{remove.holes}
\title{Remove or return polygon holes}
\usage{
remove.holes(x, only.holes = FALSE)
}
\arguments{
\item{x}{sf POLYGON or MULTIPOLYGON object}

\item{only.holes}{Delete holes (FALSE) or returns only holes (FALSE)}
}
\value{
sf POLYGON object
}
\description{
Removes or returns all holes (null geometry) in sf polygon 
class objects
}
\details{
A hole is considered a polygon within a polygon (island) representing null 
geometry. If you want to return only holes, no longer NULL, use keep = TRUE. 
To delete holes use default only.holes = FALSE. Single part features will be 
returned regardless of input.
}
\examples{
library(sf)

p <- sf::st_as_sf(sf::st_sfc(
  sf::st_polygon(list(
  cbind(c(2,4,4,1,2),c(2,3,5,4,2)),
  cbind(c(2.33, 2.05, 3.25, 3.25, 2.33), 
      c(3.00, 3.56, 3.95, 3.46, 3.00)))),
  sf::st_polygon(list(	
    cbind(c(5,4,2,5),c(2,3,2,2)))),
  sf::st_polygon(list(  
    cbind(c(4,4,5,10,4),c(5,3,2,5,5)),
    cbind(c(5,6,6,5,5),c(4,4,3,3,4)) 
))))
  p$ID <- 1:3

rh <- remove.holes(p)
kh <- remove.holes(p, only.holes=TRUE)

opar <- par(no.readonly=TRUE)
   par(mfrow=c(2,2))
     plot(st_geometry(p), main="Original with holes")
     plot(st_geometry(rh), main="holes removed only.holes=FALSE")
	 plot(st_geometry(kh), main="return holes only.holes=TRUE")
par(opar)

}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/z_normalization.R
\name{z_normalization}
\alias{z_normalization}
\title{z normalization}
\usage{
z_normalization(
  x,
  method = c("original", "modified"),
  na = c("keep", "remove")
)
}
\arguments{
\item{x}{Timeseries vector to normalize}

\item{method}{("original", "modified") Use the standard z normalization or modified version}

\item{na}{Behavior of NA values, keep or remove}
}
\value{
vector of z normalized data
}
\description{
Z-normalizes a time series by subtracting its mean and dividing by the standard deviation or meadian and MAD
}
\note{
The original method is to subtract x rom the mean then divide by sd whereas, modified uses the median and mad. 
For NA's if the argument is keep, an NA index is built and the NA's are re-inserted in to the resulting vector.
The behavior of remove will entirely remove them and result in a vector length different than the input.
}
\examples{
 # add data
 data(EuStockMarkets)
 d <- as.vector(EuStockMarkets[,1])

# Calculate Z score
summary(d)
summary(Z_org <- z_normalization(d) ) 
summary(Z_mod <- z_normalization(d, method="modified") )
  par(mfrow=c(3,1))
    plot(density(d), main="original timeseries")
    plot(density(Z_org), main="original z norm")
    plot(density(Z_mod), main="modified z norm")

# Check NA behavior, insert some NA's
d[c(100, 500, 1000, 1400)] <- NA
length(d)
length( z_normalization(d, na="keep") ) 
length( z_normalization(d, na="remove") ) 
 
}
\references{
Iglewicz, B. & D.C. Hoaglin (1993) How to Detect and Handle Outliers, 
  American Society for Quality Control, Milwaukee, WI.
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/proximity.index.R
\name{proximity.index}
\alias{proximity.index}
\title{Proximity Index}
\usage{
proximity.index(x, y = NULL, min.dist = 0, max.dist = 1000, background = NULL)
}
\arguments{
\item{x}{A polygon class sp or sf object}

\item{y}{Optional column in data containing classes}

\item{min.dist}{Minimum threshold distance}

\item{max.dist}{Maximum neighbor distance}

\item{background}{Optional value in y column indicating background value}
}
\value{
A vector equal to nrow(x) of proximity index values, if a background value is 
specified NA values will be returned in the position(s) of the specified class
}
\description{
Calculates proximity index for a set of polygons
}
\examples{
\donttest{
 library(sf)
 if(require(sp, quietly = TRUE)) {
   data(meuse, package = "sp")
   meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, 
                     agr = "constant")
   meuse <- st_buffer(meuse, dist = meuse$elev * 5)
     meuse$LU <- sample(c("forest","nonforest"), nrow(meuse), 
                       replace=TRUE) 

 # All polygon proximity index 1000 radius	
 ( pidx <- proximity.index(meuse, min.dist = 1) )
   pidx[pidx > 1000] <- 1000
 
 # Class-level proximity index 1000 radius
 ( pidx.class <- proximity.index(meuse, y = "LU", min.dist = 1) )
   
 # plot index for all polygons
 meuse$pidx <- pidx
   plot(meuse["pidx"])
 
 # plot index for class-level polygons 
 meuse$cpidx <- pidx.class
   plot(meuse["cpidx"])
 
 # plot index for just forest class
 forest <- meuse[meuse$LU == "forest",]
  plot(forest["cpidx"])

} else { 
  cat("Please install sp package to run example", "\n")
}
}   
}
\references{
Gustafson, E.J., & G.R. Parker (1994) Using an Index of Habitat Patch Proximity 
  for Landscape Design. Landscape and Urban Planning 29:117-130
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/divergence.R
\name{divergence}
\alias{divergence}
\title{divergence}
\usage{
divergence(x, y, type = c("Kullback-Leibler", "cross-entropy"))
}
\arguments{
\item{x}{a vector of integer values, defining observed}

\item{y}{a vector of integer values, defining estimates}

\item{type}{Type of divergence statistic c("Kullback-Leibler", 
"cross-entropy")}
}
\value{
single value vector with divergence statistic
}
\description{
Kullback-Leibler Divergence (Cross-entropy)
}
\examples{
x <- round(runif(10,1,4),0)
y <- round(runif(10,1,4),0)

divergence(x, y) 
divergence(x, y, type = "cross-entropy") 

}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/date_seq.R
\name{date_seq}
\alias{date_seq}
\title{date sequence}
\usage{
date_seq(
  start,
  end,
  step = c("day", "week", "month", "quarter", "year", "minute"),
  rm.leap = FALSE
)
}
\arguments{
\item{start}{Start date in "yyyy/mm/dd" character format}

\item{end}{End date in "yyyy/mm/dd" character format}

\item{step}{Time step, options are c("day", "week", "month", "quarter", 
"year", "minute")}

\item{rm.leap}{Remove extra days in leap years}
}
\value{
A date vector of class POSIXct for minute and Date for other options
}
\description{
creates date sequence given start and stop dates
}
\examples{
# monthly steps 1990/01/01 - 2019/12/31
d <- date_seq("1990/01/01", "2019/12/31", step="month")

# daily steps 1990/01/01 - 2019/12/31
d <- date_seq("1990/01/01", "2019/12/31", step="day")

# daily steps 1990/01/01 - 2019/12/31 with leap days removed
d <- date_seq("1990/01/01", "2019/12/31", step="day", rm.leap=TRUE)

# daily step 2008/12/29 - 2008/12/31, 2008 is leap year
d <- date_seq("2008/12/29", "2008/12/31")

# minutes step 2008/12/29 - 2008/12/31, 2008 is leap year
d <- date_seq("2008/12/29", "2008/12/31", step="minute")

}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nni.R
\name{nni}
\alias{nni}
\title{Average Nearest Neighbor Index (NNI)}
\usage{
nni(x, win = c("hull", "extent"))
}
\arguments{
\item{x}{An sf point object}

\item{win}{Type of window 'hull' or 'extent'}
}
\value{
list object containing NNI = nearest neighbor index, z.score = Z
Score value, p = p value, expected.mean.distance = Expected mean
distance, observed.mean.distance = Observed meand distance.
}
\description{
Calculates the NNI as a measure of clustering or dispersal
}
\details{
The nearest neighbor index is expressed as the ratio of the observed distance
divided by the expected distance. The expected distance is the average distance
between neighbors in a hypothetical random distribution. If the index is less than 1,
the pattern exhibits clustering; if the index is greater than 1, the trend is toward
dispersion or competition. The Nearest Neighbor Index is calculated as:
\itemize{
\item Mean Nearest Neighbor Distance (observed) D(nn) = sum(min(Dij)/N)
\item Mean Random Distance (expected) D(e) = 0.5 SQRT(A/N)
\item Nearest Neighbor Index NNI = D(nn)/D(e)
Where; D=neighbor distance, A=Area
}
}
\examples{

p = c("sf", "sp")
  if(any(!unlist(lapply(p, requireNamespace, quietly=TRUE)))) { 
    m = which(!unlist(lapply(p, requireNamespace, quietly=TRUE)))
    message("Can't run examples, please install ", paste(p[m], collapse = " "))
  } else {
  invisible(lapply(p, require, character.only=TRUE))

  data(meuse, package = "sp")
  meuse <- sf::st_as_sf(meuse, coords = c("x", "y"),  
                        crs = 28992, agr = "constant")
  nni(meuse)
  }

}
\references{
Clark, P.J., and F.C. Evans (1954) Distance to nearest neighbour as a measure
of spatial relationships in populations. Ecology 35:445-453

Cressie, N (1991) Statistics for spatial data. Wiley & Sons, New York.
}
\author{
Jeffrey S. Evans  \href{mailto:sage_insights@outlook.com}{sage_insights@outlook.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sampleTransect.R
\name{sampleTransect}
\alias{sampleTransect}
\title{Sample transect}
\usage{
sampleTransect(
  x,
  min.dist,
  max.dist,
  distance = NULL,
  azimuth = NULL,
  id = NULL,
  ...
)
}
\arguments{
\item{x}{A sf point object}

\item{min.dist}{Minimum length of transect(s)}

\item{max.dist}{Maximum length of transect(s)}

\item{distance}{A vector of distances, same length as x, used
to define transect distances (length)}

\item{azimuth}{A vector of azimuths, same length as x, used
to define transect direction}

\item{id}{A unique identification column in x}

\item{...}{Additional arguments passed to st_sample}
}
\value{
A list object contaning sf LINES and POINTS objects representing random transects 
and sample points along each transect. The "ID" column links the resulting data.
}
\description{
Creates random transects from points and generates 
             sample points along each transect
}
\details{
Function create lines and samples using random or defined direction 
and length transects and then creates a point sample along each transect. 
The characteristic of the sample points are defined by arguments passed 
to the sf::st_sample function. The distance and azimuth arguments allow
for specifying the exact length and direction for each points transect.
}
\examples{
if(require(sp, quietly = TRUE)) {
library(sf)
  data(meuse, package = "sp")
  meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, 
                    agr = "constant")
meuse <- meuse[sample(1:nrow(meuse),10),]

transects <- sampleTransect(meuse, min.dist=200, max.dist=500, 
                            type="regular", size=20)
   plot(st_geometry(transects$transects))
     plot(st_geometry(meuse), pch=19, cex=2, add=TRUE)
       plot(st_geometry(transects$samples), 
	        col="red", pch=19, add=TRUE)

} else { 
  cat("Please install sp package to run example", "\n")
}

}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chae.R
\name{chae}
\alias{chae}
\title{Canine-Human Age Equivalent}
\usage{
chae(x)
}
\arguments{
\item{x}{numeric vector, dog age}
}
\value{
numeric vector, equivalent human age
}
\description{
Calculates canines equivalent human age (for fun)
}
\examples{
dat <- data.frame(DogAge = seq(0,18,0.25),
             HumanAge=chae(seq(0,18,0.25)))[-1,]

plot(dat$DogAge, dat$HumanAge, "l",
     main="Canine-Human Age Equivalence",
	 ylab="Human Age", xlab="Dog Age")
  points( 15, chae(15), col="red", pch=19, cex=1.5)
  points( 10, chae(10), col="blue", pch=19, cex=1.5)
  points( 3, chae(3), col="black", pch=19, cex=1.5)
legend("bottomright", legend=c("Camas (15-YO)", "Kele (10-YO)", "Aster (3-YO)"), 
      pch=c(19,19,19), cex=c(1.5,1.5,1.5), 
	     col=c("red","blue","black"))  

}
\references{
Wang, T., J. M, A.N. Hogan, S. Fong, K. Licon et al.  (2020) quantitative 
  translation of dog-to-human aging by conserved remodeling of epigenetic 
  networks. Cell Systems 11(2)176-185
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/polyPerimeter.R
\name{polyPerimeter}
\alias{polyPerimeter}
\title{Polygon perimeter}
\usage{
polyPerimeter(x)
}
\arguments{
\item{x}{sf POLYGON class object}
}
\value{
A vector of polygon perimeters in projection units
}
\description{
Calculates the perimeter length(s) for a polygon object
}
\examples{
library(sf)
  polys <- st_read(system.file("shape/nc.shp", package="sf"))
    polys <- suppressWarnings(st_cast(polys[c(10,100),], "POLYGON"))
 
 polyPerimeter(polys)

}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/daymet.point.R
\name{daymet.point}
\alias{daymet.point}
\title{DAYMET point values}
\usage{
daymet.point(
  lat,
  long,
  start.year,
  end.year,
  site = NULL,
  files = FALSE,
  echo = FALSE
)
}
\arguments{
\item{lat}{latitude of point (decimal degrees WGS84)}

\item{long}{longitude pf point (decimal degrees WGS84)}

\item{start.year}{First year of data}

\item{end.year}{Last year of data}

\item{site}{Unique identification value that is appended to data}

\item{files}{(TRUE/FALSE) Write file to disk}

\item{echo}{(TRUE/FALSE) Echo progress}
}
\value{
A data.frame with geographic coordinate point-level climate results
}
\description{
Downloads DAYMET climate variables for specified point and time-period
}
\details{
data is available for Long -131.0 W and -53.0 W; lat 52.0 N and 14.5 N
Function uses the Single Pixel Extraction tool and returns year, yday, 
      dayl(s), prcp (mm/day), srad (W/m^2), swe (kg/m^2), tmax (deg c), 
      tmin (deg c), vp (Pa)
Metadata for DAYMET single pixel extraction: 
\url{ https://daymet.ornl.gov/files/UserGuides/current/readme_singlepointextraction.pdf }
}
\examples{
\donttest{
( d <- daymet.point(lat = 36.0133, long = -84.2625, start.year = 2013, 
                    end.year=2014, site = "1", files = FALSE, echo = FALSE) )
}

}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/similarity.R
\name{similarity}
\alias{similarity}
\title{Ecological similarity}
\usage{
similarity(
  x,
  k = 4,
  method = "mahalanobis",
  frequency = TRUE,
  scale = TRUE,
  ID = NULL
)
}
\arguments{
\item{x}{data.frame containing ecological measures}

\item{k}{Number of k nearest neighbors (kNN)}

\item{method}{Method to compute multivariate distances c("mahalanobis", "raw", 
"euclidean", "ica")}

\item{frequency}{Calculate frequency of each reference row (TRUE/FALSE)}

\item{scale}{Scale multivariate distances to standard range (TRUE/FALSE)}

\item{ID}{Unique ID vector to use as reference ID's (rownames). Must be 
unique and same length as number of rows in x}
}
\value{
data.frame with k similar targets and associated distances. If frequency = TRUE  the 
freq column represents the number of times a row (ID) was selected as a neighbor.
}
\description{
Uses row imputation to identify "k" ecological similar observations
}
\details{
This function uses row-based imputation to identify k similar neighbors for each 
observation. Has been used to identify offsets based on ecological similarity.
}
\examples{
\donttest{ 
 library(sf)
 data(pu)
 kNN <- similarity(st_drop_geometry(pu[2:ncol(pu)]), k = 4, 
                   frequency = TRUE, ID = pu$UNIT_ID)  
 p <- kNN$freq   
 clr <- c("#3288BD", "#99D594", "#E6F598", "#FEE08B", 
          "#FC8D59", "#D53E4F")   
 p <- ifelse(p <= 0, clr[1], 
        ifelse(p > 0 & p < 10, clr[2],
          ifelse(p >= 10 & p < 20, clr[3],
 	       ifelse(p >= 20 & p < 50, clr[4],
 	         ifelse(p >= 50 & p < 100, clr[5],
 	           ifelse(p >= 100, clr[6], NA))))))
 plot(st_geometry(pu), col=p, border=NA)
   legend("topleft", legend=c("None","<10","10-20",
          "20-50","50-100",">100"),
          fill=clr, cex=0.6, bty="n") 
   box()
}
 
}
\references{
Evans, J.S., S.R. Schill, G.T. Raber (2015) A Systematic Framework for Spatial 
  Conservation Planning and Ecological Priority Design in St. Lucia, Eastern 
  Caribbean. Chapter 26 in Central American Biodiversity : Conservation, Ecology 
  and a Sustainable Future. F. Huettman (eds). Springer, NY.
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spectral.separability.R
\name{spectral.separability}
\alias{spectral.separability}
\title{spectral separability}
\usage{
spectral.separability(x, y, jeffries.matusita = TRUE)
}
\arguments{
\item{x}{data.frame, matrix or vector of spectral values must,
match classes defined in y}

\item{y}{A vector or factor with grouping classes, must match
row wise values in x}

\item{jeffries.matusita}{(TRUE/FALSE) Return J-M distance (default) else Bhattacharyya}
}
\value{
A matrix of class-wise Jeffries-Matusita or Bhattacharyya distance separability values
}
\description{
Calculates spectral separability by class
}
\details{
Available statistics:
\itemize{
\item Bhattacharyya distance (Bhattacharyya 1943; Harold 2003) measures the similarity
of two discrete or continuous probability distributions.
\item Jeffries-Matusita (default) distance (Bruzzone et al., 2005; Swain et al., 1971)
is a scaled (0-2) version of Bhattacharyya. The J-M distance is asymptotic to 2,
where 2 suggest complete separability.
}
}
\examples{
require(MASS)                
# Create example data 
d <- 6                 # Number of bands
n.class <- 5           # Number of classes
n <- rep(1000, 5)                
mu <- round(matrix(rnorm(d*n.class, 128, 1), 
            ncol=n.class, byrow=TRUE), 0)

x <- matrix(double(), ncol=d, nrow=0)
  classes <- integer()
    for (i in 1:n.class) {
      f <- svd(matrix(rnorm(d^2), ncol=d))
      sigma <- t(f$v) \%*\% diag(rep(10, d)) \%*\% f$v
      x <- rbind(x, mvrnorm(n[i], mu[, i], sigma))
      classes <- c(classes, rep(i, n[i]))
    }
colnames(x) <- paste0("band", 1:6)
classes <- factor(classes, labels=c("water", "forest",
                  "shrub", "urban", "ag"))

# Separability for multi-band (multivariate) spectra 
spectral.separability(x, classes)

# Separability for single-band (univariate) spectra 
spectral.separability(x[,1], classes)

}
\references{
Bhattacharyya, A. (1943) On a measure of divergence between two statistical
populations defined by their probability distributions'. Bulletin of the
Calcutta Mathematical Society 35:99-109

Bruzzone, L., F. Roli, S.B. Serpico (1995) An extension to multiclass cases of
the Jefferys-Matusita distance. IEEE Transactions on Pattern Analysis and
Machine Intelligence 33:1318-1321

Kailath, T., (1967) The Divergence and Bhattacharyya measures in signal
selection. IEEE Transactions on Communication Theory 15:52-60
}
\author{
Jeffrey S. Evans  \href{mailto:sage_insights@outlook.com}{sage_insights@outlook.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/erase.points.R
\name{erase.point}
\alias{erase.point}
\title{Erase points}
\usage{
erase.point(y, x, inside = TRUE)
}
\arguments{
\item{y}{A sf POINT object}

\item{x}{A sf POLYGON object}

\item{inside}{(TRUE/FALSE) Remove points inside polygon, else outside polygon}
}
\value{
An sf POINT object
}
\description{
Removes points intersecting a polygon feature class
}
\details{
Used to erase points that intersect polygon(s). The default of inside=TRUE
erases points inside the polygons however, if inside=FALSE then
the function results in an intersection where points that
intersect the polygon are retained.
}
\examples{

library(sf)
  
if (require(sp, quietly = TRUE)) {
  data(meuse, package = "sp")
  meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, agr = "constant")

  s <- st_as_sf(st_sample(st_as_sfc(st_bbox(meuse)), size=1000, 
                 type = "regular"))
    s$id <- 1:nrow(s)
  b <- st_buffer(s[sample(1:nrow(s),5),], dist=300)
    b$id <- 1:nrow(b)
  
# Erase points based on polygons
in.erase <- erase.point(s, b)
out.erase <- erase.point(s, b, inside = FALSE)

 opar <- par(no.readonly=TRUE)
 par(mfrow=c(2,2))
   plot(st_geometry(s), pch=20, main="original data")
     plot(st_geometry(b),add=TRUE)
   plot(st_geometry(in.erase), pch=20, main="erased data")
     plot(st_geometry(b),add=TRUE)
   plot(st_geometry(out.erase), pch=20,  
        main="erased data using inside=FALSE")
     plot(st_geometry(b),add=TRUE)
 par(opar)

} else { 
  cat("Please install sp package to run example", "\n")
}

}
\author{
Jeffrey S. Evans    <jeffrey_evans<at>tnc.org>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/collinear.R
\name{collinear}
\alias{collinear}
\title{Collinearity test}
\usage{
collinear(x, p = 0.85, nonlinear = FALSE, p.value = 0.001)
}
\arguments{
\item{x}{A data.frame or matrix containing continuous data}

\item{p}{The correlation cutoff (default is 0.85)}

\item{nonlinear}{A boolean flag for calculating nonlinear correlations 
(FALSE/TRUE)}

\item{p.value}{If nonlinear is TRUE, the p value to accept as the 
significance of the correlation}
}
\value{
Messages and a vector of correlated variables
}
\description{
Test for linear or nonlinear collinearity/correlation in data
}
\details{
Evaluation of the pairwise linear correlated variables to remove is 
accomplished through calculating the mean correlations of each variable 
and selecting the variable with higher mean. If nonlinear = TRUE, pairwise
nonlinear correlations are evaluated by fitting y as a semi-parametrically 
estimated function of x using a generalized additive model and testing 
whether or not that functional estimate is constant, which would indicate 
no relationship between y and x thus, avoiding potentially arbitrary decisions
regarding the order in a polynomial regression.
}
\examples{
data(cor.data)

# Evaluate linear correlations on linear dataCollinearity between
head( dat <- cor.data[[4]] ) 
pairs(dat, pch=20)
  ( cor.vars <- collinear( dat ) )

# Remove identified variable(s)
head( dat[,-which(names(dat) \%in\% cor.vars)] )

\donttest{
# Evaluate linear correlations on nonlinear data
#   using nonlinear correlation function
plot(cor.data[[1]], pch=20) 
  collinear(cor.data[[1]], p=0.80, nonlinear = TRUE ) 	
}	       

}
\author{
Jeffrey S. Evans  <jeffrey_evans<at>tnc.org>

Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/raster.Zscore.R
\name{raster.Zscore}
\alias{raster.Zscore}
\title{Modified z-score for a raster}
\usage{
raster.Zscore(x, p.value = FALSE, file.name = NULL, ...)
}
\arguments{
\item{x}{A raster class object}

\item{p.value}{Return p-value rather than z-score 
raster (FALSE/TRUE)}

\item{file.name}{Name of raster written to disk}

\item{...}{Additional arguments passed to writeRaster}
}
\value{
raster class object or raster written to disk
}
\description{
Calculates the modified z-score for raster values
}
\note{
Since this functions needs to operate on all of the raster values, 
it is not memory safe
}
\examples{
\donttest{
library(terra)
r <- rast(nrows=500, ncols=500)
  r[] <- runif(ncell(r), 0, 1)

# Modified z-score
( z <- raster.Zscore(r) )

# P-value
( p <- raster.Zscore(r, p.value = TRUE) )
}	

}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/topo.distance.R
\name{topo.distance}
\alias{topo.distance}
\title{Topographic distance}
\usage{
topo.distance(x, r, echo = FALSE)
}
\arguments{
\item{x}{sf LINESTRING object}

\item{r}{terra SpatRaster class elevation raster}

\item{echo}{(FALSE/TRUE) print progress to screen}
}
\value{
Vector of corrected topographic distances same length as nrow(x)
}
\description{
Calculates topographic corrected distance for a line object
}
\details{
This function corrects straight-line (euclidean) distances for topographic-slope effect.
}
\examples{
 library(sf)
 library(terra)
 
 # create example data
 elev <- rast(system.file("extdata/elev.tif", package="spatialEco"))
   names(elev) <- "elev"

 lns <- lapply(1:5, function(i) {
   p <- st_combine(st_as_sf(spatSample(elev, size=2, as.points=TRUE)))
   st_as_sf(st_cast(p, "LINESTRING")) }) 
 lns <- do.call(rbind, lns) 
  
  plot(elev)
    plot(st_geometry(lns), add=TRUE)
      
 # Calculate topographical distance  
 ( tdist <- topo.distance(lns, elev) )
 ( lgt <- as.numeric(st_length(lns)) ) 
 
 # Increase in corrected distance
 tdist - lgt
 
 # Percent increase in corrected distance
 ((tdist - lgt) / lgt) * 100

}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pp.subsample.R
\name{pp.subsample}
\alias{pp.subsample}
\title{Point process random subsample}
\usage{
pp.subsample(
  x,
  n,
  window = "hull",
  sigma = "Scott",
  wts = NULL,
  gradient = 1,
  edge = FALSE
)
}
\arguments{
\item{x}{An sf POINT class}

\item{n}{Number of random samples to generate}

\item{window}{Type of window (hull or extent)}

\item{sigma}{Bandwidth selection method for KDE, default is 'Scott'.
Options are 'Scott', 'Stoyan', 'Diggle', 'likelihood',
and 'geometry'}

\item{wts}{Optional vector of weights corresponding to point pattern}

\item{gradient}{A scaling factor applied to the sigma parameter used to
adjust the gradient decent of the density estimate. The
default is 1, for no adjustment (downweight < 1 | upweight > 1)}

\item{edge}{Apply Diggle edge correction (TRUE/FALSE)}
}
\value{
sf class POINT geometry containing random subsamples
}
\description{
Generates random subsample based on density estimate
of observations
}
\details{
The window type creates a convex hull by default or, optionally, uses the maximum
extent (envelope). The resulting bandwidth can vary widely by method. the 'diggle'
method is intended for  bandwidth representing 2nd order spatial variation whereas
the 'scott' method will represent 1st order trend. the 'geometry' approach will also
represent 1st order trend. for large datasets, caution should be used with the 2nd
order 'likelihood' approach, as it is slow and computationally expensive. finally,
the 'stoyan' method will produce very strong 2nd order results. '

Available bandwidth selection methods are:
\itemize{
\item Scott - (Scott 1992), Scott's Rule for Bandwidth Selection (1st order)
\item Diggle - (Berman & Diggle 1989), Minimise the mean-square error via cross
validation (2nd order)
\item likelihood - (Loader 1999), Maximum likelihood cross validation (2nd order)
\item geometry - Bandwidth is based on simple window geometry (1st order)
\item Stoyan - (Stoyan & Stoyan 1995), Based on pair-correlation function (strong 2nd order)
\item User defined - using a numeric value for sigma
}
}
\examples{
 
library(sf) 
if(require(spatstat.explore, quietly = TRUE)) { 
data(bei, package = "spatstat.data")

trees <- st_as_sf(bei)
  trees <- trees[-1,]

n=round(nrow(trees) * 0.10, digits=0)       
trees.wrs <- pp.subsample(trees, n=n, window='hull')
  plot(st_geometry(trees), pch=19, col='black')
    plot(st_geometry(trees.wrs), pch=19, col='red', add=TRUE) 
      box()
       title('10\% subsample')
    legend('bottomright', legend=c('Original sample', 'Subsample'), 
                 col=c('black','red'),pch=c(19,19))   

} else { 
  cat("Please install spatstat.explore package to run example", "\n")
}

}
\references{
Berman, M. and Diggle, P. (1989) Estimating weighted integrals of the second-order
intensity of a spatial point process. Journal of the Royal Statistical Society,
series B 51, 81-92.

Fithian, W & T. Hastie (2013) Finite-sample equivalence in statistical models for
presence-only data. Annals of Applied Statistics 7(4): 1917-1939

Hengl, T., H. Sierdsema, A. Radovic, and A. Dilo (2009) Spatial prediction of species
distributions from occurrence-only records: combining point pattern analysis,
ENFA and regression-kriging. Ecological Modelling, 220(24):3499-3511

Loader, C. (1999) Local Regression and Likelihood. Springer, New York.

Scott, D.W. (1992) Multivariate Density Estimation. Theory, Practice and Visualization.
New York, Wiley.

Stoyan, D. and Stoyan, H. (1995) Fractals, random shapes and point fields: methods of
geometrical statistics. John Wiley and Sons.

Warton, D.i., and L.C. Shepherd (2010) Poisson Point Process Models Solve the Pseudo-Absence
Problem for Presence-only Data in Ecology. The Annals of Applied Statistics, 4(3):1383-1402
}
\author{
Jeffrey S. Evans  \href{mailto:sage_insights@outlook.com}{sage_insights@outlook.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/poly.regression.R
\name{poly.regression}
\alias{poly.regression}
\title{Local Polynomial Regression}
\usage{
poly.regression(
  y,
  x = NULL,
  s = 0.75,
  impute = FALSE,
  na.only = FALSE,
  ci = FALSE,
  ...
)
}
\arguments{
\item{y}{Vector to smooth or impute NA values}

\item{x}{Optional x covariate data (must match dimensions of y)}

\item{s}{Smoothing parameter (larger equates to more smoothing)}

\item{impute}{(FALSE/TRUE) Should NA values be inputed}

\item{na.only}{(FALSE/TRUE) Should only NA values be change in y}

\item{ci}{(FALSE/TRUE) Should confidence intervals be returned}

\item{...}{Additional arguments passed to loess}
}
\value{
If ci = FALSE, a vector of smoothed values,
otherwise a list object with:
\itemize{
\item loess - A vector, same length of y, representing the smoothed or
inputed data
\item lower.ci - Lower confidence interval
\item upper.ci - Upper confidence interval
}
}
\description{
Calculates a Local Polynomial Regression for smoothing
or imputation of missing data.
}
\details{
This is a wrapper function for loess that simplifies data smoothing and imputation
of missing values. The function allows for smoothing a vector, based on an index
(derived automatically) or covariates. If the impute option is TRUE NA values are
imputed, otherwise the returned vector will still have NA's present. If impute and
na.only are both TRUE the vector is returned, without being smoothed but with imputed
NA values filled in. The loess weight function is defined using the tri-cube weight
function w(x) = (1-|x|^3)^3 where; x is the distance of a data point from the point
the curve being fitted.
}
\examples{
 x <- seq(-20, 20, 0.1)
 y <- sin(x)/x + rnorm(length(x), sd=0.03)
 p <- which(y == "NaN")
 y <- y[-p]
 r <- poly.regression(y, ci=TRUE, s=0.30)
 
 plot(y,type="l", lwd=0.5, main="s = 0.10")
   y.polygon <- c((r$lower.ci)[1:length(y)], (r$upper.ci)[rev(1:length(y))])
   x.polygon <- c(1:length(y), rev(1:length(y)))
   polygon(x.polygon, y.polygon, col="#00009933", border=NA) 
      lines(r$loess, lwd=1.5, col="red")
 
 # Impute NA values, replacing only NA's
 y.na <- y
 y.na[c(100,200,300)] <- NA 
 p.y <- poly.regression(y.na, s=0.10, impute = TRUE, na.only = TRUE)
 y - p.y
 
 plot(p.y,type="l", lwd=1.5, col="blue", main="s = 0.10")
   lines(y, lwd=1.5, col="red")

}
\seealso{
\code{\link[stats]{loess}} for loess ... model options
}
\author{
Jeffrey S. Evans  \href{mailto:sage_insights@outlook.com}{sage_insights@outlook.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subsample.distance.R
\name{subsample.distance}
\alias{subsample.distance}
\title{Distance-based subsampling}
\usage{
subsample.distance(x, size, d, d.max = NULL, replacement = FALSE)
}
\arguments{
\item{x}{A POLYGON or POINT sf object}

\item{size}{Subsample size}

\item{d}{Minimum sampling distance in meters}

\item{d.max}{Maximum sampling distance in meters}

\item{replacement}{(FALSE/TRUE) Subsample with replacement}
}
\value{
A subsampled POLYGON or POINT sf object
}
\description{
Draws a minimum, and optional maximum constrained, distance sub-sampling
}
\note{
This function provides a distance constrained subsample of existing point 
      or polygon data. Please note that units are in meters regardless of input
      CRS projection units (including lat/long).
}
\examples{
     
\donttest{
if(require(sp, quietly = TRUE)) {
library(sf)
  data(meuse, package = "sp")
  meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, 
                    agr = "constant")

# Subsample with a 500m minimum sample spread 
sub.meuse <- subsample.distance(meuse, size = 10, d = 500)  
  plot(st_geometry(meuse), pch=19, main="min dist = 500")
    plot(st_geometry(sub.meuse), pch=19, col="red", add=TRUE) 
 
# Check distances	
dm <- st_distance(sub.meuse)
  diag(dm) <- NA
cat("\n", "Min distance for subsample", min(dm, na.rm=TRUE), "\n")  
cat("Max distance for subsample", max(dm, na.rm=TRUE), "\n")  

} else { 
  cat("Please install sp package to run example", "\n")
}
}
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trasp.R
\name{trasp}
\alias{trasp}
\title{Solar-radiation Aspect Index}
\usage{
trasp(x, ...)
}
\arguments{
\item{x}{A terra SpatRaster object}

\item{...}{Additional arguments passed to terra::app}
}
\value{
A terra SpatRaster object of Roberts and Cooper (1989) Solar-radiation Aspect Index
}
\description{
Calculates the Roberts and Cooper (1989) Solar-radiation Aspect Index
}
\details{
Roberts and Cooper (1989) rotates (transforms) the circular aspect to assign a 
value of zero to land oriented in a north-northeast direction, (typically the 
coolest and wettest orientation), and a value of one on the hotter, dryer 
south-southwesterly slopes. The result is a continuous variable between 0 - 1. 
The metric is defined as: trasp = ( 1 - cos((pi/180)(a-30) ) / 2 
where; a = aspect in degrees
}
\examples{
library(terra)
elev <- rast(system.file("extdata/elev.tif", package="spatialEco"))
  s <- trasp(elev)
    plot(s)
    
}
\references{
Roberts. D.W., and Cooper, S.V. (1989). Concepts and techniques of vegetation mapping. 
  In Land Classifications Based on Vegetation: Applications for Resource Management.
  USDA Forest Service GTR INT-257, Ogden, UT, pp 90-96
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parea.sample.R
\name{parea.sample}
\alias{parea.sample}
\title{Percent area sample}
\usage{
parea.sample(x, pct = 0.1, join = FALSE, sf = 4046.86, stype = "random", ...)
}
\arguments{
\item{x}{An sf POLYGON object}

\item{pct}{Percent of area sampled}

\item{join}{{FALSE/TRUE} Join polygon attributed to point sample}

\item{sf}{Scaling factor (default is meters to acres conversion factor)}

\item{stype}{Sampling type ('random', 'regular', 'nonaligned', 'hexagonal')}

\item{...}{Additional arguments passed to spsample}
}
\value{
An sf POINT object
}
\description{
Creates a point sample of polygons where n is based 
             on percent area
}
\details{
This function results in an adaptive sample based on the area of 
each polygon. The default scaling factor (sf) converts meters to
acres. You can set sf=1 to stay in the native projection units
}
\examples{
library(sf)
nc <- st_read(system.file("shape/nc.shp", package="sf"))
  nc <- suppressWarnings(st_cast(nc[c(10,100),], "POLYGON"))
  
 ( ars <- parea.sample(nc, pct=0.001, join = TRUE, stype='random') ) 
     plot(st_geometry(nc))
        plot(st_geometry(ars), pch=19, add=TRUE)  

}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spatial.select.R
\name{spatial.select}
\alias{spatial.select}
\title{Spatial Select}
\usage{
spatial.select(
  x,
  y = NULL,
  distance = NULL,
  predicate = c("intersection", "intersect", "contains", "covers", "touches",
    "proximity", "contingency"),
  neighbors = c("queen", "rook")
)
}
\arguments{
\item{x}{An sp or sf polygon(s) object that defines the spatial query}

\item{y}{A sp or sf feature class that will be subset by the query of x}

\item{distance}{A proximity distance of features to select (within distance)}

\item{predicate}{Spatial predicate for intersection}

\item{neighbors}{If predicate = "contingency" type of neighbors options are
c("queen", "rook")}
}
\value{
An sf object representing a subset of y based on the spatial query of x or,
if predicate = contingency a sparse matrix representing neighbor indexes
}
\description{
Performs a spatial select (feature subset) between a polygon(s) and other
feature class
}
\details{
Performs a spatial select of features based on an overlay of a polygon (x),
which can represent multiple features, and a polygon, point or line feature
classes (y). User can specify a partial or complete intersection, using within
argument, or within a distance, using distance argument, predicated on the
query polygon. This function is similar to ArcGIS/Pro spatial select. Please note
that for point to point neighbor selections use the knn function.
Valid spatial predicates include: intersect, touches, covers, contains, proximity
and contingency.
See DE-9IM topology model for detailed information on following data predicates.
\itemize{
\item intersection  Create a spatial intersection between two features
\item intersect     Boolean evaluation of features intersecting
\item contains      Boolean evaluation of x containing y
\item covers        Boolean evaluation of x covering y
\item touches       Boolean evaluation of x touching y
\item proximity     Evaluation of distance-based proximity of x to y (x and y can be the same)
\item contingency   Evaluation of polygon contingency (eg., 1st, 2nd order)
}
}
\examples{
if(require(sp, quietly = TRUE)) {
library(sf)
  data(meuse, package = "sp")
  meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, 
                    agr = "constant")

spolys <- hexagons(meuse, res=100)
  spolys$ID <- 1:nrow(spolys)
    p <- st_as_sf(st_sample(spolys, 500))
	  p$PTID <- 1:nrow(p) 
	  sf::st_geometry(p) <- "geometry"

  plot(st_geometry(spolys), main="all data")
    plot(st_geometry(p), pch=20, add=TRUE)
	
sub.int <- spatial.select(p, spolys, predicate = "intersect")
  plot(st_geometry(sub.int), main="intersects")
    plot(st_geometry(p), pch=20, add=TRUE)	

sub.prox <- spatial.select(p, spolys, distance=100, predicate = "proximity")	
  plot(st_geometry(sub.int), main="intersects")
    plot(st_geometry(p), pch=20, add=TRUE)

# For rook or queen polygon contingency 	
plot( spolys <- sf::st_make_grid(sf::st_sfc(sf::st_point(c(0,0)), 
                 sf::st_point(c(3,3))), n = c(3,3)) )
  
spatial.select(x=spolys, predicate = "contingency")
spatial.select(spolys, predicate = "contingency", neighbors = "rook") 

} else { 
  cat("Please install sp package to run example", "\n")
}

}
\seealso{
\code{\link[sf]{st_intersection}} for details on intersection predicate

\code{\link[sf]{st_intersects}} for details on intersect predicate

\code{\link[sf]{st_contains}} for details on contain predicate

\code{\link[sf]{st_covers}} for details on covers predicate

\code{\link[sf]{st_touches}} for details on touches predicate

\code{\link[sf]{st_is_within_distance}} for details on proximity predicate

\url{https://en.wikipedia.org/wiki/DE-9IM} for details on DE-9IM topology model
}
\author{
Jeffrey S. Evans  \href{mailto:sage_insights@outlook.com}{sage_insights@outlook.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annulus.matrix.R
\name{annulus.matrix}
\alias{annulus.matrix}
\title{Annulus matrix}
\usage{
annulus.matrix(scale = 3, inner.scale = 0, outer.scale = 0, null.value = 0)
}
\arguments{
\item{scale}{Number of rings (defines dimensions of matrix)}

\item{inner.scale}{Number of inner rings to set to null.value}

\item{outer.scale}{Number of outer rings to set to null.value}

\item{null.value}{Value to set inner and outer scale(s) to}
}
\value{
A matrix object with defined null.value and 1, representing retained rings
}
\description{
Creates a square matrix representing annulus position values of 1 
             and defined null
}
\details{
This function will return a matrix of 1 and defined null.value based on a specification
of the scale, inner scale and outer scale. The scale defines how many rings will be
represented in the matrix based on (2 * scale - 1). So, a scale of 3 will result in a
5x5 matrix. The inner.scale and outer.scale arguments represent the > and < rings that
will be set to the defined null.value (see examples). The resulting matrix can be used
as the specified window in a focal function.
}
\examples{
annulus.matrix(5)                   # 5 concentric rings
annulus.matrix(5, 3)                # 5 concentric rings with the 3 inner set to 0
annulus.matrix(5, 3, null.value=NA) # 5 concentric rings with the 3 inner set to NA
annulus.matrix(5, 3, 5)             # 5 rings with 3 inner and 5 outer set to 0
annulus.matrix(9, 3, 7)             # 9 rings with 3 inner and 7 outer set to 0

}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ants-data.R
\docType{data}
\name{ants}
\alias{ants}
\title{Ant Biodiversity Data}
\format{
A data.frame with 82 rows (species) and 5 columns (covertypes):
\describe{
  \item{species}{Ant species (family)}
  \item{Primary.Forest}{Primary forest type}
  \item{Abandoned.cacao.plantations}{Abandoned cacao plantations type}
  \item{Productive.cacao.plantations}{Active cacao plantations type}
  \item{Banana.plantations}{Active banana plantations type}
}
}
\description{
Roth et al., (1994) Costa Rican ant diversity data
}
\references{
Roth, D. S., I. Perfecto, and B. Rathcke (1994) The effects of management systems on 
            ground-foraging ant diversity in Costa Rica. Ecological Applications 4(3):423-436.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classBreaks.R
\name{classBreaks}
\alias{classBreaks}
\title{Class breaks}
\usage{
classBreaks(x, n, type = c("equal", "quantile", "std", "geometric"))
}
\arguments{
\item{x}{A vector to find breaks for}

\item{n}{Number of breaks}

\item{type}{Statistic used to find breaks c("equal", "quantile", "std", "geometric")}
}
\value{
A vector containing class break values the length is n+1 to allow for 
specification of ranges
}
\description{
Finds class breaks in a distribution
}
\details{
The robust std method uses sqrt(sum(x^2)/(n-1)) to center the data before deriving "pretty" breaks.
}
\examples{
 y <- rnbinom(100, 10, 0.5)
   classBreaks(y, 10)  
   classBreaks(y, 10, type="quantile")
 
opar <- par(no.readonly=TRUE)
   par(mfrow=c(2,2))
     d <- density(y)
     plot(d, type="n", main="Equal Area breaks")
       polygon(d, col="cyan")
       abline(v=classBreaks(y, 10)) 
     plot(d, type="n", main="Quantile breaks")
       polygon(d, col="cyan")
       abline(v=classBreaks(y, 10, type="quantile"))
     plot(d, type="n", main="Robust Standard Deviation breaks")
       polygon(d, col="cyan")
       abline(v=classBreaks(y, 10, type="std"))
     plot(d, type="n", main="Geometric interval breaks")
       polygon(d, col="cyan")
       abline(v=classBreaks(y, 10, type="geometric"))
 par(opar)
	
 ( y.breaks <- classBreaks(y, 10) )   	
 cut(y, y.breaks, include.lowest = TRUE, labels = 1:10)

}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rasterDistance.R
\name{rasterDistance}
\alias{rasterDistance}
\title{Raster Distance}
\usage{
rasterDistance(x, y, scale = FALSE)
}
\arguments{
\item{x}{A terra SpatRast or sf class object}

\item{y}{Value(s) in x to to calculate distance to}

\item{scale}{(FALSE/TRUE) Perform a row standardization on results}
}
\value{
A terra SpatRast raster representing distances
}
\description{
Calculates the Euclidean distance of a defined raster class and
             all the other cells in a taster
}
\details{
This replicates the terra distance function but uses the Arya & Mount
Approximate Near Neighbor (ANN) C++ library for calculating distances. Where this
results in a notable increase in performance it is not memory safe, needing to read
in the entire raster and does not use the GeographicLib (Karney, 2013) spheroid 
distance method for geographic data.
}
\examples{
\donttest{
library(sf)
library(terra)

# read, project and subset 10 polygons
nc <- suppressWarnings(st_cast(st_read(system.file("shape/nc.shp", 
         package="sf")), "POLYGON"))
  nc <- st_transform(nc, st_crs("ESRI:102008"))
    nc.sub <- nc[sample(1:nrow(nc),10),]

# create 1000m reference raster, rasterize subset polygons
ref <- rast(ext(nc), resolution=1000)
  rnc <- mask(rasterize(vect(nc.sub), field="CNTY_ID",
              ref, background=9999), vect(nc)) 
    crs(rnc) <- "ESRI:102008"  
  
# Calculate distance to class 1 in rnc raster, plot results
ids <- nc.sub$CNTY_ID 
rd <- rasterDistance(rnc, y=ids) 
  plot(rd)
    plot( st_geometry(nc.sub), add=TRUE)

}
}
\references{
Arya S., Mount D. M., Netanyahu N. S., Silverman R. and Wu A. Y (1998), An 
  optimal algorithm for approximate nearest neighbor searching, Journal of 
  the ACM, 45, 891-923.
}
\seealso{
\code{\link[terra]{distance}, \link[terra]{distance}}
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/raster.mds.R
\name{raster.mds}
\alias{raster.mds}
\title{Raster multidimensional scaling (MDS)}
\usage{
raster.mds(r, s = 5, window.median = FALSE, ...)
}
\arguments{
\item{r}{A terra SpatRaster class object}

\item{s}{Window size (may be a vector of 1 or 2) of 
n x n dimension.}

\item{window.median}{(FALSE/TRUE) Return the median of the MDS 
matrix values.}

\item{...}{Additional arguments passed to terra::focal}
}
\value{
A terra SpatRaster class object
}
\description{
Multidimensional scaling of raster values within an N x N focal window
}
\details{
An MDS focal function. If only one value provided for s, then a square matrix 
(window) will be used. If window.median = FALSE then the center value of the 
matrix is returned and not the median of the matrix
}
\examples{
\donttest{ 
 library(terra)
 r <- rast(system.file("ex/elev.tif", package="terra"))
   r <- r[[1]] / max(global(r, "max", na.rm=TRUE)[,1])
 
 diss <- raster.mds(r)
 diss.med <- raster.mds(r, window.median = TRUE)

opar <- par(no.readonly=TRUE)
   par(mfrow=c(2,2))
   plot(r)
     title("Elevation")
   plot( focal(r, w = matrix(1, nrow=5, ncol=5), fun = var) )
     title("Variance")		 
     plot(diss)
       title("MDS")
     plot(diss.med)
       title("Median MDS")
par(opar)
}

}
\references{
Quinn, G.P., & M.J. Keough (2002) Experimental design and data analysis 
  for biologists. Cambridge University Press. Ch. 18. Multidimensional 
  scaling and cluster analysis.
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/swvi.R
\name{swvi}
\alias{swvi}
\title{Senescence weighted Vegetation Index (swvi)}
\usage{
swvi(
  red,
  nir,
  swir,
  green = NULL,
  mtvi = FALSE,
  senescence = 0,
  threshold = NULL,
  weight.factor = NULL,
  ...
)
}
\arguments{
\item{red}{SpatRaster, Red band (0.636 - 0.673mm), landsat 5&7 band 3, OLI
(landsat 8) band 4}

\item{nir}{SpatRaster, Near infrared band (0.851 - 0.879mm) landsat 5&7 band 4,
OLI (landsat 8) band 5}

\item{swir}{SpatRaster, short-wave infrared band 1 (1.566 - 1.651mm), landsat 5&7
band 5, OLI (landsat 8) band 6}

\item{green}{Green band if MTVI = TRUE}

\item{mtvi}{(FALSE | TRUE) Use Modified Triangular Vegetation Index 2
instead of MSAVI}

\item{senescence}{The critical value, in NDSVI, representing senescent vegetation}

\item{threshold}{Threshold value for defining NA based on < p}

\item{weight.factor}{Apply partial weights (w * weight.factor) to the NDSVI weights}

\item{...}{Additional arguments passed to terra::lapp function}
}
\value{
A terra SpatRaster class object of the weighted MSAVI metric
}
\description{
Modified Soil-adjusted Vegetation Index (MSAVI) or Modified Triangular
Vegetation Index 2 (MTVI) weighted by the Normalized difference senescent
vegetation index (NDSVI)
}
\details{
The intent of this index is to correct the MSAVI or MTVI index for bias associated
with senescent vegetation. This is done by:
1 deriving the NDSVI
2 applying a threshold to limit NDSVI to values associated with senescent vegetation
3 converting the index to inverted weights (-1*(NDSVI/sum(NDSVI)))
4 applying weights to MSAVI or MTVI

The MSAVI formula follows the modification proposed by Qi et al. (1994),
often referred to as MSAVI2. MSAVI index reduces soil noise and increases
the dynamic range of the vegetation signal. The implemented modified version
(MSAVI2) is based on an inductive method that does not use a constant L value, in
separating soil effects, an highlights healthy vegetation. The MTVI(2) index follows
Haboudane et al., (2004) and represents the area of a hypothetical triangle in spectral
space that connects (1) green peak reflectance, (2) minimum chlorophyll absorption, and
(3) the NIR shoulder. When chlorophyll absorption causes a decrease of red reflectance,
and leaf tissue abundance causes an increase in NIR reflectance, the total area of the
triangle increases. It is good for estimating green LAI, but its sensitivity to chlorophyll
increases with an increase in canopy density. The modified version of the index accounts
for the background signature of soils while preserving sensitivity to LAI  and resistance
to the influence of chlorophyll.

The Normalized difference senescent vegetation index (NDSVI) follows methods from
Qi et a., (2000). The senescence is used to threshold the NDSVI. Values less then this value
will be NA. The threshold argument is used to apply a threshold to MSAVI. The default is NULL
but if specified all values (MSAVI <= threshold) will be NA. Applying a weight.factor can be
used to change the influence of the weights on MSAVI.
}
\examples{
\donttest{
library(terra)
lsat <- rast(system.file("/extdata/Landsat_TM5.tif", package="spatialEco"))
 
# Using Modified Soil-adjusted Vegetation Index (MSAVI)
( wmsavi <- swvi(red = lsat[[3]], nir = lsat[[4]], swir = lsat[[5]]) )
    plotRGB(lsat, r=6,g=5,b=2, scale=1, stretch="lin")
      plot(wmsavi, legend=FALSE, col=rev(terrain.colors(100, alpha=0.35)), add=TRUE )

# Using Modified Triangular Vegetation Index 2 (MTVI) 
( wmtvi <- swvi(red = lsat[[3]], nir = lsat[[4]], swir = lsat[[5]],
                          green = lsat[[3]], mtvi = TRUE) )
  plotRGB(lsat, r=6,g=5,b=2, scale=1, stretch="lin")
    plot(wmtvi, legend=FALSE, col=rev(terrain.colors(100, alpha=0.35)), add=TRUE )
}

}
\references{
Haboudane, D., et al. (2004) Hyperspectral Vegetation Indices and Novel Algorithms
for Predicting Green LAI of Crop Canopies: Modeling and Validation in the Context
of Precision Agriculture. Remote Sensing of Environment 90:337-352.

Qi J., Chehbouni A., Huete A.R., Kerr Y.H., (1994). Modified Soil Adjusted Vegetation
Index (MSAVI). Remote Sens Environ 48:119-126.

Qi J., Kerr Y., Chehbouni A., (1994). External factor consideration in vegetation
index development. Proc. of Physical Measurements and Signatures in Remote Sensing,
ISPRS, 723-730.

Qi, J., Marsett, R., Moran, M.S., Goodrich, D.C., Heilman, P., Kerr, Y.H., Dedieu,
G., Chehbouni, A., Zhang, X.X. (2000). Spatial and temporal dynamics of vegetation
}
\author{
Jeffrey S. Evans  \href{mailto:sage_insights@outlook.com}{sage_insights@outlook.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sar.R
\name{sar}
\alias{sar}
\title{Surface Area Ratio}
\usage{
sar(x, s = NULL, scale = TRUE)
}
\arguments{
\item{x}{A terra SpatRaster object}

\item{s}{cell resolution (default is NULL and not needed if projection 
is in planar units)}

\item{scale}{(TRUE/FALSE) Scale (row standardize) results}
}
\value{
A terra SpatRaster class object of the Surface Area Ratio
}
\description{
Calculates the Berry (2002) Surface Area Ratio based on slope
}
\details{
SAR is calculated as: resolution^2 * cos( (degrees(slope) * (pi / 180)) )
}
\examples{
 library(terra)
 elev <- rast(system.file("extdata/elev.tif", package="spatialEco"))
 ( surface.ratio <- sar(elev) )
   plot(surface.ratio)
    
}
\references{
Berry, J.K. (2002). Use surface area for realistic calculations. Geoworld 15(9):20-1.
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/raster.kendall.R
\name{raster.kendall}
\alias{raster.kendall}
\title{Kendall tau trend with continuity correction for raster time-series}
\usage{
raster.kendall(
  x,
  intercept = TRUE,
  p.value = TRUE,
  confidence = TRUE,
  tau = TRUE,
  min.obs = 6,
  method = c("zhang", "yuepilon", "none"),
  ...
)
}
\arguments{
\item{x}{A multiband terra SpatRaster object with at least 5 layers}

\item{intercept}{(FALSE/TRUE) return a raster with the pixel
wise intercept values}

\item{p.value}{(FALSE/TRUE) return a raster with the pixel
wise p.values}

\item{confidence}{(FALSE/TRUE) return a raster with the pixel
wise 95 pct confidence levels}

\item{tau}{(FALSE/TRUE) return a raster with the pixel wise
tau correlation values}

\item{min.obs}{The threshold of minimum number of observations (default 6)}

\item{method}{Kendall method to use c("zhang", "yuepilon","none"), see kendall function}

\item{...}{Additional arguments passed to the terra app function}
}
\value{
Depending on arguments, a raster layer or rasterBrick object containing:
\itemize{
\item raster layer 1 - slope for trend, always returned
\item raster layer 2 - Kendall's tau two-sided test, reject null at 0, if tau TRUE
\item raster layer 3 - intercept for trend if intercept TRUE
\item raster layer 4 - p value for trend fit if p.value TRUE
\item raster layer 5 - lower confidence level at 95 pct, if confidence TRUE
\item raster layer 6 - upper confidence level at 95 pct, if confidence TRUE
}
}
\description{
Calculates a nonparametric statistic for a monotonic trend
based on the Kendall tau statistic and the Theil-Sen slope
modification
}
\details{
This function implements Kendall's nonparametric test for a monotonic trend
using the Theil-Sen (Theil 1950; Sen 1968; Siegel 1982) method to estimate
the slope and related confidence intervals.
}
\examples{
\donttest{
 library(terra)

 # note; nonsense example with n=9
 r <- c(rast(system.file("ex/logo.tif", package="terra")),
        rast(system.file("ex/logo.tif", package="terra")),
        rast(system.file("ex/logo.tif", package="terra"))) 
 
 # Calculate trend slope with p-value and confidence level(s)
 # ("slope","intercept", "p.value","z.value", "LCI","UCI","tau")
   k <- raster.kendall(r, method="none")
   plot(k)
}

}
\references{
Theil, H. (1950) A rank invariant method for linear and polynomial regression
analysis.  Nederl. Akad. Wetensch. Proc. Ser. A 53:386-392 (Part I),
53:521-525 (Part II), 53:1397-1412 (Part III).

Sen, P.K. (1968) Estimates of Regression Coefficient Based on Kendall's tau.
Journal of the American Statistical Association. 63(324):1379-1389.

Siegel, A.F. (1982) Robust Regression Using Repeated Medians.
Biometrika, 69(1):242-244
}
\seealso{
\code{\link[zyp]{zyp.trend.vector}} for model details

\code{\link[terra]{app}} for available ... arguments
}
\author{
Jeffrey S. Evans  \href{mailto:sage_insights@outlook.com}{sage_insights@outlook.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loess.ci.R
\name{loess.ci}
\alias{loess.ci}
\title{Loess with confidence intervals}
\usage{
loess.ci(y, x, p = 0.95, plot = FALSE, ...)
}
\arguments{
\item{y}{Dependent variable, vector}

\item{x}{Independent variable, vector}

\item{p}{Percent confidence intervals (default is 0.95)}

\item{plot}{Plot the fit and confidence intervals}

\item{...}{Arguments passed to loess}
}
\value{
A list object with:
\itemize{
\item loess Predicted values
\item se Estimated standard error for each predicted value
\item lci Lower confidence interval
\item uci Upper confidence interval
\item df Estimated degrees of freedom
\item rs Residual scale of residuals used in computing the
standard errors
}
}
\description{
Calculates a local polynomial regression fit
with associated confidence intervals
}
\examples{
 x <- seq(-20, 20, 0.1)
 y <- sin(x)/x + rnorm(length(x), sd=0.03)
 p <- which(y == "NaN")
   y <- y[-p]	
   x <- x[-p]
 
opar <- par(no.readonly=TRUE)
  par(mfrow=c(2,2))  
    lci <- loess.ci(y, x, plot=TRUE, span=0.10)
    lci <- loess.ci(y, x, plot=TRUE, span=0.30)
    lci <- loess.ci(y, x, plot=TRUE, span=0.50)
    lci <- loess.ci(y, x, plot=TRUE, span=0.80)
par(opar)

}
\references{
W. S. Cleveland, E. Grosse and W. M. Shyu (1992) Local regression models.
Chapter 8 of Statistical Models in S eds J.M. Chambers and T.J. Hastie,
Wadsworth & Brooks/Cole.
}
\author{
Jeffrey S. Evans  \href{mailto:sage_insights@outlook.com}{sage_insights@outlook.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.loess.boot.R
\name{print.loess.boot}
\alias{print.loess.boot}
\title{Print Loess bootstrap model}
\usage{
\method{print}{loess.boot}(x, ...)
}
\arguments{
\item{x}{Object of class loess.boot}

\item{...}{Ignored}
}
\value{
same as summary lowess.boot of data.frame including;
\itemize{
\item nreps        Number of bootstrap replicates
\item confidence   Confidence interval (region)
\item span         alpha (span) parameter used loess fit
\item degree       polynomial degree used in loess fit
\item normalize    Normalized data (TRUE/FALSE)
\item family       Family of statistic used in fit
\item parametric   Parametric approximation (TRUE/FALSE)
\item surface      Surface fit, see loess.control
\item data         data.frame of x,y used in model
\item fit          data.frame including:
\enumerate{
\item x - Equally-spaced x index
\item y.fit - loess fit
\item up.lim - Upper confidence interval
\item low.lim - Lower confidence interval
\item stddev - Standard deviation of loess fit at each x value
}
}
}
\description{
print method for class "loess.boot"
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sf_dissolve.R
\name{sf_dissolve}
\alias{sf_dissolve}
\title{Dissolve polygons}
\usage{
sf_dissolve(x, y = NULL, overlaps = FALSE)
}
\arguments{
\item{x}{An sf POLYGON or MULTIPOLYGON object}

\item{y}{An attribute in x to dissolve by, default is NULL}

\item{overlaps}{(FALSE/TRUE) Dissolve overlapping polygons, negates using attribute}
}
\value{
A dissolved POLYGON or MULTIPOLYGON object
}
\description{
Dissolve polygon feature calss
}
\details{
If a dissolve attribute is defined, the result will be a 
MULTIPOLYGON with the grouping attribute column. If y=NULL all polygons
will be dissolved into a single attribute, unless there is spatial
discontinuity (eg., gaps) in the data. The intent of overlaps=TRUE is to 
provide functionality for dissolving overlapping polygons and should only
be used in this specialized case.
}
\examples{
library(sf)
nc <- st_read(system.file("shape/nc.shp", package="sf"))
  nc$group <- ifelse(nc$CNTY_ <= 1902, 1, 
                ifelse(nc$CNTY_ > 1902 & nc$CNTY_ <= 1982, 2, 
				     ifelse(nc$CNTY_ > 1982, 3, NA))) 

# Dissolve by group attribute
d <- sf_dissolve(nc, "group")
  plot(st_geometry(nc), border="grey")
    plot(st_geometry(d), border="red", col=NA, 
         lwd=2, add=TRUE) 

# Dissolve all polygons
d <- sf_dissolve(nc)
  plot(st_geometry(nc), border="grey")
    plot(st_geometry(d), border="red", col=NA, 
         lwd=2, add=TRUE)

# Dissolve overlapping polygons
sq <- function(pt, sz = 1) st_polygon(list(rbind(c(pt - sz), 
     c(pt[1] + sz, pt[2] - sz), c(pt + sz), c(pt[1] - sz, pt[2] + sz), 
	 c(pt - sz))))
pol <- st_sf(box = 1:6, st_sfc(sq(c(4.2,4.2)), sq(c(0,0)), sq(c(1, -0.8)), 
          sq(c(0.5, 1.7)), sq(c(3,3)), sq(c(-3, -3))))
 st_geometry(pol) <- "geometry" 		  

plot(pol)

d <- sf_dissolve(pol, overlaps=TRUE)
  plot(d["diss"])

}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/background.R
\name{background}
\alias{background}
\title{Background sample}
\usage{
background(
  x,
  p = 1000,
  known = NULL,
  d = NULL,
  type = c("regular", "random", "hexagon", "nonaligned")
)
}
\arguments{
\item{x}{A sf class polygon defining sample region}

\item{p}{Size of sample}

\item{known}{An sf POINT class of known locations with same CSR as x}

\item{d}{Threshold distance for known proximity}

\item{type}{Type of sample c("systematic", "random", "hexagon", "nonaligned")}
}
\value{
A sf POINT feature class or data.frame with x,y coordinates
}
\description{
Creates a point sample that can be used as 
             a NULL for SDM's and other modeling approaches.
}
\details{
This function creates a background point sample based on an extent 
or polygon sampling region. The known argument can be used with d 
to remove sample points based on distance-based proximity to existing  
locations (eg., known species locations). The size (p) of the resulting 
sample will be dependent on the known locations and the influence of 
the distance threshold (d). As such, if the know and d arguments are
provided the exact value provided in p will not be returned.
}
\examples{
library(sf)

# define study area
sa <- suppressWarnings(st_cast(st_read(
        system.file("shape/nc.shp", 
        package="sf")), "POLYGON"))
  sa <- sa[10,]

# create "known" locations  
locs <- st_sample(sa, 50)
  st_crs(locs) <- st_crs(sa)

# systematic sample using extent polygon
e <- st_as_sf(st_as_sfc(st_bbox(sa)))
  st_crs(e) <- st_crs(sa)
s <- background(e, p=1000, known=locs, d=1000)
  plot(st_geometry(s), pch=20)
    plot(st_geometry(locs), pch=20, col="red", add=TRUE)

# systematic sample using irregular polygon
s <- background(sa, p=1000, known=locs, d=1000)
  plot(st_geometry(sa)) 
    plot(st_geometry(s), pch=20, add=TRUE)
      plot(st_geometry(locs), pch=20, col="red", add=TRUE)

# random sample using irregular polygon
s <- background(sa, p=500, known=locs, 
                d=1000, type="random")
  plot(st_geometry(sa)) 
    plot(st_geometry(s), pch=20, add=TRUE)
      plot(st_geometry(locs), pch=20, col="red", add=TRUE)

}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parse.bits.R
\name{parse.bits}
\alias{parse.bits}
\title{Parse bits}
\usage{
parse.bits(x, bit, depth = 8, order = c("reverse", "none"))
}
\arguments{
\item{x}{Integer value}

\item{bit}{A single or vector of bits to return}

\item{depth}{The depth (length) of the bit range, default is 8}

\item{order}{c("reverse", "none") sort order for the bits}
}
\value{
a vector or data.frame of parsed interger value(s) associated with input bit
}
\description{
Returns specified bit value based on integer input

Data such as MODIS the QC band are stored in bits. This function returns the 
value(s) for specified bit. For example, the MODIS QC flag are bits 0-1 with 
the bit value 00 representing the "LST produced, good quality" flag. When 
exported from HDF the QC bands are often in an 8 bit integer range (0-255). 
With this function you can parse the values for each bit to assign the 
flag values.
}
\examples{
# Return value for bit 5 for integer value 100
parse.bits(100, 5)
 
# Return value(s) for bits 0 and 1 for integer value 100
parse.bits(100, c(0,1))

# Return value(s) for bits 0 and 1 for integer values 0-255
for(i in 0:255) { print(parse.bits(i, c(0,1))) }
 
\donttest{
#### Applied Example using Harmonized Landsat Sentinel-2 QC 

# Create dummy data and qc band
library(terra)
r <- rast(nrow=100, ncol=100)
  r[] <- round(runif(ncell(r), 0,1)) 
qc <- rast(nrow=100, ncol=100)
  qc[] <- round(runif(ncell(qc), 64,234)) 

# Calculate bit values from QC table
( qc_bits <- data.frame(int=0:255, 
	cloud = unlist(lapply(0:255, FUN=parse.bits, bit=1)),
	shadow = unlist(lapply(0:255, FUN=parse.bits, bit=3)),
	acloud = unlist(lapply(0:255, FUN=parse.bits, bit=2)),
	cirrus = unlist(lapply(0:255, FUN=parse.bits, bit=0)),
	aerosol = unlist(lapply(0:255, FUN=parse.bits, bit=c(7,6)))) )
		
# Query the results to create a vector of integer values indicating what to mask 
#  cloud is bit 1 and shadow bit 3	
m <- sort(unique(qc_bits[c(which(qc_bits$cloud == 1),
                           which(qc_bits$shadow == 1)
						   ),]$int))

# Apply queried integer values to mask image with QA band
qc[qc \%in\% m] <- NA
r <- mask(r, qc)
}

}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sieve.R
\name{sieve}
\alias{sieve}
\title{Sieve raster data}
\usage{
sieve(x, a, unit = c("m", "km", "ha"))
}
\arguments{
\item{x}{An integer terra SpatRaster}

\item{a}{Query area to remove}

\item{unit}{The unit to use for area query options are c("m", "km", "ha")}
}
\value{
A terra SpatRaster with cells < a set to NA
}
\description{
Removes contiguous cells < specified query area
}
\details{
A sieve can be used to establish a minimal mapping unit where
contiguous cells < specified query area are set to NA. These NA
values can then be filled using focal (majority, median, mean)
}
\examples{
\donttest{
library(terra)
elev <- rast(system.file("extdata/elev.tif", package="spatialEco"))
m <- matrix(c(100,200,1,200,300,2,300,400,3,400,
       500,4, 500,600,5), ncol=3, byrow=TRUE)
  x <- classify(elev, m)

# Sieve to a MMU of 60km
sv <- spatialEco::sieve(x, a = 60, unit = "km")
  plot(c(x, sv))
} 
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/raster.invert.R
\name{raster.invert}
\alias{raster.invert}
\title{Invert raster}
\usage{
raster.invert(x)
}
\arguments{
\item{x}{A terra SpatRaster object}
}
\value{
A terra SpatRaster object with inverted (flipped) raster values
}
\description{
Inverts (flip) the values of a raster
}
\details{
Inverts raster values using the formula: (((x - max(x)) * -1) + min(x)
}
\examples{
library(terra)
r <- rast(nrows=500, ncols=500, xmin=571823, xmax=616763, 
            ymin=4423540, ymax=4453690)
 crs(r) <- "epsg:9001"
r[] <- runif(ncell(r), 1000, 2500)
r <- focal(r, focalMat(r, 150, "Gauss") )

r.inv <- raster.invert(r)

opar <- par(no.readonly=TRUE)
    par(mfrow=c(1,2))
      plot(r, main="original raster")
      plot(r.inv, main="inverted raster") 
par(opar)   

}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stratified.random.R
\name{stratified.random}
\alias{stratified.random}
\title{Stratified random sample}
\usage{
stratified.random(x, strata, n = 10, reps = 1, replace = FALSE)
}
\arguments{
\item{x}{An sf class object}

\item{strata}{Column in x with stratification factor}

\item{n}{Number of random samples}

\item{reps}{Number of replicates per strata}

\item{replace}{(TRUE/FALSE) Sampling with replacement}
}
\value{
An sf class object containing random samples
}
\description{
Creates a stratified random sample of an sp class object
}
\details{
If replace=FALSE features are removed from consideration in subsequent replicates.
Conversely, if replace=TRUE, a feature can be selected multiple times across 
replicates. Not applicable if rep=1.
}
\examples{
if(require(sp, quietly = TRUE)) {
library(sf)
  data(meuse, package = "sp")
  meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, 
                    agr = "constant")

# Create stratified variable using quartile breaks
x1 <- cut(meuse$cadmium, summary(meuse$cadmium)[-4], 
          include.lowest=TRUE)
  levels(x1) <- seq(1,nlevels(x1),1)
x2 <- cut(meuse$lead, summary(meuse$lead)[-4], 
          include.lowest=TRUE)
  levels(x2) <- seq(1,nlevels(x2),1) 
meuse$STRAT <- paste(x1, x2, sep='.')

# Counts for each full strata (note; 2 strata have only 1 observation)
tapply(meuse$STRAT, meuse$STRAT, length)
   
# 2 replicates, 2 samples with replacement
ssample <- stratified.random(meuse, strata='STRAT', n=2, reps=2, 
                             replace=TRUE)
  tapply(ssample$STRAT, ssample$STRAT, length)

# 2 replicates, 2 samples no replacement
ssample.nr <- stratified.random(meuse, strata='STRAT', n=2, reps=2)
  tapply(ssample.nr$STRAT, ssample.nr$STRAT, length)

# n=1 and reps=10 for sequential numbering of samples 
ssample.ct <- stratified.random(meuse, strata='STRAT', n=1, reps=10, 
                                replace=TRUE)
  tapply(ssample.ct$STRAT, ssample.ct$STRAT, length)

# Plot random samples colored by replacement
ssample$REP <- factor(ssample$REP)
  plot(ssample['REP'], pch=20)

} else { 
  cat("Please install sp package to run example", "\n")
}

}
\references{
Hudak, A.T., N.L. Crookston, J.S. Evans, M.J. Falkowski, A.M.S. Smith, P. Gessler 
  and P. Morgan. (2006) Regression modelling and mapping of coniferous forest basal 
  area and tree density from discrete-return lidar and multispectral satellite data. 
  Canadian Journal of Remote Sensing 32: 126-138.
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.poly.trend.R
\name{print.poly.trend}
\alias{print.poly.trend}
\title{Print poly_trend}
\usage{
\method{print}{poly.trend}(x, ...)
}
\arguments{
\item{x}{Object of class poly.trend}

\item{...}{Ignored}
}
\value{
Prints trend model summary, order and trend confidence intervals
}
\description{
print method for class "poly.trend"
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/srr.R
\name{srr}
\alias{srr}
\title{Surface Relief Ratio}
\usage{
srr(x, s = 5, ...)
}
\arguments{
\item{x}{A terra SpatRaster object}

\item{s}{Focal window size}

\item{...}{Additional arguments passed to terra::lapp}
}
\value{
A terra SpatRaster object of Pike's (1971) Surface Relief Ratio
}
\description{
Calculates the Pike (1971) Surface Relief Ratio
}
\details{
Describes rugosity in continuous raster surface within a specified window. 
The implementation of SRR can be shown as: (mean(x) - min(x)) / (max(x) - min(x))
}
\examples{
\donttest{
library(terra)
elev <- rast(system.file("extdata/elev.tif", package="spatialEco"))
  r.srr <- srr(elev, s=5)
    plot(r.srr, main="Surface Relief Ratio") 
 }     
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bbox_poly.R
\name{bbox_poly}
\alias{bbox_poly}
\title{Bounding box polygon}
\usage{
bbox_poly(x)
}
\arguments{
\item{x}{An sf or terra object or vector of bounding coordinates}
}
\value{
A single feature sf class polygon object
}
\description{
Creates a polygon from a vector or raster extent
}
\details{
If not a spatial object, expected order of input for x is: xmin, ymin, 
xmax, ymax. Where; xmin, ymin and the coordinates of top left corner of the 
bounding box and xmax, ymax represent the bottom right corner. The maximum 
value of xmax is width of the extent while maximum value of ymax is the height 
of the extent.
}
\examples{
if(require(sp, quietly = TRUE)) {
library(terra)
library(sf)
  data(meuse, package = "sp")
  meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, 
                    agr = "constant")

# raster (terra)
r <- rast(ext(meuse))
  r[] <- runif(ncell(r))
 crs(r) <- "epsg:28992"
e <- bbox_poly(r)

plot(r)
  plot(st_geometry(e), border="red", add=TRUE)

# extent vector
e <- bbox_poly(c(178605, 329714, 181390, 333611)) 
  plot(e)

# vector bounding box
e <- bbox_poly(meuse)

plot(st_geometry(meuse), pch=20)
  plot(st_geometry(e), add=TRUE)

} else { 
  cat("Please install sp package to run this example", "\n")
}

}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/outliers.R
\name{outliers}
\alias{outliers}
\title{Outliers}
\usage{
outliers(x, s = 1.4826)
}
\arguments{
\item{x}{A numeric vector}

\item{s}{Scaling factor for mad statistic}
}
\value{
value for the modified Z-score
}
\description{
Identify outliers using modified Z-score
}
\examples{
 # Create data with 3 outliers
    x <- seq(0.1, 5, length=100) 
    x[98:100] <- c(100, 55, 250)
 
 # Calculate Z score
     Z <- outliers(x) 
 
 # Show number of extreme outliers using Z-score
     length(Z[Z > 9.9])
 
 # Remove extreme outliers 
     x <- x[-which(Z > 9.9)]

}
\references{
Iglewicz, B. & D.C. Hoaglin (1993) How to Detect and Handle Outliers, 
  American Society for Quality Control, Milwaukee, WI.
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/all_pairwise.R
\name{all_pairwise}
\alias{all_pairwise}
\title{All pairwise combinations}
\usage{
all_pairwise(x)
}
\arguments{
\item{x}{A numeric or character vector}
}
\value{
A list object with increasing all combination objects,
        the first list element are the pairwise comparisons
}
\description{
Creates all pairwise combinations list for iteration
}
\details{
This returns a list of vector combinations starting with
pairwise, as the first nested list element, then in groups of 
 threes, fours, to length of the vector.
}
\examples{
classes <- paste0("class", 1:10)

all_pairwise(classes)[[1]]

#### How to use as an iterator
# dataframe with 4 cols, 100 rows
d <- as.data.frame(matrix(runif(100*4), 100, 4)) 
  names(d) <- paste0("class", 1:4) 

( idx <- all_pairwise(colnames(d))[[1]] ) 

opar <- par(no.readonly=TRUE)
  par(mfrow=c(2,3))
    lapply(idx, function(i) {
      plot(d[,i[1]], d[,i[2]], main=paste0(i[1], " vs ", i[2]) )
    })	
par(opar)

}
\author{
Jeffrey S. Evans    <jeffrey_evans<at>tnc.org>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/combine.R
\name{combine}
\alias{combine}
\title{raster combine}
\usage{
combine(x)
}
\arguments{
\item{x}{raster stack/brick or SpatialPixelsDataFrame object}
}
\value{
A ratified (factor) terra SpatRaster representing unique combinations.
}
\description{
Combines rasters into all unique combinations of inputs
}
\details{
A single ratified raster object is returned with the summary table
as the raster attribute table, this is most similar to the ESRI 
format resulting from their combine function. 
    
Please note that this is not a memory safe function that utilizes
out of memory in the manner that the terra package does.
}
\examples{
library(terra)

# Create example data (with a few NA's introduced)
 r1 <- rast(nrows=100, ncol=100)
   names(r1) <- "LC1"
   r1[] <- round(runif(ncell(r1), 1,4),0)
     r1[c(8,10,50,100)] <- NA
 r2 <- rast(nrows=100, ncol=100)
   names(r2) <- "LC2"
   r2[] <- round(runif(ncell(r2), 2,6),0)
     r2[c(10,50,100)] <- NA   
 r3 <- rast(nrows=100, ncol=100)
   names(r3) <- "LC3"
   r3[] <- round(runif(ncell(r3), 2,6),0)
     r3[c(10,50,100)] <- NA   
 r <- c(r1,r2,r3)  
   names(r) <- c("LC1","LC2","LC3")

 # Combine rasters with a multilayer stack
 cr <- combine(r)
   head(cr$summary)
   plot(cr$combine)

# or, from separate layers
 cr <- combine(c(r1,r3))

}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shift.R
\name{shift}
\alias{shift}
\title{shift}
\usage{
shift(x, lag = 1, pad = NA)
}
\arguments{
\item{x}{A vector}

\item{lag}{Number of lagged offsets, default is 1}

\item{pad}{Value to fill the lagged offset with, default is NA}
}
\value{
A vector, length equal to x, with offset length filled with pad values
}
\description{
Shift a vector by specified positive or negative lag
}
\examples{
x <- 1:10

shift(x, 1)     # shift positive (from beginning of vector) by 1
shift(x, -1)    # shift negative (from end of vector) by 1
shift(x, 5, 0)  # Shift by 5 and fill (pad) with 0

}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is.empty.R
\name{is.empty}
\alias{is.empty}
\title{is.empty}
\usage{
is.empty(x, all.na = FALSE, na.empty = TRUE, trim = TRUE)
}
\arguments{
\item{x}{A vector to evaluate elements}

\item{all.na}{(FALSE / TRUE) Return a TRUE if all elements are NA}

\item{na.empty}{(TRUE / FALSE) Return TRUE if element is NA}

\item{trim}{(TRUE / FALSE) Trim empty strings}
}
\value{
A Boolean indicating empty elements in a vector, if all.na = FALSE
a TRUE/FALSE value will be returned for each element in the vector
}
\description{
evaluates empty elements in a vector
}
\details{
This function evaluates if an element in a vector is empty the na.empty argument 
allows for evaluating NA values (TRUE if NA) and all.na returns a TRUE if all elements 
are NA. The trim argument trims a character string to account for the fact that c(" ") 
is not empty but, a vector with c("") is empty. Using trim = TRUE will force both 
to return TRUE
}
\examples{
is.empty( c("") )
is.empty( c(" ") )
is.empty( c(" "), trim=FALSE )

is.empty( c("",NA,1) )
is.empty( c("",NA,1), na.empty=FALSE)

is.empty( c(NA,NA,NA) )
is.empty( c(NA,NA,NA), all.na=TRUE )
is.empty( c(NA,2,NA), all.na=TRUE )

any( is.empty( c("",2,3) ) )
any( is.empty( c(1,2,3) ) )

}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/max_extent.R
\name{max_extent}
\alias{max_extent}
\title{Maximum extent of multiple rasters}
\usage{
max_extent(x, ...)
}
\arguments{
\item{x}{terra SpatRaster class object}

\item{...}{additional SpatRaster class objects in same projection}
}
\value{
An sf POLYGON class object representing maximum extents
}
\description{
returns a extent polygon representing maximum extent of
             input rasters
}
\details{
Creates a maximum extent polygon of all specified rasters
}
\examples{
library(terra)

r1 <- rast(ext(61.87125, 76.64458, 23.90153, 37.27042))
r2 <- rast(ext(67.66625, 81.56847, 20.38458, 35.67347))
r3 <- rast(ext(72.64792,84.38125,5.91125,28.13347 ))

( e <- max_extent(r1, r2, r3) )
plot(e, border=NA)
  plot(ext(r1), border="red", add=TRUE)
  plot(ext(r2), border="green", add=TRUE)
  plot(ext(r3), border="blue", add=TRUE)
  plot(e, border="black", add=TRUE)
		    
 sf::st_bbox(e) # full extent

}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/partial.cor.R
\name{partial.cor}
\alias{partial.cor}
\title{Partial and Semi-partial correlation}
\usage{
partial.cor(
  x,
  y,
  z,
  method = c("partial", "semipartial"),
  statistic = c("kendall", "pearson", "spearman")
)
}
\arguments{
\item{x}{A vector, data.frame or matrix with 3 columns}

\item{y}{A vector same length as x}

\item{z}{A vector same length as x}

\item{method}{Type of correlation: "partial" or "semipartial"}

\item{statistic}{Correlation statistic, options are: "kendall",
"pearson", "spearman"}
}
\value{
data.frame containing:
\itemize{
\item correlation - correlation coefficient
\item p.value - p-value of correlation
\item test.statistic - test statistic
\item n - sample size
\item Method - indicating partial or semipartial correlation
\item Statistic - the correlation statistic used
}
}
\description{
Calculates a partial or semi-partial correlation
with parametric and nonparametric options
}
\details{
Partial and semipartial correlations show the association between two
variables when one or more peripheral variables are controlled
to hold them constant.

Suppose we have three variables, X, Y, and Z. Partial correlation holds
constant one variable when computing the relations two others. Suppose we
want to know the correlation between X and Y holding Z constant for both
X and Y. That would be the partial correlation between X and Y controlling
for Z. Semipartial correlation holds Z constant for either X or Y, but not
both, so if we wanted to control X for Z, we could compute the semipartial
correlation between X and Y holding Z constant for X.
}
\examples{
air.flow = stackloss[,1]
water.temperature = stackloss[,2]
acid = stackloss[,3]

# Partial using Kendall (nonparametric) correlation
partial.cor(air.flow, water.temperature, acid)

scholar <- data.frame(
  HSGPA=c(3.0, 3.2, 2.8, 2.5, 3.2, 3.8, 3.9, 3.8, 3.5, 3.1), 
	 FGPA=c(2.8, 3.0, 2.8, 2.2, 3.3, 3.3, 3.5, 3.7, 3.4, 2.9),
  SATV =c(500, 550, 450, 400, 600, 650, 700, 550, 650, 550)) 

# Standard Pearson's correlations between HSGPA and FGPA  
cor(scholar[,1], scholar[,2])

# Partial correlation using Pearson (parametric) between HSGPA 
#   and FGPA, controlling for SATV
partial.cor(scholar, statistic="pearson")

# Semipartial using Pearson (parametric) correlation 
partial.cor(x=scholar[,2], y=scholar[,1], z=scholar[,3], 
            method="semipartial", statistic="pearson")

}
\author{
Jeffrey S. Evans  \href{mailto:sage_insights@outlook.com}{sage_insights@outlook.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.effect.size.R
\name{summary.effect.size}
\alias{summary.effect.size}
\title{Summarizing effect size}
\usage{
\method{summary}{effect.size}(object, ...)
}
\arguments{
\item{object}{Object of class effect.size}

\item{...}{Ignored}
}
\value{
Prints the output data.frame contaning; effect size with upper and lower confidence 
and, mean and sd by group
}
\description{
Summary method for class "effect.size".
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/built.index.R
\name{built.index}
\alias{built.index}
\title{built index}
\usage{
built.index(
  green,
  red,
  nir,
  swir1,
  swir2,
  L = 0.5,
  method = c("Bouhennache", "Zha", "Xu")
)
}
\arguments{
\item{green}{Green band (0.53 - 0.59mm), landsat 5&7 band 3, OLI
(landsat 8) band 3}

\item{red}{Red band (0.636 - 0.673mm), landsat 5&7 band 3, OLI
(landsat 8) band 4}

\item{nir}{Near infrared band (0.851 - 0.879mm) landsat 5&7 band 4,
OLI (landsat 8) band 5}

\item{swir1}{short-wave infrared band 1 (1.566 - 1.651mm), landsat 5&7
band 5, OLI (landsat 8) band 6}

\item{swir2}{short-wave infrared band 2 (2.11 - 2.29mm), landsat 5&7
band 7, OLI (landsat 8) band 7}

\item{L}{The L factor for the savi index}

\item{method}{Method to use for index options are "Bouhennache", "Zha", "Xu"}
}
\value{
A terra raster object of the built index
}
\description{
Remote sensing built-up index
}
\details{
This function calculates the built-up index. Three methods are available:
\itemize{
\item Bouhennache is a new method that uses a larger portion of the VIR/NIR
following OLI bands (((b3+b4+b7)-b6)/3) / (((b3+b4+b7)+b6)/3)
\item Zha is the original band ratio method using TM5 ndbi = (b5 - b4) / (b5 + b4)
\item Xu is a modification to eliminate noise using ETM+7
(ndbi-((savi-nndwi)/2) / (ndbi+((savi-nndwi)/2)
}

Generally water has the highest values where built-up areas will occur in the
mid portion of the distribution. Since Bouhennache et al (2018) index exploits
a larger portion of the visible (Vis) and infra red spectrum, vegetation will
occur as the  lowest values and barren will exhibit greater values than the
vegetation and lower values than the built-up areas.

Band wavelength (nanometers) designations for landsat
TM4, TM5 and ETM+7
\itemize{
\item band-2 0.52-0.60 (green)
\item band-3 0.63-0.69 (red)
\item band-4 0.76-0.90 (NIR)
\item band-5 1.55-1.75 (SWIR 1)
\item band-7 2.09-2.35 (SWIR 2)
}

OLI (Landsat 8)
\itemize{
\item band-3 0.53-0.59 (green)
\item band-4 0.64-0.67 (red)
\item band-5 0.85-0.88 (NIR)
\item band-6 1.57-1.65 (SWIR 1)
\item band-7 2.11-2.29 (SWIR 2)
}
}
\examples{
\donttest{
library(terra)
 lsat <- rast(system.file("/extdata/Landsat_TM5.tif", package="spatialEco"))
   plotRGB(lsat, r=3, g=2, b=1, scale=1.0, stretch="lin")
			   
 # Using Bouhennache et al., (2018) method (needs green, red, swir1 and swir2) 
 ( bouh <- built.index(red = lsat[[3]], green = lsat[[2]], swir1 = lsat[[5]], 
                      swir2 = lsat[[6]]) )
    plotRGB(lsat, r=3, g=2, b=1, scale=1, stretch="lin")
      plot(bouh, legend=FALSE, col=rev(terrain.colors(100, alpha=0.35)), 
	       add=TRUE )

 # Using simple Zha et al., (2003) method (needs nir and swir1)
 ( zha <- built.index(nir = lsat[[4]], swir1 = lsat[[5]], method = "Zha") )
   plotRGB(lsat, r=3, g=2, b=1, scale=1, stretch="lin")
     plot(zha, legend=FALSE, col=rev(terrain.colors(100, alpha=0.35)), add=TRUE )

 # Using Xu (2008) normalized modification of Zha (needs green, red, nir and swir1)
 ( xu <- built.index(green= lsat[[2]], red = lsat[[3]], nir = lsat[[4]], 
                     swir1 = lsat[[5]], , method = "Xu") )
   plotRGB(lsat, r=3, g=2, b=1, scale=1, stretch="lin")
     plot(xu, legend=FALSE, col=rev(terrain.colors(100, alpha=0.35)), add=TRUE ) 
}

}
\references{
Bouhennache, R., T. Bouden, A. Taleb-Ahmed & A. Chaddad(2018) A new spectral index
for the extraction of built-up land features from Landsat 8 satellite imagery,
Geocarto International 34(14):1531-1551

Xu H. (2008) A new index for delineating built-up land features in satellite imagery.
International Journal Remote Sensing 29(14):4269-4276.

Zha G.Y., J. Gao, & S. Ni (2003) Use of normalized difference built-up index in
automatically mapping urban areas from TM imagery. International Journal of
Remote Sensing 24(3):583-594
}
\author{
Jeffrey S. Evans  \href{mailto:sage_insights@outlook.com}{sage_insights@outlook.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/raster.vol.R
\name{raster.vol}
\alias{raster.vol}
\title{Raster Percent Volume}
\usage{
raster.vol(
  x,
  p = 0.75,
  sample = FALSE,
  spct = 0.05,
  type = c("random", "regular")
)
}
\arguments{
\item{x}{A terra SpatRaster class object}

\item{p}{percent raster-value volume}

\item{sample}{(FALSE/TRUE) base volume on systematic point sample}

\item{spct}{sample percent, if sample (TRUE)}

\item{type}{If sample=TRUE type of sample, options are "random" or "regular"}
}
\value{
if sample (FALSE) binary raster object with 1 representing designated 
percent volume else, if sample (TRUE) n sf POINT object with points 
that represent the percent volume of the sub-sample
}
\description{
Calculates a percent volume on a raster or based on a 
             systematic sample
}
\note{
Since this model needs to operate on all of the raster values, it is not memory safe
}
\examples{
\donttest{
library(terra)
  r <- rast(ncols=100, nrows=100)
    r[] <- runif(ncell(r), 0, 1)
    r <- focal(r, w=focalMat(r, 6, "Gauss"))
  #r[sample(1:ncell(r)),10] <- NA
  
  # full raster percent volume 
  p30 <- raster.vol(r, p=0.30)
  p50 <- raster.vol(r, p=0.50)
  p80 <- raster.vol(r, p=0.80)

opar <- par(no.readonly=TRUE)
    par(mfrow=c(2,2))
    plot(r, col=cm.colors(10), main="original raster")
    plot(p30, breaks=c(0,0.1,1), col=c("cyan","red"), legend=FALSE,
      main="30\% volume")
    plot(p50, breaks=c(0,0.1,1), col=c("cyan","red"), legend=FALSE,
      main="50\% volume")
    plot(p80, breaks=c(0,0.1,1), col=c("cyan","red"), legend=FALSE,
      main="80\% volume")
par(opar)  
}

}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kl.divergence.R
\name{kl.divergence}
\alias{kl.divergence}
\title{Kullback-Leibler divergence (relative entropy)}
\usage{
kl.divergence(object, eps = 10^-4, overlap = TRUE)
}
\arguments{
\item{object}{Matrix or dataframe object with >=2 columns}

\item{eps}{Probabilities below this threshold are replaced by this 
threshold for numerical stability.}

\item{overlap}{Logical, do not determine the KL divergence for those 
pairs where for each point at least one of the densities 
has a value smaller than eps.}
}
\value{
pairwise Kullback-Leibler divergence index (matrix)
}
\description{
Calculates the Kullback-Leibler divergence (relative entropy)
}
\details{
Calculates the Kullback-Leibler divergence (relative entropy) between 
unweighted theoretical component distributions. Divergence is calculated 
as: int [f(x) (log f(x) - log g(x)) dx] for distributions with densities 
f() and g().
}
\examples{
x <- seq(-3, 3, length=200)
y <- cbind(n=dnorm(x), t=dt(x, df=10))
  matplot(x, y, type='l')
    kl.divergence(y)
   
# extract value for last column
  kl.divergence(y[,1:2])[3:3]

}
\references{
Kullback S., and R. A. Leibler (1951) On information and sufficiency.  
  The Annals of Mathematical Statistics 22(1):79-86
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/raster.modified.ttest.R
\name{raster.modified.ttest}
\alias{raster.modified.ttest}
\title{Dutilleul moving window bivariate raster correlation}
\usage{
raster.modified.ttest(
  x,
  y,
  d = "auto",
  sample = c("none", "random", "hexagonal", "regular"),
  p = 0.1,
  size = NULL
)
}
\arguments{
\item{x}{A terra SpatRaster class object}

\item{y}{A terra SpatRaster class object, same dimensions as x}

\item{d}{Distance for finding neighbors}

\item{sample}{Apply sub-sampling options; c("none", "random", "hexagonal", "regular")}

\item{p}{If sample != "none", what proportion of population
should be sampled}

\item{size}{Fixed sample size (default NULL)}
}
\value{
A terra SpatRaster or sf POINT class object with the following attributes:
\itemize{
\item corr - Correlation
\item Fstat - The F-statistic calculated as degrees of freedom unscaled F-statistic
\item p.value - p-value for the test
\item moran.x - Moran's-I for x
\item moran.y - Moran's-I for y
}
}
\description{
A bivarate raster correlation using Dutilleul's
modified t-test

This function provides a bivariate moving window correlation using the modified
t-test to account for spatial autocorrelation. Point based subsampling is provided
for computation tractability. The hexagon sampling is recommended as it it good
at capturing spatial process that includes nonstationarity and anistropy.
}
\examples{
\donttest{
 p = c("sf", "sp", "terra", "gstat")
 if(any(!unlist(lapply(p, requireNamespace, quietly=TRUE)))) { 
   m = which(!unlist(lapply(p, requireNamespace, quietly=TRUE)))
   message("Can't run examples, please install ", paste(p[m], collapse = " "))
 } else {
   invisible(lapply(p, require, character.only=TRUE))

data(meuse, package = "sp")
meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, 
                  agr = "constant") 
data(meuse.grid, package = "sp")                                      
meuse.grid <- st_as_sf(meuse.grid, coords = c("x", "y"), crs = 28992, 
                  agr = "constant") 

ref <- rast(ext(meuse.grid), resolution = 40)
  crs(ref) <- crs(meuse)
e <- ext(179407.8, 181087.9, 331134.4, 332332.1)
                                       
# GRID-1 log(copper): 
v1 <- variogram(log(copper) ~ 1, meuse) 
  x1 <- fit.variogram(v1, vgm(1, "Sph", 800, 1))           
  G1 <- krige(zinc ~ 1, meuse, meuse.grid, x1, nmax = 30)
G1 <- crop(rasterize(G1, ref, "var1.pred"),e)
names(G1) <- "copper"
 
 # GRID-2 log(elev):
v2 <- variogram(log(elev) ~ 1, meuse) 
  x2 <- fit.variogram(v2, vgm(1, "Sph", 800, 1))           
  G2 <- krige(zinc ~ 1, meuse, meuse.grid, x2, nmax = 30)
G2 <- crop(rasterize(G2, ref, "var1.pred"),e)
names(G2) <- "elev"

# Raster corrected correlation 
acor <- raster.modified.ttest(G1, G2)
  plot(acor)
 
# Sample-based corrected correlation
( cor.hex <- raster.modified.ttest(G1, G2, sample = "hexagonal") )	 
  plot(cor.hex["corr"], pch=20)
}
}
}
\references{
Clifford, P., S. Richardson, D. Hemon (1989), Assessing the significance of the
correlationbetween two spatial processes. Biometrics 45:123-134.

Dutilleul, P. (1993), Modifying the t test for assessing the correlation between
two spatial processes. Biometrics 49:305-314.
}
\seealso{
\code{\link[SpatialPack]{modified.ttest}} for test details
}
\author{
Jeffrey S. Evans  \href{mailto:sage_insights@outlook.com}{sage_insights@outlook.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.effect.size.R
\name{plot.effect.size}
\alias{plot.effect.size}
\title{Plot effect size}
\usage{
\method{plot}{effect.size}(x, ...)
}
\arguments{
\item{x}{A effect.size object}

\item{...}{Additional arguments passed to plot}
}
\value{
Plot of effect size object with group effect sizes and 95% confidence
}
\description{
Plot function for effect.size object
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/local.min.max.R
\name{local.min.max}
\alias{local.min.max}
\title{Local minimum and maximum}
\usage{
local.min.max(x, dev = mean, plot = TRUE, add.points = FALSE, ...)
}
\arguments{
\item{x}{A numeric vector}

\item{dev}{Deviation statistic (mean or median)}

\item{plot}{plot the minimum and maximum values with the
distribution (TRUE/FALSE)}

\item{add.points}{Should all points of x be added to
plot (TRUE/FALSE)}

\item{...}{Arguments passed to plot}
}
\value{
A list object with:
\itemize{
\item minima - minimum local values of x
\item maxima - maximum local values of x
\item mindev - Absolute deviation of minimum from specified deviation
statistic (dev argument)
\item maxdev - Absolute deviation of maximum from specified deviation
statistic (dev argument)
}
}
\description{
Calculates the local minimums and maximums in a numeric vector,
indicating inflection points in the distribution.
}
\details{
Useful function for identifying inflection or enveloping points in
a distribution
}
\examples{
                                                 
x <- rnorm(100,mean=1500,sd=800) 
( lmm <- local.min.max(x, dev=mean, add.points=TRUE, 
                       main="Local Minima and Maxima") )

# return only local minimum values
   local.min.max(x)$minima 
                                           
}
\author{
Jeffrey S. Evans  \href{mailto:sage_insights@outlook.com}{sage_insights@outlook.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spherical.sd.R
\name{spherical.sd}
\alias{spherical.sd}
\title{Spherical Variance or Standard Deviation of Surface}
\usage{
spherical.sd(r, d, variance = FALSE, ...)
}
\arguments{
\item{r}{A terra SpatRaster class object}

\item{d}{Size of focal window or a matrix to use in focal function}

\item{variance}{(FALSE|TRUE) Output spherical variance rather than standard deviation}

\item{...}{Additional arguments passed to terra:app (can write raster to disk here)}
}
\value{
A terra SpatRaster class object of the spherical standard deviation
}
\description{
Derives the spherical standard deviation of a raster surface
}
\details{
Surface variability using spherical variance/standard deviation. 
The variation can be assessed using the spherical standard deviation of the normal 
direction within a local neighborhood. This is found by expressing the normal 
directions on the surfaces cells in terms of their displacements in a Cartesian (x,y,z) 
coordinate system. Averaging the x-coordinates, y-coordinates, and z-coordinates 
separately gives a vector (xb, yb, zb) pointing in the direction of the average 
normal. This vector will be shorter when there is more variation of the normals and 
it will be longest--equal to unity--when there is no variation. Its squared length 
is (by the Pythagorean theorem) given by: R^2 = xb^2 + yb^2 + zb^2
   where; x = cos(aspect) * sin(slope) and xb = nXn focal mean of x 
          y = sin(aspect) * sin(slope) and  yb = nXn focal mean of y
          z = cos(slope) and zb = nXn focal mean of z 	   

The slope and aspect values are expected to be in radians. 
The value of (1 - R^2), which will lie between 0 and 1, is the spherical variance. 
and it's square root can be considered the spherical standard deviation.
}
\examples{
\donttest{
 library(terra)
 elev <- rast(system.file("extdata/elev.tif", package="spatialEco"))
 
 ssd <- spherical.sd(elev, d=5)
 
 slope <- terrain(elev, v='slope')
 aspect <- terrain(elev, v='aspect')
 hill <- shade(slope, aspect, 40, 270)
 plot(hill, col=grey(0:100/100), legend=FALSE, 
      main='terrain spherical standard deviation')
   plot(ssd, col=rainbow(25, alpha=0.35), add=TRUE)
}
 
}
\seealso{
\code{\link[terra]{app}} for details on ... arguments
}
\author{
Jeffrey S. Evans <jeffrey_evans<at>tnc.org>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/squareBuffer.R
\name{squareBuffer}
\alias{squareBuffer}
\title{Square buffer}
\usage{
squareBuffer(x, a, ...)
}
\arguments{
\item{x}{An sf object}

\item{a}{Numeric single or vector indicating buffer distance(s)}

\item{...}{Additional arguments passed to st_buffer}
}
\value{
A single feature sf class polygon object
}
\description{
Creates a square buffer of a feature class
}
\details{
Function creates a square buffer of feature class.
}
\examples{
library(sf)
xy <- st_as_sf(data.frame(x = c(1,3,6,7),
               y = c(3,2,7,8), z = c(38,23,12,12),
               area = c(32,23,45,67)),
			      coords = c("x", "y"), 
			      agr = "constant") 

# With fixed buffer
sb <- squareBuffer(xy, 32)
  plot(st_geometry(sb))
    plot(st_geometry(xy), pch=20, add=TRUE)
  
# With variable buffer
sb.var <- squareBuffer(xy, xy$area)
  plot(st_geometry(sb.var))
    plot(st_geometry(xy), pch=20, add=TRUE)
  
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tpi.R
\name{tpi}
\alias{tpi}
\title{Topographic Position Index (tpi)}
\usage{
tpi(x, scale = 3, win = "rectangle", normalize = FALSE, zero.correct = FALSE)
}
\arguments{
\item{x}{A terra SpatRaster object}

\item{scale}{focal window size (n-cell x n-cell for rectangle or 
distance for circle)}

\item{win}{Window type. Options are "rectangle" and "circle"}

\item{normalize}{Apply deviation correction that normalizes to local 
surface roughness}

\item{zero.correct}{Apply correction for zero values in matrix weights}
}
\value{
A terra SpatRaster object of tpi A terra SpatRaster object
}
\description{
Calculates topographic position using mean deviations
}
\examples{
\donttest{
library(terra)
elev <- rast(system.file("extdata/elev.tif", package="spatialEco"))

# calculate tpi and plot 
  tpi7 <- tpi(elev, scale=7) 
  tpi025 <- tpi(elev, win = "circle", scale=2500)
  tpi025.zc <- tpi(elev, win = "circle", scale=2500, 
                   zero.correct = TRUE)

opar <- par(no.readonly=TRUE)
    par(mfrow=c(2,2))
      plot(elev, main="original raster")
      plot(tpi7, main="tpi 7x7")
      plot(tpi025, main="tpi Circular window d=2500m")
   plot(tpi025, main="tpi Circular window d=2500m, zero correct")
par(opar)
}

}
\references{
De Reu, J., J. Bourgeois, M. Bats, A. Zwertvaegher, V. Gelorini, et al., (2014) 
  Application of the topographic position index to heterogeneous landscapes. 
  Geomorphology, 186:39-49.
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cgls_urls.R
\name{cgls_urls}
\alias{cgls_urls}
\title{Provide URL's for Copernicus Global Land Service datasets}
\usage{
cgls_urls(...)
}
\arguments{
\item{...}{not used}
}
\description{
Returns URL's of a product/version/resolution
}
\details{
Given the changes to ESA-Copenricus data distribution, using digests is
no longer reliable. Plese use the Sentinel Hub or a STAC API for data 
acccess
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dispersion.R
\name{dispersion}
\alias{dispersion}
\title{Dispersion (H-prime)}
\usage{
dispersion(x)
}
\arguments{
\item{x}{data.frame object of target values}
}
\value{
data.frame with columns H values for each target, H , sH, sHmax
}
\description{
Calculates the dispersion ("rarity") of targets associated 
             with planning units
}
\details{
The dispersion index (H-prime) is calculated H = sum( sqrt(p) / sqrt(a) ) 
where; P = (sum of target in planning unit / sum of target across all 
planning units) and a = (count of planning units containing 
target / number of planning units)
}
\examples{
\donttest{ 
 library(sf)
   data(pu)
  
 d <- dispersion(st_drop_geometry(pu[,2:ncol(pu)]))  
   p <- d[,"H"]
 clr <- c("#3288BD", "#99D594", "#E6F598", "#FEE08B", 
          "#FC8D59", "#D53E4F")      
 clrs <- ifelse(p < 0.5524462, clr[1], 
           ifelse(p >= 0.5524462 & p < 1.223523, clr[2],
             ifelse(p >= 1.223523 & p < 2.465613, clr[3],
 	          ifelse(p >= 2.465613 & p < 4.76429, clr[4],
 	            ifelse(p >= 4.76429 & p < 8.817699, clr[5],
 	              ifelse(p >= 8.817699, clr[6], NA))))))
 plot(st_geometry(pu), col=clrs, border=NA)
   legend("bottomleft", legend=rev(c("Very Rare","Rare","Moderately Rare",
          "Somewhat Common","Common","Over Dispersed")),
          fill=clr, cex=0.6, bty="n") 
   box()
}

}
\references{
Evans, J.S., S.R. Schill, G.T. Raber (2015) A Systematic Framework for Spatial 
  Conservation Planning and Ecological Priority Design in St. Lucia, Eastern Caribbean. 
  Chapter 26 in Central American Biodiversity : Conservation, Ecology and a Sustainable 
  Future. F. Huettman (eds). Springer, NY.
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/insert.values.R
\name{insert.values}
\alias{insert.values}
\title{Insert Values}
\usage{
insert.values(x, value, index)
}
\arguments{
\item{x}{A vector to insert values}

\item{value}{Values to insert into x}

\item{index}{Index position(s) to insert y values into x}
}
\value{
A vector with values of y inserted into x and the position(s) 
defined by the index
}
\description{
Inserts new values into a vector at 
             specified positions
}
\details{
This function inserts new values at specified positions in a vector. It 
does not replace existing values. If a single value is provided for y 
and l represents multiple positions y will be replicated for the length 
of l. In this way you can insert the same value at multiple locations.
}
\examples{
(x=1:10)

 # Insert single value in one location
 insert.values(x, 100, 2) 

 # Insert multiple values in multiple locations 
 insert.values(x, c(100,200), c(2,8)) 

 # Insert single value in multiple locations 
 insert.values(x, NA, c(2,8))

}
\author{
Jeffrey S. Evans <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bearing.distance.R
\name{bearing.distance}
\alias{bearing.distance}
\title{Bearing and Distance}
\usage{
bearing.distance(x, y, distance, azimuth, EastOfNorth = TRUE)
}
\arguments{
\item{x}{x coordinate}

\item{y}{y coordinate}

\item{distance}{Distance to new point (in same units as x,y)}

\item{azimuth}{Azimuth to new point}

\item{EastOfNorth}{Specified surveying convention}
}
\value{
a new point representing location of baring and distance
}
\description{
Calculates a new point [X,Y] based on defined 
             bearing and distance
}
\details{
East of north is a surveying convention and defaults to true.
}
\examples{
 pt <- cbind( x=480933, y=4479433)
 bearing.distance(pt[1], pt[2], 1000, 40)

}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/poly_trend.R
\name{poly_trend}
\alias{poly_trend}
\title{Polynomial trend}
\usage{
poly_trend(x, y, degree, ci = 0.95, plot = TRUE, ...)
}
\arguments{
\item{x}{Vector of x}

\item{y}{Vector of y}

\item{degree}{Polynomial order (default 3)}

\item{ci}{+/- confidence interval (default 0.95)}

\item{plot}{Plot results (TRUE/FALSE)}

\item{...}{Additional arguments passed to plot}
}
\value{
A poly.trend class (list) containing
\itemize{
\item trend        data.frame of fit polynomial and upper/lower confidence intervals
\item model        Class lm model object fit with poly term
\item prameterCI   Intercept confidence intervals of Nth order polynomials
\item order        Specified polynomial order
}
}
\description{
Fits a polynomial trend using specified order
}
\details{
A fit using a lm(y ~ x + I(X^2) + I(X^3)) form will be correlated which,
can cause problems. The function avoids undue correlation using orthogonal
polynomials
}
\examples{
 set.seed(42)
 x <- seq(from=0, to=20, by=0.1)
 y <- (500 + 0.4 * (x-10)^3)
 noise <- y + rnorm(length(x), mean=10, sd=80) 
 
 p <- poly_trend(x, noise, degree = 3, ci = 0.95,
                 main="3rd degree polynomial")
 
 dev.new(height=6, width=12)
   layout(matrix(c(1,2), 1, 2, byrow = TRUE))
   p <- poly_trend(x, noise, degree = 3, 
                   main="3rd degree polynomial")
   p <- poly_trend(x, noise, degree = 6, 
                   main="6th degree polynomial")

cat("Confidence intervals for", "1 -", p$order, "polynomials",  "\n")
  p$prameterCI

}
\author{
Jeffrey S. Evans  \href{mailto:sage_insights@outlook.com}{sage_insights@outlook.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/elev-data.R
\docType{data}
\name{elev}
\alias{elev}
\title{Elevation raster}
\format{
A raster RasterLayer class object:
\describe{
  \item{resoultion}{5 arc-minute 0.00833 (10000m)}
  \item{nrow}{264}
  \item{ncol}{564}
  \item{ncell}{148896}
  \item{xmin}{5.9}
  \item{xmax}{10.6}
  \item{ymin}{45.7}
  \item{ymax}{47.9}
  \item{proj4string}{+proj=longlat +ellps=WGS84}
}
}
\source{
\url{https://diva-gis.org/data.html}
}
\description{
elevation raster of Switzerland
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rm.ext.R
\name{rm.ext}
\alias{rm.ext}
\title{Remove extension}
\usage{
rm.ext(x)
}
\arguments{
\item{x}{A character vector representing a file with extension}
}
\value{
The file name with extension and file path stripped off
}
\description{
Removes file extension (and path) from string
}
\examples{
rm.ext("C:/path/file.txt")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/daymet.tiles.R
\name{daymet.tiles}
\alias{daymet.tiles}
\title{DAYMET Tile ID's}
\usage{
daymet.tiles(...)
}
\arguments{
\item{...}{ignored}
}
\value{
Vector of DAYMET tile IDS or if sp = TRUE a sp class SpatialPolygonsDataFrame
}
\description{
Returns a vector of DAYMET tile id's within a specified extent
}
\note{
Function accepts sp, raster or extent class object or bounding coordinates. 
All input must be in the same projection as the tile index SpatialPolygonsDataFrame. 
The library includes the DAYMAT tile index "DAYMET_tiles" which can be add using 
data(), see examples.
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/raster.gaussian.smooth.R
\name{raster.gaussian.smooth}
\alias{raster.gaussian.smooth}
\title{Gaussian smoothing of raster}
\usage{
raster.gaussian.smooth(
  x,
  s = 2,
  n = 5,
  scale = FALSE,
  type = c("mean", "median", "sd", "convolution"),
  ...
)
}
\arguments{
\item{x}{A terra SpatRaster raster object}

\item{s}{Standard deviation (sigma) of kernel (default is 2)}

\item{n}{Size of the focal matrix, single value (default is 
5 for 5x5 window)}

\item{scale}{(FALSE/TRUE) Scale sigma to the resolution of the raster}

\item{type}{The statistic to use in the smoothing operator; 
"mean", "median", "sd", "convolution"}

\item{...}{Additional arguments passed to terra::focal}
}
\value{
A terra SpatRaster class object of the local distributional moment
}
\description{
Applies a Gaussian smoothing kernel to smooth raster.
}
\details{
This applies a Gaussian Kernel smoother. The convolution option performs
a Gaussian decomposition whereas the other options use the kernel
as weights for the given statistic.
}
\examples{
library(terra)
elev <- rast(system.file("extdata/elev.tif", package="spatialEco"))

# Gaussian smoothing with sigma = 2 and 7x7 window
g1 <- raster.gaussian.smooth(elev, s = 2, n = 7)
    plot(c(elev,g1))

# g1 <- raster.gaussian.smooth(elev, s = 2, n = 7, type = "convolution")

}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aspline.downscale.R
\name{aspline.downscale}
\alias{aspline.downscale}
\title{Raster Downscale using adaptive regression splines}
\usage{
aspline.downscale(
  x,
  y,
  add.coords = TRUE,
  keep.model = FALSE,
  grid.search = FALSE,
  plot = FALSE,
  ...
)
}
\arguments{
\item{x}{A terra SpatRaster object representing independent variable(s)}

\item{y}{A terra SpatRaster object representing dependent variable}

\item{add.coords}{(FALSE/TRUE) Add spatial coordinates to model}

\item{keep.model}{(FALSE/TRUE) Keep MARS model (earth class object)}

\item{grid.search}{(FALSE/TRUE) perform a hyper-parameter grid se}

\item{plot}{(FALSE/TRUE) Plot results}

\item{...}{Additional arguments passed to earth}
}
\value{
A list object containing:
\itemize{
\item downscale         Downscaled terra SpatRaster object
\item GCV Generalized   Cross Validation (GCV)
\item GRSq              Estimate of the predictive power
\item RSS               Residual sum-of-squares (RSS)
\item RSq               R-square
\item model             earth MARS model object (if keep.model = TRUE)
}
}
\description{
Downscales a raster to a higher resolution raster multivariate adaptive
regression splines (MARS).
}
\details{
This function uses Multivariate Adaptive Regression Splines, to downscale a raster based
on higher-resolution or more detailed raster data specified as covariate(s). This is similar
to the raster.downsample function which uses a robust regression and is a frequentest model for
fitting linear asymptotic relationships whereas, this approach is for fitting nonparametric
functions and should be used when the distributional relationship are complex/nonlinear.
Using add.coords adds spatial coordinates to the model, including creating the associated
rasters for prediction.
}
\examples{
\dontrun{
if (require(geodata, quietly = TRUE)) {
library(terra)
library(geodata)

# Download example data (requires geodata package)
  elev <- elevation_30s(country="SWZ",  path=tempdir())
    slp <- terrain(elev, v="slope")
	  x <- c(elev,slp)
        names(x) <- c("elev","slope")
  tmax <- worldclim_country(country="SWZ", var="tmax", 
                                     path=tempdir())
    tmax <- crop(tmax[[1]], ext(elev))
	  names(tmax) <- "tmax"
	  
tmax.ds <- aspline.downscale(x, tmax, add.coords=TRUE, keep.model=TRUE)
  plot(tmax.ds$model) 

  # plot prediction and parameters	
  opar <- par(no.readonly=TRUE)
    par(mfrow=c(2,2))
      plot(tmax, main="Original Temp max")
      plot(x[[1]], main="elevation")
      plot(x[[2]], main="slope")
      plot(tmax.ds$downscale, main="Downscaled Temp max")
  par(opar)

} else { 
  cat("Please install geodata package to run example", "\n")
}
}

}
\references{
Friedman (1991) Multivariate Adaptive Regression Splines (with discussion)
Annals of Statistics 19(1):1–141
}
\author{
Jeffrey S. Evans  \href{mailto:sage_insights@outlook.com}{sage_insights@outlook.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/optimized.sample.variance.R
\name{optimized.sample.variance}
\alias{optimized.sample.variance}
\title{Optimized sample variance}
\usage{
optimized.sample.variance(x, n, type = "maximized")
}
\arguments{
\item{x}{A vector to draw a sample from}

\item{n}{Number of samples to draw}

\item{type}{Type of sample variance optimization 
c("maximized", "minimized")}
}
\value{
A data.frame with "idx" representing the index of the original vector  
and "y" is the value of the sampled data
}
\description{
Draws an optimal sample that minimizes or maximizes the sample variance
}
\examples{
library(sf)
if (require(sp, quietly = TRUE)) {
  data(meuse, package = "sp")
  meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, agr = "constant")

 n = 15
 # Draw n samples that maximize the variance of y
 ( max.sv <- optimized.sample.variance(meuse$zinc, 15) )
 
 # Draw n samples that minimize the variance of y
 ( min.sv <- optimized.sample.variance(meuse$zinc, 15, 
               type="minimized") )
 
 # Plot results
 plot(st_geometry(meuse), pch=19, col="grey")
   plot(st_geometry(meuse[max.sv$idx,]), col="red", add=TRUE, pch=19)
     plot(st_geometry(meuse[min.sv$idx,]), col="blue", add=TRUE, pch=19)
 	  box()
     legend("topleft", legend=c("population","maximized variance", 
            "minimized variance"), col=c("grey","red","blue"),  
            pch=c(19,19,19))  
 
} else { 
  cat("Please install sp package to run example", "\n")
}
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/group.pdf.R
\name{group.pdf}
\alias{group.pdf}
\title{Probability density plot by group}
\usage{
group.pdf(
  x,
  y,
  col = NULL,
  lty = NULL,
  lwd = NULL,
  lx = "topleft",
  ly = NULL,
  ...
)
}
\arguments{
\item{x}{Numeric, character or factorial vector of grouping 
variable (must be same length as y)}

\item{y}{Numeric vector (density variable)}

\item{col}{Optional line colors (see par, col)}

\item{lty}{Optional line types (see par, lty)}

\item{lwd}{Optional line widths (see par, lwd)}

\item{lx}{Position of legend (x coordinate or 'topright', 'topleft', 
'bottomright', 'bottomleft')}

\item{ly}{Position of legend (y coordinate)}

\item{...}{Additional arguments passed to plot}
}
\value{
Plot of grouped PDF's
}
\description{
Creates a probability density plot of y for 
             each group of x
}
\examples{
y=dnorm(runif(100))
x=rep(c(1,2,3), length.out=length(y)) 
group.pdf(x=as.factor(x), y=y, main='Probability Density of y by group(x)', 
ylab='PDF', xlab='Y', lty=c(1,2,3))

}
\references{
Simonoff, J. S. (1996). Smoothing Methods in Statistics. Springer-Verlag, New York.
}
\author{
Jeffrey S. Evans  <jeffrey_evans<at>tnc.org>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/crossCorrelation.R
\name{crossCorrelation}
\alias{crossCorrelation}
\title{Spatial cross correlation}
\usage{
crossCorrelation(
  x,
  y = NULL,
  coords = NULL,
  w = NULL,
  type = c("LSCI", "GSCI"),
  k = 999,
  dist.function = c("inv.power", "neg.exponent", "none"),
  scale.xy = TRUE,
  scale.partial = FALSE,
  scale.matrix = FALSE,
  alpha = 0.05,
  clust = TRUE,
  return.sims = FALSE
)
}
\arguments{
\item{x}{Vector of x response variables}

\item{y}{Vector of y response variables, if not specified the
univariate  statistic is returned}

\item{coords}{A matrix of coordinates corresponding to (x,y), only
used if w = NULL. Can also be an sp object with relevant
x,y coordinate slot (ie., points or polygons)}

\item{w}{Spatial neighbors/weights in matrix format. Dimensions
must match (n(x),n(y)) and be symmetrical. If w is not defined
then a default method is used.}

\item{type}{c("LSCI","GSCI") Return Local Spatial Cross-correlation Index (LSCI)
or Global Spatial cross-correlation Index (GSCI)}

\item{k}{Number of simulations for calculating permutation distribution
under the null hypothesis of no spatial autocorrelation}

\item{dist.function}{("inv.power", "neg.exponent", "none") If w = NULL, the default method
for deriving spatial weights matrix, options are: inverse power
or negative exponent, none is for use with a provided matrix}

\item{scale.xy}{(TRUE/FALSE) scale the x,y vectors, if FALSE it is assumed that
they are already scaled following Chen (2015)}

\item{scale.partial}{(FALSE/TRUE) rescale partial spatial autocorrelation statistics}

\item{scale.matrix}{(FALSE/TRUE) If a neighbor/distance matrix is passed, should it
be scaled using (w/sum(w))}

\item{alpha}{= 0.05     confidence interval (default is 95 pct)}

\item{clust}{(FALSE/TRUE) Return approximated lisa clusters}

\item{return.sims}{(FALSE/TRUE) Return randomizations vector n = k}
}
\value{
When not simulated k=0, a list containing:
\itemize{
\item I - Global autocorrelation statistic
\item SCI - - A data.frame with two columns representing the xy and yx autocorrelation
\item nsim - value of NULL to represent p values were derived from observed data (k=0)
\item p - Probability based observations above/below confidence interval
\item t.test - Probability based on t-test
\item clusters - If "clust" argument TRUE, vector representing LISA clusters
}

When simulated (k>0), a list containing:
\itemize{
\item I - Global autocorrelation statistic
\item SCI - A data.frame with two columns representing the xy and yx autocorrelation
\item nsim - value representing number of simulations
\item global.p - p-value of global autocorrelation statistic
\item local.p - Probability based simulated data using successful rejection of t-test
\item range.p - Probability based on range of probabilities resulting from paired t-test
\item clusters - If "clust" argument TRUE, vector representing lisa clusters
}
}
\description{
Calculates univariate or bivariate spatial cross-correlation using
local Moran's-I (LISA), following Chen (2015)
}
\details{
In specifying a distance matrix, you can pass a coordinates matrix or spatial
object to coords or alternately, pass a distance or spatial weights matrix to
the w argument. If the w matrix represents spatial weights dist.function="none"
should be specified. Otherwise, w is assumed to represent distance and will be
converted to spatial weights using inv.power or neg.exponent. The w distances
can represent an alternate distance hypothesis (eg., road, stream, network distance)
Here are example argument usages for defining a matrix.
\itemize{
\item IF coords=x, w=NULL, dist.function= c("inv.power", "neg.exponent")
A distance matrix is derived using the data passed to coords then
spatial weights derived using one of the dist.function options
\item IF cords=NULL, w=x, dist.function= c("inv.power", "neg.exponent")
It is expected that the distance matrix specified with w represent
some form of distance then the spatial weights are derived using
one of the dist.function options
\item IF cords=NULL, w=x, dist.function="none"
It is assumed that the matrix passed to w already represents
the spatial weights
}
}
\examples{
# replicate Chen (2015)
 data(chen)
( r <- crossCorrelation(x=chen[["X"]], y=chen[["Y"]], w = chen[["M"]],  
                        clust=TRUE, type = "LSCI", k=0, 
                        dist.function = "inv.power") ) 

\donttest{
library(sf)
library(spdep)
 
  if (require(sp, quietly = TRUE)) {
   data(meuse, package = "sp")
   meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, agr = "constant")
  } 

#### Using a default spatial weights matrix method (inverse power function)
( I <- crossCorrelation(meuse$zinc, meuse$copper, 
             coords = st_coordinates(meuse)[,1:2], k=99) )
  meuse$lisa <- I$SCI[,"lsci.xy"]
    plot(meuse["lisa"], pch=20)

#### Providing a distance matrix
if (require(units, quietly = TRUE)) {
  Wij <- units::drop_units(st_distance(meuse))
 ( I <- crossCorrelation(meuse$zinc, meuse$copper, w = Wij, k=99) )

#### Providing an inverse power function weights matrix
  Wij <- 1 / Wij
    diag(Wij) <- 0 
      Wij <- Wij / sum(Wij) 
        diag(Wij) <- 0
 ( I <- crossCorrelation(meuse$zinc, meuse$copper, w = Wij, 
                         dist.function = "none", k=99) )
}
} 

}
\references{
Chen, Y.G. (2012) On the four types of weight functions for spatial contiguity
matrix. Letters in Spatial and Resource Sciences 5(2):65-72

Chen, Y.G. (2013) New approaches for calculating Moran’s index of spatial
autocorrelation. PLoS ONE 8(7):e68336

Chen, Y.G. (2015) A New Methodology of Spatial Cross-Correlation Analysis.
PLoS One 10(5):e0126158. doi:10.1371/journal.pone.0126158
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/idw.smoothing.R
\name{idw.smoothing}
\alias{idw.smoothing}
\title{Inverse Distance Weighted smoothing}
\usage{
idw.smoothing(x, y, d, k)
}
\arguments{
\item{x}{An sf POINT class object}

\item{y}{Numeric data column in x to be smoothed}

\item{d}{Distance constraint}

\item{k}{Maximum number of k-nearest neighbors within d}
}
\value{
A vector, same length as nrow(x), of smoothed y values
}
\description{
Distance weighted smoothing of a variable in a 
         spatial point object
}
\details{
Smoothing is conducted with a weighted-mean where; weights represent inverse 
standardized distance lags Distance-based or neighbour-based smoothing can be 
specified by setting the desired neighbour smoothing method to a specified value 
then the other parameter to the potential maximum. For example; a constraint 
distance, including all neighbors within 1000 (d=1000) would require k to equal 
all of the potential neighbors (n-1 or k=nrow(x)-1).
}
\examples{

library(sf)
if(require(sp, quietly = TRUE)) {
  data(meuse, package = "sp")
  meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, 
                    agr = "constant")
      
 # Calculate distance weighted mean on cadmium variable in meuse data   
   cadmium.idw <- idw.smoothing(meuse, 'cadmium', k=nrow(meuse), d = 1000)                
   meuse$cadmium.wm <- cadmium.idw
 
   opar <- par(no.readonly=TRUE)
     par(mfrow=c(2,1)) 
       plot(density(meuse$cadmium), main='Cadmium')
       plot(density(meuse$cadmium.wm), main='IDW Cadmium')
   par(opar)

plot(meuse[c("cadmium","cadmium.wm")], pch=20)   

} else { 
  cat("Please install sp package to run example", "\n")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shannons.R
\name{shannons}
\alias{shannons}
\title{Shannon's Diversity (Entropy) Index}
\usage{
shannons(x, counts = TRUE, ens = FALSE, margin = "row")
}
\arguments{
\item{x}{data.frame object containing counts or proportions}

\item{counts}{Are data counts (TRUE) or relative proportions (FALSE)}

\item{ens}{Calculate effective number of species (TRUE/FALSE)}

\item{margin}{Calculate diversity for rows or columns. c("row", "col")}
}
\value{
data.frame with "H" (Shannon's diversity) and "evenness" (Shannon's 
evenness where H / max( sum(x) ) ) and ESN
}
\description{
Calculates Shannon's Diversity Index and Shannon's Evenness Index
}
\details{
The expected for H is 0-3+ where a value of 2 has been suggested as medium-high diversity, 
for evenness is 0-1 with 0 signifying no evenness and 1, complete evenness.
}
\examples{
# Using Costa Rican ant diversity data from Roth et al. (1994)
data(ants)
  
# Calculate diversity for each covertype ("col") 
shannons(ants[,2:ncol(ants)], ens = TRUE, counts = FALSE, margin = "col")

# Calculate diversity for each species ("row") 
ant.div <- shannons(ants[,2:ncol(ants)], ens = TRUE, counts = FALSE, 
                    margin = "row")
  row.names(ant.div) <- ants[,1]
  ant.div

}
\references{
Shannon, C. E. and W. Weaver (1948) A mathematical theory of communication. The Bell 
  System Technical Journal, 27:379-423.

Simpson, E. H. (1949) Measurement of diversity. Nature 163:688

Roth, D. S., I. Perfecto, and B. Rathcke (1994) The effects of management systems on 
  ground-foraging ant diversity in Costa Rica. Ecological Applications 4(3):423-436.
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sf.kde.R
\name{sf.kde}
\alias{sf.kde}
\alias{sp.kde}
\title{Spatial kernel density estimate}
\usage{
sf.kde(
  x,
  y = NULL,
  bw = NULL,
  ref = NULL,
  res = NULL,
  standardize = FALSE,
  scale.factor = 10000,
  mask = FALSE
)
}
\arguments{
\item{x}{sf POINT object}

\item{y}{Optional values, associated with x coordinates, 
to be used as weights}

\item{bw}{Distance bandwidth of Gaussian Kernel, must be units 
of projection}

\item{ref}{A terra SpatRaster, vect, ext or sf vector, bbox 
vector to estimate the kde extent}

\item{res}{Resolution of raster when ref not SpatRaster}

\item{standardize}{Standardize results to 0-1 (FALSE/TRUE)}

\item{scale.factor}{Numeric scaling factor for the KDE (defaults to 10000),
to account for very small estimate values}

\item{mask}{(TRUE/FALSE) mask resulting raster if ref is provided
as a SpatRaster}
}
\value{
a terra SpatRaster class object containing kernel density estimate
}
\description{
A weighted or unweighted Gaussian Kernel Density estimate 
             for point spatial data
}
\details{
Please note that ks methods for estimation has been reverted to the Gussian method proposed
in Venables & Ripley (2002). There was not enought evendence that the Chacon & Duong (2018)
multivariate method(s) for bandwidth selection and kernal estimation were suitable for 
spatial random fields.
}
\examples{
\donttest{ 
library(sf) 
library(terra) 
  
data(meuse, package = "sp")
meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, 
                  agr = "constant") 
  			
# Unweighted KDE (spatial locations only) with 40m resoultion  				
pt.kde <- sf.kde(x = meuse, bw = 1000, standardize = TRUE, res=40)
  plot(pt.kde, main="Unweighted kde")
    plot(st_geometry(meuse), pch=20, col="red", add=TRUE) 

# cadmium weighted KDE usign extent with 40m resoultion and 500m and 1000m bw 
cadmium.kde.500 <- sf.kde(x = meuse, y = meuse$cadmium, res=40, 
                          bw = 500, standardize = TRUE)
cadmium.kde.1000 <- sf.kde(x = meuse, y = meuse$cadmium, res=40, 
                          bw = 1000, standardize = TRUE)						  
  plot(c(cadmium.kde.500, cadmium.kde.1000))

# Using defined raster
r <- terra::rast(terra::ext(meuse), resolution =  40, 
		               crs=terra::crs(meuse))   
  pt.kde <- sf.kde(x = meuse, bw = 1000, standardize = TRUE,  ref = r) 
 
}

}
\references{
Duong, T. & Hazelton, M.L. (2005) Cross-validation bandwidth matrices for multivariate 
  kernel density estimation. Scandinavian Journal of Statistics, 32, 485-506. 

Wand, M.P. & Jones, M.C. (1994) Multivariate plug-in bandwidth selection. Computational 
  Statistics, 9, 97-116. 

Venables, W. N. and Ripley, B. D. (2002) Modern Applied Statistics with S. 
  Fourth edition. Springer.
}
\author{
Jeffrey S. Evans  <jeffrey_evans@tnc.org>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/winsorize.R
\name{winsorize}
\alias{winsorize}
\title{Winsorize transformation}
\usage{
winsorize(
  x,
  min.value = NULL,
  max.value = NULL,
  p = c(0.05, 0.95),
  na.rm = FALSE
)
}
\arguments{
\item{x}{A numeric vector}

\item{min.value}{A fixed lower bounds, all values lower than this will be 
replaced by this value. The default is set to the 5th-quantile 
of x.}

\item{max.value}{A fixed upper bounds, all values higher than this will be replaced 
by this value. The default is set to the 95th-quantile of x.}

\item{p}{A numeric vector of 2 representing the probabilities used in the 
quantile function.}

\item{na.rm}{(FALSE/TRUE) should NAs be omitted?}
}
\value{
A transformed vector the same length as x, unless na.rm is TRUE, then x is length 
minus number of NA's
}
\description{
Removes extreme outliers using a winsorization transformation
}
\details{
Winsorization is the transformation of a distribution by limiting extreme values 
to reduce the effect of spurious outliers. This is done by shrinking outlying 
observations to the border of the main part of the distribution.
}
\examples{
set.seed(1234)     
x <- rnorm(100)     
x[1] <- x[1] * 10  
winsorize(x)       

plot(x, type="l", main="Winsorization transformation")
  lines(winsorize(x), col="red", lwd=2)
    legend("bottomright", legend=c("Original distribution","With outliers removed"),
        lty=c(1,1), col=c("black","red"))

# Behavior with NA value(s)
x[4] <- NA
winsorize(x)             # returns x with original NA's 
winsorize(x, na.rm=TRUE) # removes NA's 

}
\references{
Dixon, W.J. (1960) Simplified Estimation from Censored Normal Samples. Annals of Mathematical 
  Statistics. 31(2):385-391
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/curvature.R
\name{curvature}
\alias{curvature}
\title{Surface curvature}
\usage{
curvature(x, type = c("planform", "profile", "total", "mcnab", "bolstad"), ...)
}
\arguments{
\item{x}{A terra SpatRaster object}

\item{type}{Method used c("planform", "profile", "total", "mcnab", "bolstad")}

\item{...}{Additional arguments passed to focal}
}
\value{
raster class object of surface curvature
}
\description{
Calculates Zevenbergen & Thorne, McNab's or Bolstad's curvature
}
\details{
The planform and profile curvatures are the second derivative(s) of the 
elevation surface, or the slope of the slope. Profile curvature is in 
the direction of the maximum slope, and the planform curvature is 
perpendicular to the direction of the maximum slope.
Negative values in the profile curvature indicate the surface is upwardly
convex whereas, positive values indicate that the surface is upwardly concave.  
Positive values in the planform curvature indicate an that the surface is 
laterally convex whereas, negative values indicate that the surface is 
laterally concave.

Total curvature is the sigma of the profile and planform curvatures. A value of 
0 in profile, planform or total curvature, indicates the surface is flat. 
The planform, profile and total curvatures are derived using 
Zevenbergen & Thorne (1987) via a quadratic equation fit to eight neighbors 
as such, the s (focal window size) argument is ignored. 

McNab's and Bolstad's variants of the surface curvature (concavity/convexity) 
index (McNab 1993; Bolstad & Lillesand 1992; McNab 1989). The index is based 
on features that confine the view from the center of a 
3x3 window. In the Bolstad equation, edge correction is addressed 
by dividing by the radius distance to the outermost cell (36.2m).
}
\examples{
\donttest{
  library(terra)
  elev <- rast(system.file("extdata/elev.tif", package="spatialEco"))

  crv <- curvature(elev, type = "planform")
  mcnab.crv <- curvature(elev, type = "mcnab")
      plot(mcnab.crv, main="McNab's curvature") 
}

}
\references{
Bolstad, P.V., and T.M. Lillesand (1992). Improved classification of forest 
  vegetation in northern Wisconsin through a rule-based combination of soils, 
  terrain, and Landsat TM data. Forest Science. 38(1):5-20.

Florinsky, I.V. (1998). Accuracy of Local Topographic Variables Derived from 
  Digital Elevation Models. International Journal of Geographical Information 
  Science, 12(1):47-62.

McNab, H.W. (1989). Terrain shape index: quantifying effect of minor landforms 
  on tree height. Forest Science. 35(1):91-104.

McNab, H.W. (1993). A topographic index to quantify the effect of mesoscale 
  landform on site productivity. Canadian Journal of Forest Research. 23:1100-1107.

Zevenbergen, L.W. & C.R. Thorne (1987). Quantitative Analysis of Land Surface 
  Topography. Earth Surface Processes and Landforms, 12:47-56.
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dahi.R
\name{dahi}
\alias{dahi}
\title{Diurnal Anisotropic Heat Index}
\usage{
dahi(x, amax = 202.5)
}
\arguments{
\item{x}{An elevation raster of class terra SpatRaster}

\item{amax}{The Alpha Max (amax) parameter in degrees defined 
as: minimum = 0, maximum = 360 with the default = 202.500}
}
\value{
terra SpatRaster class object Diurnal Anisotropic Heat Index
}
\description{
Simple approximation of the anisotropic diurnal heat (Ha) distribution
}
\details{
The Diurnal Anisotropic Heat Index is based on this equation.  
Ha = cos(amax - a) * arctan(b)
  Where; amax defines the aspect with the maximum total heat  
  surplus, a is the aspect and b is the slope angle. Please
  note that all parameters are converted to radians.
}
\examples{
library(terra)
elev <- rast(system.file("extdata/elev.tif", package="spatialEco"))
Ha <- dahi(elev)
  plot(Ha, col=grey(0:100/100), smooth=TRUE)

}
\references{
Boehner, J., and Antonic, O. (2009) Land-surface parameters specific to 
   topo-climatology. In: Hengl, T., & Reuter, H. (Eds.), Geomorphometry -
   Concepts, Software, Applications. Developments in Soil Science, 
   33:195-226
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TM5-data.R
\docType{data}
\name{TM5}
\alias{TM5}
\title{Landsat 5 TM Scene}
\format{
A tif (inst/extdata/Landsat_TM5.tif) with 30m 6 bands:
\describe{
  \item{Blue}{0.45 - 0.52 µm}
  \item{Green}{0.52 - 0.60 µm}
  \item{Red}{0.63 - 0.69 µm}
  \item{NIR Near-Infrared}{0.76 - 0.90 µm}
  \item{SWIR1 Near-Infrared}{1.55 - 1.75 µm}
  \item{SWIR2 Mid-Infrared}{2.08 - 2.35 µm}
}
}
\description{
Subset of Landsat 5 TM Scene: LT52240631988227CUB02 Contains six bands of surface reflectance
path 224/row 63 acquisition date: 1988-08-14 13:00:47 GMT, EPSG:32622
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rotate.polygon.R
\name{rotate.polygon}
\alias{rotate.polygon}
\title{Rotate polygon}
\usage{
rotate.polygon(
  p,
  angle = 45,
  sp = FALSE,
  anchor = c("center", "lower.left", "upper.right")
)
}
\arguments{
\item{p}{A polygon object of sf or sp class}

\item{angle}{Rotation angle in degrees}

\item{sp}{(FALSE | TRUE) Output sp class object}

\item{anchor}{Location to rotate polygon on options are "center", 
"lower.left" and "upper.right"}
}
\value{
an sp or sf polygon object with rotated polygon
}
\description{
rotates polygon by specified angle
}
\details{
The anchor is the location that the rotation is anchored to. The center
is the centroid where the lower.left and upper.right are based on the 
min or max of the coordinates respectively.
}
\examples{
library(sf)
 
data(meuse, package = "sp")
  meuse <- st_as_sf(meuse, coords = c("x", "y"), 
                    crs = 28992, agr = "constant")
 
 e <- st_convex_hull(st_union(meuse))
   e30 <- rotate.polygon(e, angle=30)
 
 plot(e, main="rotated 30 degrees")
   plot(e30, add=TRUE)
 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/morans.plot.R
\name{morans.plot}
\alias{morans.plot}
\title{Autocorrelation Plot}
\usage{
morans.plot(
  x,
  y = NULL,
  coords = NULL,
  type.ac = c("xy", "yx"),
  dist.function = "inv.power",
  scale.xy = TRUE,
  scale.morans = FALSE,
  ...
)
}
\arguments{
\item{x}{Vector of x response variables}

\item{y}{Vector of y response variables}

\item{coords}{A matrix of coordinates corresponding to [x,y]}

\item{type.ac}{Type of autocorrelation plot ("xy", "yx")}

\item{dist.function}{("inv.power", "neg.exponent")}

\item{scale.xy}{(TRUE/FALSE) scale the x,y vectors}

\item{scale.morans}{(FALSE/TRUE) standardize the Moran's index to an 
expected [-1 to 1]?}

\item{...}{Additional arguments passed to plot}
}
\value{
A plot of the scaled variable against its spatially lagged values.
}
\description{
Autocorrelation plot (Anselin 1996), following Chen (2015), 
             aka, Moran's-I plot (univariate or bivariate)
}
\details{
The argument "type" controls the plot for x influencing y (type="xy") or y 
influencing x (type="yx"). If y is not defined then the statistic is univariate 
and only the "xy" plot will be available. The linear relationship between x and 
its spatial lag (Wx) is indicative of the spatial autoregressive process, underlying 
the spatial dependence. The statistic can be autocorrelation (univariate or 
cross-correlation (bivariate). The quadrants are the zero intercept for random 
autocorrelation and the red line represents the trend in autocorrelation. The quadrants 
in the plot indicate the type of spatial association/interaction (Anselin 1996). 
For example the upper-left quadrant represents negative associations of low values 
surrounded by high and the lower-right quadrant represents negative associations of
high values surrounded by low.  

If y is not specified the univariate statistic for x is returned. the coords argument 
is only used if k = NULL. Can also be an sp object with relevant x,y coordinate slot 
(ie., points or polygons). If w = NULL, the default method for deriving spatial weights 
matrix, options are: inverse power or negative exponent. If scale.xy = FALSE it is 
assumed that they are already scaled following Chen (2015).
}
\examples{
 p = c("sf", "sp", "spdep")
 if(any(!unlist(lapply(p, requireNamespace, quietly=TRUE)))) { 
   m = which(!unlist(lapply(p, requireNamespace, quietly=TRUE)))
   message("Can't run examples, please install ", paste(p[m], collapse = " "))
 } else {
 invisible(lapply(p, require, character.only=TRUE))
 
 data(meuse, package = "sp")
 meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, agr = "constant") 

# Autocorrelation (univariate)  
morans.plot(meuse$zinc, coords = st_coordinates(meuse)[,1:2])   

# Cross-correlation of: x influencing y and y influencing x
opar <- par(no.readonly=TRUE)
  par(mfrow=c(1,2)) 
    morans.plot(x=meuse$zinc, y=meuse$copper, coords = st_coordinates(meuse)[,1:2], 
                scale.morans = TRUE)
    morans.plot(x=meuse$zinc, y=meuse$copper, coords = st_coordinates(meuse)[,1:2],
                scale.morans = TRUE, type.ac="yx") 
par(opar)
}                       
}
\references{
Chen., Y. (2015) A New Methodology of Spatial Cross-Correlation Analysis. 
  PLoS One 10(5):e0126158. doi:10.1371/journal.pone.0126158
 
Anselin, L. (1996) The Moran scatterplot as an ESDA tool to assess local instability 
  in spatial association. pp. 111-125 in M. M. Fischer, H. J. Scholten and D. Unwin (eds) 
  Spatial analytical perspectives on GIS, London, Taylor and Francis

Anselin, L. (1995) Local indicators of spatial association, Geographical Analysis, 
  27:93-115
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mean_angle.R
\name{mean_angle}
\alias{mean_angle}
\title{Mean Angle}
\usage{
mean_angle(a, angle = c("degree", "radians"))
}
\arguments{
\item{a}{vector of angle values}

\item{angle}{("degree", "radians") to define angle in degrees or radians}
}
\value{
A vector of mean angle
}
\description{
Calculates the mean angle of a vector
}
\details{
The arithmetic mean is not correct for calculating the central tendency of
angles. This function is intended to return the mean angle for slope or aspect,
which could be used in a focal or zonal function.
}
\examples{
library(terra)
mean_angle(c(180, 10))
  mean(c(180, 10))
mean_angle(c(90, 180, 70, 60))
  mean(c(90, 180, 70, 60))
mean_angle(c(90, 180, 270, 360))
  mean(c(90, 180, 270, 360))

elev <- rast(system.file("extdata/elev.tif", package="spatialEco"))
asp <- terrain(elev, v="aspect")
s <- buffer(spatSample(asp, 20, as.points=TRUE, 
            na.rm=TRUE, values=FALSE), 5000)

plot(asp)
  plot(s, add=TRUE)

d <- extract(asp, s)
cat("Mean angles of aspect", "\n")
  tapply(d[,2], d[,1], mean_angle) 
cat("arithmetic means of aspect", "\n")
  tapply(d[,2], d[,1], mean, na.rm=TRUE) 

}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.loess.boot.R
\name{plot.loess.boot}
\alias{plot.loess.boot}
\title{Plot Loess Bootstrap}
\usage{
\method{plot}{loess.boot}(x, ...)
}
\arguments{
\item{x}{A loess.boot object}

\item{...}{Additional arguments passed to plot}
}
\value{
plot of lowess bootstrap
}
\description{
Plot function for loess.boot object
}
\examples{
 n=1000
 x <- seq(0, 4, length.out=n)	 
 y <- sin(2*x)+ 0.5*x + rnorm(n, sd=0.5)
 sb <- loess.boot(x, y, nreps = 99, confidence = 0.90, span = 0.40)
 plot(sb)
                   
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dissection.R
\name{dissection}
\alias{dissection}
\title{Dissection}
\usage{
dissection(x, s = 5, ...)
}
\arguments{
\item{x}{A terra SpatRaster class object}

\item{s}{Focal window size}

\item{...}{Additional arguments passed to terra::lapp}
}
\value{
A SpatRaster class object of Martonne's modified dissection
}
\description{
Calculates the Evans (1972) Martonne's modified 
             dissection
}
\details{
Dissection is calculated as: 
( z(s) - min(z(s)) ) / ( max(z(s)) - min(z(s)) )
}
\examples{
\donttest{ 
library(terra)
elev <- rast(system.file("extdata/elev.tif", package="spatialEco"))
  d <- dissection(elev, s=3)
    plot(d, main="dissection") 
}     

}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/moments.R
\name{moments}
\alias{moments}
\title{moments}
\usage{
moments(x, plot = FALSE)
}
\arguments{
\item{x}{numeric vector}

\item{plot}{plot of distribution (TRUE/FALSE)}
}
\value{
A vector with the following values:
\itemize{
\item min Minimum
\item 25th  25th percentile
\item mean  Arithmetic mean
\item gmean  Geometric mean
\item hmean  Harmonic mean
\item median  50th percentile
\item 7th5  75th percentile
\item max  Maximum
\item stdv  Standard deviation
\item var  Variance
\item cv  Coefficient of variation (percent)
\item mad  Median absolute deviation
\item skew  Skewness
\item kurt  Kurtosis
\item nmodes  Number of modes
\item mode  Mode (dominate)
}
}
\description{
Calculate statistical moments of a distribution
}
\examples{
x <- runif(1000,0,100)
( d <- moments(x, plot=TRUE) )
( mode.x <- moments(x, plot=FALSE)[16] )
 
}
\author{
Jeffrey S. Evans  \href{mailto:sage_insights@outlook.com}{sage_insights@outlook.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kendall.R
\name{kendall}
\alias{kendall}
\title{Kendall tau trend with continuity correction for time-series}
\usage{
kendall(
  y,
  tau = TRUE,
  intercept = TRUE,
  p.value = TRUE,
  confidence = TRUE,
  method = c("zhang", "yuepilon", "none"),
  threshold = 6,
  ...
)
}
\arguments{
\item{y}{A vector representing a timeseries with >= 8 obs}

\item{tau}{(FALSE/TRUE) return tau values}

\item{intercept}{(FALSE/TRUE) return intercept values}

\item{p.value}{(FALSE/TRUE) return p.values}

\item{confidence}{(FALSE/TRUE) return 95 pct confidence levels}

\item{method}{Method for deriving tau and slope ("zhang", "yuepilon", "none")}

\item{threshold}{The threshold for number of minimum observations in the time-series}

\item{...}{Not used}
}
\value{
Depending on arguments, a vector containing:
\itemize{
\item Theil-Sen slope, always returned
\item Kendall's tau two-sided test, if tau TRUE
\item intercept for trend if intercept TRUE
\item p value for trend fit if p.value TRUE
\item lower confidence level at 95-pct if confidence TRUE
\item upper confidence level at 95-pct if confidence TRUE
}
}
\description{
Calculates a nonparametric statistic for a monotonic trend based
on the Kendall tau statistic and the Theil-Sen slope modification
}
\details{
This function implements Kendall's nonparametric test for a monotonic trend
using the Theil-Sen (Theil 1950; Sen 1968; Siegel 1982) method to estimate
the slope and related confidence intervals. Critical values are Z > 1.96
representing a significant increasing trend and a Z < -1.96 a significant
decreasing trend (p < 0.05). The null hypothesis can be rejected if Tau = 0.
Autocorrelation in the time-series is addressed using a prewhitened linear trend
following the Zhang et al., (2000) or  Yue & Pilon (2002) methods. If you do not
have autocorrelation in the data, the "none" or "yuepilon" method is recommended.
Please note that changing the threshold to fewer than 6 observations (ideally 8) may
prevent the function from failing but, will likely invalidate the statistic.
A threshold of <=4 will yield all NA values. If method= "none" a modification of the
EnvStats::kendallTrendTest code is implemented.
}
\examples{
data(EuStockMarkets)
d <- as.vector(EuStockMarkets[,1])
kendall(d)

}
\references{
Theil, H. (1950) A rank invariant method for linear and polynomial regression
analysis. Nederl. Akad. Wetensch. Proc. Ser. A 53:386-392 (Part I),
53:521-525 (Part II), 53:1397-1412 (Part III).

Sen, P.K. (1968) Estimates of Regression Coefficient Based on Kendall's tau.
Journal of the American Statistical Association. 63(324):1379-1389.

Siegel, A.F. (1982) Robust Regression Using Repeated Medians.
Biometrika, 69(1):242-244

Yue, S., P. Pilon, B. Phinney and G. Cavadias, (2002) The influence of autocorrelation
on the ability to detect trend in hydrological series.
Hydrological Processes, 16: 1807-1829.

Zhang, X., Vincent, L.A., Hogg, W.D. and Niitsoo, A., (2000) Temperature
and Precipitation Trends in Canada during the 20th Century.
Atmosphere-Ocean 38(3): 395-429.
}
\seealso{
\code{\link[zyp]{zyp.trend.vector}} for model details
}
\author{
Jeffrey S. Evans  \href{mailto:sage_insights@outlook.com}{sage_insights@outlook.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/smooth.time.series.R
\name{smooth.time.series}
\alias{smooth.time.series}
\title{Smooth Raster Time-series}
\usage{
smooth.time.series(x, f = 0.8, smooth.data = FALSE, ...)
}
\arguments{
\item{x}{A terra SpatRaster with > 8 layers}

\item{f}{Smoothing parameter (see loess span argument)}

\item{smooth.data}{(FALSE/TRUE) Smooth all of the data or just impute NA values}

\item{...}{Additional arguments passed to terra::app (for 
writing results to disk)}
}
\value{
A terra SpatRaster containing imputed or smoothed data.
}
\description{
Smooths pixel-level data in raster time-series and can impute 
             missing (NA) values.
}
\details{
This function uses a LOESS regression to smooth the time-series. If the data is 
smoothed, (using the smooth.data = TRUE argument) it will be entirely replaced by  
a loess estimate of the time-series (estimated distribution at the pixel-level). 
Alternately, with smooth.data = FALSE, the function can be used to impute missing 
pixel data (NA) in raster time-series (stacks/bricks).
The results can dramatically be effected by the choice of the smoothing 
parameter (f) so caution is warranted and the effect of this parameter tested.
}
\examples{
\donttest{
library(terra)
  random.raster <- function(rows=50, cols=50, l=20, min=0, max=1){ 
    do.call(c, replicate(l, rast(matrix(runif(rows * cols, min, max), 
	        rows , cols))))
  }
r <- random.raster()

#### Smooth time-series using raster stack/brick 
r.smooth <- smooth.time.series(r, f = 0.4, smooth.data = TRUE)  

# extract pixel 100 for plotting
y <- as.numeric(r[100])
ys <- as.numeric(r.smooth[100])

# plot results	
plot(y, type="l")
  lines(ys, col="red")
    legend("bottomright", legend=c("original","smoothed"),
         lty=c(1,1), col=c("black","red"))	
}
}
\seealso{
\code{\link[stats]{loess}} for details on the loess regression

\code{\link[terra]{app}} for details on additional (...) arguments

\code{\link[spatialEco]{impute.loess}} for details on imputation model
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pseudo.absence.R
\name{pseudo.absence}
\alias{pseudo.absence}
\title{Pseudo-absence random samples}
\usage{
pseudo.absence(
  x,
  n,
  window = "hull",
  ref = NULL,
  s = NULL,
  sigma = "Scott",
  wts = NULL,
  KDE = FALSE,
  gradient = 1,
  p = NULL,
  edge = FALSE
)
}
\arguments{
\item{x}{An sf POINT geometry object}

\item{n}{Number of random samples to generate}

\item{window}{Type of window (hull OR extent), overridden if mask provided}

\item{ref}{Optional terra SpatRaster class raster. The resolution of the
density estimate will match mask.}

\item{s}{Optional resolution passed to window argument. Caution should be
used due to long processing times associated with high
resolution. In contrast, coarse resolution can exclude
known points.}

\item{sigma}{Bandwidth selection method for KDE, default is 'Scott'.
Options are 'Scott', 'Stoyan', 'Diggle', 'likelihood',
and 'geometry'}

\item{wts}{Optional vector of weights corresponding to point pattern}

\item{KDE}{Return KDE raster (TRUE/FALSE)}

\item{gradient}{A scaling factor applied to the sigma parameter used to
adjust the gradient decent of the density estimate. The
default is 1, for no adjustment (downweight < 1 | upweight > 1)}

\item{p}{Minimum value for probability distribution (must be >  0)}

\item{edge}{Apply Diggle edge correction (TRUE/FALSE)}
}
\value{
A list class object with the following components:
\itemize{
\item sample  A sf POINT geometry object containing random samples
\item kde     A terra SpatRaster class of inverted Isotropic KDE estimates
used as sample weights (IF KDE = TRUE)
\item sigma   Selected bandwidth of KDE
}
}
\description{
Generates pseudo-absence samples based on density estimate
of known locations
}
\details{
The window type creates a convex hull by default or, optionally, uses the
maximum extent (envelope). If a mask is provided the kde will represent
areas defined by the mask and defines the area that pseudo absence data
will be generated.

Available bandwidth selection methods are:
\itemize{
\item Scott (Scott 1992), Scott's Rule for Bandwidth Selection (1st order)
\item Diggle (Berman & Diggle 1989), Minimize the mean-square error via cross
\item validation (2nd order)
\item likelihood (Loader 1999), Maximum likelihood cross validation (2nd order)
\item geometry, Bandwidth is based on simple window geometry (1st order)
\item Stoyan (Stoyan & Stoyan 1995), Based on pair-correlation function (strong 2nd order)
\item User defined numeric distance bandwidth
}
}
\note{
resulting bandwidth can vary widely by method. the 'diggle' method
is intended for selecting bandwidth representing 2nd order spatial variation
whereas the 'scott' method will represent 1st order trend. the 'geometry' approach
will also represent 1st order trend. For large datasets, caution should be used with
the 2nd order 'likelihood' approach, as it is slow and computationally expensive.
finally, the 'stoyan' method will produce very strong 2nd order results.
}
\examples{
 p = c("sf", "sp", "terra", "spatstat.data")
 if(any(!unlist(lapply(p, requireNamespace, quietly=TRUE)))) { 
   m = which(!unlist(lapply(p, requireNamespace, quietly=TRUE)))
   message("Can't run examples, please install ", paste(p[m], collapse = " "))
 } else {
   invisible(lapply(p, require, character.only=TRUE))
  
data(meuse, package = "sp")
meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, 
                  agr = "constant") 

# Using a raster mask   
r <- rast(ext(meuse), resolution=40, crs=crs(meuse))
  r[] <- rep(1,ncell(r))
  
pa <- pseudo.absence(meuse, n=100, window='hull', KDE=TRUE, ref = r, 
                     sigma='Diggle', s=50) 
  col.br <- colorRampPalette(c('blue','yellow'))
    plot(pa$kde, col=col.br(10))
      plot(st_geometry(meuse), pch=20, cex=1, add=TRUE)
        plot(st_geometry(pa$sample), col='red', pch=20, cex=1, add=TRUE)
          legend('top', legend=c('Presence', 'Pseudo-absence'), 
                 pch=c(20,20),col=c('black','red'), bg="white")

# With clustered data
data(bei, package = "spatstat.data")
  trees <- st_as_sf(bei)
    trees <- trees[-1,]

trees.abs <- pseudo.absence(trees, n=100, window='extent', KDE=TRUE)
  col.br <- colorRampPalette(c('blue','yellow'))
    plot(trees.abs$kde, col=col.br(10))
     plot(st_geometry(trees), pch=20, cex=0.50, add=TRUE)
        plot(st_geometry(trees.abs$sample), col='red', pch=20, cex=1, add=TRUE)
          legend('top', legend=c('Presence', 'Pseudo-absence'), 
                 pch=c(20,20),col=c('black','red'),bg="white")
}     
}
\references{
Berman, M. and Diggle, P. (1989) Estimating weighted integrals of the second-order
intensity of a spatial point process. Journal of the Royal Statistical Society,
series B 51, 81-92.

Fithian, W & T. Hastie (2013) Finite-sample equivalence in statistical models for
presence-only data. Annals of Applied Statistics 7(4): 1917-1939

Hengl, T., H. Sierdsema, A. Radovic, and A. Dilo (2009) Spatial prediction of species
distributions from occurrence-only records: combining point pattern analysis,
ENFA and regression-kriging. Ecological Modelling, 220(24):3499-3511

Loader, C. (1999) Local Regression and Likelihood. Springer, New York.

Scott, D.W. (1992) Multivariate Density Estimation. Theory, Practice and Visualization.
New York, Wiley.

Stoyan, D. and Stoyan, H. (1995) Fractals, random shapes and point fields: methods of
geometrical statistics. John Wiley and Sons.

Warton, D.i., and L.C. Shepherd (2010) Poisson Point Process Models Solve the Pseudo-Absence
Problem for Presence-only Data in Ecology. The Annals of Applied Statistics, 4(3):1383-1402
}
\author{
Jeffrey S. Evans  \href{mailto:sage_insights@outlook.com}{sage_insights@outlook.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quadrats.R
\name{quadrats}
\alias{quadrats}
\title{Quadrats}
\usage{
quadrats(x, s = 250, n = 100, r = NULL, sp = FALSE)
}
\arguments{
\item{x}{An sf POLYGONS object defining extent}

\item{s}{Radius defining single or range of sizes of quadrats}

\item{n}{Number of quadrats}

\item{r}{A rotation factor for random rotation, default is NULL}

\item{sp}{(FALSE | TRUE) Output sp class object}
}
\value{
an sf POLYGONS object with rotated polygon(s)
}
\description{
Creates quadrat polygons for sampling or analysis
}
\details{
The radius (s) parameter can be a single value or a range of values, 
representing a randomization range of resulting quadrat sizes. The
rotation (r) parameter can also be used to defined a fixed rotation or
random range of quadrat rotations. You can specify each of these parameters
using an explicit vector that will be sampled eg., seq(100,300,0.5)
}
\examples{
library(sf)
library(terra) 

# read meuse data and create convex hull 
if (require(sp, quietly = TRUE)) {
  data(meuse, package = "sp")
  meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, agr = "constant")
  e <- st_convex_hull(st_union(meuse))

 # Fixed size 250 and no rotation 
 s <- quadrats(e, s = 250, n = 10)
   plot(st_geometry(s))
 
\donttest{   
# Variable sizes 100-300 and rotation of 0-45 degrees
s <- quadrats(e, s = c(100,300), n = 10, r = c(0,45))
  plot(st_geometry(s))
 
# Variable sizes 100-300 and no rotation 
s <- quadrats(e, s = c(100,300), n = 10)
  plot(st_geometry(s))
}   

} else { 
  cat("Please install sp package to run example", "\n")
}
 
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hli.pt.R
\name{hli.pt}
\alias{hli.pt}
\title{Point estimate of Heat Load Index}
\usage{
hli.pt(
  alpha,
  theta,
  latitude,
  direct = FALSE,
  scaled = TRUE,
  force.hemisphere = c("none", "southern", "northern")
)
}
\arguments{
\item{alpha}{Aspect in degrees}

\item{theta}{Slope in degrees}

\item{latitude}{A latitude representing the centrality of the data}

\item{direct}{Boolean (FALSE/TRUE) Return direct incident radiation 
else HLI (default)}

\item{scaled}{Boolean (TRUE/FALSE) Apply arithmetic scale using EXP(h)}

\item{force.hemisphere}{If country is split at the equator, force southern 
or northern hemisphere equation c("southern", "northern")}
}
\value{
Vector of McCune & Keon (2002) Heat Load Index
}
\description{
Calculates the McCune & Keon (2002) Heat Load Index
}
\details{
Describes A southwest facing slope should have warmer temperatures than a 
southeast facing slope, even though the amount of solar radiation they receive 
is equivalent. The McCune and Keon (2002) method accounts for this by "folding" 
the aspect so that the highest values are southwest and the lowest values are  
northeast. Additionally, this method account for steepness of slope, which is 
not addressed in most other aspect rescaling equations. HLI values range 
from 0 (coolest) to 1 (hottest). 

The equations follow McCune (2007) and support northern and southern hemisphere 
calculations. The folded aspect for northern hemispheres use (180 - (Aspect – 225) ) 
and for Southern hemisphere  ( 180 - ( Aspect – 315) ). If a country is split at the 
equator you can use the force.hemisphere argument to choose which equation to use. 
Valid values for this argument are "southern" and "northern" with the default "none".
}
\examples{

# Single point input
hli.pt(theta=180, alpha=30, latitude=40) 

# Multiple input, returns results from 
#   McCune, B., and D. Keon (2002)
# Raw -0.2551 -0.6280 0.0538 -0.6760 -1.1401 -0.2215
# arithmetic scale 0.7748 0.5337 1.0553 0.5086 0.3198 0.8013

slp = c(0, 30, 30, 0, 30, 30)
asp =c(0, 0, 180, 0, 0, 180)
lat =c(40, 40, 40, 60, 60, 60)
hli.pt(theta = slp, alpha = asp, latitude = lat)
  
}
\references{
McCune, B., and D. Keon (2002) Equations for potential annual direct 
  incident radiation and heat load index. Journal of Vegetation 
  Science. 13:603-606.

McCune, B. (2007). Improved estimates of incident radiation and heat load 
  using non-parametric regression against topographic variables. Journal 
  of Vegetation Science 18:751-754.
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/raster.downscale.R
\name{raster.downscale}
\alias{raster.downscale}
\title{Raster Downscale}
\usage{
raster.downscale(
  x,
  y,
  scatter = FALSE,
  full.res = FALSE,
  residuals = FALSE,
  se = FALSE,
  p = 0.95,
  uncertainty = c("none", "prediction", "confidence")
)
}
\arguments{
\item{x}{A terra SpatRaster object representing independent variable(s)}

\item{y}{A terra SpatRaster object representing dependent variable}

\item{scatter}{(FALSE/TRUE) Optional scatter plot}

\item{full.res}{(FALSE/TRUE) Use full resolution of x (see notes)}

\item{residuals}{(FALSE/TRUE) Output raster residual error raster,
at same resolution as y}

\item{se}{(FALSE/TRUE) Output standard error raster, using prediction or
confidence interval}

\item{p}{The confidence/prediction interval (default is 95\%)}

\item{uncertainty}{Output uncertainty raster(s) of confidence or prediction interval,
at same resolution as y. Options are c("none", "prediction", "confidence")}
}
\value{
A list object containing:
\itemize{
\item downscale - downscaled terra SpatRaster object
\item model - MASS rlm model object
\item MSE - Mean Square Error
\item AIC - Akaike information criterion
\item parm.ci - Parameter confidence intervals
\item residuals - If residuals = TRUE, a SpatRaster of the residual error
\item uncertainty - If pred.int = TRUE, SpatRaster's of the lower/upper prediction intervals
\item std.error - If se = TRUE, SpatRaster's of the standard error
}
}
\description{
Downscales a raster to a higher resolution raster using
a robust regression
}
\details{
This function uses a robust regression, fit using an M-estimation with Tukey's biweight
initialized by a specific S-estimator, to downscale a raster based on higher-resolution
or more detailed raster data specified as covariate(s). You can optionally output residual
error, standard error and/or uncertainty rasters. However, please note that when choosing
the type of uncertainty, using a confidence interval (uncertainty around the mean predictions)
when you should be using the prediction interval (uncertainty around a single values) will
greatly underestimate the uncertainty in a given predicted value (Bruce & Bruce 2017).
The full.res = TRUE option uses the x data to sample y rather than y to sample x. THis makes
the problem much more computationally and memory extensive and should be used with caution.
There is also the question of pseudo-replication (sample redundancy) in the dependent variable.
Statistically speaking one would expect to capture the sample variation of x by sampling at the
frequency of y thus supporting the downscaling estimate. Note that if uncertainty is not defined
the prediction interval for standard error defaults to "confidence" else is the same output as
uncertainty (eg., prediction or confidence).
}
\examples{
\dontrun{
if (require(geodata, quietly = TRUE)) {
library(terra)
library(geodata)

# Download example data (requires geodata package)
elev <- elevation_30s(country="SWZ", path=tempdir())
slp <- terrain(elev, v="slope")
tmax <- worldclim_country(country="SWZ", var="tmax", path=tempdir())
  tmax <- crop(tmax[[1]], ext(elev))

# Downscale temperature
x=c(elev,slp)
  names(x) <- c("elev","slope")
y=tmax
  names(y) <- c("tmax")

tmax.ds <- raster.downscale(x, y, scatter=TRUE, residuals = TRUE,
                            uncertainty = "prediction", se = TRUE)
	
  # plot prediction and parameters	
  opar <- par(no.readonly=TRUE)
    par(mfrow=c(2,2))
      plot(tmax, main="Temp max")
      plot(x[[1]], main="elevation")
      plot(x[[2]], main="slope")
      plot(tmax.ds$downscale, main="Downscaled Temp max")
  par(opar)

  # Plot residual error and raw prediction +/- intervals
  opar <- par(no.readonly=TRUE)
    par(mfrow=c(2,2))
      plot(tmax.ds$std.error, main="Standard Error")
      plot(tmax.ds$residuals, main="residuals")
      plot(tmax.ds$uncertainty[[1]], 
	       main="lower prediction interval")
      plot(tmax.ds$uncertainty[[2]], 
	       main="upper prediction interval")
  par(opar)
  
  # plot prediction uncertainty
  opar <- par(no.readonly=TRUE)
    par(mfrow=c(2,1))
      plot(tmax.ds$downscale - tmax.ds$uncertainty[[1]], 
	       main="lower prediction interval")
      plot(tmax.ds$downscale - tmax.ds$uncertainty[[2]], 
	       main="upper prediction interval")  
  par(opar)  

} else { 
  cat("Please install geodata package to run example", "\n")
}
}
}
\references{
Bruce, P., & A. Bruce. (2017). Practical Statistics for Data Scientists. O’Reilly Media.
}
\author{
Jeffrey S. Evans  \href{mailto:sage_insights@outlook.com}{sage_insights@outlook.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wt.centroids.R
\name{wt.centroid}
\alias{wt.centroid}
\title{Weighted centroid}
\usage{
wt.centroid(x, p = NULL, spatial = TRUE)
}
\arguments{
\item{x}{sf POINT class object}

\item{p}{Weights column in x}

\item{spatial}{(TRUE/FALSE) Output sf POINT object}
}
\value{
An x,y coordinate or sf POINT object representing the weighted or unweighted 
coordinate centroid
}
\description{
Creates centroid of [x,y] coordinates with optional
             weights field
}
\details{
The weighted centroid is calculated as:
[Xw]=[X]*[p], [Yw]=[Y]*[p], [sXw]=SUM[Xw], [sYw]=SUM[Yw], [sP]=SUM[p]
   wX=[sXw]/[sP], wY=[sYw]/[sP]    
   where; X=X coordinate(S), Y=Y coordinate(S), p=WEIGHT
}
\examples{
p = c("sf", "sp")
if(any(!unlist(lapply(p, requireNamespace, quietly=TRUE)))) { 
  m = which(!unlist(lapply(p, requireNamespace, quietly=TRUE)))
  message("Can't run examples, please install ", paste(p[m], collapse = " "))
} else {
invisible(lapply(p, require, character.only=TRUE))

data(meuse, package = "sp")
meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, 
                  agr = "constant")

wt.copper <- wt.centroid(meuse, p='copper') 
wt.zinc <- wt.centroid(meuse,  p='zinc') 

plot(st_geometry(meuse), pch=20, cex=0.75, 
     main='Weighted centroid(s)')
  plot(st_geometry(wt.copper), pch=19, col='red', 
       cex=1.5, add=TRUE)  
  plot(st_geometry(wt.zinc), pch=19, col='blue', 
       cex=1.5, add=TRUE)
legend('topleft', legend=c('all','copper', 'zinc'), 
       pch=c(20,19,19),col=c('black','red','blue'))
}
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/raster.transformations.R
\name{raster.transformation}
\alias{raster.transformation}
\title{Statistical transformation for rasters}
\usage{
raster.transformation(x, trans = "norm", smin = 0, smax = 255)
}
\arguments{
\item{x}{A terra SpatRaster class object}

\item{trans}{Transformation method: "norm", "rstd", "std", "stretch",
"nl", "slog", "sr" (please see notes)}

\item{smin}{Minimum value for stretch}

\item{smax}{Maximum value for stretch}
}
\value{
A terra SpatRaster class object of specified transformation
}
\description{
Transforms raster to a specified statistical transformation
}
\details{
Transformation option details:
\itemize{
\item norm - (Normalization_ (0-1): if min(x) < 0 ( x - min(x) ) / ( max(x) - min(x) )
\item rstd - (Row standardize) (0-1): if min(x) >= 0 x / max(x) This normalizes data
\item 

\if{html}{\out{<div class="sourceCode">}}\preformatted{   with negative distributions
}\if{html}{\out{</div>}}
\item std - (Standardize) (x - mean(x)) / sdv(x)
\item stretch - (Stretch) ((x - min(x)) * max.stretch / (max(x) - min(x)) + min.stretch)
This will stretch values to the specified minimum and maximum values
(eg., 0-255 for 8-bit)
\item nl - (Natural logarithms) if min(x) > 0 log(x)
\item slog - (Signed log 10) (for skewed data): if min(x) >= 0 ifelse(abs(x) <= 1, 0,
sign(x)*log10(abs(x)))
\item sr - (Square-root) if min(x) >= 0 sqrt(x)
}
}
\examples{
\donttest{
library(terra)
r <- rast(nrows=500, ncols=500, xmin=571823, xmax=616763, 
            ymin=4423540, ymax=4453690)
  crs(r) <- "epsg:9001"
r[] <- runif(ncell(r), 1000, 2500)

 # Positive values so, can apply any transformation    
 for( i in c("norm", "rstd", "std", "stretch", "nl", "slog", "sr")) {
   print( raster.transformation(r, trans = i) ) 
 }

 # Negative values so, can't transform using "nl", "slog" or "sr"
 r[] <- runif(ncell(r), -1, 1)
   for( i in c("norm", "rstd", "std", "stretch", "nl", "slog", "sr")) {
  try( print( raster.transformation(r, trans = i) ) ) 
   }
}

}
\author{
Jeffrey S. Evans  \href{mailto:sage_insights@outlook.com}{sage_insights@outlook.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sobel.R
\name{sobal}
\alias{sobal}
\title{Sobel-Feldman operator}
\usage{
sobal(x, method = "intensity", ...)
}
\arguments{
\item{x}{A raster class object}

\item{method}{Type of operator ("intensity", "direction", "edge")}

\item{...}{Additional arguments passed to raster::overlay or, 
if method="edge", raster::focal (if you want a file 
 written to disk use filename = "" argument)}
}
\value{
A raster class object or raster written to disk
}
\description{
An isotropic image gradient operator using a 3x3 window
}
\details{
The Sobel-Feldmanh operator is a discrete differentiation operator, deriving an 
approximation of the gradient of the intensity function. abrupt discontinuity 
in the gradient function represents edges, making this a common approach for edge 
detection. The Sobel-Feldman operator is based on convolving the image with a small, 
separable, and integer matrix in the horizontal and vertical directions. The operator 
uses two 3x3 kernels which are convolved with the original image to calculate 
approximations of the derivatives - one for horizontal changes, and one for vertical. 
Where x is defined here as increasing in the right-direction, and y as increasing in 
the down-direction. At each pixel in the raster, the resulting gradient can be combined 
to give the gradient intensity, using: SQRT( Gx^2 Gy^2 ). This can be expanded into the 
gradient direction using atan(Gx/Gy)
}
\examples{
library(terra)

r <- rast(system.file("ex/logo.tif", package="terra"))  
  s.int <- sobal(r[[1]])
  s.dir <- sobal(r[[1]], method = "direction")
  s.edge <- sobal(r[[1]], method = "edge")

opar <- par(no.readonly=TRUE)
par(mfrow=c(2,2))
  plot(r[[1]])
  plot(s.int, main="intensity") 
  plot(s.dir, main="direction") 
  plot(s.edge, main="edge")
par(opar)   

}
\references{
Sobel, I., & G. Feldman, (1969) A 3x3 Isotropic Gradient Operator for Image Processing, 
  presented at the Stanford Artificial Intelligence Project (SAIL).
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/csi.R
\name{csi}
\alias{csi}
\title{Cosine Similarity Index}
\usage{
csi(x, y = NULL, angular = TRUE)
}
\arguments{
\item{x}{A vector or matrix object}

\item{y}{If x is a vector, then a vector object}

\item{angular}{Booliean (TRUE/FALSE) return angular similarity}
}
\value{
If x is a matrix, a list object with: similarity and angular.similarity matrices or,  
if x and y are vectors, a vector of similarity and angular.similarity
}
\description{
Calculates the cosine similarity and angular similarity on  
             two vectors or a matrix
}
\details{
The cosine similarity index is a measure of similarity between two 
vectors of an inner product space. This index is bested suited for high-dimensional 
positive variable space. One useful application of the index is to measure separability 
of clusters derived from algorithmic approaches (e.g., k-means). It is a good common 
practice to center the data before calculating the index. It should be noted that the 
cosine similarity index is mathematically, and often numerically, equivalent to the 
Pearson's correlation coefficient   

The cosine similarity index is derived: 
s(xy) = x * y / ||x|| * ||y||, where the expected is 1.0 (perfect similarity)  
to -1.0 (perfect dissimilarity). A normalized angle between the vectors can 
be used as a bounded similarity function within [0,1] 
angular similarity  = 1 - (cos(s)^-1/pi)
}
\examples{
# Compare two vectors (centered using scale)
x=runif(100)
y=runif(100)^2
csi(as.vector(scale(x)),as.vector(scale(y)))
  
# Compare columns (vectors) in a matrix (centered using scale)
x <- matrix(round(runif(100),0),nrow=20,ncol=5)
( s <- csi(scale(x)) )
    
# Compare vector (x) to each column in a matrix (y)
y <- matrix(round(runif(500),3),nrow=100,ncol=5)
x=runif(100) 
csi(as.vector(scale(x)),scale(y))

}
\author{
Jeffrey S. Evans  <jeffrey_evans@tnc.org>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/random.raster.R
\name{random.raster}
\alias{random.raster}
\title{Random raster}
\usage{
random.raster(
  r = NULL,
  n.row = 50,
  n.col = 50,
  n.layers = 1,
  x = seq(1, 10),
  min = 0,
  max = 1,
  mean = 0,
  sd = 1,
  p = 0.5,
  s = 1.5,
  mask = TRUE,
  distribution = c("random", "normal", "seq", "binominal", "gaussian")
)
}
\arguments{
\item{r}{Optional existing terra raster defining nrow/ncol}

\item{n.row}{Number of rows}

\item{n.col}{Number of columns}

\item{n.layers}{Number of layers in resulting raster stack}

\item{x}{A vector of values to sample if distribution is "sample"}

\item{min}{Minimum value of raster}

\item{max}{Maximum value of raster}

\item{mean}{Mean of centered distribution}

\item{sd}{Standard deviation of centered distribution}

\item{p}{p-value for binominal distribution}

\item{s}{sigma value for Gaussian distribution}

\item{mask}{(TRUE/FALSE) If r is provided, mask results to r}

\item{distribution}{Available distributions, c("random", "normal", 
"seq", "binominal", "gaussian", "sample")}
}
\value{
terra SpatRaster object with random rasters
}
\description{
Create a random raster or raster stack using specified 
             distribution
}
\details{
Options for distributions are; random, normal, seq, binominal, gaussian and sample raster(s)
}
\examples{
library(terra)

# Using existing raster to create random binominal  
r <- rast(system.file("ex/elev.tif", package="terra"))    
( rr <- random.raster(r, n.layers = 3, distribution="binominal") )
  plot(c(r,rr)) 

# default; random, nrows=50, ncols=50, n.layers=5
( rr <- random.raster() )

# specified; binominal, nrows=20, ncols=20, nlayers=5
( rr <- random.raster(n.layer=5, n.col=20, n.row=20,  
                     distribution="binominal") )

# specified; gaussian, nrows=50, ncols=50, nlayers=1
( rr <- random.raster(n.col=50, n.row=50, s=8,  
                      distribution="gaussian") )
   plot(rr)
 
# specified; sample, nrows=50, ncols=50, nlayers=1
( rr <- random.raster(n.layer=1, x=c(2,6,10,15), 
                     distribution="sample" ) )
  freq(rr)

}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trend.line.R
\name{trend.line}
\alias{trend.line}
\title{trend.line}
\usage{
trend.line(x, y, type = "linear", plot = TRUE, ...)
}
\arguments{
\item{x}{Vector of x}

\item{y}{Vector of y}

\item{type}{Trend line types are: 'linear', 'exponential',
'logarithmic', 'polynomial'}

\item{plot}{plot results (TRUE/FALSE)}

\item{...}{Additional arguments passed to plot}
}
\value{
A list class object with the following components:
\itemize{
\item for type = 'linear'  x is slope and y is intercept
\item for type = 'exponential', 'logarithmic', or 'polynomial'
x is original x variable and y is vector of fit
regression line
}
}
\description{
Calculated specified trend line of x,y
}
\examples{
x <- 1:10
y <- jitter(x^2)

opar <- par(no.readonly=TRUE)
  par(mfcol=c(2,2))
    trend.line(x,y,type='linear',plot=TRUE,pch=20,main='Linear')
    trend.line(x,y,type='exponential',plot=TRUE,pch=20,main='Exponential')
    trend.line(x,y,type='logarithmic',plot=TRUE,pch=20,main='Logarithmic')
    trend.line(x,y,type='polynomial',plot=TRUE,pch=20,main='Polynomial')
 par(opar)

}
\author{
Jeffrey S. Evans  \href{mailto:sage_insights@outlook.com}{sage_insights@outlook.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/optimal.k.R
\name{optimal.k}
\alias{optimal.k}
\title{optimalK}
\usage{
optimal.k(x, nk = 10, plot = TRUE, cluster = TRUE, clara = FALSE, ...)
}
\arguments{
\item{x}{Numeric dataframe, matrix or vector}

\item{nk}{Number of clusters to test (2:nk)}

\item{plot}{(TRUE / FALSE) Plot cluster silhouettes(TRUE/FALSE)}

\item{cluster}{(TRUE / FALSE) Create cluster object with optimal k}

\item{clara}{(FALSE / TRUE) Use clara model for large data}

\item{...}{Additional arguments passed to clara}
}
\value{
Object of class clust "pam" or "clara" with tested silhouette values
}
\description{
Find optimal k of k-Medoid partitions using 
             silhouette widths
}
\examples{
if (require(cluster, quietly = TRUE)) {
  x <- rbind(cbind(rnorm(10,0,0.5), rnorm(10,0,0.5)),
             cbind(rnorm(15,5,0.5), rnorm(15,5,0.5)))

  clust <- optimal.k(x, 20, plot=TRUE, cluster=TRUE)
    plot(silhouette(clust$model), col = c('red', 'green'))
      plot(clust$model, which.plots=1, main='K-Medoid fit')

# Extract multivariate and univariate mediods (class centers)
  clust$model$medoids
    pam(x[,1], 1)$medoids  

# join clusters to data
  x <- data.frame(x, k=clust$model$clustering) 

} else { 
  cat("Please install cluster package to run example", "\n")
}

}
\references{
Theodoridis, S. & K. Koutroumbas(2006) Pattern Recognition 3rd ed.
}
\seealso{
\code{\link[cluster]{pam}} for details on Partitioning Around Medoids (PAM)

\code{\link[cluster]{clara}} for details on Clustering Large Applications (clara)
}
\author{
Jeffrey S. Evans  <jeffrey_evans<at>tnc.org>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.cross.cor.R
\name{summary.cross.cor}
\alias{summary.cross.cor}
\title{Summary of spatial cross correlation}
\usage{
\method{summary}{cross.cor}(object, ...)
}
\arguments{
\item{object}{Object of class cross.cor}

\item{...}{Ignored}
}
\value{
When not simulated k=0, prints functions list object containing:
\itemize{
\item I - Global autocorrelation statistic
\item SCI - - A data.frame with two columns representing the xy and yx autocorrelation
\item nsim - value of NULL to represent p values were derived from observed data (k=0)
\item p - Probability based observations above/below confidence interval
\item t.test - Probability based on t-test
}

When simulated (k>0), prints functions list object containing:
\itemize{
\item I - Global autocorrelation statistic
\item SCI - A data.frame with two columns representing the xy and yx autocorrelation
\item nsim - value representing number of simulations
\item global.p - p-value of global autocorrelation statistic
\item local.p - Probability based simulated data using successful rejection of t-test
\item range.p - Probability based on range of probabilities resulting from paired t-test
}
}
\description{
summary method for class "cross.cor"
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/raster.entropy.R
\name{raster.entropy}
\alias{raster.entropy}
\title{Raster Entropy}
\usage{
raster.entropy(x, d = 5, categorical = FALSE, global = FALSE, ...)
}
\arguments{
\item{x}{A terra SpatRaster object (requires integer raster)}

\item{d}{Size of matrix (window)}

\item{categorical}{Is the data categorical or continuous (FALSE/TRUE)}

\item{global}{Should the model use a global or local n to calculate 
entropy (FALSE/TRUE)}

\item{...}{Optional arguments passed terra focal function}
}
\value{
terra SpatRaster class object
}
\description{
Calculates entropy on integer raster (i.e., 8 bit 0-255)
}
\details{
Entropy calculated as: H = -sum(Pi*ln(Pi)) where; Pi, Proportion of one value 
to total values Pi=n(p)/m and m, Number of unique values. Expected range: 
0 to log(m) H=0 if window contains the same value in all cells.
H increases with the number of different values in the window. The ellipsis
arguments can be used to write to disk using the filename argument.
}
\examples{
library(terra)
  r <- rast(ncols=100, nrows=100)
    r[] <- round(runif(ncell(r), 1,8), digits=0)

rEnt <- raster.entropy(r, d=5, categorical = TRUE, global = TRUE)
  opar <- par(no.readonly=TRUE)
    par(mfcol=c(2,1))
      plot(r)
        plot(rEnt)
  par(opar)

# Maximum entropy is reached when all values are different, same as log(m)
#   for example; log( length( unique(x) ) ) 

}
\references{
Fuchs M., R. Hoffmann, F. Schwonke (2008) Change Detection with GRASS 
  GIS - Comparison of images taken by different sensor.
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/separability.R
\name{separability}
\alias{separability}
\title{separability}
\usage{
separability(
  x,
  y,
  plot = FALSE,
  cols = c("red", "blue"),
  clabs = c("Class1", "Class2"),
  ...
)
}
\arguments{
\item{x}{X vector}

\item{y}{Y vector}

\item{plot}{plot separability (TRUE/FALSE)}

\item{cols}{colors for plot (must be equal to number of classes)}

\item{clabs}{labels for two classes}

\item{...}{additional arguments passes to plot}
}
\value{
A data.frame with the following separability metrics:
\itemize{
\item B - Bhattacharryya distance statistic
\item JM - Jeffries-Matusita distance statistic
\item M - M-Statistic
\item D - Divergence index
\item TD - Transformed Divergence index
}
}
\description{
Calculates variety of two-class sample separability metrics
}
\details{
Available statistics:
\itemize{
\item M-Statistic (Kaufman & Remer 1994) - This is a measure of the difference of the
distributional peaks. A large M-statistic indicates good separation between the
two classes as within-class variance is minimized and between-class variance
maximized (M <1 poor, M >1 good).
\item Bhattacharyya distance (Bhattacharyya 1943; Harold 2003) - Measures the similarity
of two discrete or continuous probability distributions.
\item Jeffries-Matusita distance (Bruzzone et al., 2005; Swain et al., 1971) - The J-M
distance is a function of separability that directly relates to the probability of
how good a resultant classification will be. The J-M distance is asymptotic to v2,
where values of v2 suggest complete separability
\item Divergence and transformed Divergence (Du et al., 2004) - Maximum likelihood approach.
Transformed divergence gives an exponentially decreasing weight to increasing distances
between the classes.
}
}
\examples{
   norm1 <- dnorm(seq(-20,20,length=5000),mean=0,sd=1) 
   norm2 <- dnorm(seq(-20,20,length=5000),mean=0.2,sd=2)                          
     separability(norm1, norm2) 
           
   s1 <- c(1362,1411,1457,1735,1621,1621,1791,1863,1863,1838)
   s2 <- c(1362,1411,1457,10030,1621,1621,1791,1863,1863,1838)
     separability(s1, s2, plot=TRUE) 
      
}
\references{
Anderson, M. J., & Clements, A. (2000) Resolving environmental disputes: a
statistical method for choosing among competing cluster models. Ecological
Applications 10(5):1341-1355

Bhattacharyya, A. (1943) On a measure of divergence between two statistical
populations defined by their probability distributions'. Bulletin of the
Calcutta Mathematical Society 35:99-109

Bruzzone, L., F. Roli, S.B. Serpico (1995) An extension to multiclass cases of
the Jefferys-Matusita distance. IEEE Transactions on Pattern Analysis and
Machine Intelligence 33:1318-1321

Du, H., C.I. Chang, H. Ren, F.M. D'Amico, J. O. Jensen, J., (2004) New
Hyperspectral Discrimination Measure for Spectral Characterization. Optical
Engineering 43(8):1777-1786.

Kailath, T., (1967) The Divergence and Bhattacharyya measures in signal
selection. IEEE Transactions on Communication Theory 15:52-60

Kaufman Y., and L. Remer (1994) Detection of forests using mid-IR reflectance:
An application for aerosol studies. IEEE T. Geosci.Remote. 32(3):672-683.
}
\author{
Jeffrey S. Evans  \href{mailto:sage_insights@outlook.com}{sage_insights@outlook.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract_vertices.R
\name{extract.vertices}
\alias{extract.vertices}
\title{Extract vertices for polygons or lines}
\usage{
extract.vertices(x, join = TRUE)
}
\arguments{
\item{x}{An sf line or polygon class object}

\item{join}{(TRUE/FALSE) Joint attributes from original object}
}
\value{
An sf POINT object of extrated line or polygon vertices
}
\description{
Extracts [x,y] vertices from an sf line or polygon object
}
\details{
This function returns the vertices of a line or polygon object, as opposed
to the polygon centroids or line start/stop coordinates
}
\examples{

library(sf)
nc <- sf::st_read(system.file("shape/nc.shp", package="sf"))
  nc <- suppressWarnings(sf::st_cast(nc, "POLYGON"))
    nc <- nc[c(10,50),]
  
( v <- extract.vertices(nc) )
  plot(st_geometry(nc))
    plot(st_geometry(v), pch=20, cex=2, col="red", add=TRUE)

}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tri.R
\name{tri}
\alias{tri}
\title{Terrain Ruggedness Index}
\usage{
tri(r, s = 3, exact = TRUE, ...)
}
\arguments{
\item{r}{A terra SpatRaster class object}

\item{s}{Scale of window. Must be odd number, can represent 2
dimensions (eg., s=c(3,5) would represent a 3 x 5 window)}

\item{exact}{Calculate (TRUE/FALSE) the exact TRI or an algebraic
approximation.}

\item{...}{Additional arguments passed to terra::focal or terra::app}
}
\value{
A terra SpatRaster class object of the TRI
}
\description{
Implementation of the Riley et al (1999) Terrain Ruggedness Index
}
\details{
The algebraic approximation is considerably faster. However, because
inclusion of the center cell, the larger the scale the larger the divergence
of the minimum value. Resuls are driven by local variations so, fixed thresholds
are not very reliable. However there are some reccomended breaks (eg., Riley et al., 1999).

Riley et al., (1999) ranges for classifying Topographic Ruggedness Index:
\itemize{
\item 0-80 - level terrain surface.
\item 81-116 - nearly level surface.
\item 117-161 - slightly rugged surface.
\item 162-239 - intermediately rugged surface.
\item 240-497 - moderately rugged surface.
\item 498-958 - highly rugged surface.
\item gt 959 - extremely rugged surface.
}
}
\examples{
\donttest{
library(terra)
elev <- rast(system.file("extdata/elev.tif", package="spatialEco"))
  ( tri.ext <- tri(elev) )
  ( tri.app <- tri(elev, exact = FALSE) )
  plot(c(tri.ext, tri.app))
}

}
\references{
Riley, S.J., S.D. DeGloria and R. Elliot (1999) A terrain
ruggedness index that quantifies topographic heterogeneity,
Intermountain Journal of Sciences 5(1-4):23-27.
}
\author{
Jeffrey S. Evans  \href{mailto:sage_insights@outlook.com}{sage_insights@outlook.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cor.data.R
\docType{data}
\name{cor.data}
\alias{cor.data}
\title{Various correlation structures}
\format{
A list object with 4 elements containing data.frames:
\describe{
  \item{example 1}{two columns with nonlinear wave function relationship}
  \item{example 2}{two columns with simple nonlinear relationship}
  \item{example 3}{two columns with nonlinear multi-level wave function relationship}
  \item{example 4}{4 columns with first two having linear relationship}
}
}
\description{
linear and nonlinear correlated data examples

A list object with various linear and nonlinear correlation structures
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gaussian.kernel.R
\name{gaussian.kernel}
\alias{gaussian.kernel}
\title{Gaussian Kernel}
\usage{
gaussian.kernel(sigma = 2, s = 5)
}
\arguments{
\item{sigma}{sigma (standard deviation) of kernel (defaults 2)}

\item{s}{scale defining the number of rows and columns for kernel (default 5)}
}
\value{
Symmetrical (NxN) matrix of a Gaussian distribution
}
\description{
Creates a Gaussian Kernel of specified size and sigma
}
\examples{
opar <- par()
  par(mfrow=c(2,2))
  persp(gaussian.kernel(sigma=1, s=27), theta = 135, 
        phi = 30, col = "grey", ltheta = -120, shade = 0.6, 
        border=NA )
  persp(gaussian.kernel(sigma=2, s=27), theta = 135, phi = 30,
        col = "grey", ltheta = -120, shade = 0.6, border=NA )	
  persp(gaussian.kernel(sigma=3, s=27), theta = 135, phi = 30,
        col = "grey", ltheta = -120, shade = 0.6, border=NA )	
  persp(gaussian.kernel(sigma=4, s=27), theta = 135, phi = 30,
        col = "grey", ltheta = -120, shade = 0.6, border=NA )	
 par(opar) 			
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spatialEcoNews.R
\name{spatialEcoNews}
\alias{spatialEcoNews}
\title{spatialEco news}
\usage{
spatialEcoNews(...)
}
\arguments{
\item{...}{not used}
}
\value{
Shows package NEWS file
}
\description{
Displays release notes
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/oli.aws.R
\name{oli.aws}
\alias{oli.aws}
\title{Query AWS-OLI}
\usage{
oli.aws(...)
}
\arguments{
\item{...}{not used}
}
\description{
Query of Amazon AWS OLI-Landsat 8 cloud service
}
\details{
Given the changes to AWS Registry of Open Data, using the data  
digest is no longer reliable. Plese use the AWS Data Exchange 
or a STAC API for data acccess
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conf.interval.R
\name{conf.interval}
\alias{conf.interval}
\title{Confidence interval for mean or median}
\usage{
conf.interval(x, cl = 0.95, stat = "mean", std.error = TRUE)
}
\arguments{
\item{x}{Vector to calculate confidence interval for}

\item{cl}{Percent confidence level (default = 0.95)}

\item{stat}{Statistic (mean or median)}

\item{std.error}{Return standard error (TRUE/FALSE)}
}
\value{
data.frame contaning:
\itemize{
\item lci - Lower confidence interval value
\item uci - Upper confidence interval value
\item mean - If stat = "mean", mean value of distribution
\item mean - Value of the mean or median
\item conf.level - Confidence level used for confidence interval
\item std.error - If std.error = TRUE standard error of distribution
}
}
\description{
Calculates confidence interval for the mean or median of a distribution with
unknown population variance
}
\examples{
 x <- runif(100)
 cr <- conf.interval(x, cl = 0.97) 
 print(cr)

 d <- density(x)
 plot(d, type="n", main = "PDF with mean and 0.97 confidence interval")
   polygon(d, col="cyan3")
   abline(v=mean(x, na.rm = TRUE), lty = 2)
   segments( x0=cr[["lci"]], y0=mean(d$y), x1=cr[["uci"]], 
             y1=mean(d$y), lwd = 2.5, 
             col = "black")
 	legend("topright", legend = c("mean", "CI"), 
 	       lty = c(2,1), lwd = c(1,2.5)) 

}
\author{
Jeffrey S. Evans  \href{mailto:sage_insights@outlook.com}{sage_insights@outlook.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/remove_duplicates.R
\name{remove_duplicates}
\alias{remove_duplicates}
\title{Remove duplicate geometries}
\usage{
remove_duplicates(x, threshold = 0.00001)
}
\arguments{
\item{x}{An sf POINT, POLYGON or LINESTRING object}

\item{threshold}{A distance threshold indicating fuzzy duplication,
default i 0.00001}
}
\value{
sf object, of same feature class as x, with duplicate geometries removed
}
\description{
Removes duplicate geometries in a single-part feature class
}
\details{
This function removes duplicate geometries based on order and not "non null" 
attribution or other factors, the first feature gets to stay. If one needs to 
know which points were removed sf::st_difference can be used between original 
data and results of the function.
}
\examples{
library(sf)

# data with 10 duplicate obs
s <- data.frame(x = runif(100), y = runif(100))
  s <- data.frame(rbind(s, s[sample(1:nrow(s), 10),]) ) 
    s <- st_as_sf(s, coords = c("x", "y"))
      s$ID <- 1:nrow(s)

nrow(s) 
nrow( srmd <- remove_duplicates(s) )

}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/overlap.R
\name{overlap}
\alias{overlap}
\title{Niche overlap (Warren's-I)}
\usage{
overlap(x, y)
}
\arguments{
\item{x}{A matrix or SpatRaster raster class object}

\item{y}{A matrix or SpatRaster raster class object
with the same dimensions of x}
}
\value{
A vector (single value) representing the I similarity statistic
}
\description{
Similarity Statistic for Quantifying Niche Overlap using Warren's-I
}
\details{
The overlap function computes the I similarity statistic (Warren et al. 2008)  
of two overlapping niche estimates. Similarity is based on the Hellenger distance. 
It is assumed that the input data share the same extent and cellsize and all values 
are positive.

The I similarity statistic sums the pair-wise differences between two
predictions to create a single value representing the similarity of the two
distributions. The I similarity statistic ranges from a value of 0, where
two distributions have no overlap, to 1 where two distributions are
identical (Warren et al., 2008). The function is based on code  
from Jeremy VanDerWal
}
\examples{
  
# add degree of separation in two matrices 
p1 <- abs(matrix(1:50,nr=50,nc=50) + 
         runif(n = 2500, min = -1, max = 1))
p2 <- abs(matrix(1:50,nr=50,nc=50) + 
         rnorm(n = 2500, mean = 1, sd = 1))
 
# High overlap/similarity 
( I <- overlap(p1,p2) ) 

}
\references{
Warren, D. L., R. E. Glor, M. Turelli, and D. Funk. (2008).
  Environmental Niche Equivalency versus Conservatism: Quantitative 
  Approaches to Niche Evolution. Evolution 62:2868-2883.
}
\author{
Jeffrey Evans <sage_insights@outlook.com> and Jeremy VanDerWal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/knn.R
\name{knn}
\alias{knn}
\title{Spatial K nearest neighbor}
\usage{
knn(
  y,
  x,
  k = 1,
  d = NULL,
  ids = NULL,
  weights.y = NULL,
  weights.x = NULL,
  indexes = FALSE
)
}
\arguments{
\item{y}{Spatial sf object or coordinates matrix}

\item{x}{Spatial points or polygons object or coordinates matrix}

\item{k}{Number of neighbors}

\item{d}{Optional search radius}

\item{ids}{Optional column of ID's in x}

\item{weights.y}{A vector or matrix representing covariates of y}

\item{weights.x}{A vector or matrix representing covariates of x}

\item{indexes}{(FALSE/TRUE) Return row indexes of x neighbors}
}
\value{
A data.frame with row indexes (optional), rownames, ids (optional) and 
distance of k
}
\description{
Find K nearest neighbors for two spatial objects
}
\details{
Finds nearest neighbor in x based on y and returns rownames, index and distance,
If ids is NULL, rownames of x are returned. If coordinate matrix provided, 
columns need to be ordered [X,Y]. If a radius for d is specified than a maximum 
search radius is imposed. If no neighbor is found, a neighbor is not returned  

You can specify weights to act as covariates for x and y. The vectors or matrices
must match row dimensions with x and y as well as columns matching between weights.
In other words, the covariates must match and be numeric.
}
\examples{
\donttest{
if(require(sp, quietly = TRUE)) {
library(sf)
  data(meuse, package = "sp")
  meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, 
                    agr = "constant")
 
# create reference and target obs
idx <- sample(1:nrow(meuse), 10) 
  pts <- meuse[idx,]
    meuse <- meuse[-idx,]
    meuse$IDS <- 1:nrow(meuse)
 
# Find 2 neighbors in meuse
( nn <- knn(pts, meuse, k=2, ids = "IDS", indexes = TRUE) )
  plot( st_geometry(pts), pch=19, main="KNN")
    plot(st_geometry(meuse[nn[,1],]), pch=19, col="red", add=TRUE)

# Using covariates (weights)
wx = as.matrix(st_drop_geometry(meuse[,1:3]))
wy = as.matrix(st_drop_geometry(pts[,1:3]))

( nn <- knn(pts, meuse, k=2, ids = "IDS", indexes = TRUE,
            weights.y=wy, weights.x=wx) )
  plot(st_geometry(pts), pch=19, main="KNN")
    plot(st_geometry(meuse[nn[,1],]), pch=19, col="red")
	  
# Using coordinate matrices
y <- st_coordinates(pts)[,1:2]
x <- st_coordinates(meuse)[,1:2]
knn(y, x, k=2)

} else { 
  cat("Please install sp package to run example", "\n")
}
}
  
}
\seealso{
\code{\link[RANN]{nn2}} for details on search algorithm
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/breeding.density.R
\name{breeding.density}
\alias{breeding.density}
\title{Breeding density areas (aka, core habitat areas)}
\usage{
breeding.density(x, pop, p = 0.75, bw = 6400, b = 8500, self = TRUE)
}
\arguments{
\item{x}{sf POINT object}

\item{pop}{Population count/density column in x}

\item{p}{Target percent of population}

\item{bw}{Bandwidth distance for the kernel estimate (default 8500)}

\item{b}{Buffer distance (default 8500)}

\item{self}{(TRUE/FALSE) Should source observations be included in 
density (default TRUE)}
}
\value{
A list object with:
\itemize{ 
\item pop.pts   sf POINT object with points identified within the specified p
\item pop.area  sf POLYGON object of buffered points specified by parameter b
\item bandwidth Specified distance bandwidth used in identifying neighbor counts 
\item buffer    Specified buffer distance used in buffering points for pop.area  
\item p         Specified population percent
}
}
\description{
Calculates breeding density areas base on population counts and 
             spatial point density.
}
\details{
The breeding density areas model identifies the Nth-percent population exhibiting 
the highest spatial density and counts/frequency. It then buffers these points by 
a specified distance to produce breeding area polygons. If you would like to recreate 
the results in Doherty et al., (2010), then define bw = 6400m and b[if p < 0.75 
b = 6400m, | p >= 0.75 b = 8500m]
}
\examples{
library(sf)

n=1500
bb <- rbind(c(-1281299,-761876.5),c(1915337,2566433.5))
bb.mat <- round(cbind(c(bb[1,1], bb[1,2], bb[1,2], bb[1,1]),
                  c(bb[2,1], bb[2,1], bb[2,2], bb[2,2])),2)
 bbp <- st_sfc(st_polygon(list(rbind(bb.mat, bb.mat[1,]))))
   pop <- st_as_sf(st_sample(bbp, n, type = "random"))
  st_geometry(pop) <- "geometry"
     pop$ID <- 1:nrow(pop)
  pop$counts <- round(runif(nrow(pop), 1,250),0)
   
    bd75 <- breeding.density(pop, pop='counts', p=0.75, b=8500, bw=6400)	 
      plot(st_geometry(bd75$pop.area), border = NA,  
        main='75\% breeding density areas', col="grey")
         plot(st_geometry(pop), pch=20, col='black', add=TRUE)
         plot(st_geometry(bd75$pop.pts), pch=20, col='red', add=TRUE)
      legend("bottomright", legend=c("selected areas","selected sites", "all sites"),
             bg="white", fill=c("grey","red", "black"), pt.cex = 2) 

}
\references{
Doherty, K.E., J.D. Tack, J.S. Evans, D.E. Naugle (2010) Mapping breeding densities of 
  greater  sage-grouse: A tool for range-wide conservation planning.  
  Bureau of Land Management. Number L10PG00911
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geo.buffer.R
\name{geo.buffer}
\alias{geo.buffer}
\title{Buffer geographic data}
\usage{
geo.buffer(x, r, ...)
}
\arguments{
\item{x}{A sf or sp vector class object}

\item{r}{Buffer radius in meters}

\item{...}{Additional arguments passed to sf::st_buffer}
}
\value{
an sp or sf polygon class object representing buffer for each feature
}
\description{
Buffers data in geographic (Latitude/Longitude) projection
}
\details{
Projects (Latitude/Longitude) data in decimal-degree geographic projection 
using an on-the-fly azimuthal equidistant projection in meters centered on
}
\examples{
library(sf)
e <- c(61.87125, 23.90153, 76.64458, 37.27042)
  names(e) <- c("xmin", "ymin", "xmax", "ymax")
  s <- st_as_sf(st_sample(st_as_sfc(st_bbox(e)), size=100, 
                 type = "regular"))
	st_crs(s) <- st_crs(4326)
      s$id <- 1:nrow(s)

b <- geo.buffer(x=s, r=1000)
  plot(st_geometry(b[1,]))
     plot(st_geometry(s[1,]), pch=20,cex=2, add=TRUE)	 

}
\seealso{
\code{\link[sf]{st_buffer}} for st_buffer ... arguments
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fuzzySum.R
\name{fuzzySum}
\alias{fuzzySum}
\title{Fuzzy Sum}
\usage{
fuzzySum(x)
}
\arguments{
\item{x}{Vector of values to apply fuzzy sum}
}
\value{
Value of fuzzy sum
}
\description{
Calculates the fuzzy sum of a vector
}
\details{
The fuzzy sum is an increasing linear combination of values. 
This can be used to sum probabilities or results of multiple 
density functions.
}
\examples{
p = c(0.8,0.76,0.87)
  fuzzySum(p)
  sum(p)

p = c(0.3,0.2,0.1)
  fuzzySum(p)
  sum(p)  

}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/effect.size.R
\name{effect.size}
\alias{effect.size}
\title{Cohen's-d effect size}
\usage{
effect.size(y, x, pooled = TRUE, conf.level = 0.95)
}
\arguments{
\item{y}{A character or factor vector}

\item{x}{A numeric vector, same length as y}

\item{pooled}{Pooled or population standard deviation (TRUE/FALSE)}

\item{conf.level}{Specified confidence interval. Default is 0.95}
}
\value{
An effect.size class object with x, y and a data.frame with columns for effect 
size, lower confidence interval, lower confidence interval. The row names of 
the data frame represent the levels in y
}
\description{
Cohen's-d effect size with pooled sd for a control and experimental group
}
\note{
This implementation will iterate through each class in y and treating a given class 
as the experimental group and all other classes as a control case. Each class had d 
and the confidence interval derived. A negative d indicate directionality with same 
magnitude. The expected range for d is 0 - 3 
d is derived; ( mean(experimental group) - mean(control group) ) / sigma(p)
pooled standard deviation is derived; 
sqrt(  ( (Ne - 1) * sigma(e)^2 + (Nc - 1) * sigma(c)^2 ) / (Ne + Nc - 2) )   
  where; Ne, Nc = n of experimental and control groups.
}
\examples{
 ( es <- effect.size(iris$Species, iris$Sepal.Length) )
   plot(es)

}
\references{
Cohen, J., (1988) Statistical Power Analysis for the Behavioral Sciences 
  (second ed.). Lawrence Erlbaum Associates.

Cohen, J (1992) A power primer. Psychological Bulletin 112(1):155-159
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/insert.R
\name{insert}
\alias{insert}
\title{Insert a row or column into a data.frame}
\usage{
insert(x, MARGIN = 1, value = NULL, idx, name = NULL)
}
\arguments{
\item{x}{Existing data.frame}

\item{MARGIN}{Insert a 1 = row or 2 = column}

\item{value}{A vector of values equal to the length of MARGIN,
if nothing specified values with be NA}

\item{idx}{Index position to insert row or column}

\item{name}{Name of new column (not used for rows, MARGIN=1)}
}
\value{
A data.frame with the new row or column inserted
}
\description{
Inserts a new row or column into a data.frame at a specified location
}
\details{
Where there are methods to easily add a row/column to the end or beginning of a data.frame, 
it is not straight forward to insert data at a specific location within the data.frame. 
This function allows for inserting a vector at a specific location eg., between columns or 
rows 1 and 2 where row/column 2 is moved to the 3rd position and a new vector of values is 
inserted into the 2nd position.
}
\examples{
d <- data.frame(ID=1:10, y=runif(10))

# insert row
insert(d, idx=2)
insert(d, value=c(20,0), idx=2)

# insert column
insert(d, MARGIN=2, idx=2)
insert(d, MARGIN = 2, value = rep(0,10), idx=2, name="x")

}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rasterCorrelation.R
\name{rasterCorrelation}
\alias{rasterCorrelation}
\title{Raster correlation}
\usage{
rasterCorrelation(x, y, s = 3, type = "pearson")
}
\arguments{
\item{x}{A terra SpatRaster class object for x}

\item{y}{A terra SpatRasterclass object for y}

\item{s}{Scale of window. Can be a single value, two 
values for uneven window or a custom matrix. 
Must be odd number (eg., s=3, for 3x3 window or 
s=c(3,5) for 3 x 5 window)}

\item{type}{Type of output, options are: "pearson", "spearman", 
"covariance"}
}
\value{
A terra SpatRaster class object
}
\description{
Performs a moving window correlation between two rasters
}
\note{
The NA behavior is set to na.rm = TRUE to make default outputs
consistent between the terra and raster packages.
}
\examples{
\donttest{
library(terra)
 
r <- rast(system.file("ex/logo.tif", package="terra"))  
  x <- r[[1]]
  y <- r[[3]]
  
 r.cor <- rasterCorrelation(x, y, s = 5, type = "spearman")
   plot(r.cor)
}

}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/impute.loess.R
\name{impute.loess}
\alias{impute.loess}
\title{Impute loess}
\usage{
impute.loess(y, s = 0.2, smooth = FALSE)
}
\arguments{
\item{y}{A vector to impute}

\item{s}{Smoothing parameter ()}

\item{smooth}{(FALSE/TRUE) Smooth data, else only replace NA's}
}
\value{
A vector the same length as x with NA values filled or the data smoothed (or both).
}
\description{
Imputes missing data or smooths using Loess regression
}
\details{
Performs a local polynomial regression to smooth data or to 
impute NA values. The minimal number of non-NA observations to reliably
impute/smooth values is 6. There is not a reliably way to impute NA's
on the tails of the distributions so if the missing data is in the
first or last position of the vector it will remain NA. Please note
that smooth needs to be TRUE to return a smoothed vector, else only
NA's will be imputed.
}
\examples{
data(cor.data)
d <- cor.data[[1]][,2]
  plot(d, type="l")
  lines(impute.loess(d, s=0.3, smooth=TRUE), lwd=2, col="red")
 
# add some NA's
d <- d[1:100]
  d[sample(30:70, 5)] <- NA 
  d
  
impute.loess(d, s=0.2)
  
}
\author{
Jeffrey S. Evans  <jeffrey_evans<at>tnc.org>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/raster.deviation.R
\name{raster.deviation}
\alias{raster.deviation}
\title{Raster local deviation from the global trend}
\usage{
raster.deviation(
  x,
  type = c("trend", "min", "max", "mean", "median"),
  s = 3,
  degree = 1,
  global = FALSE
)
}
\arguments{
\item{x}{A terra SpatRaster object}

\item{type}{The global statistic to represent the local deviation
options are: "trend", "min", "max", "mean", "median"}

\item{s}{Size of matrix (focal window), not used with type="trend"}

\item{degree}{The polynomial degree if type is trend, default is 1st order.}

\item{global}{Use single global value for deviation or cell-level values 
(FALSE/TRUE). Argument is ignored for type="trend"}
}
\value{
A SpatRaster class object representing local deviation from the raster or the 
specified global statistic
}
\description{
Calculates the local deviation from the raster, a specified global statistic 
or a polynomial trend of the raster.
}
\details{
The deviation from the trend is derived as [y-hat - y] where; y-hat is the 
Nth-order polynomial. Whereas the deviation from a global statistic is [y - y-hat] 
where; y-hat is the local (focal) statistic. The global = TRUE argument allows 
one to evaluate the local deviation from the global statistic [stat(x) - y-hat] 
where; stat(x) is the global value of the specified statistic and y-hat is the 
specified focal statistic.
}
\examples{
library(terra)
elev <- rast(system.file("extdata/elev.tif", package="spatialEco"))

# local deviation from first-order trend, global mean and raw value
r.dev.trend <- raster.deviation(elev, type="trend", degree=1) 
r.dev.mean <- raster.deviation(elev, type="mean", s=5)
r.gdev.mean <- raster.deviation(elev, type="mean", s=5, global=TRUE)

opar <- par(no.readonly=TRUE)
par(mfrow=c(2,2))
  plot(elev, main="original")
  plot(r.dev.trend, main="dev from trend")
  plot(r.dev.mean, main="dev of mean from raw values")
  plot(r.gdev.mean, main="local dev from global mean")
par(opar)

}
\references{
Magee, Lonnie (1998). Nonlocal Behavior in Polynomial Regressions. The American 
  Statistician. American Statistical Association. 52(1):20-22

 Fan, J. (1996). Local Polynomial Modelling and Its Applications: From linear  
   regression to nonlinear regression. Monographs on Statistics and Applied 
   Probability. Chapman and  Hall/CRC. ISBN 0-412-98321-4
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/correlogram.R
\name{correlogram}
\alias{correlogram}
\title{Correlogram}
\usage{
correlogram(x, v, dist = 5000, ns = 99, ...)
}
\arguments{
\item{x}{A sf POINT object}

\item{v}{Test variable in x}

\item{dist}{Distance of correlation lags, if latlong=TRUE units are
great circle in kilometers}

\item{ns}{Number of simulations to derive simulation envelope}

\item{...}{Arguments passed to cor ('pearson', 'kendall' or 'spearman')}
}
\value{
Plot of correlogram and a list object containing:
\itemize{
\item autocorrelation is a data.frame object with the following components
\item autocorrelation - Autocorrelation value for each distance lag
\item dist - Value of distance lag
\item lci - Lower confidence interval (p=0.025)
\item uci - Upper confidence interval (p=0.975)
\item CorrPlot recordedplot object to recall plot
}
}
\description{
Calculates and plots a correlogram
}
\examples{
library(sf)
if(require(sp, quietly = TRUE)) {
  data(meuse, package = "sp")
  meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, 
                    agr = "constant")
}

zinc.cg <- correlogram(x = meuse, v = meuse$zinc, dist = 250, ns = 9)

}
\author{
Jeffrey S. Evans  \href{mailto:sage_insights@outlook.com}{sage_insights@outlook.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/raster.moments.R
\name{raster.moments}
\alias{raster.moments}
\title{Raster moments}
\usage{
raster.moments(x, type = "mean", s = 3, p = 0.75, ...)
}
\arguments{
\item{x}{A terra SpatRaster object}

\item{type}{The global statistic to represent the local deviation
options are: "min", "min", "mean", "median", "var, "sd", 
"mad", "kurt", "skew", "quantile"}

\item{s}{Size of matrix (focal window), can be single value or two 
values defining the [x,y] dimensions of the focal matrix}

\item{p}{if type="quantile", the returned percentile.}

\item{...}{Additional arguments passed to terra::focal}
}
\value{
A terra SpatRaster object representing the local distributional moment
}
\description{
Calculates focal statistical moments of a raster
}
\details{
This is a simple wrapper for the terra focal function, returning local statistical moments
}
\examples{
\donttest{
library(terra)
r <- rast(nrows=500, ncols=500, xmin=571823, xmax=616763, 
            ymin=4423540, ymax=4453690)
  crs(r) <- "epsg:9001"
r[] <- runif(ncell(r), 1000, 2500)

# Calculate 10th percentile for 3x3 window
r.p10 <- raster.moments(r, type="quantile", p=0.10) 
}

}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nth.values.R
\name{nth.values}
\alias{nth.values}
\title{Nth values}
\usage{
nth.values(x, N = 2, smallest = FALSE)
}
\arguments{
\item{x}{Numeric vector}

\item{N}{Number of (Nth) values returned}

\item{smallest}{(FALSE/TRUE) Return the highest, else smallest values}
}
\value{
Numeric vector of Nth values
}
\description{
Returns the Nth highest or lowest values in a vector
}
\details{
This function returns n lowest or highest elements in a vector
}
\examples{
nth.values(1:20, N=3, smallest = TRUE)                 
nth.values(1:20, N=3)

}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lai.R
\name{lai}
\alias{lai}
\title{Leaf Area Index}
\usage{
lai(ndvi, method = c("Jonckheere", "Chen"))
}
\arguments{
\item{ndvi}{NDVI in floating point standard scale range (-1 to 1)}

\item{method}{Method to use for index options c("Jonckheere", "Chen")}
}
\value{
A terra SpatRaster object with derived LAI vaues
}
\description{
Remote sensing measure of LAI (leaf area per ground-unit area)
}
\details{
This function calculates the Leaf Area Index (LAI) representing the amount of leaf area 
per unit of ground area. This is an important parameter for understanding the structure 
and function of vegetation, as it affects processes such as photosynthesis, transpiration, 
and carbon cycling. These two approaches are based on the empirical relationship between 
NDVI and LAI, which has been observed in many studies, and it is a widely used method for 
estimating LAI from remote sensing data. The formulas are derived from the fact that vegetation 
with higher LAI tends to have higher reflectance in the near-infrared (NIR) band and lower 
reflectance in the red band, resulting in higher NDVI values. But still, the exact relationship 
between NDVI and LAI can vary depending on factors such as vegetation type, canopy structure, 
and environmental conditions.
}
\examples{
\donttest{
library(terra)
lsat <- rast(system.file("/extdata/Landsat_TM5.tif", package="spatialEco"))
   plotRGB(lsat, r=3, g=2, b=1, scale=1.0, stretch="lin")
	
  ndvi <-  ( lsat[[4]] - lsat[[3]] ) / (lsat[[4]] + lsat[[3]]) 
		   
 # Using Jonckheere et al., (2004) method
 lai01 <- lai(ndvi)
   plot(lai01)

}

}
\references{
Jonckheere, I., Fleck, S., Nackaerts, K., Muys, B., Coppin, P. (2004). A comparison of two 
  methods to retrieve the leaf area index (LAI) from SPOT-4 HRVIR data. International 
  Journal of Remote Sensing, 25(21):4407–4425.
 
Chen, J. M., Liu, R., & Ju, W. (2014). A simple and effective method for estimating 
  leaf area index from Landsat imagery. Remote Sensing of Environment, 152:538–548.
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sa.trans.R
\name{sa.trans}
\alias{sa.trans}
\title{Trigonometric  transformation of a slope and aspect interaction}
\usage{
sa.trans(
  slope,
  aspect,
  type = "cos",
  slp.units = "degrees",
  asp.units = "degrees"
)
}
\arguments{
\item{slope}{slope values in degrees, radians or percent}

\item{aspect}{aspect values in degrees or radians}

\item{type}{Type of transformation, options are: "cos", "sin"}

\item{slp.units}{Units of slope values, options are: "degrees", 
"radians" or "percent"}

\item{asp.units}{Units of aspect values, options are: 
"degrees" or "radians"}
}
\value{
A vector of the modeled value
}
\description{
The Trigonometric  Stage (1978) 
             [slope * cos(aspect)] or [slope * sin(aspect)]
}
\details{
An a priori assumption of a maximum in the NW quadrant (45 azimuth)
and a minimum in the SW quadrant can be replaced by an empirically
determined location of the optimum without repeated calculations of
the regression fit. In addition it is argued that expressions for
the effects of aspect should always be considered as terms involving
an interaction with slope (Stage, 1976)

For slopes from 0% - 100%, The functions are linearized and
bounded from -1 to 1. Greater than 100% slopes are treated
out of the -1 to 1 range.

An alternative for slopes with values approaching infinity is
to take the square root of slope/100 to reduce the range of
values.By default this model test all values greater than 100%
to 101% and flat areas (-1) to nodata.
}
\examples{
library(terra)
sa.trans(slope = 48.146, aspect = 360.000)

# Example of creating slope*cos(aspect) raster
elev <- rast(system.file("extdata/elev.tif", package="spatialEco"))
sa <- terra::terrain(elev, v=c("slope", "aspect"), unit="degrees")
scosa <- terra::lapp(c(sa[[1]], sa[[2]]), fun = sa.trans)

}
\references{
Stage, A. R. 1976. An Expression of the Effects of Aspect, Slope, 
  and Habitat Type on Tree Growth. Forest Science 22(3):457-460.
}
\author{
Jeffrey S. Evans  <jeffrey_evans@tnc.org>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logistic.regression.R
\name{logistic.regression}
\alias{logistic.regression}
\title{Logistic and Auto-logistic regression}
\usage{
logistic.regression(
  ldata,
  y,
  x,
  penalty = TRUE,
  autologistic = FALSE,
  coords = NULL,
  bw = NULL,
  type = "inverse",
  style = "W",
  longlat = FALSE,
  ...
)
}
\arguments{
\item{ldata}{data.frame object containing variables}

\item{y}{Dependent variable (y) in ldata}

\item{x}{Independent variable(s) (x) in ldata}

\item{penalty}{Apply regression penalty (TRUE/FALSE)}

\item{autologistic}{Add auto-logistic term (TRUE/FALSE)}

\item{coords}{Geographic coordinates for auto-logistic model matrix
or sp object.}

\item{bw}{Distance bandwidth to calculate spatial lags (if empty neighbors
result, need to increase bandwidth). If not provided it will be
calculated automatically based on the minimum distance that includes
at least one neighbor.}

\item{type}{Neighbor weighting scheme (see autocov_dist)}

\item{style}{Type of neighbor matrix (Wij), default is mean of neighbors}

\item{longlat}{Are coordinates (coords) in geographic, lat/long (TRUE/FALSE)}

\item{...}{Additional arguments passed to lrm}
}
\value{
A list class object with the following components:
\itemize{
\item model - lrm model object (rms class)
\item bandwidth - If AutoCov = TRUE returns the distance bandwidth used for the
auto-covariance function
\item diagTable - data.frame of regression diagnostics
\item coefTable - data.frame of regression coefficients (log-odds)
\item Residuals - data.frame of residuals and standardized residuals
\item AutoCov - If an auto-logistic model, AutoCov represents lagged
auto-covariance term
}
}
\description{
Performs a logistic (binomial) or auto-logistic
(spatially lagged binomial) regression using maximum
likelihood or penalized maximum likelihood estimation.
}
\details{
It should be noted that the auto-logistic model (Besag 1972) is intended for
exploratory analysis of spatial effects. Auto-logistic are know to underestimate
the effect of environmental variables and tend to be unreliable (Dormann 2007).
Wij matrix options under style argument - B is the basic binary coding, W is row
standardized (sums over all links to n), C is globally standardized (sums over
all links to n), U is equal to C divided by the number of neighbours (sums over
all links to unity) and S is variance-stabilizing. Spatially lagged y defined as:
W(y)ij=sumj_(Wij yj)/ sumj_(Wij) where; Wij=1/Euclidean(i,j)
If the object passed to the function is an sp class there is no need to call the data
slot directly via "object@data", just pass the object name.
}
\examples{
 p = c("sf", "sp", "spdep", "rms")
 if(any(!unlist(lapply(p, requireNamespace, quietly=TRUE)))) { 
   m = which(!unlist(lapply(p, requireNamespace, quietly=TRUE)))
   message("Can't run examples, please install ", paste(p[m], collapse = " "))
 } else {
   invisible(lapply(p, require, character.only=TRUE))
 
 data(meuse, package = "sp")
 meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, 
                   agr = "constant")
   meuse$DepVar <- rbinom(nrow(meuse), 1, 0.5)
 
 #### Logistic model
 lmodel <- logistic.regression(meuse, y='DepVar', 
                   x=c('dist','cadmium','copper')) 
   lmodel$model
     lmodel$diagTable
       lmodel$coefTable
 
 #### Logistic model with factorial variable
 lmodel <- logistic.regression(meuse, y='DepVar', 
             x=c('dist','cadmium','copper', 'soil')) 
   lmodel$model
     lmodel$diagTable
       lmodel$coefTable
 
  ### Auto-logistic model using 'autocov_dist' in 'spdep' package
  lmodel <- logistic.regression(meuse, y='DepVar', 
              x=c('dist','cadmium','copper'), autologistic=TRUE, 
              coords=st_coordinates(meuse), bw=5000) 
    lmodel$model
      lmodel$diagTable
        lmodel$coefTable
    est <- predict(lmodel$model, type='fitted.ind')
  
  #### Add residuals, standardized residuals and estimated probabilities
  VarNames <- rownames(lmodel$model$var)[-1]
    meuse$AutoCov <- lmodel$AutoCov 
    meuse$Residual <- lmodel$Residuals[,1]
    meuse$StdResid <- lmodel$Residuals[,2]
    meuse$Probs <- predict(lmodel$model, 
                          sf::st_drop_geometry(meuse[,VarNames]),
 	                     type='fitted')  
  
 #### Plot fit and probabilities
 
 resid(lmodel$model, "partial", pl="loess") 
 
 # plot residuals
 resid(lmodel$model, "partial", pl=TRUE) 
  
 # global test of goodness of fit 
 resid(lmodel$model, "gof")
  
 # Approx. leave-out linear predictors
 lp1 <- resid(lmodel$model, "lp1")            
  
 # Approx leave-out-1 deviance            
 -2 * sum(meuse$DepVar * lp1 + log(1-plogis(lp1)))
  
 # plot estimated probabilities at points
 plot(meuse['Probs'], pch=20)
 
}
}
\references{
Besag, J.E., (1972) Nearest-neighbour systems and the auto-logistic model for binary
data. Journal of the Royal Statistical Society, Series B Methodological 34:75-83

Dormann, C.F., (2007) Assessing the validity of autologistic regression. Ecological
Modelling 207:234-242

Le Cessie, S., Van Houwelingen, J.C., (1992) Ridge estimators in logistic regression.
Applied Statistics 41:191-201

Shao, J., (1993) Linear model selection by cross-validation. JASA 88:486-494
}
\seealso{
\code{\link[rms]{lrm}}

\code{\link[spdep]{autocov_dist}}
}
\author{
Jeffrey S. Evans  \href{mailto:sage_insights@outlook.com}{sage_insights@outlook.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vrm.R
\name{vrm}
\alias{vrm}
\title{Vector Ruggedness Measure (VRM)}
\usage{
vrm(x, s = 3)
}
\arguments{
\item{x}{A terra SpatRaster class object}

\item{s}{Scale of window. Must be odd number, can represent 2 dimensions 
(eg., s=c(3,5) would represent a 3 x 5 window)}
}
\value{
A terra SpatRaster class object of the VRI
}
\description{
Implementation of the Sappington et al., (2007) vector 
             ruggedness measure
}
\details{
This function measures terrain ruggedness by calculating the vector 
ruggedness measure
}
\examples{
library(terra)
elev <- rast(system.file("extdata/elev.tif", package="spatialEco"))
   vrm3 <- vrm(elev) 
   vrm5 <- vrm(elev, s=5)
   plot(c(vrm3, vrm5))

}
\references{
Sappington, J.M., K.M. Longshore, D.B. Thomson (2007). Quantifying Landscape 
  Ruggedness for Animal Habitat Analysis: A case Study Using Bighorn Sheep in 
  the Mojave Desert. Journal of Wildlife Management. 71(5):1419-1426
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hexagons.R
\name{hexagons}
\alias{hexagons}
\title{Hexagons}
\usage{
hexagons(x, res = 100)
}
\arguments{
\item{x}{sf class object indicating extent}

\item{res}{Area of resulting hexagons}
}
\value{
sf POLYGONS object
}
\description{
Create hexagon polygons
}
\details{
Based on extent of x, creates a hexagon mesh with size of hexagons defined by res argumnet
}
\examples{
library(sf)
if(require(sp, quietly = TRUE)) {
  data(meuse, package = "sp")
  meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, 
                    agr = "constant")

hex <- hexagons(meuse, res=300)   
  plot(st_geometry(hex))
    plot(st_geometry(meuse),pch=20,add=TRUE)

# subset hexagons to intersection with points
idx <- which(apply(st_intersects(hex, meuse, sparse=FALSE), 1, any))
hex.sub <- hex[idx,] 
  plot(st_geometry(hex.sub))
    plot(st_geometry(meuse),pch=20,add=TRUE)

} else { 
  cat("Please install sp package to run example", "\n")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hybrid.kmeans.R
\name{hybrid.kmeans}
\alias{hybrid.kmeans}
\title{Hybrid K-means}
\usage{
hybrid.kmeans(x, k = 2, hmethod = "ward.D", stat = mean, ...)
}
\arguments{
\item{x}{A data.frame or matrix with data to be clustered}

\item{k}{Number of clusters}

\item{hmethod}{The agglomeration method used in hclust}

\item{stat}{The statistic to aggregate class centers (mean or median)}

\item{...}{Additional arguments passed to \code{\link[stats]{kmeans}}}
}
\value{
returns an object of class "kmeans" which has a print and a fitted method
}
\description{
Hybrid K-means clustering using hierarchical clustering to define 
cluster-centers
}
\details{
This method uses hierarchical clustering to define the cluster-centers in the K-means 
clustering algorithm. This mitigates some of the know convergence issues in K-means.
}
\note{
options for hmethod are: "ward.D", "ward.D2", "single", 
"complete", "average", mcquitty", "median", "centroid"
}
\examples{
x <- rbind(matrix(rnorm(100, sd = 0.3), ncol = 2),
           matrix(rnorm(100, mean = 1, sd = 0.3), ncol = 2))

# Compare k-means to hybrid k-means with k=4		   
km <- kmeans(x, 4)		   
hkm <- hybrid.kmeans(x,k=4)		   

opar <- par(no.readonly=TRUE)
par(mfrow=c(1,2))
  plot(x[,1],x[,2], col=km$cluster,pch=19, main="K-means")
  plot(x[,1],x[,2], col=hkm$cluster,pch=19, main="Hybrid K-means")
par(opar)

}
\references{
Singh, H., & K. Kaur (2013) New Method for Finding Initial Cluster Centroids in 
  K-means Algorithm. International Journal of Computer Application. 74(6):27-30

Ward, J.H., (1963) Hierarchical grouping to optimize an objective function. Journal 
  of the American Statistical Association. 58:236-24
}
\seealso{
\code{\link[stats]{kmeans}} for available ... arguments and function details

\code{\link[stats]{hclust}} for details on hierarchical clustering
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loess.boot.R
\name{loess.boot}
\alias{loess.boot}
\title{Loess Bootstrap}
\usage{
loess.boot(x, y, nreps = 100, confidence = 0.95, ...)
}
\arguments{
\item{x}{Independent variable}

\item{y}{Dependent variable}

\item{nreps}{Number of bootstrap replicates}

\item{confidence}{Fraction of replicates contained in confidence
region}

\item{...}{Additional arguments passed to loess function}
}
\value{
list object containing
\itemize{
\item nreps        Number of bootstrap replicates
\item confidence   Confidence interval (region)
\item span         alpha (span) parameter used loess fit
\item degree       polynomial degree used in loess fit
\item normalize    Normalized data (TRUE/FALSE)
\item family       Family of statistic used in fit
\item parametric   Parametric approximation (TRUE/FALSE)
\item surface      Surface fit, see loess.control
\item data         data.frame of x,y used in model
\item fit          data.frame including:
\enumerate{
\item x - Equally-spaced x index (see NOTES)
\item y.fit - loess fit
\item up.lim - Upper confidence interval
\item low.lim - Lower confidence interval
\item stddev - Standard deviation of loess fit at each x value
}
}
}
\description{
Bootstrap of a Local Polynomial Regression (loess)
}
\details{
The function fits a loess curve and then calculates a symmetric nonparametric
bootstrap with a confidence region. Fitted curves are evaluated at a fixed number
of equally-spaced x values, regardless of the number of x values in the data. Some
replicates do not include the values at the lower and upper end of the range of x
values. If the number of such replicates is too large, it becomes impossible to
construct a confidence region that includes a fraction "confidence" of the bootstrap
replicates. In such cases, the left and/or right portion of the confidence region
is truncated.
}
\examples{
 n=1000
 x <- seq(0, 4, length.out=n)	 
 y <- sin(2*x)+ 0.5*x + rnorm(n, sd=0.5)
 sb <- loess.boot(x, y, nreps=99, confidence=0.90, span=0.40)
 plot(sb)
                     	
}
\references{
Cleveland, WS, (1979) Robust Locally Weighted Regression and Smoothing Plots Journal
of the American Statistical Association 74:829-836

Efron, B., and R. Tibshirani (1993) An Introduction to the Bootstrap Chapman and
Hall, New York

Hardle, W., (1989) Applied Nonparametric Regression Cambridge University Press, NY.

Tibshirani, R. (1988) Variance stabilization and the bootstrap.
Biometrika 75(3):433-44.
}
\author{
Jeffrey S. Evans  \href{mailto:sage_insights@outlook.com}{sage_insights@outlook.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/time_to_event.R
\name{time_to_event}
\alias{time_to_event}
\title{Time to event}
\usage{
time_to_event(
  x,
  y = 1,
  dir = c("LR", "RL"),
  int = FALSE,
  up.to = FALSE,
  na.action = c("fail", "ignore")
)
}
\arguments{
\item{x}{A vector, representing time-series, to evaluate}

\item{y}{Threshold value tor return position for}

\item{dir}{Direction of evaluation c("LR", "RL")}

\item{int}{FALSE | TRUE - Evaluate as integer (rounds to 0 decimal places)}

\item{up.to}{FALSE | TRUE - Return value before event}

\item{na.action}{c("fail", "ignore"), if "fail" function will return error with NA's
with "ignore" NA values will be included in count to event}
}
\value{
A vector value representing the time to event
}
\description{
Returns the time (sum to position) to a specified value
}
\details{
The time to event represents the sum of positions, in the vector,
until the specified value is found ie., (0,0,1) would be 3 or, 
2 with up.to=TRUE. The int argument allows for rounding a continuous  
variable. Since it may be difficult to find an exact match to a floating 
point value rounding mitigates the problem. If you want a specific rounding 
value (eg., 1 decimal place) you can apply it to x first then pass it to 
the function. The up.to argument will stop one value before the specified value 
of (y) regardless of integer or float. For NA handling, na.action defines the
function behavior, causing it to fail or count NAs. Note that it makes no
sense to actually remove NAs as it will make the run uninterpretable.
}
\examples{
# Binomial instance
time_to_event(c(0,0,0,0,1,0,0,0,1,0))
time_to_event(c(0,0,0,0,1,0,0,0,1,0), up.to = TRUE)
time_to_event(c(0,0,0,0,1,0,0,0,1,0), dir="RL")
time_to_event(c(NA,0,0,0,1,0,0,0,1,0), na.action="ignore")

# Continuous threshold instance
( x <- runif(100, 0,7) ) 
time_to_event(x, y = 5, int=TRUE)

# raster example
library(terra)

# Binomial instance
r <- do.call(c, replicate(20,terra::rast(matrix(sample(
             c(0,1), 1000, replace=TRUE), 100, 100))))             
  ( t2e <- app(r, fun=time_to_event) )

# Continuous threshold instance
r <- do.call(c, replicate(20,terra::rast(matrix(
              runif(1000,0,7), 100, 100))))
  ( t2e <- app(r, fun=time_to_event, y=5) )

}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chen-data.R
\docType{data}
\name{chen}
\alias{chen}
\title{Cross-correlation data from Chen (2015)}
\format{
A list object with 3 elements:
\describe{
  \item{X}{per capita GRP(yuan)}
  \item{Y}{Level of urbanization percent}
  \item{M}{Railway Distance (km) matrix of 29 Chinese regions}
}
}
\source{
\url{https://journals.plos.org/plosone/article?id=10.1371/journal.pone.0126158}
}
\description{
Urbanization and economic development data from Chen (2015)
compiled from, National Bureau of Statistics of China
}
\references{
Chen, Y.G. (2012) On the four types of weight functions for spatial contiguity 
   matrix. Letters in Spatial and Resource Sciences 5(2):65-72

Chen, Y.G. (2013) New approaches for calculating Moran’s index of spatial 
  autocorrelation. PLoS ONE 8(7):e68336

Chen, Y.G. (2015) A New Methodology of Spatial Cross-Correlation Analysis. 
  PLoS One 10(5):e0126158. doi:10.1371/journal.pone.0126158
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pu-data.R
\docType{data}
\name{pu}
\alias{pu}
\title{Biodiversity Planning Units}
\format{
A sp SpatialPolygonsDataFrame with 5919 rows and 46 variables:
\describe{
  \item{UNIT_ID}{Unique planning unit ID}
  \item{DR_Dr_A}{Biodiversity target}
  \item{DR_Dr_L}{Biodiversity target}
  \item{Ht_Dr_A}{Biodiversity target}
  \item{Ht_Dr_L}{Biodiversity target}
  \item{DR_Ms_A}{Biodiversity target}
  \item{DR_Ms_L}{Biodiversity target}
  \item{Ht_Ms_L}{Biodiversity target}
  \item{DR_LM_M}{Biodiversity target}
  \item{H_LM_M_L}{Biodiversity target}
  \item{H_LM_R_L}{Biodiversity target}
  \item{DR_LM_R_L}{Biodiversity target}
  \item{DR_Rn_L}{Biodiversity target}
  \item{DR_LM_R_S}{Biodiversity target}
  \item{DR_Rn_S}{Biodiversity target}
  \item{DR_Ms_S}{Biodiversity target}
  \item{Ht_Ms_A}{Biodiversity target}
  \item{DR_Ms_E}{Biodiversity target}
  \item{DR_Ms_I}{Biodiversity target}
  \item{DR_Rn_E}{Biodiversity target}
  \item{DR_Rn_I}{Biodiversity target}
  \item{H_LM_R_E}{Biodiversity target}
  \item{Ht_Ms_E}{Biodiversity target}
  \item{Ht_Rn_E}{Biodiversity target}
  \item{DR_Rn_A}{Biodiversity target}
  \item{Ht_Rn_A}{Biodiversity target}
  \item{Ht_Rn_I}{Biodiversity target}
  \item{Ht_Dr_E}{Biodiversity target}
  \item{Ht_Ms_S}{Biodiversity target}
  \item{Ht_Dr_S}{Biodiversity target}
  \item{Ht_Rn_L}{Biodiversity target}
  \item{Ht_Th_A}{Biodiversity target}
  \item{Ht_Th_L}{Biodiversity target}
  \item{Ht_Th_S}{Biodiversity target}
  \item{Ht_Dr_U}{Biodiversity target}
  \item{Ht_Dr_I}{Biodiversity target}
  \item{Ht_Ms_I}{Biodiversity target}
  \item{H_LM_M_A}{Biodiversity target}
  \item{H_LM_M_E}{Biodiversity target}
  \item{H_LM_R_A}{Biodiversity target}
  \item{H_LM_M_S}{Biodiversity target}
  \item{H_LM_R_I}{Biodiversity target}
  \item{H_LM_R_S}{Biodiversity target}
  \item{Ht_Rn_S}{Biodiversity target}
  \item{Ht_Ms_U}{Biodiversity target}
  \item{Ht_Rn_U}{Biodiversity target}
}
}
\source{
"The Nature Conservancy"
}
\description{
Subset of biodiversity planning units for Haiti ecoregional spatial reserve plan
}
\references{
Evans, J.S., S.R. Schill, G.T. Raber (2015) A Systematic Framework for Spatial Conservation Planning and Ecological Priority Design in St. Lucia, Eastern Caribbean. Chapter 26 in Central American Biodiversity : Conservation, Ecology and a Sustainable Future. F. Huettman (eds). Springer, NY.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/o.ring.R
\name{o.ring}
\alias{o.ring}
\title{Inhomogeneous O-ring}
\usage{
o.ring(x, inhomogeneous = FALSE, ...)
}
\arguments{
\item{x}{spatstat ppp object}

\item{inhomogeneous}{(FALSE/TRUE) Run homogeneous (pcf) or inhomogeneous 
(pcfinhom)}

\item{...}{additional arguments passed to pcf or pcfinhom}
}
\value{
plot of o-ring and data.frame with plot labels and descriptions
}
\description{
Calculates the inhomogeneous O-ring point pattern statistic 
            (Wiegand & Maloney 2004)
}
\details{
The function K(r) is the expected number of points in a circle of radius r centered 
at an arbitrary point (which is not counted), divided by the intensity l of the pattern. 
The alternative pair correlation function g(r), which arises if the circles of 
Ripley's K-function are replaced by rings, gives the expected number of points at 
distance r from an arbitrary point, divided by the intensity of the pattern. Of special 
interest is to determine whether a pattern is random, clumped, or regular. 

Using rings instead of circles has the advantage that one can isolate specific 
distance classes, whereas the cumulative K-function confounds effects at larger 
distances with effects at shorter distances. Note that the K-function and the O-ring 
statistic respond to slightly different biological questions. The accumulative 
K-function can detect aggregation or dispersion up to a given distance r and is 
therefore appropriate if the process in question (e.g., the negative effect of 
competition) may work only up to a certain distance, whereas the O-ring statistic 
can detect aggregation or dispersion at a given distance r. The O-ring statistic 
has the additional advantage that it is a probability density function (or a 
conditioned probability spectrum) with the interpretation of a neighborhood 
density, which is more intuitive than an accumulative measure.
}
\examples{
if (require(spatstat.explore, quietly = TRUE)) {
data(lansing)
  x  <- spatstat.geom::unmark(split(lansing)$maple)
  o.ring(x)

} else { 
  cat("Please install spatstat.explore package to run example", "\n")
}

}
\references{
Wiegand T., and K. A. Moloney (2004) Rings, circles and null-models for point 
pattern analysis in ecology. Oikos 104:209-229
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/concordance.R
\name{concordance}
\alias{concordance}
\title{Concordance test for binomial models}
\usage{
concordance(y, p)
}
\arguments{
\item{y}{vector of binomial response variable used in model}

\item{p}{estimated probabilities from fit binomial model}
}
\value{
list object with: concordance, discordance, tied and pairs
}
\description{
Performs a concordance/disconcordance (C-statistic) test on 
             binomial models.
}
\details{
Test of binomial regression for the hypothesis that probabilities of all 
positives [1], are greater than the probabilities of the nulls [0]. The 
concordance would be 100% for a perfect model where, disconcordance is the 
inverse of concordance, representing the null. The C-statistic has been  
show to be comparable to the area under an ROC

Results are: concordance - percent of positives that are greater than 
probabilities of nulls. discordance - concordance inverse of concordance 
representing the null class, tied - number of tied probabilities and 
pairs - number of pairs compared
}
\examples{
data(mtcars)
dat <- subset(mtcars, select=c(mpg, am, vs))
glm.reg <- glm(vs ~ mpg, data = dat, family = binomial)
concordance(dat$vs, predict(glm.reg, type = "response")) 

}
\references{
Austin, P.C. & E.W. Steyerberg (2012) Interpreting the concordance statistic of a 
  logistic regression model: relation to the variance and odds ratio of a continuous 
  explanatory variable. BMC Medical Research Methodology, 12:82

Harrell, F.E. (2001) Regression modelling strategies. Springer, New York, NY.

Royston, P. & D.G. Altman (2010) Visualizing and assessing discrimination in the 
  logistic  regression model. Statistics in Medicine 29(24):2508-2520
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/raster.change.R
\name{raster.change}
\alias{raster.change}
\title{Raster change between two nominal rasters}
\usage{
raster.change(
  x,
  y,
  s = 3,
  stat = c("kappa", "t.test", "cor", "entropy", "cross-entropy", "divergence"),
  ...
)
}
\arguments{
\item{x}{A terra SpatRaster}

\item{y}{A terra SpatRaster for comparison to x}

\item{s}{Integer or matrix for defining Kernel,
must be odd but not necessarily square}

\item{stat}{Statistic to use in comparison, please see details for
options.}

\item{...}{Additional arguments passed to terra::focalPairs}
}
\value{
A terra SpatRaster layer containing one of the following layers:
\itemize{
\item kappa - Kappa or Weighted Kappa statistic (if stat = "kappa")
\item correlation - Paired t.test statistic  (if stat = "cor")
\item entropy - Local entropy  (if stat = "entropy")
\item divergence - Kullback-Leibler divergence (if stat = "divergence")
\item cross.entropy - Local Cross-entropy (if stat = "cross.entropy")
\item t.test - Paired t.test statistic  (if stat = "t.test")
\item p.value - p-value of the paired t.test statistic (if stat = "t.test")
}
}
\description{
Compares two categorical rasters with a variety of statistical options
}
\details{
This function provides a various statistics for comparing two classified maps.
Valid options are:
\itemize{
\item kappa - Cohen's Kappa
\item t.test - Two-tailed paired t-test
\item cor - Persons Correlation
\item entropy - Delta entropy
\item cross-entropy - Cross-entropy loss function
\item divergence - Kullback-Leibler divergence (relative entropy)
}

Kappa and t-test values < 0 are reported as 0. For a weighted kappa, a matrix must
be provided that correspond to the pairwise weights for all values in both rasters.
Delta entropy is derived by calculating Shannon's on each focal window then
differencing  them (e(x) - e(y)). The s argument can be a single scalar, defining
a symmetrical kernel, two scalers defining the dimensions of the kernel eg., c(3,5)
or a matrix defining the kernel say, resulting from terra::focalMat
}
\examples{
\donttest{
 library(sf) 
 library(terra) 
  
 e <- ext(179407.8, 181087.9, 331134.4, 332332.1)
 r1 <- rast(e, resolution=20)
   r1[] <- sample(1:5, ncell(r1), replace=TRUE)
 r2 <- rast(e, resolution=20)
   r2[] <- sample(1:5, ncell(r2), replace=TRUE)
 	  
 d = 5 # kernel    
 ( r.kappa <- raster.change(r1, r2, s = d) )   
 ( r.ttest <- raster.change(r1, r2, s = d, stat="t.test") )
 ( r.ent <- raster.change(r1, r2, s = d, stat="entropy") )   
 ( r.cor <- raster.change(r1, r2, s = d, stat="cor") )
 ( r.ce <- raster.change(r1, r2, s = d, stat = "cross-entropy") )
 ( r.kl <- raster.change(r1, r2, s = d, stat = "divergence") )	
       
   opar <- par(no.readonly=TRUE)
   par(mfrow=c(3,2))
     plot(r.kappa, main="Kappa")
     plot(r.ttest[[1]], main="Paired t-test")
     plot(r.ent, main="Delta Entropy")
     plot(r.cor, main="Rank Correlation")
     plot(r.kl, main="Kullback-Leibler")
     plot(r.ce, main="cross-entropy")
   par(opar) 
}

}
\references{
Cohen, J. (1960). A coefficient of agreement for nominal scales. Educational
and Psychological Measurement, 20:37-46

McHugh M.L. (2012) Interrater reliability: the kappa statistic.
Biochemia medica, 22(3):276–282.

Kullback, S., R.A. Leibler (1951). On information and sufficiency. Annals of
Mathematical Statistics. 22(1):79–86
}
\author{
Jeffrey S. Evans  \href{mailto:sage_insights@outlook.com}{sage_insights@outlook.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hsp.R
\name{hsp}
\alias{hsp}
\title{Hierarchical Slope Position}
\usage{
hsp(
  x,
  min.scale = 3,
  max.scale = 27,
  inc = 4,
  win = "rectangle",
  normalize = FALSE
)
}
\arguments{
\item{x}{A terra SpatRaster class object}

\item{min.scale}{Minimum scale (window size)}

\item{max.scale}{Maximum scale (window size)}

\item{inc}{Increment to increase scales}

\item{win}{Window type, options are "rectangle" or "circle"}

\item{normalize}{Normalize results to 0-1 scale (FALSE | TRUE)}
}
\value{
terra SpatRaster class object of slope position
}
\description{
Calculates a hierarchical scale decomposition of topographic 
position index
}
\details{
if win  = "circle" units are distance, if win = "rectangle" units 
are number of cells
}
\examples{
\donttest{ 
  library(terra)
  elev <- rast(system.file("extdata/elev.tif", package="spatialEco"))
  hsp27 <- hsp(elev, 3, 27, 4, normalize = TRUE)
  plot(hsp27)
 }

}
\references{
Murphy M.A., J.S. Evans, and A.S. Storfer (2010) Quantify Bufo boreas 
  connectivity in Yellowstone National Park with landscape genetics. 
  Ecology 91:252-261
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sample.annulus.R
\name{sample.annulus}
\alias{sample.annulus}
\title{Sample annulus}
\usage{
sample.annulus(x, r1, r2, size = 10, ...)
}
\arguments{
\item{x}{An sf POINT class object}

\item{r1}{Numeric value defining inner radius of annulus 
(in projection units)}

\item{r2}{Numeric value defining outer radius of annulus 
(in projection units)}

\item{size}{Number of samples}

\item{...}{Additional arguments passed to sf::st_sample}
}
\value{
sf POINTS object
}
\description{
Creates sample points based on annulus with defined 
inner and outer radius
}
\details{
Function can be used for distance based sampling which is a sampling method 
that can be used to capture spatially lagged variation.
}
\examples{
 library(sf)
 if(require(sp, quietly = TRUE)) {
   data(meuse, package = "sp")
   meuse <- st_as_sf(meuse, coords = c("x", "y"), crs = 28992, 
                     agr = "constant")

 xy <- meuse[2,]
 rs100 <- sample.annulus(xy, r1=50, r2=100, size = 50)
 rs200 <- sample.annulus(xy, r1=100, r2=200, size = 50)
 
 plot(st_geometry(rs200), pch=20, col="red")
   plot(st_geometry(rs100), pch=20, col="blue", add=TRUE)
   plot(st_geometry(xy), pch=20, cex=2, col="black", add=TRUE)
 legend("topright", legend=c("50-100m", "100-200m", "source"), 
        pch=c(20,20,20), col=c("blue","red","black"))

\donttest{
# Run on multiple points
rs100 <- sample.annulus(meuse[1:3,], r1=50, r2=100, 
                        size = 50)
rs200 <- sample.annulus(meuse[1:3,], r1=50, r2=200, 
                        size = 50)
plot(st_geometry(rs200), pch=20, col="red")
  plot(st_geometry(rs100), pch=20, col="blue", add=TRUE)
    plot(st_geometry(meuse[1:3,]), pch=20, cex=2, col="black", add=TRUE)
 legend("topright", legend=c("50-100m", "100-200m", "source"), 
        pch=c(20,20,20), col=c("blue","red","black"))
}
} else { 
  cat("Please install sp package to run example", "\n")
}

}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cross.tab.R
\name{cross.tab}
\alias{cross.tab}
\title{Class comparison between two nominal rasters}
\usage{
cross.tab(x, y, values = NULL, labs = NULL, pct = FALSE, ...)
}
\arguments{
\item{x}{A terra SpatRaster class object}

\item{y}{A terra SpatRaster class object to compare to x}

\item{values}{Expected values in both rasters}

\item{labs}{Labels associated with values argument}

\item{pct}{(TRUE/FALSE) return proportions rather than counts}

\item{...}{Additional arguments}
}
\value{
a table with the cross tabulated counts
}
\description{
Creates a labeled cross tabulation between two nominal rasters
}
\details{
This function returns a cross tabulation between two nominal rasters. 
Arguments allow for labeling the results and returning proportions 
rather than counts. It also accounts for asymmetrical classes between
the two rasters
}
\examples{
library(terra)
 
e <- ext(179407.8, 181087.9, 331134.4, 332332.1)
lulc2010 <- rast(e, resolution=20)
  lulc2010[] <- sample(1:5, ncell(lulc2010), replace=TRUE)
lulc2020 <- rast(e, resolution=20)
  lulc2020[] <- sample(1:5, ncell(lulc2020), replace=TRUE)
 
 ( v = sort(unique(c(lulc2010[], lulc2020[]))) )
 l = c("water","urban","forest",
       "ag","barren")

cross.tab(lulc2010, lulc2020) 
cross.tab(lulc2010, lulc2020, values = v, labs = l)
cross.tab(lulc2010, lulc2020, values = v, labs = l, pct=TRUE)

# Create asymmetrical classes 
na.idx <- which(!is.na(lulc2010[]))
lulc2020[na.idx] <- sample(c(1,2,4,5), length(na.idx), replace=TRUE)
cross.tab(lulc2010, lulc2020, values = v, labs = l, pct=TRUE)


}
\references{
Pontius Jr, R.G., Shusas, E., McEachern, M. (2004). Detecting important categorical land changes
}
\author{
Jeffrey S. Evans  <sage_insights@outlook.com>
}
